package game.core.gui;

import game.engine.GUIEvent;
import game.engine.GUIEventListener;
import game.engine.GameflowController;
import game.engine.geometry.GeometryFactory;

import java.awt.BorderLayout;
import java.lang.reflect.InvocationTargetException;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;

public class LoadingPanel extends ImagePanel {

	private static LoadingPanel instance = null;

	public static final String PANEL_ID = "LoadingPanel";

	public static final String ICON_ID = "loading_game";

	private Border border = BorderFactory.createLoweredBevelBorder();

	private Vector guiEventListenerList = new Vector(1, 10);
	
	private JTextField description = new JTextField();

	private LoadingPanel(String iconName) {
		super(iconName);
		this.createPanel();
		//this.addGUIEventListener(GameModel.getInstance());
		
	}

	public static LoadingPanel getInstance() {
		if (instance == null) {
			instance = new LoadingPanel(LoadingPanel.ICON_ID);
		}

		return instance;
	}

	private void createPanel() {
		this.setLayout(new BorderLayout());

		JScrollPane textScroll  = new JScrollPane(description);
		textScroll.setOpaque(false);
		description.setOpaque(false);
		
		//this.add(textScroll, BorderLayout.CENTER);
		this.add(this.createProgressPanel(), BorderLayout.SOUTH);
	}

	private JPanel createProgressPanel() {
		JPanel progressPanel = new JPanel();
		progressPanel.setOpaque(false);

		JProgressBar progressBar = new JProgressBar();
		progressBar.setBorder(border);
		progressBar.setOpaque(false);

		progressPanel.add(Box.createVerticalStrut(10), BorderLayout.NORTH);
		progressPanel.add(Box.createHorizontalStrut(10), BorderLayout.WEST);
		progressPanel.add(progressBar, BorderLayout.CENTER);
		progressPanel.add(Box.createHorizontalStrut(10), BorderLayout.EAST);
		progressPanel.add(Box.createVerticalStrut(10), BorderLayout.SOUTH);

		return progressPanel;
	}

	public void loadSession(String name) {
		
	}

	public void addGUIEventListener(GUIEventListener guiEventListener) {
		this.guiEventListenerList.addElement(guiEventListener);
	}

	public void removeGUIEventListener(GUIEventListener guiEventListener) {
		this.guiEventListenerList.removeElement(guiEventListener);
	}

	public void fireGUIEvent(GUIEvent event) {

		for (int i = 0; i < this.guiEventListenerList.size(); i++) {
			GUIEventListener guiEventListener = (GUIEventListener) this.guiEventListenerList
					.elementAt(i);

			guiEventListener.executeGUIEvent(event);
		}

	}

}
