package game.core.gui;

import game.core.IconProperties;
import game.engine.GUIEvent;
import game.engine.GUIEventListener;
import game.engine.util.IconFactory;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.DisplayMode;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Toolkit;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

public class MainFrame extends JFrame {

	private static final int DONT_CARE = -1;

	private static final int WIDTH = 1024;

	private static final int HEIGHT = 768;

	private static MainFrame instance = null;

	private IconFactory iconFactory = IconFactory.getFactory();

	public final CardLayout CARD_LAYOUT = new CardLayout();

	public final JPanel CONTENT_PANEL = new JPanel();

	private Vector GUIEventListenerList = new Vector(1, 10);

	private int width = -1;

	private int height = -1;

	private MainFrame() {
		super("Descent: Journeys in the Dark");
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		this.setSize((int) screenSize.getWidth(), (int) screenSize.getHeight());

		ImageIcon titleIcon = iconFactory.getIcon("title_icon");
		this.setIconImage(titleIcon.getImage());

		this.centerWindow();
		this.setResizable(false);
	}

	public void createLayout() {

		this.CONTENT_PANEL.setLayout(this.CARD_LAYOUT);
		this.CONTENT_PANEL.add(MainPanel.getInstance(), MainPanel.PANEL_ID);
		this.CONTENT_PANEL.add(LoadGamePanel.getInstance(),
				LoadGamePanel.PANEL_ID);
		this.CONTENT_PANEL.add(LoadingPanel.getInstance(),
				LoadingPanel.PANEL_ID);
		this.CONTENT_PANEL.add(DescentGamePanel.getInstance(),
				DescentGamePanel.PANEL_ID);

		Container contentPane = this.getContentPane();
		contentPane.setLayout(new BorderLayout());
		contentPane.add(this.CONTENT_PANEL, BorderLayout.CENTER);

		this.CARD_LAYOUT.first(this.CONTENT_PANEL);
	}

	private void centerWindow() {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = this.getSize();

		if (frameSize.width > screenSize.width) {
			frameSize.width = screenSize.width;
		}

		if (frameSize.height > screenSize.height) {
			frameSize.height = screenSize.height;
		}

		this.width = frameSize.width;
		this.height = frameSize.height;

		this.setLocation((screenSize.width - frameSize.width) >> 1,
				(screenSize.height - frameSize.height) >> 1);
	}

	public void fireGUIEvent(GUIEvent event) {

		for (int i = 0; i < this.GUIEventListenerList.size(); i++) {
			GUIEventListener listener = (GUIEventListener) this.GUIEventListenerList
					.elementAt(i);

			listener.executeGUIEvent(event);
		}
	}

	public CardLayout getLayoutManager() {
		return this.CARD_LAYOUT;
	}

	public int getMainFrameHeight() {
		return this.height;
	}

	public int getMainFrameWidth() {
		return this.width;
	}

	public static MainFrame getInstance() {
		if (instance == null) {
			instance = new MainFrame();
		}

		return instance;
	}

	public static void main(String args[]) {
		MainFrame mainFrame = MainFrame.getInstance();

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		//mainFrame.setUndecorated(true);

		mainFrame.createLayout();
		mainFrame.setVisible(true);
		
		 /*GraphicsDevice usedDevice = GraphicsEnvironment
				.getLocalGraphicsEnvironment().getDefaultScreenDevice();
		usedDevice.setFullScreenWindow(mainFrame);
		usedDevice.setDisplayMode(findDisplayMode(usedDevice.getDisplayModes(),
				MainFrame.WIDTH, MainFrame.HEIGHT, usedDevice.getDisplayMode()
						.getBitDepth(), usedDevice.getDisplayMode()
						.getRefreshRate()));*/

	}

	private static DisplayMode findDisplayMode(DisplayMode[] displayModes,
			int requestWidth, int requestHeight, int requestDepth,
			int requestFreshRate) {

		DisplayMode displayMode = findDisplayModeInternal(displayModes,
				requestWidth, requestHeight, requestDepth, requestFreshRate);

		if (displayMode == null) {
			displayMode = findDisplayModeInternal(displayModes, requestWidth,
					requestHeight, MainFrame.DONT_CARE, MainFrame.DONT_CARE);
		}

		if (displayMode == null) {
			displayMode = findDisplayModeInternal(displayModes, requestWidth,
					MainFrame.DONT_CARE, MainFrame.DONT_CARE,
					MainFrame.DONT_CARE);
		}

		if (displayMode == null) {
			displayMode = findDisplayModeInternal(displayModes,
					MainFrame.DONT_CARE, MainFrame.DONT_CARE,
					MainFrame.DONT_CARE, MainFrame.DONT_CARE);
		}

		return null;
	}

	private static DisplayMode findDisplayModeInternal(DisplayMode[] displayModes,
			int requestWidth, int requestHeight, int requestDepth,
			int requestRefreshRate) {

		DisplayMode displayModeToUse = null;

		for (int i = 0; i < displayModes.length; i++) {
			DisplayMode displayMode = displayModes[i];

			if ((requestWidth == MainFrame.DONT_CARE || displayMode.getWidth() == requestWidth)
					&& (requestHeight == MainFrame.DONT_CARE || displayMode
							.getHeight() == requestHeight)
					&& (requestDepth == MainFrame.DONT_CARE || displayMode
							.getBitDepth() == requestDepth)) {

				displayModeToUse = displayMode;

			}
		}

		return displayModeToUse;
	}
}
