package game.core.gui.component;

import game.engine.GameflowController;
import game.engine.PlayGround;
import game.engine.SceneGraph;
import game.engine.geometry.Cube;
import game.engine.geometry.GeometryFactory;
import game.engine.geometry.Quad;
import game.engine.renderer.BufferController;
import game.engine.renderer.Camera;
import game.engine.renderer.GarbageCollector;
import game.engine.renderer.Renderer;
import game.engine.renderer.RendererFactory;
import game.engine.renderer.TextureController;
import game.engine.renderer.ViewFrustrum;
import game.engine.spatial.LinearAlgebra4D;
import game.engine.spatial.Transformation;

import java.awt.Component;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.SwingUtilities;

import net.java.games.jogl.GL;
import net.java.games.jogl.GLCanvas;
import net.java.games.jogl.GLDrawable;
import net.java.games.jogl.GLEventListener;
import net.java.games.jogl.GLU;

public class DrawableGLEventListener implements GLEventListener, KeyListener,
		MouseListener {

	private GLCanvas glc = null;

	private GL gl = null;

	private GLU glu = null;

	private Renderer renderer = null;

	private Camera camera = null;

	public float[] translateZ = { 0, 0, 1, 1 };

	public float[] translateInversZ = { 0, 0, -1, 1 };

	public float[] translateX = { 1, 0, 0, 1 };

	public float[] translateInversX = { -1, 0, 0, 1 };

	public void init(GLDrawable drawable) {

		float[] clearColor = { 1f, 1f, 1f, 1.0f };

		camera = new Camera();

		renderer = RendererFactory.createRenderer(Renderer.OPEN_GL, drawable);
		renderer.setBackgroundColor(clearColor);
		renderer.setClearDepth(1);

		GarbageCollector.createInstance(renderer);

		renderer.setCamera(camera);
		renderer.initRenderer();

		GeometryFactory.createFactory(renderer);
		TextureController.createTextureController(renderer);
		BufferController.createBufferController(renderer);

		GameflowController.getInstance().addGameflowEventListener(
				this.renderer.sceneGraph);

		renderer.addUserInputEventListener(GameflowController.getInstance());

		ViewFrustrum.createViewFrustrum(camera, this.renderer.sceneGraph);
	}

	public void display(GLDrawable drawable) {
		GL gl = drawable.getGL();
		GLU glu = drawable.getGLU();

		this.gl = gl;
		this.glu = glu;

		if (renderer.beginScene()) {
			renderer.drawScene(renderer.sceneGraph);
			renderer.endScene();
		}

		this.getGlc().repaint();
	}

	public void reshape(GLDrawable arg0, int arg1, int arg2, int arg3, int arg4) {

	}

	public void displayChanged(GLDrawable arg0, boolean arg1, boolean arg2) {

	}

	public GLCanvas getGlc() {
		return glc;
	}

	public void setGlc(GLCanvas glc) {
		this.glc = glc;
	}

	public void keyTyped(KeyEvent e) {
		if (e.getKeyChar() == KeyEvent.VK_8) {

			if (check(camera.eyePoint, translateZ)) {
				Transformation trans = camera.cameraTransformation;
				trans.setIdentity();

				trans.translate(translateZ);

				camera.applyTransformationToEyePoint();
				camera.applyTransformationToLookAtPoint();
			}
		}

		if (e.getKeyChar() == KeyEvent.VK_5) {

			if (check(camera.eyePoint, translateInversZ)) {
				Transformation trans = camera.cameraTransformation;
				trans.setIdentity();

				trans.translate(translateInversZ);

				camera.applyTransformationToEyePoint();
				camera.applyTransformationToLookAtPoint();
			}
		}

		if (e.getKeyChar() == KeyEvent.VK_4) {
			Transformation trans = camera.cameraTransformation;
			trans.setIdentity();

			float[] destPoint = new float[4];

			LinearAlgebra4D.calculatePointVectorSum(camera.eyePoint,
					LinearAlgebra4D.UNION_VECTOR_Y, destPoint);

			trans.rotateDegree(camera.eyePoint, destPoint, 15);

			camera.applyTransformationToLookAtPoint();

			float[] directionVector = new float[4];
			float[] p1 = { camera.eyePoint[0], camera.lookAtPoint[1],
					camera.eyePoint[2], 1 };

			LinearAlgebra4D.calculateVector(p1, camera.lookAtPoint,
					directionVector);
			LinearAlgebra4D.normalizeVector(directionVector, translateZ);
			LinearAlgebra4D.calculateInversVector(translateZ, translateInversZ);

			float[] translate2 = new float[4];
			LinearAlgebra4D.calculateInversVector(p1, translate2);

			trans.setIdentity();
			trans.translate(translate2);
			trans.rotateDegree(LinearAlgebra4D.ORIGIN,
					LinearAlgebra4D.UNION_VECTOR_Y, 90);

			float[] temp = new float[4];

			LinearAlgebra4D.calculateTableVectorProduct(trans.compositeMatrix,
					directionVector, temp);
			LinearAlgebra4D.normalizeVector(temp, translateX);

			trans.setIdentity();
			trans.translate(translate2);
			trans.rotateDegree(LinearAlgebra4D.ORIGIN,
					LinearAlgebra4D.UNION_VECTOR_Y, -90);

			LinearAlgebra4D.calculateTableVectorProduct(trans.compositeMatrix,
					directionVector, temp);
			LinearAlgebra4D.normalizeVector(temp, translateInversX);

		}

		if (e.getKeyChar() == KeyEvent.VK_6) {
			Transformation trans = camera.cameraTransformation;
			trans.setIdentity();

			float[] destPoint = new float[4];

			LinearAlgebra4D.calculatePointVectorSum(camera.eyePoint,
					LinearAlgebra4D.UNION_VECTOR_Y, destPoint);

			trans.rotateDegree(camera.eyePoint, destPoint, -15);

			camera.applyTransformationToLookAtPoint();

			float[] directionVector = new float[4];
			float[] p1 = { camera.eyePoint[0], camera.lookAtPoint[1],
					camera.eyePoint[2], 1 };

			LinearAlgebra4D.calculateVector(p1, camera.lookAtPoint,
					directionVector);
			LinearAlgebra4D.normalizeVector(directionVector, directionVector);
			LinearAlgebra4D.normalizeVector(directionVector, translateZ);
			LinearAlgebra4D.calculateInversVector(translateZ, translateInversZ);

			float[] translate2 = new float[4];
			LinearAlgebra4D.calculateInversVector(p1, translate2);

			trans.setIdentity();
			trans.translate(translate2);
			trans.rotateDegree(LinearAlgebra4D.ORIGIN,
					LinearAlgebra4D.UNION_VECTOR_Y, 90);

			float[] temp = new float[4];

			LinearAlgebra4D.calculateTableVectorProduct(trans.compositeMatrix,
					directionVector, temp);
			LinearAlgebra4D.normalizeVector(temp, translateX);

			trans.setIdentity();
			trans.translate(translate2);
			trans.rotateDegree(LinearAlgebra4D.ORIGIN,
					LinearAlgebra4D.UNION_VECTOR_Y, -90);

			LinearAlgebra4D.calculateTableVectorProduct(trans.compositeMatrix,
					directionVector, temp);
			LinearAlgebra4D.normalizeVector(temp, translateInversX);
		}

		this.getGlc().repaint();
	}

	public void keyPressed(KeyEvent arg0) {
		// TODO Auto-generated method stub

	}

	public void keyReleased(KeyEvent arg0) {
		// TODO Auto-generated method stub

	}

	public Renderer getRenderer() {
		return this.renderer;
	}

	public void mouseClicked(MouseEvent e) {

		if (SwingUtilities.isLeftMouseButton(e)) {
			this.getRenderer().processMouseSelection(e.getX(), e.getY());
		}

	}

	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	private boolean check(float[] vector, float[] direction) {

		float[] temp = vector.clone();
		Transformation trans = new Transformation();
		
		trans.setIdentity();
		trans.translate(direction);
		
		LinearAlgebra4D.calculateTableVectorProduct(
				trans.compositeMatrix, temp, temp);
		
		for (int i = 0; i < 3; i++) {
			if (temp[i] < 3 || temp[i] > 37) {
				return false;
			}
		}

		return true;

	}

}
