package game.core.gui.component;

import game.core.IconProperties;
import game.core.gui.DescentGamePanel;
import game.core.gui.MainFrame;
import game.engine.util.IconFactory;

import java.awt.event.ActionEvent;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JToolBar;

public class OptionToolBar extends JToolBar implements UserInputEventGenerator {

	private ClassLoader cl = this.getClass().getClassLoader();

	private IconProperties iconProp = IconProperties.getInstance();

	private static OptionToolBar instance = null;

	private Vector userInputListener = new Vector(5, 5);

	JButton load_button = null;

	JButton save_button = null;

	JButton quit_button = null;

	private OptionToolBar() {
		this.setOpaque(false);
		this.setFloatable(false);
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.createOptionToolBar();
	}

	public static OptionToolBar getInstance() {
		if (instance == null) {
			instance = new OptionToolBar();
		}

		return instance;
	}

	private void createOptionToolBar() {
		load_button = new JButton(new AbstractAction("", IconFactory
				.getFactory().getIcon("load_action")) {

			public void actionPerformed(ActionEvent arg0) {
				DescentGamePanel.getInstance().glCanvas.requestFocusInWindow();
			}
		});
		load_button.setOpaque(false);
		ImageIcon load_icon = IconFactory.getFactory().getIcon("load_action");
		load_button.setIcon(load_icon);
		load_button.setToolTipText("load game");
		
		save_button = new JButton(new AbstractAction("", IconFactory
				.getFactory().getIcon("load_action")) {

			public void actionPerformed(ActionEvent arg0) {
				DescentGamePanel.getInstance().glCanvas.requestFocusInWindow();
			}
		});
		save_button.setOpaque(false);
		ImageIcon save_icon = IconFactory.getFactory().getIcon("save_action");
		save_button.setIcon(save_icon);
		save_button.setToolTipText("save game");
		
		quit_button = new JButton(new AbstractAction("", IconFactory
				.getFactory().getIcon("load_action")) {

			public void actionPerformed(ActionEvent arg0) {
				MainFrame.getInstance().setVisible(false);
				MainFrame.getInstance().dispose();

				System.exit(0);
			}
		});

		quit_button.setOpaque(false);
		ImageIcon quit_icon = IconFactory.getFactory().getIcon("quit_action");
		quit_button.setIcon(quit_icon);
		quit_button.setToolTipText("quit_game");
		
		this.add(load_button);
		this.add(save_button);
		this.add(Box.createHorizontalStrut(10));
		this.add(quit_button);
	}

	private void lock() {
		load_button.setEnabled(false);
	}

	private void unlock() {
		load_button.setEnabled(true);
		// WorldFrame.glCanvas.requestFocusInWindow();
	}

	public void addUserInputEventListener(Object object) {
		this.userInputListener.addElement(object);
	}

	public void removeUserInputEventListener(Object object) {
		this.userInputListener.removeElement(object);
	}

	public void fireUserInputEvent(UserInputEvent userInputEvent) {
		for (int i = 0; i < this.userInputListener.size(); i++) {
			UserInputListener userInputListener = (UserInputListener) this.userInputListener
					.elementAt(i);

			userInputListener.executeUserEvent(userInputEvent);
		}
	}
}
