package game.core.gui.component;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.util.Vector;

import game.core.MessageProperties;
import game.engine.GameflowController;
import game.engine.Node;
import game.engine.PlayGroundElement;
import game.engine.SelectionController;
import game.engine.creatures.Creature;
import game.engine.creatures.CreatureFactory;
import game.engine.util.IconFactory;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

public class StatusPanel extends JPanel {

	private static StatusPanel instance = null;

	private JLabel label_hero = new JLabel();

	private Border border = BorderFactory.createLoweredBevelBorder();

	private Border raisedBorder = BorderFactory.createRaisedBevelBorder();

	private JLabel label_melee = null;

	private JLabel label_ranged = null;

	private JLabel label_magic = null;

	private JLabel label_wounds = null;

	private JLabel label_name = null;

	public JLabel label_status = null;

	private JTextField textField_wounds = null;

	private JTextField textField_armor = null;

	private JTextField textField_points = null;

	private JTextField textField_melee = null;

	private JTextField textField_range = null;

	public TransparentList messageTable = new TransparentList();

	private static final String OBJECT = "object";

	private static final String MESSAGE = "message";

	private static final String TARGET = "target";

	private StatusPanel() {
		this.setLayout(new BorderLayout());
		this.setOpaque(false);

		this.add(Box.createVerticalStrut(10), BorderLayout.NORTH);
		this.add(this.createContentPanel(), BorderLayout.CENTER);
		this.add(Box.createHorizontalStrut(10), BorderLayout.WEST);
		this.add(Box.createHorizontalStrut(10), BorderLayout.EAST);
		this.add(Box.createVerticalStrut(10), BorderLayout.SOUTH);
	}

	private JPanel createContentPanel() {
		JPanel contentPanel = new JPanel();
		contentPanel.setLayout(new BorderLayout());
		contentPanel.setOpaque(false);

		contentPanel.add(this.createHeroPanel(), BorderLayout.WEST);
		contentPanel.add(this.createTraitPanel(), BorderLayout.CENTER);
		contentPanel.add(this.createActionPanel(), BorderLayout.EAST);

		return contentPanel;
	}

	private JPanel createHeroPanel() {
		JPanel heroPanel = new JPanel();

		heroPanel.setLayout(new BoxLayout(heroPanel, BoxLayout.X_AXIS));
		heroPanel.setOpaque(false);

		label_hero.setBorder(border);

		ImageIcon icon = IconFactory.getFactory().getIcon("entry");
		label_hero.setIcon(icon);

		heroPanel.add(label_hero);

		return heroPanel;
	}

	private JPanel createTraitPanel() {

		JPanel traitPanel = new JPanel();
		traitPanel.setLayout(null);
		traitPanel.setOpaque(false);
		traitPanel.setBorder(border);

		label_wounds = new JLabel();
		ImageIcon icon_wounds = IconFactory.getFactory().getIcon("wounds");
		label_wounds.setIcon(icon_wounds);
		label_wounds.setText("  wounds: ");
		label_wounds.setForeground(Color.ORANGE);
		label_wounds.setBounds(5, 5, 120, 50);
		label_wounds.setBorder(raisedBorder);

		textField_wounds = new JTextField();
		textField_wounds.setBounds(140, 23, 50, 20);
		textField_wounds.setOpaque(false);
		textField_wounds.setEditable(false);
		textField_wounds.setForeground(Color.WHITE);

		JLabel label_points = new JLabel();
		ImageIcon icon_points = IconFactory.getFactory().getIcon(
				"points_action");
		label_points.setIcon(icon_points);
		label_points.setText("fatigue:");
		label_points.setForeground(Color.ORANGE);
		label_points.setBounds(200, 7, 110, 50);
		label_points.setBorder(raisedBorder);

		textField_points = new JTextField();
		textField_points.setBounds(315, 20, 50, 20);
		textField_points.setOpaque(false);
		textField_points.setEditable(false);
		textField_points.setForeground(Color.WHITE);

		JLabel label_armor = new JLabel();
		ImageIcon icon_armor = IconFactory.getFactory().getIcon("armor");
		label_armor.setIcon(icon_armor);
		label_armor.setText("  armor:");
		label_armor.setForeground(Color.ORANGE);
		label_armor.setBounds(5, 70, 110, 50);
		label_armor.setBorder(raisedBorder);

		textField_armor = new JTextField();
		textField_armor.setBounds(140, 85, 50, 20);
		textField_armor.setOpaque(false);
		textField_armor.setEditable(false);
		textField_armor.setForeground(Color.WHITE);

		traitPanel.add(label_wounds);
		traitPanel.add(textField_wounds);
		traitPanel.add(label_armor);
		traitPanel.add(textField_armor);
		traitPanel.add(label_points);
		traitPanel.add(textField_points);

		label_melee = new JLabel();
		ImageIcon icon_melee = IconFactory.getFactory().getIcon("melee_icon");
		label_melee.setIcon(icon_melee);
		label_melee.setText("melee: ");
		label_melee.setForeground(Color.ORANGE);
		label_melee.setBounds(5, 135, 110, 50);
		label_melee.setBorder(raisedBorder);

		textField_melee = new JTextField();
		textField_melee.setBounds(140, 155, 50, 20);
		textField_melee.setOpaque(false);
		textField_melee.setEditable(false);
		textField_melee.setForeground(Color.WHITE);

		label_ranged = new JLabel();
		ImageIcon range_icon = IconFactory.getFactory().getIcon("range_icon");
		label_ranged.setIcon(range_icon);
		label_ranged.setText("range: ");
		label_ranged.setForeground(Color.ORANGE);
		label_ranged.setBounds(200, 135, 110, 50);
		label_ranged.setBorder(raisedBorder);

		textField_range = new JTextField();
		textField_range.setBounds(315, 155, 50, 20);
		textField_range.setOpaque(false);
		textField_range.setEditable(false);
		textField_range.setForeground(Color.WHITE);

		traitPanel.add(label_melee);
		traitPanel.add(textField_melee);
		traitPanel.add(label_ranged);
		traitPanel.add(textField_range);

		label_name = new JLabel();
		label_name.setText("  name: ");
		label_name.setForeground(Color.ORANGE);
		label_name.setBounds(390, 5, 130, 50);
		label_name.setBorder(raisedBorder);

		label_status = new JLabel();
		label_status.setForeground(Color.ORANGE);
		label_status.setBounds(390, 135, 130, 50);
		label_status.setBorder(raisedBorder);
		label_status.setName(" status: ");
		
		//traitPanel.add(label_name);
		traitPanel.add(label_status);

		return traitPanel;
	}

	private JPanel createActionPanel() {
		JPanel actionPanel = new JPanel();
		actionPanel.setLayout(new BorderLayout());
		actionPanel.setOpaque(false);
		actionPanel.setBorder(border);

		messageTable.setEnabled(false);
		JScrollPane tableScroll = new JScrollPane(messageTable);
		tableScroll.setMaximumSize(new Dimension(250, 100));
		tableScroll.setPreferredSize(new Dimension(250, 100));
		tableScroll.setOpaque(false);

		actionPanel.add(tableScroll, BorderLayout.EAST);

		return actionPanel;
	}

	public final void addNewMessage(ImageIcon icon, String message) {

		Object[] messageField = new Object[2];
		messageField[0] = message;
		messageField[1] = icon;

		DefaultListModel listModel = (DefaultListModel) messageTable.getModel();
		listModel.addElement(messageField);

	}

	public final void setValues(PlayGroundElement element) {

		Creature creature = CreatureFactory.getCreature(element.creatureInfo);

		this.label_hero.setIcon(creature.icon);
		this.textField_melee.setText("" + creature.melee_trait);
		this.textField_range.setText("" + creature.ranged_trait);

		this.textField_armor.setText("" + creature.armor);
		this.textField_points.setText("" + creature.speed_trait);
		this.textField_wounds.setText("" + element.wounds);

		this.label_name.setText("   " + creature.name);

		this.label_status.setIcon(null);
		
		if (element.action == Creature.ACTION_1) {
			this.label_status.setIcon(IconFactory.getFactory().getIcon(
					creature.actionIcon_1));
		}

		if (element.action == Creature.ACTION_2) {
			this.label_status.setIcon(IconFactory.getFactory().getIcon(
					creature.actionIcon_2));
		}

		if (element.action == Creature.ACTION_3) {
			this.label_status.setIcon(IconFactory.getFactory().getIcon(
					creature.actionIcon_3));
		}

		if (element.action == Creature.ACTION_4) {
			this.label_status.setIcon(IconFactory.getFactory().getIcon(
					creature.actionIcon_4));
		}
	}

	public static StatusPanel getInstance() {
		if (instance == null) {
			instance = new StatusPanel();
		}

		return instance;
	}
}
