package game.engine;

import game.core.MessageProperties;
import game.core.gui.LoadGamePanel;
import game.core.gui.LoadingPanel;
import game.core.gui.MainFrame;
import game.core.gui.component.ControllerToolBar;
import game.core.gui.component.StatusPanel;
import game.core.gui.component.TransparentListModel;
import game.core.gui.component.UserInputEvent;
import game.core.gui.component.UserInputListener;
import game.engine.ai.PathFinder;
import game.engine.creatures.Creature;
import game.engine.creatures.CreatureFactory;
import game.engine.geometry.GeometryFactory;
import game.engine.spatial.LinearAlgebra4D;
import game.engine.spatial.Spatial;
import game.engine.spatial.Transformation;
import game.engine.util.IconFactory;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Random;
import java.util.Vector;

public class GameflowController implements UserInputListener {

	private Vector gameflowListenerList = new Vector();

	public Vector monsterList = new Vector();

	public Hashtable heroTable = new Hashtable();

	public Vector heroList = new Vector();

	public GameflowQueue queue = new GameflowQueue();

	private static GameflowController instance = null;

	private Vector creatureEventList = new Vector();

	private String path = null;

	private Random random = new Random();
	
	public Vector entryPointList = new Vector();

	int level = -1;

	public final void createPlayGround(int level) {
		GameflowEvent event = new GameflowEvent();

		this.level = level;

		event.message = GameflowEvent.INITIAL_PLAYGROUND;
		event.playGroundLevel = level;

		this.queue.addEventToQueue(event);
	}

	public final static GameflowController getInstance() {
		if (instance == null) {
			instance = new GameflowController();
		}

		return instance;
	}

	public final void removeGameElement(Node node) {

		GameflowEvent event = new GameflowEvent();

		event.message = GameflowEvent.REMOVE_GAME_ELEMENT;
		event.node = node;

		this.queue.addEventToQueue(event);
	}

	public final void removeTemporaryGameElement(Node node) {
		GameflowEvent event = new GameflowEvent();

		event.message = GameflowEvent.REMOVE_TEMPORARY_GAME_ELEMENT;
		event.node = node;

		this.queue.addEventToQueue(event);
	}

	public final void addGameflowEventListener(
			GameflowListener gameflowEventListener) {
		this.gameflowListenerList.add(gameflowEventListener);
	}

	public final void fireGameflowEvent(GameflowEvent event) {

		for (int i = 0; i < this.gameflowListenerList.size(); i++) {
			GameflowListener gameflowListener = (GameflowListener) this.gameflowListenerList
					.elementAt(i);
			gameflowListener.executeGameflowEvent(event);
		}
	}

	public final void startExecution() {
		for (int i = 0; i < this.queue.size(); i++) {
			GameflowEvent event = (GameflowEvent) this.queue
					.getEventFromQueue();
			this.fireGameflowEvent(event);
		}
	}

	public final void executeUserEvent(UserInputEvent event) {
		int message = event.getMessage();

		if (message == UserInputEvent.NEXT_TURN) {
			this.exectueNextTurnEvent();
		}

		if (message == UserInputEvent.DISPLAY_MOVEMAP) {
			this.displayMoveMap(event);
		}

		if (message == UserInputEvent.REMOVE_MONSTER) {
			this.removeMonster(event.node);
		}

		if (message == UserInputEvent.DISPLAY_MARK) {
			this.displayMark(event.playGroundElement);
		}

		event = null;
	}

	private final void displayMark(PlayGroundElement playGroundElement) {

		GameflowEvent gameflowEvent = new GameflowEvent();
		gameflowEvent.message = GameflowEvent.ADD_NEW_GAME_ELEMENT;
		gameflowEvent.playGroundElement = playGroundElement;

		this.queue.addEventToQueue(gameflowEvent);
	}

	private final void removeMonster(Node node) {
		GameflowEvent gameflowEvent = new GameflowEvent();
		gameflowEvent.message = GameflowEvent.REMOVE_GAME_ELEMENT;
		gameflowEvent.node = node;

		this.queue.addEventToQueue(gameflowEvent);
	}

	private final void displayMoveMap(UserInputEvent event) {

		PlayGroundElement playGroundElement = new PlayGroundElement(0, 0, 0,
				GeometryFactory.MOVE_MAP);

		playGroundElement.element = event.playGroundElement.element;
		playGroundElement.moveMapList = event.moveMap;

		GameflowEvent gameflowEvent = new GameflowEvent();
		gameflowEvent.message = GameflowEvent.ADD_NEW_GAME_ELEMENT;
		gameflowEvent.playGroundElement = playGroundElement;

		this.queue.addEventToQueue(gameflowEvent);

	}

	private final void exectueNextTurnEvent() {
		
		this.testHeroDead();
		this.testHeroWin();
		TransparentListModel model = (TransparentListModel) StatusPanel
				.getInstance().messageTable.getModel();
		model.removeAllElements();

		StatusPanel.getInstance().addNewMessage(
				IconFactory.getFactory().getIcon("next_turn_action"),
				MessageProperties.getInstance().getProperty("next_turn"));

		this.initializeSpecialAbility();
		this.executeSpecialAbility();
		this.executeMonsterAttack();
		//this.executeUniqueMonsterAttack();
		this.testHeroDead();
		this.resetSpecialAbilities();

		ControllerToolBar.getInstance().currentHero = 0;

		Node node = SelectionController.getInstance().formerSelectedHero;

		if (node != null) {
			if (node.playGroundelement.moveMap != null) {
				this.removeTemporaryGameElement(node.playGroundelement.moveMap);
			}
		}

		MotionMonitor.getInstance().currentIndex = 0;
		MotionMonitor.getInstance().monsterMovment = true;
		this.executeMonsterList();

		this.resetHeroValues();
		this.executeEntryPointList();

		SelectionController.getInstance().selectHero(
				(Node) this.heroList.elementAt(0));

		GameflowEvent event = new GameflowEvent();
		event.message = GameflowEvent.QUEUE_EMPTY;

		this.queue.addEventToQueue(event);

	}

	private final void testHeroWin() {

		for (int i = 25; i < 28; i++) {
			for (int j = 37; j < 39; j++) {
				int charakter = PlayGround.getInstance().playGround[i][j];

				if (charakter == PlayGround.HERO) {
					MainFrame.getInstance().CARD_LAYOUT.show(MainFrame
							.getInstance().CONTENT_PANEL,
							LoadGamePanel.PANEL_ID);
				}
			}
		}
	}

	private final void testHeroDead() {
		for (int i = 0; i < this.heroList.size(); i++) {
			Node heroNode = (Node) this.heroList.elementAt(i);

			PlayGroundElement hero = heroNode.playGroundelement;

			if (hero.wounds <= 0) {
				MainFrame.getInstance().CARD_LAYOUT.show(MainFrame
						.getInstance().CONTENT_PANEL, LoadingPanel.PANEL_ID);
			}
		}
	}

	private final void executeUniqueMonsterAttack() {
		Vector targetList = new Vector();

		for (int i = 0; i < this.heroList.size(); i++) {
			Node heroNode = (Node) this.heroList.elementAt(i);

			PlayGroundElement hero = heroNode.playGroundelement;

			if (hero.useSpecialAbility || !hero.hasMoved) {
				targetList.addElement(heroNode);
			}
		}

		Random r = new Random();
		int index = r.nextInt(5);

		if (index < targetList.size()) {

			Node heroNode = (Node) targetList.elementAt(index);
			PlayGroundElement heroPlayGroundElement = heroNode.playGroundelement;

			Creature octopus = CreatureFactory
					.getCreature(GeometryFactory.OCTOPUS);
			Creature hero = CreatureFactory
					.getCreature(heroPlayGroundElement.creatureInfo);

			int hitValue = octopus.melee_trait + r.nextInt(7);

			int hitCount = hitValue - hero.armor;

			if (hitCount > 0) {

				heroPlayGroundElement.wounds -= hitCount;

				StatusPanel.getInstance().addNewMessage(
						IconFactory.getFactory().getIcon(hero.smallIcon),
						hero.name
								+ " "
								+ MessageProperties.getInstance().getProperty(
										"hurt") + " " + octopus.name + " ( "
								+ hitCount + " ) ");
			}

		}
	}

	private final void initializeSpecialAbility() {

		for (int i = 0; i < this.heroList.size(); i++) {
			Node heroNode = (Node) this.heroList.elementAt(i);

			PlayGroundElement heroPlayGroundElement = heroNode.playGroundelement;

			if (heroPlayGroundElement.useSpecialAbility) {
				Creature heroCreature = CreatureFactory
						.getCreature(heroNode.playGroundelement.geometryFactoryType);

				CreatureEventElement creatureEvent = new CreatureEventElement();
				creatureEvent.actionIndex = heroPlayGroundElement.action;
				creatureEvent.node = heroNode;

				switch (heroPlayGroundElement.action) {
				case Creature.ACTION_1: {
					creatureEvent.counter = heroCreature.action_1_time;
					creatureEvent.actionProcessTime = heroCreature.action_process_time_1;
					break;
				}

				case Creature.ACTION_2: {
					creatureEvent.counter = heroCreature.action_2_time;
					creatureEvent.actionProcessTime = heroCreature.action_process_time_2;
					break;
				}

				case Creature.ACTION_3: {
					creatureEvent.counter = heroCreature.action_3_time;
					creatureEvent.actionProcessTime = heroCreature.action_process_time_3;
					break;
				}

				case Creature.ACTION_4: {
					creatureEvent.counter = heroCreature.action_4_time;
					creatureEvent.actionProcessTime = heroCreature.action_process_time_4;
					break;
				}
				}

				this.creatureEventList.addElement(creatureEvent);
			}
		}
	}

	private final void executeSpecialAbility() {

		for (int i = 0; i < this.creatureEventList.size(); i++) {
			CreatureEventElement creatureEvent = (CreatureEventElement) this.creatureEventList
					.elementAt(i);

			if (creatureEvent.counter <= creatureEvent.actionProcessTime) {

				Creature heroCreature = CreatureFactory
						.getCreature(creatureEvent.node.playGroundelement.geometryFactoryType);
				PlayGroundElement heroPlayGroundElement = creatureEvent.node.playGroundelement;

				Node heroNode = creatureEvent.node;

				switch (creatureEvent.actionIndex) {
				case Creature.ACTION_1: {
					heroCreature.useSpecialAbility1(heroNode);
					break;
				}

				case Creature.ACTION_2: {
					heroCreature.useSpecialAbility2(heroNode);
					break;
				}

				case Creature.ACTION_3: {
					heroCreature.useSpecialAbility3(heroNode);
					break;
				}

				case Creature.ACTION_4: {
					heroCreature.useSpecialAbility4(heroNode);
					break;
				}
				}
			}
		}
	}

	private final void resetSpecialAbilities() {

		int[] indices = new int[this.creatureEventList.size()];
		int i = 0;

		for (int j = 0; j < indices.length; j++) {
			indices[j] = -1;
		}

		for (int j = 0; j < this.creatureEventList.size(); j++) {

			CreatureEventElement creatureEvent = (CreatureEventElement) this.creatureEventList
					.elementAt(j);

			if (creatureEvent.counter == 0) {
				Node heroNode = creatureEvent.node;
				Creature heroCreature = CreatureFactory
						.getCreature(heroNode.playGroundelement.geometryFactoryType);
				PlayGroundElement heroPlayGroundElement = heroNode.playGroundelement;

				switch (creatureEvent.actionIndex) {
				case Creature.ACTION_1: {
					heroCreature.resetSpecialAbility1(heroNode);
					break;
				}

				case Creature.ACTION_2: {
					heroCreature.resetSpecialAbility2(heroNode);
					break;
				}

				case Creature.ACTION_3: {
					heroCreature.resetSpecialAbility3(heroNode);
					break;
				}

				case Creature.ACTION_4: {
					heroCreature.resetSpecialAbility4(heroNode);
					break;
				}
				}

				indices[i++] = j;
			} else {
				creatureEvent.counter--;
			}
		}

		for (int j = indices.length - 1; j >= 0; j--) {
			if (indices[j] != -1) {
				this.creatureEventList.removeElementAt(j);
			}
		}

	}

	private final void executeMonsterAttack() {
		for (int i = 0; i < this.heroList.size(); i++) {
			Node hero = (Node) this.heroList.elementAt(i);
			Creature heroCreature = CreatureFactory
					.getCreature(hero.playGroundelement.geometryFactoryType);
			PlayGroundElement heroPlayGroundElement = hero.playGroundelement;

			Vector surroundingMonsterList = hero.playGroundelement.monsterList;

			for (int j = 0; j < surroundingMonsterList.size(); j++) {
				Node monsterNode = (Node) surroundingMonsterList.elementAt(j);

				PlayGroundElement monsterPlayGroundElement = monsterNode.playGroundelement;
				Creature monsterCreature = CreatureFactory
						.getCreature(monsterPlayGroundElement.geometryFactoryType);

				int monsterAttackValue = monsterCreature.melee_trait
						+ random.nextInt(7);
				int heroDefenseValue = heroCreature.armor;

				int hitValue = monsterAttackValue - heroDefenseValue;

				if (hitValue > 0) {
					heroPlayGroundElement.wounds -= hitValue;

					StatusPanel.getInstance().addNewMessage(
							IconFactory.getFactory().getIcon(
									heroCreature.smallIcon),
							heroCreature.name
									+ " "
									+ MessageProperties.getInstance()
											.getProperty("hurt") + " "
									+ monsterCreature.name + " ( " + hitValue
									+ " ) ");
				}
			}
		}

		for (int i = 0; i < this.monsterList.size(); i++) {

			Node monsterNode = (Node) this.monsterList.elementAt(i);
			PlayGroundElement monsterPlayGroundElement = monsterNode.playGroundelement;
			Creature monsterCreature = CreatureFactory
					.getCreature(monsterPlayGroundElement.geometryFactoryType);

			if (monsterCreature.ranged_trait > 1) {

				Vector heroList = PlayGround.getInstance()
						.getElementsInLineOfSight(
								monsterPlayGroundElement.posX,
								monsterPlayGroundElement.posZ,
								monsterCreature.ranged_trait, PlayGround.HERO);

				if (heroList.size() > 0) {
					Node heroNode = (Node) heroList.elementAt(0);

					PlayGroundElement heroPlayGroundElement = heroNode.playGroundelement;

					Creature heroCreature = CreatureFactory
							.getCreature(heroPlayGroundElement.geometryFactoryType);

					int monsterAttackValue = monsterCreature.melee_trait
							+ random.nextInt(7);
					int heroDefenseValue = heroCreature.armor;

					int hitValue = monsterAttackValue - heroDefenseValue;

					if (hitValue > 0) {
						heroPlayGroundElement.wounds -= hitValue;

						StatusPanel.getInstance().addNewMessage(
								IconFactory.getFactory().getIcon(
										heroCreature.smallIcon),
								heroCreature.name
										+ " "
										+ MessageProperties.getInstance()
												.getProperty("hurt") + " "
										+ monsterCreature.name + " ( "
										+ hitValue + " ) ");
					}

				}
			}
		}
	}

	private final void resetHeroValues() {

		for (int i = 0; i < this.heroList.size(); i++) {
			Node hero = (Node) this.heroList.elementAt(i);

			hero.playGroundelement.hasMoved = false;
			hero.playGroundelement.hasAttacked = false;
			hero.playGroundelement.useSpecialAbility = false;
			hero.playGroundelement.action = Creature.NO_ACTION;
		}
	}

	private final void executeEntryPointList() {
		GeometryFactory.getInstance()
				.createEntryPoints(GeometryFactory.LEVEL_1);
		
		for (int i = 0; i < entryPointList.size(); i++) {

			GameflowEvent event = new GameflowEvent();

			event.message = GameflowEvent.ADD_NEW_GAME_ELEMENT;
			event.playGroundElement = (PlayGroundElement) entryPointList
					.elementAt(i);

			if (PlayGround.getInstance()
					.isPositionFree(event.playGroundElement)) {
				this.queue.addEventToQueue(event);
			}
		}
	}

	private final void executeMonsterList() {
		MotionMonitor motionMonitor = MotionMonitor.getInstance();

		if (this.monsterList.size() > 0) {

			Node monster_node = (Node) this.monsterList.elementAt(0);

			PlayGroundElement playGroundElementMonster = monster_node.playGroundelement;
			motionMonitor.initializeMonsterMovement(monster_node,
					playGroundElementMonster);
		}
	}
}
