package game.engine;

import game.engine.ai.PathFinder;
import game.engine.creatures.Creature;
import game.engine.creatures.CreatureFactory;
import game.engine.spatial.Spatial;
import game.engine.spatial.Transformation;

public class MotionMonitor {

	private static MotionMonitor instance = null;

	public int currentIndex = 0;

	public Node currentNode = null;

	private PlayGroundElement currentPlayGroundElement = null;

	private boolean motionFinished = true;

	public Routemap currentRouteMap = null;

	public boolean monsterMovment = false;

	private MotionMonitor() {

	}

	public final static MotionMonitor getInstance() {
		if (instance == null) {
			instance = new MotionMonitor();
		}

		return instance;
	}

	public final void initializeMonsterMovement(Node node_monster,
			PlayGroundElement playGroundElement_monster) {

		GameflowController controller = GameflowController.getInstance();
		this.setMotionFinished(false);
		this.currentNode = node_monster;
		this.currentPlayGroundElement = playGroundElement_monster;

		if (controller.heroList.size() > 0) {

			int index = -1;
			int norm = Integer.MAX_VALUE;
			int length = -1;

			for (int i = 0; i < controller.heroList.size(); i++) {
				Node currentHero = (Node) controller.heroList.elementAt(i);
				PlayGroundElement playGroundElementHero = currentHero.playGroundelement;

				if ((length = PlayGround.getInstance().pathLength(
						this.currentPlayGroundElement.posX,
						this.currentPlayGroundElement.posZ,
						playGroundElementHero.posX, playGroundElementHero.posZ)) != -1) {

					if (length < norm) {
						norm = length;
						index = i;
					}
				}
			}

			if (index != -1) {
				node_monster.geometricalState = Spatial.CHANGE_POSITION;
				Node currentHero = (Node) controller.heroList.elementAt(index);
				PlayGroundElement currentHero_element = currentHero.playGroundelement;

				Creature creature = CreatureFactory.getCreature(playGroundElement_monster.creatureInfo);
				
				// alter here
				PlayGround.getInstance().calculateMonsterMovement(
						this.currentPlayGroundElement, currentHero_element, creature.speed_trait);

				this.currentPlayGroundElement.targetX = currentHero_element.routeMap.targetX;
				this.currentPlayGroundElement.targetZ = currentHero_element.routeMap.targetZ;

				PlayGround.getInstance().changePosition(
						this.currentPlayGroundElement);

				this.currentRouteMap = (Routemap) currentHero_element.routeMap
						.clone();
			}
		}
	}

	public final void changePosition() {

		if (this.currentNode.geometricalState == Spatial.CHANGE_POSITION) {
			RouteMapElement routeElement = null;

			if (this.currentRouteMap.size() > 0) {

				routeElement = (RouteMapElement) this.currentRouteMap.pop();

				if (routeElement.type == RouteMapElement.TRANSLATE) {
					if (routeElement.direction == RouteMapElement.SOUTH) {
						float[] translate = { 0, 0, routeElement.speed, 1 };

						Transformation transformation = new Transformation();
						transformation.translate(translate);

						this.currentNode
								.updateTransformationState(transformation);
					}

					if (routeElement.direction == RouteMapElement.NORTH) {
						float[] translate = { 0, 0, -routeElement.speed, 1 };

						Transformation transformation = new Transformation();
						transformation.translate(translate);

						this.currentNode
								.updateTransformationState(transformation);
					}

					if (routeElement.direction == RouteMapElement.WEST) {
						float[] translate = { -routeElement.speed, 0, 0, 1 };
						Transformation transformation = new Transformation();
						transformation.translate(translate);

						this.currentNode
								.updateTransformationState(transformation);
					}

					if (routeElement.direction == RouteMapElement.EAST) {
						float[] translate = { routeElement.speed, 0, 0, 1 };
						Transformation transformation = new Transformation();
						transformation.translate(translate);

						this.currentNode
								.updateTransformationState(transformation);
					}

					routeElement = null;
				}
			} else {
				this.setMotionFinished(true);
				this.currentNode.geometricalState = Spatial.STATIC;

				if (this.monsterMovment) {
					this.currentIndex++;

					GameflowController controller = GameflowController
							.getInstance();

					if (this.currentIndex < controller.monsterList.size()) {
						Node node = (Node) controller.monsterList
								.elementAt(this.currentIndex);
						PlayGroundElement nextMonster = node.playGroundelement;

						this.initializeMonsterMovement(node, nextMonster);
					}
				}
			}
		}
	}

	public final boolean isMotionFinished() {
		return motionFinished;
	}

	public final void setMotionFinished(boolean motionFinished) {
		this.motionFinished = motionFinished;
	}
}
