package game.engine;

import java.util.Vector;

import game.engine.geometry.Geometry;
import game.engine.renderer.Renderer;
import game.engine.spatial.LinearAlgebra4D;
import game.engine.spatial.Spatial;
import game.engine.spatial.Transformation;

public class Node extends Spatial {

	public Vector childList = new Vector();
	
	public Node() {
	}

	public final void updateTransformationState(Transformation newWorldTransformation) {
		Transformation.concateTransformations(this.transformationMatrix,
				newWorldTransformation, this.transformationMatrix);

		for (int i = 0; i < childList.size(); i++) {
			Spatial spatial = (Spatial) childList.elementAt(i);
			spatial.updateTransformationState(newWorldTransformation);
		}
	}

	public final void draw(Renderer renderer, boolean isCulled) {
		
		if(this.geometricalState == Spatial.CHANGE_POSITION){
			MotionMonitor.getInstance().changePosition();
		}
		
		for (int i = 0; i < childList.size(); i++) {
			Spatial spatial = (Spatial) childList.elementAt(i);

			if (spatial != null) {
				spatial.onDraw(renderer, false);
			}
		}
	}

	public final static void addChildrenToNode(Node node, Geometry geometry) {
		if (node != null) {
			node.childList.addElement(geometry);
			geometry.parent = node;

			node.updateTransformationState(node.transformationMatrix);
		}

	}

	public final static boolean removeChildrenSpatial(Node node, Spatial spatial) {
		return node.childList.removeElement(spatial);
	}

	public final static void addChildrenToNode(Node node, Node childNode) {
		node.childList.addElement(childNode);
		childNode.parent = node;

		node.updateTransformationState(node.transformationMatrix);
	}
}
