package game.engine;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Stack;
import java.util.Vector;

import game.engine.ai.Node;
import game.engine.ai.PathFinder;
import game.engine.geometry.Geometry;
import game.engine.geometry.GeometryFactory;
import game.engine.renderer.Renderer;
import game.engine.spatial.Spatial;

public class PlayGround {

	private static PlayGround instance = null;

	public static final int STRUCTURE = 1;

	public static final int FREE = 0;

	public static final int HERO = 2;

	public static final int MONSTER = 3;

	public static final int TRIGGER = 4;

	public HashMap geometryControllerMapper = new HashMap();

	public int[][] playGround = null;

	Renderer renderer = null;

	String pathName = null;

	private int horizontalBoxCount = -1;

	private int verticalBoxCount = -1;

	public Hashtable positionObjectMapper = new Hashtable();

	private PlayGround() {

	}

	public final static PlayGround getInstance() {
		if (instance == null) {
			instance = new PlayGround();
		}

		return instance;
	}

	public final void initialPlayGround_1() {

		playGround = new int[40][40];

		this.verticalBoxCount = 40;
		this.horizontalBoxCount = 40;

		for (int i = 0; i < playGround.length; i++) {
			for (int j = 0; j < playGround[i].length; j++) {
				playGround[i][j] = PlayGround.STRUCTURE;
			}
		}

		for (int i = 1; i < 39; i++) {
			playGround[4][i] = PlayGround.FREE;
			playGround[35][i] = PlayGround.FREE;
		}

		for (int i = 1; i < 15; i++) {
			for (int j = 10; j < 30; j++) {
				playGround[j][i] = PlayGround.FREE;
			}
		}

		for (int i = 25; i < 39; i++) {
			for (int j = 10; j < 30; j++) {
				playGround[j][i] = PlayGround.FREE;
			}
		}

		for (int i = 15; i < 25; i++) {
			playGround[15][i] = PlayGround.FREE;
			playGround[16][i] = PlayGround.FREE;
		}

		for (int i = 9; i < 12; i++) {
			for (int j = 12; j < 14; j++) {
				playGround[j][i] = PlayGround.STRUCTURE;
			}
		}

		for (int i = 9; i < 12; i++) {
			for (int j = 19; j < 21; j++) {
				playGround[j][i] = PlayGround.STRUCTURE;
			}
		}
		
		//-----------------------------------------------------
		//brunnen
		  for(int i = 8; i < 11;i++){
			  for(int j = 24; j < 27;j++){
				  playGround[j][i] = PlayGround.STRUCTURE;
			  }
		  }
		
		//-----------------------------------------------------
		//tempel
		  playGround[17][34] = PlayGround.STRUCTURE;
		  playGround[18][35] = PlayGround.STRUCTURE;
		  
		  playGround[19][35] = PlayGround.STRUCTURE;
		  playGround[14][34] = PlayGround.STRUCTURE;
		  playGround[13][34] = PlayGround.STRUCTURE;
		  playGround[13][35] = PlayGround.STRUCTURE;
		  
		//-----------------------------------------------------
		// stairs
		  
		  for(int i = 1; i < 6;i++){
			  playGround[16][i] = PlayGround.STRUCTURE;
		  }
		  
		  for(int i = 16; i < 20;i++){
			  playGround[i][5] = PlayGround.STRUCTURE;
		  }
		  
		  for(int i = 5; i > 2;i--){
			  playGround[19][i] = PlayGround.STRUCTURE;
		  }
		  
		  for(int i = 19; i < 23;i++){
			  playGround[19][3] = PlayGround.STRUCTURE;
		  }
		  
		  
		  playGround[21][1] = PlayGround.STRUCTURE;
		  playGround[21][2] = PlayGround.STRUCTURE;
		  playGround[22][1] = PlayGround.STRUCTURE;
		  playGround[22][2] = PlayGround.STRUCTURE;
		  //-----------------------------------------------------
		  
		for (int i = 9; i < 12; i++) {
			for (int j = 27; j < 29; j++) {
				playGround[j][i] = PlayGround.STRUCTURE;
			}
		}

		for (int i = 30; i < 32; i++) {
			for (int j = 12; j < 14; j++) {
				playGround[j][i] = PlayGround.STRUCTURE;
			}
		}

		for (int i = 30; i < 32; i++) {
			for (int j = 19; j < 21; j++) {
				playGround[j][i] = PlayGround.STRUCTURE;
			}
		}

		for (int i = 30; i < 32; i++) {
			for (int j = 27; j < 29; j++) {
				playGround[j][i] = PlayGround.STRUCTURE;
			}
		}

		for (int i = 1; i < 5; i++) {
			playGround[17][i] = PlayGround.STRUCTURE;
			playGround[18][i] = PlayGround.STRUCTURE;
		}

		for (int i = 1; i < 3; i++) {
			playGround[19][i] = PlayGround.STRUCTURE;
			playGround[20][i] = PlayGround.STRUCTURE;
		}

		for (int i = 3; i < 8; i++) {
			playGround[24][i] = PlayGround.STRUCTURE;
			playGround[25][i] = PlayGround.STRUCTURE;
		}
		
		//corner lamps
		//--------------------------------------------------------------
		playGround[10][13] = PlayGround.STRUCTURE;
		playGround[10][14] = PlayGround.STRUCTURE;
		playGround[11][13] = PlayGround.STRUCTURE;
		playGround[11][14] = PlayGround.STRUCTURE;

		playGround[28][13] = PlayGround.STRUCTURE;
		playGround[28][14] = PlayGround.STRUCTURE;
		playGround[29][13] = PlayGround.STRUCTURE;
		playGround[29][14] = PlayGround.STRUCTURE;
		
		playGround[10][25] = PlayGround.STRUCTURE;
		playGround[10][26] = PlayGround.STRUCTURE;
		playGround[11][25] = PlayGround.STRUCTURE;
		playGround[11][26] = PlayGround.STRUCTURE;

		playGround[28][25] = PlayGround.STRUCTURE;
		playGround[28][26] = PlayGround.STRUCTURE;
		playGround[29][25] = PlayGround.STRUCTURE;
		playGround[29][26] = PlayGround.STRUCTURE;
		//--------------------------------------------------------------
		
		PathFinder pathFinder = PathFinder.createInstance(playGround);
	}

	public final boolean isNewElementValid(PlayGroundElement playGroundElement) {

		int posX = playGroundElement.posX;
		int posZ = playGroundElement.posZ;

		int[][] playGroundInfo = playGroundElement.playGroundInfo;

		for (int i = posZ, k = 0; k < playGroundInfo.length; i++, k++) {
			for (int j = posX, l = 0; l < playGroundInfo[k].length; j++, l++) {
				if (this.playGround[i][j] != PlayGround.FREE) {
					return false;
				}
			}
		}

		return true;
	}

	public final boolean isPositionFree(PlayGroundElement playGroundElement) {
		int posX = playGroundElement.posX;
		int posZ = playGroundElement.posZ;

		if (this.playGround[posZ][posX] != PlayGround.FREE) {
			return false;
		}

		return true;

	}

	public final boolean isPositionFree(int posX, int posZ) {

		if (this.playGround[posZ][posX] != PlayGround.FREE) {
			return false;
		}

		return true;

	}

	public final void addNewElement(PlayGroundElement playGroundElement) {

		int posX = playGroundElement.posX;
		int posZ = playGroundElement.posZ;

		int[][] playGroundInfo = playGroundElement.playGroundInfo;

		for (int i = posZ, k = 0; k < playGroundInfo.length; i++, k++) {
			for (int j = posX, l = 0; l < playGroundInfo[k].length; j++, l++) {
				this.playGround[i][j] = playGroundInfo[k][l];
				String pos = "" + i + j;
				this.positionObjectMapper.put(pos, playGroundElement.element);
			}
		}

		
		//System.out.println("Playground add ");
		//PlayGround.getInstance().displayPlayGround();
	}

	public final void removeElement(PlayGroundElement gameElement) {
		int posX = gameElement.posX;
		int posZ = gameElement.posZ;

		int[][] playGroundInfo = gameElement.playGroundInfo;

		for (int i = posZ, k = 0; k < playGroundInfo.length; i++, k++) {
			for (int j = posX, l = 0; l < playGroundInfo[k].length; j++, l++) {
				this.playGround[i][j] = PlayGround.FREE;
				String pos = "" + i + j;
				this.positionObjectMapper.remove(pos);
			}
		}
		
		//System.out.println("Playground remove ");
		//PlayGround.getInstance().displayPlayGround();
	}

	public final void changePosition(PlayGroundElement gameElement) {
		int posX = gameElement.posX;
		int posZ = gameElement.posZ;

		int[][] playGroundInfo = gameElement.playGroundInfo;

		for (int i = posZ, k = 0; k < playGroundInfo.length; i++, k++) {
			for (int j = posX, l = 0; l < playGroundInfo[k].length; j++, l++) {
				this.playGround[i][j] = PlayGround.FREE;
				String pos = "" + i + j;
				this.positionObjectMapper.remove(pos);
			}
		}

		posX = gameElement.targetX;
		posZ = gameElement.targetZ;

		for (int i = posZ, k = 0; k < playGroundInfo.length; i++, k++) {
			for (int j = posX, l = 0; l < playGroundInfo[k].length; j++, l++) {
				this.playGround[i][j] = gameElement.playGroundInfo[k][l];
				String pos = "" + i + j;
				this.positionObjectMapper.put(pos, gameElement.element);

			}
		}

		gameElement.posX = gameElement.targetX;
		gameElement.posZ = gameElement.targetZ;

	}

	// test purpose not be used in program
	private void displayPlayGround() {
		for (int i = 0; i < this.verticalBoxCount; i++) {
			for (int j = 0; j < this.horizontalBoxCount; j++) {
				if (this.playGround[i][j] == PlayGround.STRUCTURE) {
					System.out.print("S");
					continue;
				}
				if (this.playGround[i][j] == PlayGround.HERO) {
					System.out.print("H");
					continue;
				}
				if (this.playGround[i][j] == PlayGround.MONSTER) {
					System.out.print("M");
					continue;
				}

				System.out.print("O");
			}
			System.out.println();
		}
	}

	public final void alterPlayGround(int horizontal, int vertical, int state) {
		this.playGround[vertical][horizontal] = state;
	}

	public final int getHorizontalBoxCount() {
		return horizontalBoxCount;
	}

	public final int getVerticalBoxCount() {
		return verticalBoxCount;
	}

	public final static int pathLength(int posX, int posZ, int targetX,
			int targetZ) {
		PathFinder pathFinder = PathFinder.getInstance();

		if (pathFinder.generatePath(posX, posZ, targetX, targetZ)) {

			int length = 0;
			Node target = pathFinder.currentNode;

			while (target.parent != null) {
				target = target.parent;
				++length;
			}

			return length;
		}
		return -1;
	}

	public final static boolean calculateMonsterMovement(
			PlayGroundElement monster, PlayGroundElement hero, int stepCount) {

		if (stepCount <= 0) {
			return false;
		}

		PathFinder pathFinder = PathFinder.getInstance();

		int posX = monster.posX;
		int posZ = monster.posZ;

		int targetX = hero.posX;
		int targetZ = hero.posZ;

		Routemap routeMap = hero.routeMap;

		int pathLength = PlayGround.pathLength(posX, posZ, targetX, targetZ);

		if (pathLength != -1) {

			int parentX = -1;
			int parentZ = -1;
			int currentX = -1;
			int currentZ = -1;

			routeMap.clear();
			RouteMapElement element = null;

			if (pathLength == 1) {

				Node target = pathFinder.currentNode;
				target = target.parent;

				routeMap.targetX = target.x;
				routeMap.targetZ = target.z;

				Spatial node = (Spatial) (monster.element);

				if (GameflowController.getInstance().monsterList.remove(node) == true) {
					hero.monsterList.addElement(node);
				}

				return true;
			}

			if (pathLength - 1 <= stepCount) {

				Node target = pathFinder.currentNode;

				target = target.parent;

				routeMap.targetX = target.x;
				routeMap.targetZ = target.z;

				Spatial node = (Spatial) (monster.element);

				if (GameflowController.getInstance().monsterList.remove(node) == true) {
					hero.monsterList.addElement(node);
				}

				while (target.parent != null) {
					parentX = target.x;
					parentZ = target.z;

					target = target.parent;

					currentX = target.x;
					currentZ = target.z;

					int dx = 0;
					int dz = 0;

					dx = parentX - currentX;
					dz = parentZ - currentZ;

					if (dz == 1 && dx == 0) {
						for (int i = 0; i < 10; i++) {
							element = new RouteMapElement();
							element.posX = currentX;
							element.posZ = currentZ;
							element.type = RouteMapElement.TRANSLATE;
							element.speed = 0.1f;
							element.direction = RouteMapElement.SOUTH;

							routeMap.push(element);
						}

					}

					if (dz == -1 && dx == 0) {
						for (int i = 0; i < 10; i++) {
							element = new RouteMapElement();
							element.posX = currentX;
							element.posZ = currentZ;
							element.type = RouteMapElement.TRANSLATE;
							element.speed = 0.1f;
							element.direction = RouteMapElement.NORTH;

							routeMap.push(element);

						}
					}

					if (dz == 0 && dx == 1) {
						for (int i = 0; i < 10; i++) {
							element = new RouteMapElement();
							element.posX = currentX;
							element.posZ = currentZ;
							element.type = RouteMapElement.TRANSLATE;
							element.speed = 0.1f;
							element.direction = RouteMapElement.EAST;

							routeMap.push(element);
						}

					}

					if (dz == 0 && dx == -1) {
						for (int i = 0; i < 10; i++) {
							element = new RouteMapElement();
							element.posX = currentX;
							element.posZ = currentZ;
							element.type = RouteMapElement.TRANSLATE;
							element.speed = 0.1f;
							element.direction = RouteMapElement.WEST;

							routeMap.push(element);
						}

					}
				}

				return true;

			} else {

				Node target = pathFinder.currentNode;

				int steps = (pathLength - 1) - stepCount;

				if (target.parent != null) {
					target = target.parent;
				}

				while (steps > 0) {
					target = target.parent;
					--steps;
				}

				routeMap.targetX = target.x;
				routeMap.targetZ = target.z;

				while (target.parent != null) {

					parentX = target.x;
					parentZ = target.z;

					target = target.parent;

					currentX = target.x;
					currentZ = target.z;

					int dx = 0;
					int dz = 0;

					dx = parentX - currentX;
					dz = parentZ - currentZ;

					if (dz == 1 && dx == 0) {
						for (int i = 0; i < 10; i++) {
							element = new RouteMapElement();
							element.posX = currentX;
							element.posZ = currentZ;
							element.type = RouteMapElement.TRANSLATE;
							element.speed = 0.1f;
							element.direction = RouteMapElement.SOUTH;

							routeMap.push(element);
						}

					}

					if (dz == -1 && dx == 0) {
						for (int i = 0; i < 10; i++) {
							element = new RouteMapElement();
							element.posX = currentX;
							element.posZ = currentZ;
							element.type = RouteMapElement.TRANSLATE;
							element.speed = 0.1f;
							element.direction = RouteMapElement.NORTH;

							routeMap.push(element);

						}
					}

					if (dz == 0 && dx == 1) {
						for (int i = 0; i < 10; i++) {
							element = new RouteMapElement();
							element.posX = currentX;
							element.posZ = currentZ;
							element.type = RouteMapElement.TRANSLATE;
							element.speed = 0.1f;
							element.direction = RouteMapElement.EAST;

							routeMap.push(element);
						}

					}

					if (dz == 0 && dx == -1) {
						for (int i = 0; i < 10; i++) {
							element = new RouteMapElement();
							element.posX = currentX;
							element.posZ = currentZ;
							element.type = RouteMapElement.TRANSLATE;
							element.speed = 0.1f;
							element.direction = RouteMapElement.WEST;

							routeMap.push(element);
						}

					}
				}

			}

			return true;
		} else {
			return false;
		}
	}

	public final static void generateMapForHeroMovement(PlayGroundElement hero,
			PlayGroundElement plate) {
		PathFinder pathFinder = PathFinder.getInstance();

		int posX = hero.posX;
		int posZ = hero.posZ;

		int targetX = plate.posX;
		int targetZ = plate.posZ;

		Routemap routeMap = hero.routeMap;

		if (pathFinder.generatePath(posX, posZ, targetX, targetZ)) {

			Node target = pathFinder.currentNode;
			int parentX = -1;
			int parentZ = -1;
			int currentX = -1;
			int currentZ = -1;

			routeMap.clear();
			RouteMapElement element = null;

			while (target.parent != null) {
				parentX = target.x;
				parentZ = target.z;

				target = target.parent;

				currentX = target.x;
				currentZ = target.z;

				int dx = 0;
				int dz = 0;

				dx = parentX - currentX;
				dz = parentZ - currentZ;

				if (dz == 1 && dx == 0) {
					for (int i = 0; i < 10; i++) {
						element = new RouteMapElement();
						element.posX = currentX;
						element.posZ = currentZ;
						element.type = RouteMapElement.TRANSLATE;
						element.speed = 0.1f;
						element.direction = RouteMapElement.SOUTH;

						routeMap.push(element);
					}

				}

				if (dz == -1 && dx == 0) {
					for (int i = 0; i < 10; i++) {
						element = new RouteMapElement();
						element.posX = currentX;
						element.posZ = currentZ;
						element.type = RouteMapElement.TRANSLATE;
						element.speed = 0.1f;
						element.direction = RouteMapElement.NORTH;

						routeMap.push(element);

					}
				}

				if (dz == 0 && dx == 1) {
					for (int i = 0; i < 10; i++) {
						element = new RouteMapElement();
						element.posX = currentX;
						element.posZ = currentZ;
						element.type = RouteMapElement.TRANSLATE;
						element.speed = 0.1f;
						element.direction = RouteMapElement.EAST;

						routeMap.push(element);
					}

				}

				if (dz == 0 && dx == -1) {
					for (int i = 0; i < 10; i++) {
						element = new RouteMapElement();
						element.posX = currentX;
						element.posZ = currentZ;
						element.type = RouteMapElement.TRANSLATE;
						element.speed = 0.1f;
						element.direction = RouteMapElement.WEST;

						routeMap.push(element);
					}

				}
			}

		}
	}

	public final static void generateMoveMap(int posX, int posZ, int speed,
			Vector pointList) {
		PathFinder pathFinder = PathFinder.getInstance();

		PlayGround.generateMovePoints(posX, posZ, speed, pointList);

		int[] indices = new int[pointList.size()];

		for (int k = 0; k < indices.length; k++) {
			indices[k] = -1;
		}

		int j = 0;

		/*for (int i = 0; i < pointList.size(); i++) {
			MoveMapElement point = (MoveMapElement) pointList.elementAt(i);

			if (!PathFinder.getInstance().generatePath(posX, posZ, point.posX,
					point.posZ)) {
				indices[j++] = i;
			}
		}

		for (int i = indices.length - 1; i >= 0; i--) {
			if (indices[i] != -1) {
				pointList.removeElementAt(indices[i]);
			}
		}*/

	}

	public final static void generateMovePoints(int x, int z, int speed,
			Vector vList) {
		int posX = x;
		int posZ = z + speed;
		int counter = 2;

		vList.addElement(new MoveMapElement(posX, posZ));
		int j = 0;

		while (j < speed) {
			--posZ;
			--posX;

			for (int i = posX; i < posX + (counter + 1); i++) {
				vList.addElement(new MoveMapElement(i, posZ));
			}

			counter += 2;
			++j;
		}

		j = 0;
		posX = x;
		posZ = z - speed;
		counter = 2;

		vList.addElement(new MoveMapElement(posX, posZ));

		while (j < speed - 1) {
			--posX;
			++posZ;

			for (int i = posX; i < posX + (counter + 1); i++) {
				vList.addElement(new MoveMapElement(i, posZ));
			}

			counter += 2;
			++j;
		}
	}

	public final boolean hasLineOfSight(int posX, int posZ, int targetX,
			int targetZ) {

		boolean flag = true;

		int dx = targetX - posX;
		int dz = targetZ - posZ;

		int steps = -1;
		int k = -1;

		float xIncrement, zIncrement;
		float x = posX;
		float z = posZ;

		if (Math.abs(dx) > Math.abs(dz)) {
			steps = Math.abs(dx);
		} else {
			steps = Math.abs(dz);
		}

		xIncrement = (float) (dx) / (float) (steps);
		zIncrement = (float) (dz) / (float) (steps);

		for (k = 0; k < steps - 1; k++) {
			x += xIncrement;
			z += zIncrement;

			if (this.playGround[(int) z][(int) x] != PlayGround.FREE) {

				return false;
			}
		}
		return flag;
	}

	private final Vector getElementInFieldOfControll(int posX, int posZ,
			int distance, int playGroundElementType) {

		Vector fieldOfControll = new Vector();
		Vector elementList = new Vector();

		this.generateMoveMap(posX, posZ, distance, fieldOfControll);

		for (int i = 0; i < fieldOfControll.size(); i++) {
			MoveMapElement element = (MoveMapElement) fieldOfControll
					.elementAt(i);

			String pos = "" + element.posZ + element.posX;

			game.engine.Node object = (game.engine.Node) this.positionObjectMapper
					.get(pos);

			if (object != null) {
				PlayGroundElement playGroundElment = object.playGroundelement;

				if (playGroundElment.playGroundInfo[0][0] == playGroundElementType) {
					elementList.addElement(object);
				}
			}
		}

		return elementList;
	}

	public final Vector getElementsInLineOfSight(int posX, int posZ,
			int distance, int playGroundType) {

		Vector elementList = this.getElementInFieldOfControll(posX, posZ,
				distance, playGroundType);

		for (int i = 0; i < elementList.size(); i++) {
			game.engine.Node monster = (game.engine.Node) elementList
					.elementAt(i);

			PlayGroundElement playGroundElement = monster.playGroundelement;

			if (!this.hasLineOfSight(posX, posZ, playGroundElement.posX,
					playGroundElement.posZ)) {
				elementList.removeElementAt(i);
			}
		}

		return elementList;
	}

	public static void main(String args[]) {

		int[][] playGround = { { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
				{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
				{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
				{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
				{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
				{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
				{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
				{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
				{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
				{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
				{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
				{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
				{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
				{ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 } };

		PlayGround playGroundObject = PlayGround.getInstance();
		playGroundObject.playGround = playGround;
		playGroundObject.verticalBoxCount = 14;
		playGroundObject.horizontalBoxCount = 14;

		PathFinder path = PathFinder.createInstance(playGround);

		int[][] info = { { PlayGround.MONSTER } };

		PlayGroundElement m1 = new PlayGroundElement(3, 2, 3,
				PlayGround.MONSTER, info);

		game.engine.Node node = new game.engine.Node();
		m1.element = node;
		node.playGroundelement = m1;

		PlayGroundElement m2 = new PlayGroundElement(5, 2, 3,
				PlayGround.MONSTER, info);

		node = new game.engine.Node();
		m2.element = node;
		node.playGroundelement = m2;

		PlayGroundElement m3 = new PlayGroundElement(2, 2, 3,
				PlayGround.MONSTER, info);

		node = new game.engine.Node();
		m3.element = node;
		node.playGroundelement = m3;

		PlayGroundElement m4 = new PlayGroundElement(4, 2, 8,
				PlayGround.MONSTER, info);

		node = new game.engine.Node();
		m4.element = node;
		node.playGroundelement = m4;

		playGroundObject.addNewElement(m1);
		playGroundObject.addNewElement(m2);
		playGroundObject.addNewElement(m3);
		playGroundObject.addNewElement(m4);

		PlayGroundElement monster = new PlayGroundElement(2, 0, 2,
				GeometryFactory.BANE_SPIDER);
		PlayGroundElement hero = new PlayGroundElement(2, 0, 3,
				GeometryFactory.DWARF_RANGED);

		System.out.println(PlayGround
				.calculateMonsterMovement(monster, hero, 5));

		System.out.println(hero.routeMap.targetX + " target "
				+ hero.routeMap.targetZ);

		/*
		 * playGroundObject.alterPlayGround(1, 1, PlayGround.HERO);
		 * playGroundObject.alterPlayGround(5, 3, PlayGround.MONSTER);
		 * 
		 * System.out.println(playGroundObject.determineLineOfSight(1, 1, 5,
		 * 3));
		 */

		/*
		 * int[][] info = { { PlayGround.MONSTER } };
		 * 
		 * PlayGroundElement m1 = new PlayGroundElement(2, 2, 2,
		 * PlayGround.MONSTER, info);
		 * 
		 * game.engine.Node node = new game.engine.Node(); m1.element = node;
		 * node.playGroundelement = m1;
		 * 
		 * PlayGroundElement m2 = new PlayGroundElement(5, 2, 5,
		 * PlayGround.MONSTER, info);
		 * 
		 * node = new game.engine.Node(); m2.element = node;
		 * node.playGroundelement = m2;
		 * 
		 * PlayGroundElement m3 = new PlayGroundElement(2, 2, 7,
		 * PlayGround.MONSTER, info);
		 * 
		 * node = new game.engine.Node(); m3.element = node;
		 * node.playGroundelement = m3;
		 * 
		 * PlayGroundElement m4 = new PlayGroundElement(4, 2, 8,
		 * PlayGround.MONSTER, info);
		 * 
		 * node = new game.engine.Node(); m4.element = node;
		 * node.playGroundelement = m4;
		 * 
		 * playGroundObject.addNewElement(m1);
		 * playGroundObject.addNewElement(m2);
		 * playGroundObject.addNewElement(m3);
		 * playGroundObject.addNewElement(m4);
		 * 
		 * Vector monsterList = playGroundObject.getMonsterInLineOfSight(1, 1,
		 * 20);
		 * 
		 * System.out.println(monsterList.size());
		 * 
		 * playGroundObject.displayPlayGround();
		 */
	}
}
