package game.engine;

import java.util.Hashtable;

import game.core.gui.component.ControllerToolBar;
import game.engine.geometry.*;
import game.engine.renderer.Renderer;
import game.engine.spatial.*;

public class SceneGraph extends Node implements GeometryConstants,
		GameflowListener {

	private Node current = null;

	public Renderer renderer = null;

	//public Hashtable idGeometryMapper = new Hashtable();

	public SceneGraph(Renderer renderer) {
		super();
		this.current = this;
		this.renderer = renderer;
	}

	private final Node getCurrentNode() {
		return this.current;
	}

	public final void setRenderer(Renderer renderer) {
		this.renderer = renderer;
	}

	private final void createPlayGroundStructure(int level) {

		GeometryFactory geometryFactory = GeometryFactory.getInstance();

		Node playGround = new Node();
		geometryFactory.createPlayGround(playGround, level);

		Node.addChildrenToNode(this.current, playGround);
	}

	public final void addNewElement(PlayGroundElement playGroundElement) {
		GeometryFactory geometryFactory = GeometryFactory.getInstance();

		Node parentNode = geometryFactory.createGeometry(playGroundElement);

		float[] translate = { playGroundElement.posX, playGroundElement.posY,
				playGroundElement.posZ, 1 };

		Transformation transformation = new Transformation();
		transformation.translate(translate);

		parentNode.updateTransformationState(transformation);

		Node.addChildrenToNode(this.current, parentNode);
	}
	
	public final void removeElement(Node node){
		Node parentNode = (Node)node.parent;
		
		Node.removeChildrenSpatial(parentNode, node);
		
		node.playGroundelement = null;
		node = null;
	}

	public final void removeTemporaryElement(Node node){
		Node parentNode = (Node)node.parent;
		
		Node.removeChildrenSpatial(parentNode, node);
	}
	
	public final void notifyGameflowController() {
		GameflowController.getInstance().startExecution();
	}

	public final void executeGameflowEvent(GameflowEvent event) {
		int message = event.message;

		if (message == GameflowEvent.INITIAL_PLAYGROUND) {
			this.createPlayGroundStructure(event.playGroundLevel);
		}

		if (message == GameflowEvent.ADD_NEW_GAME_ELEMENT) {
			this.addNewElement(event.playGroundElement);
		}
		
		if (message == GameflowEvent.REMOVE_GAME_ELEMENT) {
			this.removeElement(event.node);
		}
		
		if (message == GameflowEvent.REMOVE_TEMPORARY_GAME_ELEMENT) {
			this.removeTemporaryElement(event.node);
		}
		
		if (message == GameflowEvent.QUEUE_EMPTY) {
			ControllerToolBar.getInstance().unlock();
		}

		event = null;
	}
}
