package game.engine;

import game.core.MessageProperties;
import game.core.gui.component.ControllerToolBar;
import game.core.gui.component.StatusPanel;
import game.core.gui.component.UserInputEvent;
import game.engine.ai.PathFinder;
import game.engine.creatures.Creature;
import game.engine.creatures.CreatureFactory;
import game.engine.geometry.GeometryFactory;
import game.engine.spatial.Spatial;
import game.engine.util.IconFactory;

import java.util.Hashtable;
import java.util.Random;
import java.util.Vector;

import com.sun.java.swing.plaf.motif.MotifIconFactory;

public class SelectionController {

	private static SelectionController instance = null;

	public Hashtable idSpatialMapper = new Hashtable();

	private boolean isHeroSelected = false;

	private boolean isMonsterSelected = false;

	int counter = GeometryFactory.getInstance().INIT_SELECTION + 1;

	public Node formerSelectedHero = null;

	public int heroIndex = -1;

	private Random random = new Random();

	private SelectionController() {

	}

	public final static SelectionController getInstance() {

		if (instance == null) {
			instance = new SelectionController();
		}

		return instance;
	}

	public final void selectionOccured(int selectionID) {

		Spatial node = (Spatial) this.idSpatialMapper.get(new Integer(
				selectionID));

		if (node != null) {

			PlayGroundElement element = node.playGroundelement;

			if (element.geometryFactoryType == GeometryFactory.BANE_SPIDER) {
				this.selectMonster(node);
				return;
			}

			if (element.geometryFactoryType == GeometryFactory.GATE) {
				this.selectMonster(node);
				return;
			}

			if (element.geometryFactoryType == GeometryFactory.DWARF_RANGED) {
				this.selectHero(node);
				return;
			}

			if (element.geometryFactoryType == GeometryFactory.DWARF_GEODE) {
				this.selectHero(node);
				return;
			}

			if (element.geometryFactoryType == GeometryFactory.DWARF_WORRIOR) {
				this.selectHero(node);
				return;
			}

			if (element.geometryFactoryType == GeometryFactory.MOVE_PLATE) {
				this.selectMovePlate(element);
			}

		}

	}

	private final void selectMovePlate(PlayGroundElement playGroundElement) {

		Node hero = (Node) playGroundElement.element;
		PlayGroundElement playGroundElement_hero = hero.playGroundelement;

		if (!playGroundElement_hero.hasMoved) {

			PlayGround.generateMapForHeroMovement(playGroundElement_hero,
					playGroundElement);

			hero.geometricalState = Spatial.CHANGE_POSITION;

			playGroundElement_hero.targetX = playGroundElement.posX;
			playGroundElement_hero.targetZ = playGroundElement.posZ;

			MotionMonitor.getInstance().currentNode = hero;
			MotionMonitor.getInstance().currentRouteMap = playGroundElement_hero.routeMap;
			MotionMonitor.getInstance().monsterMovment = false;

			PlayGround.getInstance().changePosition(playGroundElement_hero);

			playGroundElement_hero.hasMoved = true;

			if (playGroundElement_hero.moveMap != null) {
				GameflowController.getInstance().removeTemporaryGameElement(
						this.formerSelectedHero.playGroundelement.moveMap);

				playGroundElement_hero.moveMap = null;
			}
		}
	}

	public final void selectHero(Spatial node) {

		StatusPanel.getInstance().setValues(node.playGroundelement);
		ControllerToolBar.getInstance()
				.selectionChanged(node.playGroundelement);

		Vector heroList = GameflowController.getInstance().heroList;

		for (int i = 0; i < heroList.size(); i++) {

			Node heroNode = (Node) heroList.elementAt(i);
			PlayGroundElement element = heroNode.playGroundelement;

			if (element.moveMap != null) {
				GameflowController.getInstance().removeTemporaryGameElement(
						this.formerSelectedHero.playGroundelement.moveMap);

				element.moveMap = null;
			}

		}

		if (this.formerSelectedHero == null) {
			this.formerSelectedHero = (Node) node;
		}

		if (this.formerSelectedHero != (Node) node) {
			if (this.formerSelectedHero.playGroundelement.moveMap != null) {
				GameflowController.getInstance().removeTemporaryGameElement(
						this.formerSelectedHero.playGroundelement.moveMap);

			}
			this.formerSelectedHero.playGroundelement.moveMap = null;
			this.formerSelectedHero = (Node) node;
		}

		this.heroIndex = GameflowController.getInstance().heroList
				.indexOf(node);

		if (node.playGroundelement.action != Creature.NO_ACTION) {
			ControllerToolBar.getInstance().enableToolBar();
		} else {
			ControllerToolBar.getInstance().disableToolBar();
		}

		if (!node.playGroundelement.hasMoved
				&& node.playGroundelement.monsterList.size() == 0
				&& !node.playGroundelement.useSpecialAbility) {
			displayMoveMap(node);
		}

	}

	private final void displayMark(Node node) {
		PlayGroundElement playGroundElement = node.playGroundelement;

		UserInputEvent event = new UserInputEvent(UserInputEvent.DISPLAY_MARK);

		event.playGroundElement = playGroundElement;

		GameflowController.getInstance().executeUserEvent(event);

	}

	private final void displayMoveMap(Spatial node_hero) {

		PlayGroundElement playGroundElement = node_hero.playGroundelement;

		if (!playGroundElement.hasMoved) {
			Vector platePositionList = new Vector();

			Creature hero = CreatureFactory
					.getCreature(node_hero.playGroundelement.geometryFactoryType);

			PlayGround
					.generateMoveMap(playGroundElement.posX,
							playGroundElement.posZ, hero.speed_trait,
							platePositionList);

			UserInputEvent event = new UserInputEvent(
					UserInputEvent.DISPLAY_MOVEMAP);
			event.moveMap = platePositionList;
			event.playGroundElement = playGroundElement;

			GameflowController.getInstance().executeUserEvent(event);
		}
	}

	private final void selectMonster(Spatial node) {

		if (this.formerSelectedHero == null) {
			StatusPanel.getInstance().setValues(node.playGroundelement);
		} else {
			if (!this.formerSelectedHero.playGroundelement.hasAttacked) {

				PathFinder path = PathFinder.getInstance();

				PlayGroundElement playGroundElement_monster = node.playGroundelement;
				PlayGroundElement playGroundElement_hero = this.formerSelectedHero.playGroundelement;

				int pathLength = PlayGround.getInstance().pathLength(
						playGroundElement_hero.posX,
						playGroundElement_hero.posZ,
						playGroundElement_monster.posX,
						playGroundElement_monster.posZ);

				Creature hero_creature = CreatureFactory
						.getCreature(this.formerSelectedHero.playGroundelement.geometryFactoryType);

				if (pathLength <= hero_creature.ranged_trait) {

					if (PlayGround.getInstance().hasLineOfSight(
							playGroundElement_hero.posX,
							playGroundElement_hero.posZ,
							playGroundElement_monster.posX,
							playGroundElement_monster.posZ)) {

						this.formerSelectedHero.playGroundelement.hasAttacked = true;

						Creature monster_creature = CreatureFactory
								.getCreature(node.playGroundelement.geometryFactoryType);

						int hero_attackValue = hero_creature.melee_trait
								+ random.nextInt(7);
						int monster_defenseValue = monster_creature.armor;
						int hitValue = hero_attackValue - monster_defenseValue;

						if (hitValue > 0) {

							int monsterWound = node.playGroundelement.wounds
									- hitValue;

							if (monsterWound <= 0) {
								PlayGround.getInstance().removeElement(
										node.playGroundelement);
								GameflowController controller = GameflowController
										.getInstance();

								if (controller.monsterList.remove(node) != true) {
									Vector heroList = controller.heroList;

									for (int i = 0; i < heroList.size(); i++) {
										Node hero = (Node) heroList
												.elementAt(i);
										Vector surroundingMonsterList = hero.playGroundelement.monsterList;

										if (surroundingMonsterList
												.removeElement(node) == true) {
											break;
										}
									}
								}

								this.idSpatialMapper.remove(node.selectionID);

								UserInputEvent event = new UserInputEvent(
										UserInputEvent.REMOVE_MONSTER);

								event.node = (Node) node;

								GameflowController.getInstance()
										.executeUserEvent(event);

								if (node.playGroundelement.geometryFactoryType == GeometryFactory.BANE_SPIDER) {
									StatusPanel
											.getInstance()
											.addNewMessage(

													IconFactory
															.getFactory()
															.getIcon(
																	monster_creature.smallIcon),
													monster_creature.name
															+ " "
															+ MessageProperties
																	.getInstance()
																	.getProperty(
																			"killed")
															+ " "
															+ hero_creature.name);
								}

								if (node.playGroundelement.geometryFactoryType == GeometryFactory.GATE) {
									StatusPanel
											.getInstance()
											.addNewMessage(

													IconFactory
															.getFactory()
															.getIcon(
																	monster_creature.smallIcon),
													monster_creature.name
															+ " "
															+ MessageProperties
																	.getInstance()
																	.getProperty(
																			"destroyed")
															+ " "
															+ hero_creature.name);
								}

							} else {
								node.playGroundelement.wounds -= hitValue;

								if (node.playGroundelement.geometryFactoryType == GeometryFactory.BANE_SPIDER) {
									StatusPanel
											.getInstance()
											.addNewMessage(

													IconFactory
															.getFactory()
															.getIcon(
																	monster_creature.smallIcon),
													monster_creature.name
															+ " "
															+ MessageProperties
																	.getInstance()
																	.getProperty(
																			"hurt")
															+ " "
															+ hero_creature.name
															+ " ( " + hitValue
															+ " ) ");
								}
								
								if (node.playGroundelement.geometryFactoryType == GeometryFactory.GATE) {
									StatusPanel
											.getInstance()
											.addNewMessage(

													IconFactory
															.getFactory()
															.getIcon(
																	monster_creature.smallIcon),
													monster_creature.name
															+ " "
															+ MessageProperties
																	.getInstance()
																	.getProperty(
																			"damaged")
															+ " "
															+ hero_creature.name
															+ " ( " + hitValue
															+ " ) ");
								}

							}

							if (!playGroundElement_hero.hasMoved) {
								this.selectHero(playGroundElement_hero.element);
							}
						}
					}
				}
			}
		}

	}

	public final void setSpecialAbility(int action) {

		Node heroNode = this.formerSelectedHero;

		if (heroNode != null) {
			PlayGroundElement heroPlayGroundElement = heroNode.playGroundelement;

			if (!heroPlayGroundElement.hasAttacked
					&& !heroPlayGroundElement.hasMoved) {

				if (heroPlayGroundElement.moveMap != null) {
					GameflowController
							.getInstance()
							.removeTemporaryGameElement(
									this.formerSelectedHero.playGroundelement.moveMap);

					heroPlayGroundElement.moveMap = null;
				}

				Creature hero_creature = CreatureFactory
						.getCreature(heroPlayGroundElement.creatureInfo);

				StatusPanel.getInstance().addNewMessage(
						IconFactory.getFactory().getIcon(
								hero_creature.smallIcon),
						hero_creature.name
								+ " "
								+ MessageProperties.getInstance().getProperty(
										"special"));

				heroPlayGroundElement.action = action;

				switch (action) {
				case Creature.ACTION_1: {
					StatusPanel.getInstance().label_status.setIcon(IconFactory
							.getFactory().getIcon(hero_creature.actionIcon_1));

					break;
				}
				case Creature.ACTION_2: {
					StatusPanel.getInstance().label_status.setIcon(IconFactory
							.getFactory().getIcon(hero_creature.actionIcon_2));
					break;
				}
				case Creature.ACTION_3: {
					StatusPanel.getInstance().label_status.setIcon(IconFactory
							.getFactory().getIcon(hero_creature.actionIcon_3));
					break;
				}
				case Creature.ACTION_4: {
					StatusPanel.getInstance().label_status.setIcon(IconFactory
							.getFactory().getIcon(hero_creature.actionIcon_4));
					break;
				}
				}

				heroPlayGroundElement.hasAttacked = true;
				heroPlayGroundElement.hasMoved = true;
				heroPlayGroundElement.useSpecialAbility = true;
				heroPlayGroundElement.action = action;

			}
		}

	}

	public final int addMonster(Spatial monsterNode) {
		monsterNode.selectionID = counter;
		this.idSpatialMapper.put(new Integer(counter++), monsterNode);

		return counter - 1;
	}

	public final void addHero(int id, Spatial heroNode) {
		this.idSpatialMapper.put(new Integer(id), heroNode);
	}

	public final int addMoveMapPlate(Spatial plate) {
		plate.selectionID = counter;
		this.idSpatialMapper.put(new Integer(counter++), plate);

		return counter - 1;
	}
}
