package game.engine.creatures;

import java.util.Random;
import java.util.Vector;

import game.core.MessageProperties;
import game.core.gui.component.StatusPanel;
import game.engine.GameflowController;
import game.engine.MoveMapElement;
import game.engine.Node;
import game.engine.PlayGround;
import game.engine.PlayGroundElement;
import game.engine.geometry.GeometryFactory;
import game.engine.spatial.Transformation;
import game.engine.util.IconFactory;

public class DwarfGeode extends Creature {

	private static DwarfGeode instance = null;

	private Random random = new Random();

	private DwarfGeode() {
		this.creatureType = GeometryFactory.DWARF_GEODE;
		this.init();
	}

	private void init() {
		this.speed_trait = 4;
		this.armor = 2;
		this.melee_trait = 5;
		this.ranged_trait = 20;
		this.wounds = 20;

		this.icon = IconFactory.getFactory().getIcon("magician");
		this.smallIcon = "magician_small";
		this.actionIcon_1 = "magician_action_1";
		this.actionIcon_2 = "magician_action_2";
		this.actionIcon_3 = "magician_action_3";
		this.actionIcon_4 = "magician_action_4";

		this.tool_text_1 = "Heal nearby dwarfs";
		this.tool_text_2 = "Teleport other dwarfs to own position";
		this.tool_text_3 = "Melt with shadow";
		this.tool_text_4 = "Destroy enemy entry point";

		this.action_1_time = 0;
		this.action_2_time = 0;
		this.action_3_time = 0;
		this.action_4_time = 0;

		this.action_process_time_1 = 0;
		this.action_process_time_2 = 0;
		this.action_process_time_3 = 0;
		this.action_process_time_4 = 0;

		this.name = "Angramosch";
	}

	public final static DwarfGeode getInstance() {

		if (instance == null) {
			instance = new DwarfGeode();
		}

		return instance;
	}

	@Override
	public void useSpecialAbility1(Node node) {

		PlayGroundElement playGroundElementHero = node.playGroundelement;

		Vector targetList = PlayGround.getInstance().getElementsInLineOfSight(
				playGroundElementHero.posX, playGroundElementHero.posZ,
				this.ranged_trait, PlayGround.HERO);

		for (int i = 0; i < targetList.size(); i++) {
			Node tempHeroNode = (Node) targetList.elementAt(i);
			PlayGroundElement tempHeroPlayGroundElement = tempHeroNode.playGroundelement;

			Creature tempHeroCreature = CreatureFactory
					.getCreature(tempHeroPlayGroundElement.geometryFactoryType);

			int healingPoints = random.nextInt(7);

			tempHeroPlayGroundElement.wounds = Math.min(
					tempHeroCreature.wounds, tempHeroPlayGroundElement.wounds
							+ healingPoints);

			if (tempHeroCreature.wounds > tempHeroPlayGroundElement.wounds
					+ healingPoints) {

				StatusPanel.getInstance().addNewMessage(
						IconFactory.getFactory().getIcon(
								tempHeroCreature.smallIcon),
						tempHeroCreature.name
								+ " "
								+ MessageProperties.getInstance().getProperty(
										"heal") + " " + this.name + " ( "
								+ healingPoints + " ) ");
			}

		}

	}

	@Override
	public void useSpecialAbility2(Node node) {
		PlayGroundElement heroPlayGroundElement = node.playGroundelement;

		Vector posList = new Vector();
		int posIndex = 0;

		PlayGround.generateMoveMap(heroPlayGroundElement.posX,
				heroPlayGroundElement.posZ, 3, posList);

		GameflowController gameflowController = GameflowController
				.getInstance();
		Vector heroList = gameflowController.heroList;
		Vector monsterList = gameflowController.monsterList;

		for (int i = 0; i < heroList.size(); i++) {
			Node heroNode = (Node) heroList.elementAt(i);
			PlayGroundElement tempHeroPlayGroundElement = heroNode.playGroundelement;

			if (tempHeroPlayGroundElement.creatureInfo != GeometryFactory.DWARF_GEODE
					&& tempHeroPlayGroundElement.monsterList.size() == 0) {
				MoveMapElement moveElement = (MoveMapElement) posList
						.elementAt(posIndex++);
				Transformation transformation = new Transformation();

				float[] translateToOrigin = { -tempHeroPlayGroundElement.posX,
						0, -tempHeroPlayGroundElement.posZ, 1 };
				float[] translate = { moveElement.posX, 0, moveElement.posZ, 1 };

				transformation.translate(translateToOrigin);
				transformation.translate(translate);

				heroNode.updateTransformationState(transformation);

				tempHeroPlayGroundElement.targetX = moveElement.posX;
				tempHeroPlayGroundElement.targetY = tempHeroPlayGroundElement.posY;
				tempHeroPlayGroundElement.targetZ = moveElement.posZ;

				PlayGround.getInstance().changePosition(
						tempHeroPlayGroundElement);
			}
		}
	}

	@Override
	public void useSpecialAbility3(Node node) {

		PlayGroundElement heroPlayGroundElement = node.playGroundelement;
		PlayGround.getInstance().alterPlayGround(heroPlayGroundElement.posX,
				heroPlayGroundElement.posZ, PlayGround.STRUCTURE);
		Vector monsterList = heroPlayGroundElement.monsterList;

		GameflowController controller = GameflowController.getInstance();

		if (monsterList.size() > 0) {
			for (int i = 0; i < monsterList.size(); i++) {
				controller.monsterList.addElement(monsterList.elementAt(i));
			}

			monsterList.removeAllElements();
		}

	}

	@Override
	public void useSpecialAbility4(Node node) {
		Vector entryPointList = GeometryFactory.getInstance().entryPointList;

		int index = -1;
		
		for (int i = 0; i < entryPointList.size(); i++) {
			PlayGroundElement entryPoint = (PlayGroundElement) entryPointList
					.elementAt(i);

			PlayGroundElement heroPlayGroundElement = node.playGroundelement;

			if (heroPlayGroundElement.posX == entryPoint.posX
					&& heroPlayGroundElement.posY == entryPoint.posY
					&& heroPlayGroundElement.posZ == entryPoint.posZ) {
				index = i;
				
			}

		}
		
		if(index != -1){
			entryPointList.removeElementAt(index);
			
			

				StatusPanel.getInstance().addNewMessage(
						IconFactory.getFactory().getIcon(
								this.smallIcon),
						"entrypoint"
								+ " "
								+ MessageProperties.getInstance().getProperty(
										"destroyed") + " " + this.name);
			
		}

	}

	@Override
	public void resetSpecialAbility1(Node node) {
		// TODO Auto-generated method stub

	}

	@Override
	public void resetSpecialAbility2(Node node) {
		// TODO Auto-generated method stub

	}

	@Override
	public void resetSpecialAbility3(Node node) {
		PlayGroundElement heroPlayGroundElement = node.playGroundelement;
		PlayGround.getInstance().alterPlayGround(heroPlayGroundElement.posX,
				heroPlayGroundElement.posZ, PlayGround.HERO);

	}

	@Override
	public void resetSpecialAbility4(Node node) {
		// TODO Auto-generated method stub

	}

}
