package game.engine.creatures;

import java.util.Queue;
import java.util.Random;
import java.util.Stack;
import java.util.Vector;

import game.core.IconProperties;
import game.core.MessageProperties;
import game.core.gui.component.StatusPanel;
import game.core.gui.component.UserInputEvent;
import game.engine.GameflowController;
import game.engine.GameflowEvent;
import game.engine.Node;
import game.engine.PlayGround;
import game.engine.PlayGroundElement;
import game.engine.SelectionController;
import game.engine.geometry.GeometryFactory;
import game.engine.util.IconFactory;

public class DwarfRanger extends Creature {

	private static DwarfRanger instance = null;

	private Random random = new Random();

	public Vector bombs = new Vector();

	private int bomb_range_trait = 4;

	private int bomb_melee_trait = 10;

	private DwarfRanger() {
		this.creatureType = GeometryFactory.DWARF_RANGED;
		this.init();
	}

	private void init() {
		this.speed_trait = 5;
		this.armor = 5;
		this.melee_trait = 5;
		this.ranged_trait = 10;
		this.wounds = 25;

		this.icon = IconFactory.getFactory().getIcon("dwarf");
		this.smallIcon = "dwarf_small";
		this.actionIcon_1 = "dwarf_action_1";
		this.actionIcon_2 = "dwarf_action_2";
		this.actionIcon_3 = "dwarf_action_3";
		this.actionIcon_4 = "dwarf_action_4";

		this.tool_text_1 = "Throw discs on nearby enemys";
		this.tool_text_2 = "Give one aimed shoot";
		this.tool_text_3 = "Install a bomb";
		this.tool_text_4 = "Duck behind heavy shield";

		this.action_1_time = 0;
		this.action_2_time = 1;
		this.action_3_time = 1;
		this.action_4_time = 0;

		this.action_process_time_1 = 0;
		this.action_process_time_2 = 1;
		this.action_process_time_3 = 0;
		this.action_process_time_4 = 0;

		this.name = "Gloin";
	}

	public static final DwarfRanger getInstance() {

		if (instance == null) {
			instance = new DwarfRanger();
		}

		return instance;
	}

	@Override
	public void useSpecialAbility1(Node node) {
		PlayGroundElement heroPlayGroundElement = node.playGroundelement;

		Vector monsterList = PlayGround.getInstance().getElementsInLineOfSight(
				heroPlayGroundElement.posX, heroPlayGroundElement.posZ,
				this.ranged_trait, PlayGround.MONSTER);

		for (int i = 0; i < monsterList.size(); i++) {
			Node monsterNode = (Node) monsterList.elementAt(i);
			PlayGroundElement monsterPlayGroundElement = monsterNode.playGroundelement;

			Creature monsterCreature = CreatureFactory
					.getCreature(monsterPlayGroundElement.creatureInfo);

			int attackValue = this.melee_trait  + random.nextInt(7);
			int defenseValue = monsterCreature.armor;

			int hitValue = attackValue - defenseValue;

			if (hitValue > 0) {

				int wounds = monsterPlayGroundElement.wounds - hitValue;

				if (wounds > 0) {
					monsterPlayGroundElement.wounds -= hitValue;

					if (monsterPlayGroundElement.creatureInfo == GeometryFactory.BANE_SPIDER) {
						StatusPanel.getInstance().addNewMessage(
								IconFactory.getFactory().getIcon(
										monsterCreature.smallIcon),
								monsterCreature.name
										+ " "
										+ MessageProperties.getInstance()
												.getProperty("hurt") + " "
										+ this.name + " " + " ( " + hitValue
										+ " ) ");
					}

					if (monsterPlayGroundElement.creatureInfo == GeometryFactory.GATE) {
						StatusPanel.getInstance().addNewMessage(
								IconFactory.getFactory().getIcon(
										monsterCreature.smallIcon),
								monsterCreature.name
										+ " "
										+ MessageProperties.getInstance()
												.getProperty("damaged") + " "
										+ this.name + " " + " ( " + hitValue
										+ " ) ");
					}

				} else {
					PlayGround.getInstance().removeElement(
							monsterNode.playGroundelement);

					if (!GameflowController.getInstance().monsterList
							.remove(monsterNode)) {
						for (int j = 0; j < GameflowController.getInstance().heroList
								.size(); j++) {
							Node hero = (Node) GameflowController.getInstance().heroList
									.elementAt(j);

							Vector heroAttachedMonsterList = hero.playGroundelement.monsterList;

							if (heroAttachedMonsterList
									.removeElement(monsterNode)) {
								break;
							}
						}
					}

					SelectionController.getInstance().idSpatialMapper
							.remove(monsterNode.selectionID);

					UserInputEvent event = new UserInputEvent(
							UserInputEvent.REMOVE_MONSTER);

					event.node = (Node) monsterNode;
					GameflowController.getInstance().executeUserEvent(event);

					if (monsterPlayGroundElement.creatureInfo == GeometryFactory.BANE_SPIDER) {
						StatusPanel.getInstance().addNewMessage(
								IconFactory.getFactory().getIcon(
										monsterCreature.smallIcon),
								monsterCreature.name
										+ " "
										+ MessageProperties.getInstance()
												.getProperty("killed") + " "
										+ this.name);
					}

					if (monsterPlayGroundElement.creatureInfo == GeometryFactory.GATE) {
						StatusPanel.getInstance().addNewMessage(
								IconFactory.getFactory().getIcon(
										monsterCreature.smallIcon),
								monsterCreature.name
										+ " "
										+ MessageProperties.getInstance()
												.getProperty("destroyed") + " "
										+ this.name);
					}
				}

			}
		}

	}

	@Override
	public void useSpecialAbility2(Node node) {
		this.melee_trait *= 2;

	}

	@Override
	public void useSpecialAbility3(Node node) {

		PlayGroundElement bomb = new PlayGroundElement(
				node.playGroundelement.posX, node.playGroundelement.posY,
				node.playGroundelement.posZ, GeometryFactory.BOMB);

		this.bombs.addElement(bomb);

	}

	@Override
	public void useSpecialAbility4(Node node) {
		PlayGroundElement playGroundElement = node.playGroundelement;

		if (playGroundElement.monsterList.size() == 0) {
			this.armor *= 4;
		}

	}

	@Override
	public void resetSpecialAbility1(Node node) {
		// TODO Auto-generated method stub

	}

	@Override
	public void resetSpecialAbility2(Node node) {
		this.melee_trait = 8;
	}

	@Override
	public void resetSpecialAbility3(Node node) {

		if (bombs.size() > 0) {
			PlayGroundElement bomb = (PlayGroundElement) this.bombs
					.elementAt(0);

			if (bomb != null) {
				Vector monsterList = PlayGround.getInstance()
						.getElementsInLineOfSight(bomb.posX, bomb.posZ,
								this.bomb_range_trait, PlayGround.MONSTER);

				for (int i = 0; i < monsterList.size(); i++) {
					Node monsterNode = (Node) monsterList.elementAt(i);
					PlayGroundElement monsterPlayGroundElement = monsterNode.playGroundelement;
					Creature monsterCreature = CreatureFactory
							.getCreature(monsterPlayGroundElement.creatureInfo);

					int attackValue = this.bomb_melee_trait + random.nextInt(7);
					int defenseValue = monsterCreature.armor;

					int hitValue = attackValue - defenseValue;

					if (hitValue > 0) {

						int wounds = monsterPlayGroundElement.wounds - hitValue;

						if (wounds > 0) {
							monsterPlayGroundElement.wounds -= hitValue;

							StatusPanel.getInstance().addNewMessage(
									IconFactory.getFactory().getIcon(
											monsterCreature.smallIcon),
									monsterCreature.name
											+ " "
											+ MessageProperties.getInstance()
													.getProperty("hurt") + " "
											+ "a bomb" + " " + " ( " + hitValue
											+ " ) ");

						} else {
							PlayGround.getInstance().removeElement(
									monsterNode.playGroundelement);

							if (!GameflowController.getInstance().monsterList
									.remove(monsterNode)) {
								for (int j = 0; j < GameflowController
										.getInstance().heroList.size(); j++) {
									Node hero = (Node) GameflowController
											.getInstance().heroList
											.elementAt(j);

									Vector heroAttachedMonsterList = hero.playGroundelement.monsterList;

									if (heroAttachedMonsterList
											.removeElement(monsterNode)) {
										break;
									}
								}
							}

							SelectionController.getInstance().idSpatialMapper
									.remove(monsterNode.selectionID);

							UserInputEvent event = new UserInputEvent(
									UserInputEvent.REMOVE_MONSTER);

							event.node = (Node) monsterNode;
							GameflowController.getInstance().executeUserEvent(
									event);

							StatusPanel.getInstance().addNewMessage(
									IconFactory.getFactory().getIcon(
											monsterCreature.smallIcon),
									monsterCreature.name
											+ " "
											+ MessageProperties.getInstance()
													.getProperty("killed")
											+ " " + "a bomb");
						}

					}
				}

			}

			if (bomb != null) {
				Vector monsterList = PlayGround.getInstance()
						.getElementsInLineOfSight(bomb.posX, bomb.posZ,
								this.bomb_range_trait, PlayGround.HERO);

				for (int i = 0; i < monsterList.size(); i++) {
					Node heroNode = (Node) monsterList.elementAt(i);
					PlayGroundElement heroPlayGroundElement = heroNode.playGroundelement;
					Creature heroCreature = CreatureFactory
							.getCreature(heroPlayGroundElement.creatureInfo);

					int attackValue = this.bomb_melee_trait + random.nextInt(7);
					int defenseValue = heroCreature.armor;

					int hitValue = attackValue - defenseValue;

					if (hitValue > 0) {

						int wounds = heroPlayGroundElement.wounds - hitValue;

						if (wounds > 0) {
							heroPlayGroundElement.wounds -= hitValue;

							StatusPanel.getInstance().addNewMessage(
									IconFactory.getFactory().getIcon(
											heroCreature.smallIcon),
									heroCreature.name
											+ " "
											+ MessageProperties.getInstance()
													.getProperty("hurt") + " "
											+ "a bomb" + " " + " ( " + hitValue
											+ " ) ");

						} else {

							heroPlayGroundElement.wounds -= hitValue;
							
							StatusPanel.getInstance().addNewMessage(
									IconFactory.getFactory().getIcon(
											heroCreature.smallIcon),
									heroCreature.name
											+ " "
											+ MessageProperties.getInstance()
													.getProperty("killed")
											+ " " + "a bomb");
						}

					}
				}
			}
		}

	}

	@Override
	public void resetSpecialAbility4(Node node) {
		// TODO Auto-generated method stub

	}
}
