package game.engine.creatures;

import java.util.Random;
import java.util.Vector;

import game.core.MessageProperties;
import game.core.gui.component.StatusPanel;
import game.core.gui.component.UserInputEvent;
import game.engine.GameflowController;
import game.engine.Node;
import game.engine.PlayGround;
import game.engine.PlayGroundElement;
import game.engine.SelectionController;
import game.engine.geometry.GeometryFactory;
import game.engine.util.IconFactory;

public class DwarfWorrior extends Creature {

	private static DwarfWorrior instance = null;

	private Random random = new Random();

	private DwarfWorrior() {
		this.creatureType = GeometryFactory.DWARF_WORRIOR;
		this.init();
	}

	private void init() {
		this.speed_trait = 3;
		this.armor = 5;
		this.melee_trait = 10;
		this.ranged_trait = 2;
		this.wounds = 20;

		this.icon = IconFactory.getFactory().getIcon("worrior");
		this.smallIcon = "worrior_small";

		this.actionIcon_1 = "worrior_action_1";
		this.actionIcon_2 = "worrior_action_2";
		this.actionIcon_3 = "worrior_action_3";
		this.actionIcon_4 = "worrior_action_4";

		this.action_1_time = 0;
		this.action_2_time = 0;
		this.action_3_time = 1;
		this.action_4_time = 1;

		this.action_process_time_1 = 0;
		this.action_process_time_2 = 0;
		this.action_process_time_3 = 1;
		this.action_process_time_4 = 0;

		this.tool_text_1 = "Attack all nearby enemies";
		this.tool_text_2 = "Balin will defend himself";
		this.tool_text_3 = "Balin will make one powerful strike in the next round";
		this.tool_text_4 = "Unarm traps";

		this.name = "Balin";
	}

	public final static DwarfWorrior getInstance() {
		if (instance == null) {
			instance = new DwarfWorrior();
		}

		return instance;
	}

	@Override
	public void useSpecialAbility1(Node heroNode) {

		Node hero = heroNode;
		PlayGroundElement playGroundElementHero = hero.playGroundelement;

		int index = playGroundElementHero.monsterList.size();

		int[] indices = new int[index];
		
		for(int i = 0; i < index; i++){
			indices[i] = -1;
		}
		
		int monsterIndex = 0;
		
		for (int i = 0; i < index; i++) {

			Node monsterNode = (Node) playGroundElementHero.monsterList
					.elementAt(i);
			PlayGroundElement playGroundElementMonster = monsterNode.playGroundelement;

			Creature monsterCreature = CreatureFactory
					.getCreature(playGroundElementMonster.geometryFactoryType);

			int heroAttackValue = this.melee_trait + random.nextInt(7);
			int monsterDefenseValue = monsterCreature.armor;

			int hitCount = heroAttackValue - monsterDefenseValue;

			if (hitCount > 0) {
				int monsterWound = playGroundElementMonster.wounds - hitCount;

				if (monsterWound <= 0) {
					PlayGround.getInstance().removeElement(
							monsterNode.playGroundelement);
					
					indices[monsterIndex++] = i;
					
					SelectionController.getInstance().idSpatialMapper
							.remove(monsterNode.selectionID);

					UserInputEvent event = new UserInputEvent(
							UserInputEvent.REMOVE_MONSTER);

					event.node = (Node) monsterNode;

					GameflowController.getInstance().executeUserEvent(event);

					StatusPanel.getInstance().addNewMessage(
							IconFactory.getFactory().getIcon(
									monsterCreature.smallIcon),
							monsterCreature.name
									+ " "
									+ MessageProperties.getInstance()
											.getProperty("killed") + " "
									+ this.name);
				} else {
					playGroundElementMonster.wounds -= hitCount;

					StatusPanel.getInstance().addNewMessage(
							IconFactory.getFactory().getIcon(
									monsterCreature.smallIcon),
							monsterCreature.name
									+ " "
									+ MessageProperties.getInstance()
											.getProperty("hurt") + " "
									+ this.name + " " + " ( " + hitCount
									+ " ) ");
				}
			}
		}
		
		for(int i = indices.length - 1; i >= 0; i--){
			if(indices[i] != -1){
				playGroundElementHero.monsterList.removeElementAt(indices[i]);
			}
		}

	}

	@Override
	public void useSpecialAbility2(Node node) {
		this.armor *= 2;

		StatusPanel.getInstance()
				.addNewMessage(
						IconFactory.getFactory().getIcon(this.smallIcon),
						this.name
								+ " "
								+ MessageProperties.getInstance().getProperty(
										"defend"));
	}

	@Override
	public void useSpecialAbility3(Node node) {
		this.melee_trait *= 2;
	}

	@Override
	public void useSpecialAbility4(Node node) {
		// TODO Auto-generated method stub

	}

	@Override
	public void resetSpecialAbility1(Node node) {
		// TODO Auto-generated method stub

	}

	@Override
	public void resetSpecialAbility2(Node node) {
		this.armor = 5;

	}

	@Override
	public void resetSpecialAbility3(Node node) {
		this.melee_trait = 10;

	}

	@Override
	public void resetSpecialAbility4(Node node) {
		// TODO Auto-generated method stub

	}
}
