package game.engine.geometry;

import game.engine.spatial.LinearAlgebra4D;
import game.engine.spatial.Transformation;

public class Arc extends Geometry {

	protected float[] center = { 0, 0, 0, 1 };

	protected int tempCounter = 0;

	public Arc(float angle_begin, float alpha, float depth, float strength,
			float diameter, int vertexCount, float texture_s_scale,
			float texture_t_scale) {
		this.geometryType = GeometryConstants.ARC;
		this.tempCounter = vertexCount;

		float step = (float) (2 * Math.PI) / (vertexCount);

		float texture_vertical_step = (float) (1 * (texture_t_scale));
		float texture_horizontal_step = (float) (1 * (texture_s_scale));

		float angle;
		
		float start = angle_begin;
		float end = angle_begin + alpha;
		
		int index = 0;
		for (angle = end; angle > start;) {
			++index;
			angle -= step;
		}
		
		vertexCount = index;
		vertexCount *= 24;
		
		
		this.VERTEX_COUNT = vertexCount;
		this.NORMAL_COUNT = vertexCount;
		this.INDEX_COUNT = vertexCount;
		this.TEXTURE_COORD_COUNT = vertexCount;
		this.DIMENSION = 4;
		this.TEXTURE_COORD_DIM = 2;

		float[][] vertices = new float[vertexCount][this.DIMENSION];
		float[][] normals = new float[vertexCount][this.DIMENSION];
		int[] indices = new int[vertexCount];
		float[][] textureCoord = new float[vertexCount][this.TEXTURE_COORD_DIM];

		float x1, y1, x2, y2, x3, y3, x4, y4;

		int count_vertex = 0;

		float texture_x = 0;
		float texture_y = 0;

		float diameter_small = diameter - strength;
		float diameter_large = diameter;

		start = angle_begin;
		end = angle_begin + alpha;

		for (angle = end; angle > start;) {

			x1 = diameter_small * (float) Math.cos(angle);
			y1 = diameter_small * (float) Math.sin(angle);

			x2 = diameter_large * (float) Math.cos(angle);
			y2 = diameter_large * (float) Math.sin(angle);

			angle -= step;

			x3 = diameter_large * (float) Math.cos(angle);
			y3 = diameter_large * (float) Math.sin(angle);

			x4 = diameter_small * (float) Math.cos(angle);
			y4 = diameter_small * (float) Math.sin(angle);

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			count_vertex++;

			vertices[count_vertex][0] = x3;
			vertices[count_vertex][1] = y3;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			count_vertex++;

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			vertices[count_vertex][0] = x3;
			vertices[count_vertex][1] = y3;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			count_vertex++;

			vertices[count_vertex][0] = x4;
			vertices[count_vertex][1] = y4;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			texture_x += texture_horizontal_step;
		}


		texture_x = 0;
		texture_y = 0;

		for (angle = end; angle > start;) {

			x1 = diameter_large * (float) Math.cos(angle);
			y1 = diameter_large * (float) Math.sin(angle);

			angle -= step;

			x2 = diameter_large * (float) Math.cos(angle);
			y2 = diameter_large * (float) Math.sin(angle);

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = depth;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			count_vertex++;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = depth;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			count_vertex++;

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = depth;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			count_vertex++;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			texture_x += texture_horizontal_step;
		}

		texture_x = 1;
		texture_y = 0;

		for (angle = start; angle < end;) {

			x1 = diameter_small * (float) Math.cos(angle);
			y1 = diameter_small * (float) Math.sin(angle);

			angle += step;

			x2 = diameter_small * (float) Math.cos(angle);
			y2 = diameter_small * (float) Math.sin(angle);

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = depth;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			count_vertex++;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = depth;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x - texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			count_vertex++;

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = depth;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x - texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			count_vertex++;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x - texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			texture_x -= texture_horizontal_step;
		}

		texture_x = 0;
		texture_y = 0;

		for (angle = start; angle < end && this.VERTEX_COUNT >= count_vertex + 6;) {

			x1 = diameter_small * (float) Math.cos(angle);
			y1 = diameter_small * (float) Math.sin(angle);

			x2 = diameter_large * (float) Math.cos(angle);
			y2 = diameter_large * (float) Math.sin(angle);

			angle += step;

			x3 = diameter_large * (float) Math.cos(angle);
			y3 = diameter_large * (float) Math.sin(angle);

			x4 = diameter_small * (float) Math.cos(angle);
			y4 = diameter_small * (float) Math.sin(angle);

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = depth;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = depth;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			count_vertex++;

			vertices[count_vertex][0] = x3;
			vertices[count_vertex][1] = y3;
			vertices[count_vertex][2] = depth;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			count_vertex++;

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = depth;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			vertices[count_vertex][0] = x3;
			vertices[count_vertex][1] = y3;
			vertices[count_vertex][2] = depth;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			count_vertex++;

			vertices[count_vertex][0] = x4;
			vertices[count_vertex][1] = y4;
			vertices[count_vertex][2] = depth;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			texture_x += texture_horizontal_step;
		}

		for (int i = 0; i < vertexCount; i++) {
			indices[i] = i;
		}

		this.vertices = vertices;
		this.normals = normals;
		this.indices = indices;
		this.textureCoord = textureCoord;

		Geometry.createVertexBuffer(this);
		Geometry.createNormalBuffer(this);
		Geometry.createVertexBufferData(this);
		this.updateNormals();
		Geometry.createNormalBufferData(this);
		Geometry.createTextureCoordBuffer(this);
		Geometry.createTextureCoordinateBufferData(this);
		Geometry.createDataBuffer(this);
		Geometry.createDataBufferData(this);
		Geometry.updateBoundingSpere(this);
	}

	public final void applyTransformation() {
		Transformation.concateTransformations(this.transformationMatrix,
				this.localTransformation, this.transformationMatrix);

		for (int i = 0; i < VERTEX_COUNT; i++) {
			LinearAlgebra4D.calculateTableVectorProduct(
					this.localTransformation.compositeMatrix, vertices[i],
					vertices[i]);
		}

		LinearAlgebra4D.calculateTableVectorProduct(
				this.localTransformation.compositeMatrix, this.center,
				this.center);

		this.localTransformation.setIdentity();

		this.updateNormals();
		Geometry.updateBoundingSpere(this);

		Geometry.updateBuffers(this);
	}

	@Override
	public final void updateNormals() {
		float[] normal = new float[4];

		int i;

		for (i = 0; i < (this.tempCounter) * 3;) {
			LinearAlgebra4D.calculateNormal(this.vertices[i + 2],
					this.vertices[i], this.vertices[i + 1], normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i] = normal.clone();
			this.normals[i + 1] = normal.clone();
			this.normals[i + 2] = normal.clone();

			i += 3;
		}

		for (; i < (this.tempCounter) * 6; i++) {
			LinearAlgebra4D.calculateVector(this.center, this.vertices[i],
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i] = normal.clone();
		}

		for (; i < (this.tempCounter) * 9; i++) {
			LinearAlgebra4D.calculateVector(this.vertices[i], this.center,
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i] = normal.clone();
		}

		for (; i < (this.tempCounter) * 12;) {
			LinearAlgebra4D.calculateNormal(this.vertices[i + 2],
					this.vertices[i], this.vertices[i + 1], normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i] = normal.clone();
			this.normals[i + 1] = normal.clone();
			this.normals[i + 2] = normal.clone();

			i += 3;
		}

	}

	@Override
	public void executeStaticState() {
		// TODO Auto-generated method stub

	}

	@Override
	public void executeDynamicState() {
		// TODO Auto-generated method stub

	}

	@Override
	public void calculateTexCoord() {
		// TODO Auto-generated method stub

	}

	public static void main(String args[]) {

		Arc cube = new Arc(0, (float) Math.PI, 2, 1, 1, 13, 1, 1);

		Transformation t = new Transformation();
		float[] t1 = { 30, 30, 30, 1 };

		t.translate(t1);
		cube.updateTransformationState(t);

		for (int i = 0; i < 4; i++) {
			System.out.println(cube.boundingVolume.center[i]);
		}

		System.out.println(cube.boundingVolume.radius[0]);

	}

}
