package game.engine.geometry;

import game.engine.GameflowEvent;
import game.engine.RouteMapElement;
import game.engine.renderer.Renderer;
import game.engine.spatial.LinearAlgebra4D;
import game.engine.spatial.Transformation;

public class Cube extends Geometry {

	public Cube(float[][] vertices, int[] indices, float[][] textCoord) {
		this.geometryType = GeometryConstants.CUBE;

		this.VERTEX_COUNT = 36;
		this.NORMAL_COUNT = 36;
		this.INDEX_COUNT = 36;
		this.TEXTURE_COORD_COUNT = 36;
		this.DIMENSION = 4;
		this.TEXTURE_COORD_DIM = 2;

		this.vertices = vertices;
		this.indices = indices;
		this.textureCoord = textCoord;
		this.updateNormals();

		Geometry.createVertexBuffer(this);
		Geometry.createNormalBuffer(this);
		Geometry.createVertexBufferData(this);
		Geometry.createNormalBufferData(this);
		Geometry.createTextureCoordBuffer(this);
		Geometry.createTextureCoordinateBufferData(this);
		Geometry.createDataBuffer(this);
		Geometry.createDataBufferData(this);
		Geometry.updateBoundingSpere(this);
	}

	public Cube(float width, float height, float depth, float texture_s_scale,
			float texture_t_scale) {
		this.geometryType = GeometryConstants.CUBE;

		this.VERTEX_COUNT = 36;
		this.NORMAL_COUNT = 36;
		this.INDEX_COUNT = 36;
		this.TEXTURE_COORD_COUNT = 36;
		this.DIMENSION = 4;
		this.TEXTURE_COORD_DIM = 2;

		this.vertices = new float[this.VERTEX_COUNT][this.DIMENSION];
		this.normals = new float[this.NORMAL_COUNT][this.DIMENSION];
		this.indices = new int[this.INDEX_COUNT];
		this.textureCoord = new float[this.TEXTURE_COORD_COUNT][this.TEXTURE_COORD_DIM];

		float x, y, z, h;
		float x1, y1, z1, x2, y2, z2, x3, y3, z3, x4, y4, z4;

		x = width;
		y = height;
		z = depth;
		h = 1;

		float[][] tempVertices = { { 0, 0, 0, h }, { x, 0, 0, h },
				{ x, y, 0, h }, { 0, y, 0, h }, { x, 0, 0, h }, { x, 0, z, h },
				{ x, y, z, h }, { x, y, 0, h }, { x, 0, z, h }, { 0, 0, z, h },
				{ 0, y, z, h }, { x, y, z, h }, { 0, 0, z, h }, { 0, 0, 0, h },
				{ 0, y, 0, h }, { 0, y, z, h }, { 0, y, 0, h }, { x, y, 0, h },
				{ x, y, z, h }, { 0, y, z, h }, { 0, 0, 0, h }, { x, 0, 0, h },
				{ x, 0, z, h }, { 0, 0, z, h } };

		int count_vertex = 0;

		float texture_x = 0;
		float texture_y = 0;

		float texture_horizontal_step = texture_s_scale;
		float texture_vertical_step = texture_t_scale;

		for (int i = 0; i < 24;) {
			x1 = tempVertices[i][0];
			y1 = tempVertices[i][1];
			z1 = tempVertices[i][2];

			x2 = tempVertices[i + 1][0];
			y2 = tempVertices[i + 1][1];
			z2 = tempVertices[i + 1][2];

			x3 = tempVertices[i + 2][0];
			y3 = tempVertices[i + 2][1];
			z3 = tempVertices[i + 2][2];

			x4 = tempVertices[i + 3][0];
			y4 = tempVertices[i + 3][1];
			z4 = tempVertices[i + 3][2];

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = z1;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y;

			++count_vertex;

			vertices[count_vertex][0] = x4;
			vertices[count_vertex][1] = y4;
			vertices[count_vertex][2] = z4;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			++count_vertex;

			vertices[count_vertex][0] = x3;
			vertices[count_vertex][1] = y3;
			vertices[count_vertex][2] = z3;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			++count_vertex;

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = z1;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y;

			++count_vertex;

			vertices[count_vertex][0] = x3;
			vertices[count_vertex][1] = y3;
			vertices[count_vertex][2] = z3;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			++count_vertex;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = z2;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y;

			++count_vertex;
			i += 4;
		}

		for (int i = 0; i < this.INDEX_COUNT; i++) {
			this.indices[i] = i;
		}

		Geometry.createVertexBuffer(this);
		Geometry.createNormalBuffer(this);
		this.updateNormals();
		Geometry.createVertexBufferData(this);
		Geometry.createNormalBufferData(this);
		Geometry.createTextureCoordBuffer(this);
		Geometry.createTextureCoordinateBufferData(this);
		Geometry.createDataBuffer(this);
		Geometry.createDataBufferData(this);
		
		Geometry.updateBoundingSpere(this);
	}

	@Override
	public final  void updateNormals() {
		this.normals = new float[this.NORMAL_COUNT][this.DIMENSION];

		float[] normal = new float[4];

		for (int i = 0; i < 24;) {
			LinearAlgebra4D.calculateNormal(this.vertices[i],
					this.vertices[i + 1], this.vertices[i + 2], normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i][0] = normal[0];
			this.normals[i][1] = normal[1];
			this.normals[i][2] = normal[2];
			this.normals[i][3] = normal[3];

			this.normals[i + 1][0] = normal[0];
			this.normals[i + 1][1] = normal[1];
			this.normals[i + 1][2] = normal[2];
			this.normals[i + 1][3] = normal[3];

			this.normals[i + 2][0] = normal[0];
			this.normals[i + 2][1] = normal[1];
			this.normals[i + 2][2] = normal[2];
			this.normals[i + 2][3] = normal[3];

			this.normals[i + 3][0] = normal[0];
			this.normals[i + 3][1] = normal[1];
			this.normals[i + 3][2] = normal[2];
			this.normals[i + 3][3] = normal[3];

			this.normals[i + 4][0] = normal[0];
			this.normals[i + 4][1] = normal[1];
			this.normals[i + 4][2] = normal[2];
			this.normals[i + 4][3] = normal[3];

			this.normals[i + 5][0] = normal[0];
			this.normals[i + 5][1] = normal[1];
			this.normals[i + 5][2] = normal[2];
			this.normals[i + 5][3] = normal[3];

			i += 6;
		}

		for (int i = 24; i < 30;) {
			LinearAlgebra4D.calculateNormal(this.vertices[i + 1],
					this.vertices[i + 2], this.vertices[i], normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i][0] = normal[0];
			this.normals[i][1] = normal[1];
			this.normals[i][2] = normal[2];
			this.normals[i][3] = normal[3];

			this.normals[i + 1][0] = normal[0];
			this.normals[i + 1][1] = normal[1];
			this.normals[i + 1][2] = normal[2];
			this.normals[i + 1][3] = normal[3];

			this.normals[i + 2][0] = normal[0];
			this.normals[i + 2][1] = normal[1];
			this.normals[i + 2][2] = normal[2];
			this.normals[i + 2][3] = normal[3];

			this.normals[i + 3][0] = normal[0];
			this.normals[i + 3][1] = normal[1];
			this.normals[i + 3][2] = normal[2];
			this.normals[i + 3][3] = normal[3];

			this.normals[i + 4][0] = normal[0];
			this.normals[i + 4][1] = normal[1];
			this.normals[i + 4][2] = normal[2];
			this.normals[i + 4][3] = normal[3];

			this.normals[i + 5][0] = normal[0];
			this.normals[i + 5][1] = normal[1];
			this.normals[i + 5][2] = normal[2];
			this.normals[i + 5][3] = normal[3];

			i += 6;
		}

		for (int i = 30; i < 36;) {
			LinearAlgebra4D.calculateNormal(this.vertices[i + 1],
					this.vertices[i + 2], this.vertices[i], normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i][0] = normal[0];
			this.normals[i][1] = normal[1];
			this.normals[i][2] = normal[2];
			this.normals[i][3] = normal[3];

			this.normals[i + 1][0] = normal[0];
			this.normals[i + 1][1] = normal[1];
			this.normals[i + 1][2] = normal[2];
			this.normals[i + 1][3] = normal[3];

			this.normals[i + 2][0] = normal[0];
			this.normals[i + 2][1] = normal[1];
			this.normals[i + 2][2] = normal[2];
			this.normals[i + 2][3] = normal[3];

			this.normals[i + 3][0] = normal[0];
			this.normals[i + 3][1] = normal[1];
			this.normals[i + 3][2] = normal[2];
			this.normals[i + 3][3] = normal[3];

			this.normals[i + 4][0] = normal[0];
			this.normals[i + 4][1] = normal[1];
			this.normals[i + 4][2] = normal[2];
			this.normals[i + 4][3] = normal[3];

			this.normals[i + 5][0] = normal[0];
			this.normals[i + 5][1] = normal[1];
			this.normals[i + 5][2] = normal[2];
			this.normals[i + 5][3] = normal[3];

			i += 6;
		}

		for (int i = 0; i < 4; i++) {
			this.normals[i] = normal.clone();
		}

	}

	public static void main(String args[]) {
		Cube cube = new Cube(2, 3, 2, 1, 1);

		Transformation t = new Transformation();
		float[] t1 = {20,20,20,1};
		
		t.translate(t1);
		cube.updateTransformationState(t);
		
		for(int i = 0; i < 4;i++){
			System.out.println(cube.boundingVolume.center[i]);
		}
		
		System.out.println(cube.boundingVolume.radius[0]);
	}

	@Override
	public void executeStaticState() {

	}

	@Override
	public void executeDynamicState() {

	}

	@Override
	public void calculateTexCoord() {
		// TODO Auto-generated method stub

	}
	

}
