package game.engine.geometry;

import game.engine.spatial.LinearAlgebra4D;
import game.engine.spatial.Transformation;

public class Cylinder extends Geometry {

	protected float[] center = { 0, 0, 0, 1 };

	public Cylinder(int vertexCount, float radius, float height,
			float texture_s_scale, float texture_t_scale) {
		this.geometryType = GeometryConstants.CYLINDER;

		float step = (float) (2 * Math.PI) / (float) (vertexCount);

		float angle;
		
		int index = 0;

		for (angle = 0; angle < (float) (2.0f * Math.PI);) {
			++index;
			angle += step;
		}

		index *= 6;
		index *= 2;
		
		vertexCount = index;
		
		float texture_x, texture_y;
		float texture_horizontal_step, texture_vertical_step;

		texture_x = 1;
		texture_y = 0;

		texture_horizontal_step = (float) 1 * texture_s_scale;
		texture_vertical_step = texture_t_scale;

		

		this.VERTEX_COUNT = vertexCount;
		this.NORMAL_COUNT = vertexCount;
		this.INDEX_COUNT = vertexCount;
		this.TEXTURE_COORD_COUNT = vertexCount;
		this.DIMENSION = 4;
		this.TEXTURE_COORD_DIM = 2;

		float[][] vertices = new float[this.VERTEX_COUNT][this.DIMENSION];
		float[][] normals = new float[this.NORMAL_COUNT][this.DIMENSION];
		int[] indices = new int[this.INDEX_COUNT];
		float[][] textureCoord = new float[this.TEXTURE_COORD_COUNT][this.TEXTURE_COORD_DIM];

		float x1, y1, z1, x2, y2, z2, x3, y3, z3, x4, y4, z4;
		

		int count_vertex = 0;

		for (angle = 0; angle < (float) (2.0f * Math.PI);) {

			x1 = radius * (float) Math.cos(angle);
			y1 = 0;
			z1 = radius * (float) Math.sin(angle);

			x2 = radius * (float) Math.cos(angle);
			y2 = height;
			z2 = radius * (float) Math.sin(angle);

			angle += step;

			x3 = radius * (float) Math.cos(angle);
			y3 = height;
			z3 = radius * (float) Math.sin(angle);

			x4 = radius * (float) Math.cos(angle);
			y4 = 0;
			z4 = radius * (float) Math.sin(angle);

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = z1;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y;

			++count_vertex;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = z2;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			++count_vertex;

			vertices[count_vertex][0] = x4;
			vertices[count_vertex][1] = y4;
			vertices[count_vertex][2] = z4;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x - texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y;

			++count_vertex;

			vertices[count_vertex][0] = x4;
			vertices[count_vertex][1] = y4;
			vertices[count_vertex][2] = z4;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x - texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y;

			++count_vertex;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = z2;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			++count_vertex;

			vertices[count_vertex][0] = x3;
			vertices[count_vertex][1] = y3;
			vertices[count_vertex][2] = z3;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x - texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			++count_vertex;

			texture_x -= texture_horizontal_step;
		}

		texture_x = 1;
		texture_y = 0;

		for (angle = (float) (2.0f * Math.PI); angle > 0;) {

			x1 = radius * (float) Math.cos(angle);
			y1 = 0;
			z1 = radius * (float) Math.sin(angle);

			x2 = radius * (float) Math.cos(angle);
			y2 = height;
			z2 = radius * (float) Math.sin(angle);

			angle -= step;

			x3 = radius * (float) Math.cos(angle);
			y3 = height;
			z3 = radius * (float) Math.sin(angle);

			x4 = radius * (float) Math.cos(angle);
			y4 = 0;
			z4 = radius * (float) Math.sin(angle);

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = z1;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y;

			++count_vertex;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = z2;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			++count_vertex;

			vertices[count_vertex][0] = x4;
			vertices[count_vertex][1] = y4;
			vertices[count_vertex][2] = z4;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x - texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y;

			++count_vertex;

			vertices[count_vertex][0] = x4;
			vertices[count_vertex][1] = y4;
			vertices[count_vertex][2] = z4;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x - texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y;

			++count_vertex;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = z2;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			++count_vertex;

			vertices[count_vertex][0] = x3;
			vertices[count_vertex][1] = y3;
			vertices[count_vertex][2] = z3;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x - texture_horizontal_step;
			textureCoord[count_vertex][1] = texture_y + texture_vertical_step;

			++count_vertex;

			texture_x -= texture_horizontal_step;
		}

		for (int i = 0; i < vertexCount; i++) {
			indices[i] = i;
		}

		this.vertices = vertices;
		this.normals = normals;
		this.indices = indices;
		this.textureCoord = textureCoord;

		Geometry.createVertexBuffer(this);
		Geometry.createNormalBuffer(this);
		Geometry.createVertexBufferData(this);
		this.updateNormals();
		Geometry.createNormalBufferData(this);
		Geometry.createTextureCoordBuffer(this);
		Geometry.createTextureCoordinateBufferData(this);
		Geometry.createDataBuffer(this);
		Geometry.createDataBufferData(this);

		Geometry.updateBoundingSpere(this);
	}

	protected final float[] getCenter() {
		return this.center;
	}

	public final void applyTransformation() {

		Transformation.concateTransformations(this.transformationMatrix,
				this.localTransformation, this.transformationMatrix);

		for (int i = 0; i < VERTEX_COUNT; i++) {
			LinearAlgebra4D.calculateTableVectorProduct(
					this.localTransformation.compositeMatrix, vertices[i],
					vertices[i]);
		}

		LinearAlgebra4D.calculateTableVectorProduct(
				this.localTransformation.compositeMatrix, this.center,
				this.center);

		this.localTransformation.setIdentity();

		this.updateNormals();
		Geometry.updateBoundingSpere(this);

		Geometry.updateBuffers(this);
	}

	@Override
	public void updateNormals() {
		float[] normal = new float[4];

		int i = 0;

		for (; i < this.VERTEX_COUNT / 2; i++) {
			LinearAlgebra4D.calculateVector(this.center, this.vertices[i],
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i][0] = normal[0];
			this.normals[i][1] = normal[1];
			this.normals[i][2] = normal[2];
			this.normals[i][3] = normal[3];
		}

		for (; i < this.VERTEX_COUNT; i++) {
			LinearAlgebra4D.calculateVector(this.vertices[i], this.center,
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i][0] = normal[0];
			this.normals[i][1] = normal[1];
			this.normals[i][2] = normal[2];
			this.normals[i][3] = normal[3];
		}
	}

	@Override
	public void executeStaticState() {
		// TODO Auto-generated method stub

	}

	@Override
	public void executeDynamicState() {
		// TODO Auto-generated method stub

	}

	@Override
	public void calculateTexCoord() {
		// TODO Auto-generated method stub

	}

	public static void main(String args[]) {
		Cylinder cube = new Cylinder(10, 2, 1, 1, 13);

		Transformation t = new Transformation();
		float[] t1 = { 30, 30, 30, 1 };

		t.translate(t1);
		cube.updateTransformationState(t);

		for (int i = 0; i < 4; i++) {
			System.out.println(cube.boundingVolume.center[i]);
		}

		System.out.println(cube.boundingVolume.radius[0]);
	}
}
