package game.engine.geometry;

import game.engine.spatial.LinearAlgebra4D;
import game.engine.spatial.Transformation;

public class DeformedConeArc extends Geometry{

	protected float[] center1 = { 0, 0, 0, 1 };

	protected float[] center2 = null;

	protected int t = 0;
	
	public DeformedConeArc(float radius1, float radius2, float length, float alpha,
			int vertexCount, float texture_s_scale, float texture_t_scale) {
		this.geometryType = GeometryConstants.DEFORMED_CONE_ARC;
		this.center2 = new float[4];

		this.center2[0] = this.center1[0];
		this.center2[1] = this.center1[1];
		this.center2[2] = length;
		this.center2[3] = 1;

		float step = (float) (2 * Math.PI) / (vertexCount);
		
		float texture_x = 0;
		float texture_y = 0;
		
		float textureStep_horizontal = 1 * texture_s_scale;
		float textureStep_vertical = (float)1 * texture_t_scale/vertexCount;
		
		float angle;
		int index = 0;
		
		for (angle = 0.0f; angle < alpha;) {
			++index;
			angle += step;
		}

		vertexCount = index;
		vertexCount *= 6;
		vertexCount += 6;
		vertexCount *= 2;
		
		
		this.VERTEX_COUNT = vertexCount ;
		this.NORMAL_COUNT = vertexCount;
		this.INDEX_COUNT = vertexCount;
		this.TEXTURE_COORD_COUNT = vertexCount;
		this.DIMENSION = 4;
		this.TEXTURE_COORD_DIM = 2;

		float[][] vertices = new float[this.VERTEX_COUNT][this.DIMENSION];
		float[][] normals = new float[this.NORMAL_COUNT][this.DIMENSION];
		int[] indices = new int[this.INDEX_COUNT];
		float[][] textureCoord = new float[this.TEXTURE_COORD_COUNT][this.TEXTURE_COORD_DIM];

		float x1, y1, x2, y2, x3, y3, x4, y4;

		int count_vertex = 0;
		int count_normal = 0;

		float[] vertex = new float[4];
		float[] normal = new float[4];

		float texture_small_diameter = (float)(radius1/radius2);

		for (angle = 0.0f; angle < alpha;) {

			x1 = radius1 * (float) Math.cos(angle);
			y1 = radius1 * (float) Math.sin(angle);

			x3 = radius2 * (float) Math.cos(angle);
			y3 = radius2 * (float) Math.sin(angle);

			angle += step;

			x2 = radius1 * (float) Math.cos(angle);
			y2 = radius1 * (float) Math.sin(angle);

			x4 = radius2 * (float) Math.cos(angle);
			y4 = radius2 * (float) Math.sin(angle);

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;
			
			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y + textureStep_vertical;

			count_vertex++;

			vertices[count_vertex][0] = x3;
			vertices[count_vertex][1] = y3;
			vertices[count_vertex][2] = length;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + textureStep_horizontal;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			vertices[count_vertex][0] = x3;
			vertices[count_vertex][1] = y3;
			vertices[count_vertex][2] = length;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + textureStep_horizontal;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y + textureStep_vertical;

			count_vertex++;

			vertices[count_vertex][0] = x4;
			vertices[count_vertex][1] = y4;
			vertices[count_vertex][2] = length;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + textureStep_horizontal;
			textureCoord[count_vertex][1] = texture_y + textureStep_vertical;

			count_vertex++;
			texture_y += textureStep_vertical;
		}

		angle = (float) (alpha - step);

		x1 = radius1 * (float) Math.cos(angle);
		y1 = radius1 * (float) Math.sin(angle);

		x3 = radius2 * (float) Math.cos(angle);
		y3 = radius2 * (float) Math.sin(angle);

		angle = (float)(alpha);

		x2 = radius1 * (float) Math.cos(angle);
		y2 = radius1 * (float) Math.sin(angle);

		x4 = radius2 * (float) Math.cos(angle);
		y4 = radius2 * (float) Math.sin(angle);

		vertices[count_vertex][0] = x1;
		vertices[count_vertex][1] = y1;
		vertices[count_vertex][2] = 0;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = texture_x;
		textureCoord[count_vertex][1] = texture_y;

		count_vertex++;

		vertices[count_vertex][0] = x2;
		vertices[count_vertex][1] = y2;
		vertices[count_vertex][2] = 0;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = texture_x;
		textureCoord[count_vertex][1] = texture_y + textureStep_vertical;

		count_vertex++;

		vertices[count_vertex][0] = x3;
		vertices[count_vertex][1] = y3;
		vertices[count_vertex][2] = length;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = texture_x + textureStep_horizontal;
		textureCoord[count_vertex][1] = texture_y;

		count_vertex++;

		vertices[count_vertex][0] = x3;
		vertices[count_vertex][1] = y3;
		vertices[count_vertex][2] = length;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = texture_x + textureStep_horizontal;
		textureCoord[count_vertex][1] = texture_y;

		count_vertex++;

		vertices[count_vertex][0] = x2;
		vertices[count_vertex][1] = y2;
		vertices[count_vertex][2] = 0;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = texture_x;
		textureCoord[count_vertex][1] = texture_y + textureStep_vertical;

		count_vertex++;

		vertices[count_vertex][0] = x4;
		vertices[count_vertex][1] = y4;
		vertices[count_vertex][2] = length;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = texture_x + textureStep_horizontal;
		textureCoord[count_vertex][1] = texture_y + textureStep_vertical;

		count_vertex++;
		
		for (angle = alpha; angle > 0.0f;) {

			x1 = radius1 * (float) Math.cos(angle);
			y1 = radius1 * (float) Math.sin(angle);

			x3 = radius2 * (float) Math.cos(angle);
			y3 = radius2 * (float) Math.sin(angle);

			angle -= step;

			x2 = radius1 * (float) Math.cos(angle);
			y2 = radius1 * (float) Math.sin(angle);

			x4 = radius2 * (float) Math.cos(angle);
			y4 = radius2 * (float) Math.sin(angle);

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = y1;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;
			
			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y + textureStep_vertical;

			count_vertex++;

			vertices[count_vertex][0] = x3;
			vertices[count_vertex][1] = y3;
			vertices[count_vertex][2] = length;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + textureStep_horizontal;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			vertices[count_vertex][0] = x3;
			vertices[count_vertex][1] = y3;
			vertices[count_vertex][2] = length;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + textureStep_horizontal;
			textureCoord[count_vertex][1] = texture_y;

			count_vertex++;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = y2;
			vertices[count_vertex][2] = 0;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x;
			textureCoord[count_vertex][1] = texture_y + textureStep_vertical;

			count_vertex++;

			vertices[count_vertex][0] = x4;
			vertices[count_vertex][1] = y4;
			vertices[count_vertex][2] = length;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_x + textureStep_horizontal;
			textureCoord[count_vertex][1] = texture_y + textureStep_vertical;

			count_vertex++;
			texture_y += textureStep_vertical;
		}

		angle = (float) (0 + step);

		x1 = radius1 * (float) Math.cos(angle);
		y1 = radius1 * (float) Math.sin(angle);

		x3 = radius2 * (float) Math.cos(angle);
		y3 = radius2 * (float) Math.sin(angle);

		angle = (float)(0);

		x2 = radius1 * (float) Math.cos(angle);
		y2 = radius1 * (float) Math.sin(angle);

		x4 = radius2 * (float) Math.cos(angle);
		y4 = radius2 * (float) Math.sin(angle);

		vertices[count_vertex][0] = x1;
		vertices[count_vertex][1] = y1;
		vertices[count_vertex][2] = 0;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = texture_x;
		textureCoord[count_vertex][1] = texture_y;

		count_vertex++;

		vertices[count_vertex][0] = x2;
		vertices[count_vertex][1] = y2;
		vertices[count_vertex][2] = 0;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = texture_x;
		textureCoord[count_vertex][1] = texture_y + textureStep_vertical;

		count_vertex++;

		vertices[count_vertex][0] = x3;
		vertices[count_vertex][1] = y3;
		vertices[count_vertex][2] = length;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = texture_x + textureStep_horizontal;
		textureCoord[count_vertex][1] = texture_y;

		count_vertex++;

		vertices[count_vertex][0] = x3;
		vertices[count_vertex][1] = y3;
		vertices[count_vertex][2] = length;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = texture_x + textureStep_horizontal;
		textureCoord[count_vertex][1] = texture_y;

		count_vertex++;

		vertices[count_vertex][0] = x2;
		vertices[count_vertex][1] = y2;
		vertices[count_vertex][2] = 0;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = texture_x;
		textureCoord[count_vertex][1] = texture_y + textureStep_vertical;

		count_vertex++;

		vertices[count_vertex][0] = x4;
		vertices[count_vertex][1] = y4;
		vertices[count_vertex][2] = length;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = texture_x + textureStep_horizontal;
		textureCoord[count_vertex][1] = texture_y + textureStep_vertical;


		for (int i = 0; i < vertexCount; i++) {
			indices[i] = i;
		}

		this.vertices = vertices;
		this.normals = normals;
		this.indices = indices;
		this.textureCoord = textureCoord;

		Geometry.createVertexBuffer(this);
		Geometry.createNormalBuffer(this);
		Geometry.createVertexBufferData(this);
		this.updateNormals();
		Geometry.createNormalBufferData(this);
		Geometry.createTextureCoordBuffer(this);
		Geometry.createTextureCoordinateBufferData(this);
		Geometry.createDataBuffer(this);
		Geometry.createDataBufferData(this);
		
		Geometry.updateBoundingSpere(this);
	}
	
	public final void applyTransformation() {

		Transformation.concateTransformations(this.transformationMatrix,
				this.localTransformation, this.transformationMatrix);

		for (int i = 0; i < VERTEX_COUNT; i++) {
			LinearAlgebra4D.calculateTableVectorProduct(
					this.localTransformation.compositeMatrix, vertices[i],
					vertices[i]);
		}

		LinearAlgebra4D.calculateTableVectorProduct(
				this.localTransformation.compositeMatrix, this.center1,
				this.center1);

		LinearAlgebra4D.calculateTableVectorProduct(
				this.localTransformation.compositeMatrix, this.center2,
				this.center2);

		this.localTransformation.setIdentity();

		this.updateNormals();
		Geometry.updateBoundingSpere(this);
		
		Geometry.updateBuffers(this);
	}
	
	@Override
	public final void updateNormals() {
		float[] normal = new float[4];

		int i;

		for (i = 0; i < this.VERTEX_COUNT /2 ;) {
			LinearAlgebra4D.calculateVector(this.center1, this.vertices[i],
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i][0] = normal[0];
			this.normals[i][1] = normal[1];
			this.normals[i][2] = normal[2];
			this.normals[i][3] = normal[3];

			LinearAlgebra4D.calculateVector(this.center1, this.vertices[i + 1],
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i + 1][0] = normal[0];
			this.normals[i + 1][1] = normal[1];
			this.normals[i + 1][2] = normal[2];
			this.normals[i + 1][3] = normal[3];
			
			LinearAlgebra4D.calculateVector(this.center2, this.vertices[i + 2],
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i + 2][0] = normal[0];
			this.normals[i + 2][1] = normal[1];
			this.normals[i + 2][2] = normal[2];
			this.normals[i + 2][3] = normal[3];
			
			LinearAlgebra4D.calculateVector(this.center2, this.vertices[i + 3],
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i + 3][0] = normal[0];
			this.normals[i + 3][1] = normal[1];
			this.normals[i + 3][2] = normal[2];
			this.normals[i + 3][3] = normal[3];
			
			LinearAlgebra4D.calculateVector(this.center1, this.vertices[i + 4],
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i + 4][0] = normal[0];
			this.normals[i + 4][1] = normal[1];
			this.normals[i + 4][2] = normal[2];
			this.normals[i + 4][3] = normal[3];
			
			LinearAlgebra4D.calculateVector(this.center2, this.vertices[i + 5],
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i + 5][0] = normal[0];
			this.normals[i + 5][1] = normal[1];
			this.normals[i + 5][2] = normal[2];
			this.normals[i + 5][3] = normal[3];

			i += 6;
		}
		
		for (; i < this.VERTEX_COUNT ;) {
			LinearAlgebra4D.calculateVector( this.vertices[i], this.center1,
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i][0] = normal[0];
			this.normals[i][1] = normal[1];
			this.normals[i][2] = normal[2];
			this.normals[i][3] = normal[3];

			LinearAlgebra4D.calculateVector( this.vertices[i + 1], this.center1,
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i + 1][0] = normal[0];
			this.normals[i + 1][1] = normal[1];
			this.normals[i + 1][2] = normal[2];
			this.normals[i + 1][3] = normal[3];
			
			LinearAlgebra4D.calculateVector( this.vertices[i + 2], this.center2,
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i + 2][0] = normal[0];
			this.normals[i + 2][1] = normal[1];
			this.normals[i + 2][2] = normal[2];
			this.normals[i + 2][3] = normal[3];
			
			LinearAlgebra4D.calculateVector( this.vertices[i + 3], this.center2,
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i + 3][0] = normal[0];
			this.normals[i + 3][1] = normal[1];
			this.normals[i + 3][2] = normal[2];
			this.normals[i + 3][3] = normal[3];
			
			LinearAlgebra4D.calculateVector( this.vertices[i + 4], this.center1,
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i + 4][0] = normal[0];
			this.normals[i + 4][1] = normal[1];
			this.normals[i + 4][2] = normal[2];
			this.normals[i + 4][3] = normal[3];
			
			LinearAlgebra4D.calculateVector( this.vertices[i + 5], this.center2,
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i + 5][0] = normal[0];
			this.normals[i + 5][1] = normal[1];
			this.normals[i + 5][2] = normal[2];
			this.normals[i + 5][3] = normal[3];
			
			i += 6;
		}
		
	}

	@Override
	public void executeStaticState() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void executeDynamicState() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void calculateTexCoord() {
		// TODO Auto-generated method stub
		
	}
	
	public static void main(String args[]) {

		DeformedConeArc cube = new DeformedConeArc(0.2f, 0.3f, 5, (float)Math.PI, 20, 1, 1);

		Transformation t = new Transformation();
		float[] t1 = { 30, 30, 30, 1 };

		t.translate(t1);
		cube.updateTransformationState(t);

		for (int i = 0; i < 4; i++) {
			System.out.println(cube.boundingVolume.center[i]);
		}

		System.out.println(cube.boundingVolume.radius[0]);

	}

}
