package game.engine.geometry;

import game.engine.spatial.LinearAlgebra4D;
import game.engine.spatial.Transformation;

public class Ellipsoid extends Geometry {

	protected float[] center = { 0, 0, 0, 1 };

	public Ellipsoid(float diameter_x, float diameter_y, float diameter_z,
			int vertexCount_vertical, int vertexCount_horizontal) {

		this.geometryType = GeometryConstants.ELLIPSOID;
		this.DIMENSION = 4;
		this.TEXTURE_COORD_DIM = 2;

		float step_vertical = (float) ((float) (2 * Math.PI) / (float) (vertexCount_vertical));

		float step_horizontal = (float) ((float) (2 * Math.PI) / (float) (vertexCount_horizontal));

		int vertexCount = (vertexCount_vertical) * (vertexCount_horizontal) * 6;

		float[][] vertices = new float[vertexCount][this.DIMENSION];
		float[][] textureCoord = new float[vertexCount][this.TEXTURE_COORD_DIM];

		float x1, y1, z1, x2, y2, z2, x3, y3, z3, x4, y4, z4;
		float angle_horizontal;
		float angle_vertical;

		float texture_vertical_step = (float) ((float) 1 / (float) vertexCount_vertical);
		float texture_horizontal_step = (float) ((float) 1 / (float) vertexCount_horizontal);

		float texture_x = 0;
		float texture_y = 0;

		int currentVertex = 0;
		
		float angle_alpha_begin = (float)-((float)Math.PI/(float)2);
		float angle_alpha_end = (float)((float)Math.PI/(float)2);
		
		float angle_beta_begin = (float)-((float)Math.PI);
		float angle_beta_end = (float)((float)Math.PI);
		
		
		for (angle_vertical = angle_alpha_begin; angle_vertical < angle_alpha_end;) {
			texture_x = 1;

			for (angle_horizontal = angle_beta_end; angle_horizontal > angle_beta_begin;) {
				x1 = diameter_x
						* (float) (Math.cos(angle_vertical) * (float) Math
								.cos(angle_horizontal));
				y1 = diameter_y
						* (float) (Math.cos(angle_vertical) * (float) Math
								.sin(angle_horizontal));
				z1 = diameter_z * (float) Math.sin(angle_vertical);

				angle_vertical += step_vertical;

				x2 = diameter_x
						* (float) (Math.cos(angle_vertical) * (float) Math
								.cos(angle_horizontal));
				y2 = diameter_y
						* (float) (Math.cos(angle_vertical) * (float) Math
								.sin(angle_horizontal));
				z2 = diameter_z * (float) Math.sin(angle_vertical);

				angle_horizontal -= step_horizontal;

				x3 = diameter_x
						* (float) (Math.cos(angle_vertical) * (float) Math
								.cos(angle_horizontal));
				y3 = diameter_y
						* (float) (Math.cos(angle_vertical) * (float) Math
								.sin(angle_horizontal));
				z3 = diameter_z * (float) Math.sin(angle_vertical);

				angle_vertical -= step_vertical;

				x4 = diameter_x
						* (float) (Math.cos(angle_vertical) * (float) Math
								.cos(angle_horizontal));
				y4 = diameter_y
						* (float) (Math.cos(angle_vertical) * (float) Math
								.sin(angle_horizontal));
				z4 = diameter_z * (float) Math.sin(angle_vertical);

				vertices[currentVertex][0] = x1;
				vertices[currentVertex][1] = y1;
				vertices[currentVertex][2] = z1;
				vertices[currentVertex][3] = 1;

				textureCoord[currentVertex][0] = texture_x;
				textureCoord[currentVertex][1] = texture_y;

				++currentVertex;

				vertices[currentVertex][0] = x2;
				vertices[currentVertex][1] = y2;
				vertices[currentVertex][2] = z2;
				vertices[currentVertex][3] = 1;

				textureCoord[currentVertex][0] = texture_x;
				textureCoord[currentVertex][1] = texture_y
						+ texture_vertical_step;

				++currentVertex;

				vertices[currentVertex][0] = x3;
				vertices[currentVertex][1] = y3;
				vertices[currentVertex][2] = z3;
				vertices[currentVertex][3] = 1;

				textureCoord[currentVertex][0] = texture_x
						- texture_vertical_step;
				textureCoord[currentVertex][1] = texture_y
						+ texture_vertical_step;

				++currentVertex;

				vertices[currentVertex][0] = x1;
				vertices[currentVertex][1] = y1;
				vertices[currentVertex][2] = z1;
				vertices[currentVertex][3] = 1;

				textureCoord[currentVertex][0] = texture_x;
				textureCoord[currentVertex][1] = texture_y;

				++currentVertex;

				vertices[currentVertex][0] = x3;
				vertices[currentVertex][1] = y3;
				vertices[currentVertex][2] = z3;
				vertices[currentVertex][3] = 1;

				textureCoord[currentVertex][0] = texture_x
						- texture_vertical_step;
				textureCoord[currentVertex][1] = texture_y
						+ texture_vertical_step;

				++currentVertex;

				vertices[currentVertex][0] = x4;
				vertices[currentVertex][1] = y4;
				vertices[currentVertex][2] = z4;
				vertices[currentVertex][3] = 1;

				textureCoord[currentVertex][0] = texture_x
						- texture_vertical_step;
				textureCoord[currentVertex][1] = texture_y;

				++currentVertex;

				texture_x -= texture_horizontal_step;
			}

			angle_horizontal = angle_beta_begin + step_horizontal;

			x1 = diameter_x
					* (float) (Math.cos(angle_vertical) * (float) Math
							.cos(angle_horizontal));
			y1 = diameter_y
					* (float) (Math.cos(angle_vertical) * (float) Math
							.sin(angle_horizontal));
			z1 = diameter_z * (float) Math.sin(angle_vertical);

			angle_vertical += step_vertical;

			x2 = diameter_x
					* (float) (Math.cos(angle_vertical) * (float) Math
							.cos(angle_horizontal));
			y2 = diameter_y
					* (float) (Math.cos(angle_vertical) * (float) Math
							.sin(angle_horizontal));
			z2 = diameter_z * (float) Math.sin(angle_vertical);

			angle_horizontal = angle_beta_begin;

			x3 = diameter_x
					* (float) (Math.cos(angle_vertical) * (float) Math
							.cos(angle_horizontal));
			y3 = diameter_y
					* (float) (Math.cos(angle_vertical) * (float) Math
							.sin(angle_horizontal));
			z3 = diameter_z * (float) Math.sin(angle_vertical);

			angle_vertical -= step_vertical;

			x4 = diameter_x
					* (float) (Math.cos(angle_vertical) * (float) Math
							.cos(angle_horizontal));
			y4 = diameter_y
					* (float) (Math.cos(angle_vertical) * (float) Math
							.sin(angle_horizontal));
			z4 = diameter_z * (float) Math.sin(angle_vertical);

			vertices[currentVertex][0] = x1;
			vertices[currentVertex][1] = y1;
			vertices[currentVertex][2] = z1;
			vertices[currentVertex][3] = 1;

			textureCoord[currentVertex][0] = texture_x;
			textureCoord[currentVertex][1] = texture_y;

			++currentVertex;

			vertices[currentVertex][0] = x3;
			vertices[currentVertex][1] = y3;
			vertices[currentVertex][2] = z3;
			vertices[currentVertex][3] = 1;

			textureCoord[currentVertex][0] = texture_x - texture_vertical_step;
			textureCoord[currentVertex][1] = texture_y + texture_vertical_step;

			++currentVertex;

			vertices[currentVertex][0] = x2;
			vertices[currentVertex][1] = y2;
			vertices[currentVertex][2] = z2;
			vertices[currentVertex][3] = 1;

			textureCoord[currentVertex][0] = texture_x;
			textureCoord[currentVertex][1] = texture_y + texture_vertical_step;

			++currentVertex;

			vertices[currentVertex][0] = x1;
			vertices[currentVertex][1] = y1;
			vertices[currentVertex][2] = z1;
			vertices[currentVertex][3] = 1;

			textureCoord[currentVertex][0] = texture_x;
			textureCoord[currentVertex][1] = texture_y;

			++currentVertex;

			vertices[currentVertex][0] = x4;
			vertices[currentVertex][1] = y4;
			vertices[currentVertex][2] = z4;
			vertices[currentVertex][3] = 1;

			textureCoord[currentVertex][0] = texture_x - texture_vertical_step;
			textureCoord[currentVertex][1] = texture_y;

			++currentVertex;

			vertices[currentVertex][0] = x3;
			vertices[currentVertex][1] = y3;
			vertices[currentVertex][2] = z3;
			vertices[currentVertex][3] = 1;

			textureCoord[currentVertex][0] = texture_x - texture_vertical_step;
			textureCoord[currentVertex][1] = texture_y + texture_vertical_step;

			++currentVertex;

			texture_y += texture_vertical_step;
			angle_vertical += step_vertical;
		}
		
		vertexCount = --currentVertex;

		
		this.VERTEX_COUNT = vertexCount;
		this.NORMAL_COUNT = vertexCount;
		this.INDEX_COUNT = vertexCount;
		this.TEXTURE_COORD_COUNT = vertexCount;

		this.indices = new int[this.VERTEX_COUNT];

		for (int i = 0; i < this.VERTEX_COUNT; i++) {
			this.indices[i] = i;
		}

		Geometry.createVertexBuffer(this);
		this.vertices = new float[this.VERTEX_COUNT][this.DIMENSION];

		for (int i = 0; i < this.VERTEX_COUNT; i++) {
			for (int j = 0; j < this.DIMENSION; j++) {
				this.vertices[i][j] = vertices[i][j];
			}
		}

		Geometry.createVertexBufferData(this);

		Geometry.createNormalBuffer(this);
		this.normals = new float[this.VERTEX_COUNT][this.DIMENSION];
		this.updateNormals();
		Geometry.createNormalBufferData(this);

		Geometry.createTextureCoordBuffer(this);
		this.textureCoord = new float[this.TEXTURE_COORD_COUNT][this.TEXTURE_COORD_DIM];

		for (int i = 0; i < this.TEXTURE_COORD_COUNT; i++) {
			for (int j = 0; j < this.TEXTURE_COORD_DIM; j++) {
				this.textureCoord[i][j] = textureCoord[i][j];
			}
		}

		Geometry.createTextureCoordinateBufferData(this);
		Geometry.createDataBuffer(this);
		Geometry.createDataBufferData(this);
		
		Geometry.updateBoundingSpere(this);
	}

	@Override
	public final void updateNormals() {
		float[] normal = new float[4];

		int i = 0;

		for (; i < this.VERTEX_COUNT ; i++) {

			LinearAlgebra4D.calculateVector(this.center, this.vertices[i],
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i] = normal.clone();
		}
	}

	@Override
	public void executeStaticState() {
		// TODO Auto-generated method stub

	}

	@Override
	public void executeDynamicState() {
		// TODO Auto-generated method stub

	}

	public float[] getCenter() {
		return this.center;
	}

	public void applyTransformation() {

		Transformation.concateTransformations(this.transformationMatrix,
				this.localTransformation, this.transformationMatrix);

		for (int i = 0; i < VERTEX_COUNT; i++) {
			LinearAlgebra4D.calculateTableVectorProduct(
					this.localTransformation.compositeMatrix, vertices[i],
					vertices[i]);
		}

		LinearAlgebra4D.calculateTableVectorProduct(
				this.localTransformation.compositeMatrix, this.center,
				this.center);

		this.localTransformation.setIdentity();

		this.updateNormals();
		Geometry.updateBoundingSpere(this);
		
		Geometry.updateBuffers(this);
	}



	@Override
	public void calculateTexCoord() {
		// TODO Auto-generated method stub
		
	}
	
	public static void main(String args[]) {

		Ellipsoid cube = new Ellipsoid(2, 2, 2, 10, 10);

		Transformation t = new Transformation();
		float[] t1 = { 30, 30, 30, 1 };

		t.translate(t1);
		cube.updateTransformationState(t);

		for (int i = 0; i < 4; i++) {
			System.out.println(cube.boundingVolume.center[i]);
		}

		System.out.println(cube.boundingVolume.radius[0]);

	}

}
