 package game.engine.geometry;

import game.engine.GameflowEvent;
import game.engine.renderer.Renderer;
import game.engine.spatial.LinearAlgebra4D;
import game.engine.spatial.Transformation;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;


public class Quad extends Geometry {

	public Quad() {
		this.geometryType = GeometryConstants.QUAD;

		this.VERTEX_COUNT = 4;
		this.NORMAL_COUNT = 4;
		this.INDEX_COUNT = 4;
		this.TEXTURE_COORD_COUNT = 4;
		this.DIMENSION = 4;
		this.TEXTURE_COORD_DIM = 2;
		
		Quad.createDefaultVertices(this);
		Quad.createDefaultNormals(this);
		Quad.createDefaultIndices(this);
		Quad.createDefaultTextureCoord(this);
		Geometry.createVertexBuffer(this);
		Geometry.createVertexBufferData(this);
		Geometry.createNormalBuffer(this);
		Geometry.createNormalBufferData(this);
		Geometry.createTextureCoordBuffer(this);
		Geometry.createTextureCoordinateBufferData(this);
		Geometry.createDataBuffer(this);
		Geometry.createDataBufferData(this);
		
		Geometry.updateBoundingSpere(this);
	}
	
	public Quad(float[][] vertices, float[][] normals, int[] indices, float[][] textureCoord){
		this.geometryType = GeometryConstants.QUAD;

		this.VERTEX_COUNT = 4;
		this.NORMAL_COUNT = 4;
		this.INDEX_COUNT = 4;
		this.TEXTURE_COORD_COUNT = 4;
		this.DIMENSION = 4;
		this.TEXTURE_COORD_DIM = 2;
		
		this.vertices = vertices;
		this.normals = normals;
		this.indices = indices;
		this.textureCoord = textureCoord;
		
		Geometry.createVertexBuffer(this);
		Geometry.createVertexBufferData(this);
		Geometry.createNormalBuffer(this);
		Geometry.createNormalBufferData(this);
		Geometry.createTextureCoordBuffer(this);
		Geometry.createTextureCoordinateBufferData(this);
		Geometry.createDataBuffer(this);
		Geometry.createDataBufferData(this);
		
		Geometry.updateBoundingSpere(this);
	}

	private final static void createDefaultVertices(Quad quad) {
		float[][] temp = { { 0, 0, 0, 1}, { 1, 0, 0, 1}, { 1, 1, 0, 1}, { 0, 1, 0, 1} };

		quad.vertices = temp;
	}

	private final static void createDefaultNormals(Quad quad) {
		float[][] temp = { { 0, 0, -1, 1}, { 0, 0, -1, 1}, { 0, 0, -1,1 },
				{ 0, 0, -1, 1} };

		quad.normals = temp;
	}

	private final static void createDefaultIndices(Quad quad) {
		int[] temp = { 0, 1, 2, 3 };

		quad.indices = temp;
	}
	
	private final static void createDefaultTextureCoord(Quad quad){
		float[][] temp = {{0,0},{1,0},{1,1},{0,1}};
		
		quad.textureCoord = temp;
	}

	@Override
	public final  void updateNormals() {
		float[] v1 = new float[4];
		float[] v2 = new float[4];
		float[] n = new float[4];
		
		LinearAlgebra4D.calculateVector(this.vertices[0], this.vertices[2], v1);
		LinearAlgebra4D.calculateVector(this.vertices[0], this.vertices[1], v2);
		LinearAlgebra4D.calculateVectorCrossProduct(v1, v2, n);
		LinearAlgebra4D.normalizeVector(n, n);
		
		this.normals[0] = n.clone();
		this.normals[1] = n.clone();
		this.normals[2] = n.clone();
		this.normals[3] = n.clone();
		
	}

	@Override
	public void executeStaticState() {
		
	}

	@Override
	public void executeDynamicState() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void calculateTexCoord() {
		// TODO Auto-generated method stub
		
	}
	
	public static void main(String args[]) {

		Ellipsoid cube = new Ellipsoid(2, 2, 2, 10, 10);

		Transformation t = new Transformation();
		float[] t1 = { 30, 30, 30, 1 };

		t.translate(t1);
		cube.updateTransformationState(t);

		for (int i = 0; i < 4; i++) {
			System.out.println(cube.boundingVolume.center[i]);
		}

		System.out.println(cube.boundingVolume.radius[0]);

	}
}
