package game.engine.geometry;

import game.engine.spatial.LinearAlgebra4D;
import game.engine.spatial.Transformation;

public class SolidDeformedCylinder extends Geometry {

	protected float[] center1 = { 0, 0, 0, 1 };

	protected float[] center2 = null;

	protected int t = 0;
	
	protected int vertexC = -1;

	public SolidDeformedCylinder(float radius1, float radius2,
			float height, int vertexCount, float textureScale_s, float textureScale_t) {
		this.geometryType = GeometryConstants.SOLID_DEFORMED_CYLINDER;
		this.center2 = new float[4];

		this.center2[0] = this.center1[0];
		this.center2[1] = height;
		this.center2[2] = this.center1[2];
		this.center2[3] = 1;

		float step = (float) (2 * Math.PI) / (vertexCount);

		float texture_s = 1;
		float texture_t = 0;
		
		float textureStep_horizontal = 1 * textureScale_s;
		float textureStep_vertical = (float) 1 * textureScale_t;
		
		vertexC = vertexCount;
		t = vertexCount * 3;
		vertexCount *= 6;
		vertexCount += t * 2 + 6;

		this.VERTEX_COUNT = vertexCount;
		this.NORMAL_COUNT = vertexCount;
		this.INDEX_COUNT = vertexCount;
		this.TEXTURE_COORD_COUNT = vertexCount;
		this.DIMENSION = 4;
		this.TEXTURE_COORD_DIM = 2;

		float[][] vertices = new float[this.VERTEX_COUNT][this.DIMENSION];
		float[][] normals = new float[this.NORMAL_COUNT][this.DIMENSION];
		int[] indices = new int[this.INDEX_COUNT];
		float[][] textureCoord = new float[this.TEXTURE_COORD_COUNT][this.TEXTURE_COORD_DIM];

		float x1, z1, x2, z2, x3, z3, x4, z4;
		float angle;

		int count_vertex = 0;

		for (angle = (float) (2.0f * Math.PI); angle > 0.0f;) {

			x1 = radius1 * (float) Math.cos(angle);
			z1 = radius1 * (float) Math.sin(angle);

			x3 = radius2 * (float) Math.cos(angle);
			z3 = radius2 * (float) Math.sin(angle);

			angle -= step;

			x2 = radius1 * (float) Math.cos(angle);
			z2 = radius1 * (float) Math.sin(angle);

			x4 = radius2 * (float) Math.cos(angle);
			z4 = radius2 * (float) Math.sin(angle);

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = 0;
			vertices[count_vertex][2] = z1;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_s;
			textureCoord[count_vertex][1] = texture_t;

			count_vertex++;

			vertices[count_vertex][0] = x4;
			vertices[count_vertex][1] = height;
			vertices[count_vertex][2] = z4;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_s - textureStep_horizontal;
			textureCoord[count_vertex][1] = texture_t + textureStep_vertical;

			count_vertex++;

			vertices[count_vertex][0] = x3;
			vertices[count_vertex][1] = height;
			vertices[count_vertex][2] = z3;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_s;
			textureCoord[count_vertex][1] = texture_t + textureStep_vertical;

			count_vertex++;

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = 0;
			vertices[count_vertex][2] = z1;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_s;
			textureCoord[count_vertex][1] = texture_t;

			count_vertex++;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = 0;
			vertices[count_vertex][2] = z2;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_s - textureStep_horizontal;
			textureCoord[count_vertex][1] = texture_t;

			count_vertex++;

			vertices[count_vertex][0] = x4;
			vertices[count_vertex][1] = height;
			vertices[count_vertex][2] = z4;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = texture_s - textureStep_horizontal;
			textureCoord[count_vertex][1] = texture_t + textureStep_vertical;

			count_vertex++;
			texture_s -= textureStep_horizontal;

		}

		texture_s = 0;
		texture_t = 0;
		
		for (angle = (float) (2.0f * Math.PI); angle > 0.0f;) {

			x1 = radius1 * (float) Math.cos(angle);
			z1 = radius1 * (float) Math.sin(angle);

			angle -= step;

			x2 = radius1 * (float) Math.cos(angle);
			z2 = radius1 * (float) Math.sin(angle);

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = 0;
			vertices[count_vertex][2] = z1;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = (float) Math.cos(angle - step);
			textureCoord[count_vertex][1] = (float) Math.sin(angle - step);

			count_vertex++;

			vertices[count_vertex][0] = this.center1[0];
			vertices[count_vertex][1] = this.center1[1];
			vertices[count_vertex][2] = this.center1[2];
			vertices[count_vertex][3] = this.center1[3];

			textureCoord[count_vertex][0] = 0.5f;
			textureCoord[count_vertex][1] = 0.5f;

			count_vertex++;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = 0;
			vertices[count_vertex][2] = z2;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = (float) Math.cos(angle);
			textureCoord[count_vertex][1] = (float) Math.sin(angle);

			count_vertex++;
			texture_t += textureStep_vertical;
		}

		angle = (float) (step);

		x1 = radius1 * (float) Math.cos(angle);
		z1 = radius1 * (float) Math.sin(angle);

		angle = (float) (0);

		x2 = radius1 * (float) Math.cos(angle);
		z2 = radius1 * (float) Math.sin(angle);

		vertices[count_vertex][0] = x1;
		vertices[count_vertex][1] = 0;
		vertices[count_vertex][2] = z1;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = (float) Math.cos(angle - step);
		textureCoord[count_vertex][1] = (float) Math.sin(angle - step);

		count_vertex++;

		vertices[count_vertex][0] = this.center1[0];
		vertices[count_vertex][1] = this.center1[1];
		vertices[count_vertex][2] = this.center1[2];
		vertices[count_vertex][3] = this.center1[3];

		textureCoord[count_vertex][0] = 0.5f;
		textureCoord[count_vertex][1] = 0.5f;

		count_vertex++;

		vertices[count_vertex][0] = x2;
		vertices[count_vertex][1] = 0;
		vertices[count_vertex][2] = z2;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = (float) Math.cos(angle);
		textureCoord[count_vertex][1] = (float) Math.sin(angle);

		count_vertex++;

		for (angle = (float) (2.0f * Math.PI); angle > 0.0f;) {

			x1 = radius2 * (float) Math.cos(angle);
			z1 = radius2 * (float) Math.sin(angle);

			angle -= step;

			x2 = radius2 * (float) Math.cos(angle);
			z2 = radius2 * (float) Math.sin(angle);

			vertices[count_vertex][0] = x1;
			vertices[count_vertex][1] = height;
			vertices[count_vertex][2] = z1;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = (float) Math.cos(angle - step);
			textureCoord[count_vertex][1] = (float) Math.sin(angle - step);

			count_vertex++;

			vertices[count_vertex][0] = x2;
			vertices[count_vertex][1] = height;
			vertices[count_vertex][2] = z2;
			vertices[count_vertex][3] = 1;

			textureCoord[count_vertex][0] = (float) Math.cos(angle);
			textureCoord[count_vertex][1] = (float) Math.sin(angle);

			count_vertex++;

			vertices[count_vertex][0] = this.center2[0];
			vertices[count_vertex][1] = this.center2[1];
			vertices[count_vertex][2] = this.center2[2];
			vertices[count_vertex][3] = this.center2[3];

			textureCoord[count_vertex][0] = 0.5f;
			textureCoord[count_vertex][1] = 0.5f;

			count_vertex++;
		}

		angle = (float) (step);

		x1 = radius2 * (float) Math.cos(angle);
		z1 = radius2 * (float) Math.sin(angle);

		angle = (float) (0);

		x2 = radius2 * (float) Math.cos(angle);
		z2 = radius2 * (float) Math.sin(angle);

		vertices[count_vertex][0] = x1;
		vertices[count_vertex][1] = height;
		vertices[count_vertex][2] = z1;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = (float) Math.cos(angle - step);
		textureCoord[count_vertex][1] = (float) Math.sin(angle - step);

		count_vertex++;

		vertices[count_vertex][0] = x2;
		vertices[count_vertex][1] = height;
		vertices[count_vertex][2] = z2;
		vertices[count_vertex][3] = 1;

		textureCoord[count_vertex][0] = (float) Math.cos(angle);
		textureCoord[count_vertex][1] = (float) Math.sin(angle);

		count_vertex++;

		vertices[count_vertex][0] = this.center2[0];
		vertices[count_vertex][1] = this.center2[1];
		vertices[count_vertex][2] = this.center2[2];
		vertices[count_vertex][3] = this.center2[3];

		textureCoord[count_vertex][0] = 0.5f;
		textureCoord[count_vertex][1] = 0.5f;

		count_vertex++;

		
		for (int i = 0; i < vertexCount; i++) {
			indices[i] = i;
		}

		this.vertices = vertices;
		this.normals = normals;
		this.indices = indices;
		this.textureCoord = textureCoord;

		Geometry.createVertexBuffer(this);
		Geometry.createNormalBuffer(this);
		Geometry.createVertexBufferData(this);
		this.updateNormals();
		Geometry.createNormalBufferData(this);
		Geometry.createTextureCoordBuffer(this);
		Geometry.createTextureCoordinateBufferData(this);
		Geometry.createDataBuffer(this);
		Geometry.createDataBufferData(this);
		
		Geometry.updateBoundingSpere(this);
	}

	public float[] getCenter1() {
		return this.center1;
	}

	public float[] getCenter2() {
		return this.center2;
	}

	public final void applyTransformation() {

		Transformation.concateTransformations(this.transformationMatrix,
				this.localTransformation, this.transformationMatrix);

		for (int i = 0; i < VERTEX_COUNT; i++) {
			LinearAlgebra4D.calculateTableVectorProduct(
					this.localTransformation.compositeMatrix, vertices[i],
					vertices[i]);
		}

		LinearAlgebra4D.calculateTableVectorProduct(
				this.localTransformation.compositeMatrix, this.center1,
				this.center1);

		LinearAlgebra4D.calculateTableVectorProduct(
				this.localTransformation.compositeMatrix, this.center2,
				this.center2);

		this.localTransformation.setIdentity();

		this.updateNormals();
		Geometry.updateBoundingSpere(this);
		Geometry.updateBuffers(this);
	}

	@Override
	public void updateNormals() {
		float[] normal = new float[4];

		int i;

		for (i = 0; i < this.VERTEX_COUNT - ((this.vertexC * 3 + 3) * 2);) {
			LinearAlgebra4D.calculateVector(this.center1, this.vertices[i],
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i][0] = normal[0];
			this.normals[i][1] = normal[1];
			this.normals[i][2] = normal[2];
			this.normals[i][3] = normal[3];

			LinearAlgebra4D.calculateVector(this.center2, this.vertices[i + 1],
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i + 1][0] = normal[0];
			this.normals[i + 1][1] = normal[1];
			this.normals[i + 1][2] = normal[2];
			this.normals[i + 1][3] = normal[3];

			LinearAlgebra4D.calculateVector(this.center2, this.vertices[i + 2],
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i + 2][0] = normal[0];
			this.normals[i + 2][1] = normal[1];
			this.normals[i + 2][2] = normal[2];
			this.normals[i + 2][3] = normal[3];

			LinearAlgebra4D.calculateVector(this.center1, this.vertices[i + 3],
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i + 3][0] = normal[0];
			this.normals[i + 3][1] = normal[1];
			this.normals[i + 3][2] = normal[2];
			this.normals[i + 3][3] = normal[3];

			LinearAlgebra4D.calculateVector(this.center1, this.vertices[i + 4],
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i + 4][0] = normal[0];
			this.normals[i + 4][1] = normal[1];
			this.normals[i + 4][2] = normal[2];
			this.normals[i + 4][3] = normal[3];

			LinearAlgebra4D.calculateVector(this.center2, this.vertices[i + 5],
					normal);
			LinearAlgebra4D.normalizeVector(normal, normal);

			this.normals[i + 5][0] = normal[0];
			this.normals[i + 5][1] = normal[1];
			this.normals[i + 5][2] = normal[2];
			this.normals[i + 5][3] = normal[3];

			i += 6;
		}
		
		for (; i < this.VERTEX_COUNT; i +=3) {
			LinearAlgebra4D.calculateNormal(this.vertices[i], this.vertices[i + 1],this.vertices[i+2],
					normal);

			this.normals[i][0] = normal[0];
			this.normals[i][1] = normal[1];
			this.normals[i][2] = normal[2];
			this.normals[i][3] = normal[3];
			
			this.normals[i + 1][0] = normal[0];
			this.normals[i + 1][1] = normal[1];
			this.normals[i + 1][2] = normal[2];
			this.normals[i + 1][3] = normal[3];
			
			
			this.normals[i + 2][0] = normal[0];
			this.normals[i + 2][1] = normal[1];
			this.normals[i + 2][2] = normal[2];
			this.normals[i + 2][3] = normal[3];
			
		}

	}

	@Override
	public void executeStaticState() {
		// TODO Auto-generated method stub

	}

	@Override
	public void executeDynamicState() {
		// TODO Auto-generated method stub

	}


	@Override
	public void calculateTexCoord() {
		// TODO Auto-generated method stub

	}
	
	public static void main(String args[]) {

		SolidDeformedCylinder cube = new SolidDeformedCylinder(2, 2, 2, 10, 1, 13);

		Transformation t = new Transformation();
		float[] t1 = { 30, 30, 30, 1 };

		t.translate(t1);
		cube.updateTransformationState(t);

		for (int i = 0; i < 4; i++) {
			System.out.println(cube.boundingVolume.center[i]);
		}

		System.out.println(cube.boundingVolume.radius[0]);

	}

}
