package game.engine.renderer;

import java.util.Vector;

import net.java.games.jogl.GLDrawable;

import game.core.gui.component.UserInputEvent;
import game.core.gui.component.UserInputListener;
import game.engine.Node;
import game.engine.SceneGraph;
import game.engine.geometry.Geometry;

public abstract class Renderer implements game.core.gui.component.UserInputEventGenerator {

	public Vector userInputEventListener = new Vector();

	private GLDrawable drawable = null;

	public SceneGraph sceneGraph = null;

	public static final String OPEN_GL = "opengl";

	public static final int RENDER_MODE = 1;

	public static final int SELECTION_MODE = 2;

	public int renderMode = Renderer.RENDER_MODE;

	public Geometry geometry = null;

	public Camera camera = null;

	public int width = 0;

	public int height = 0;

	public int clearDepth = 0;

	public abstract Texture createTexture(String name);

	public abstract void deleteTextureObjects(int[] textureObjectIDList);

	public abstract int[] generateBuffer();

	public abstract boolean beginScene();

	public abstract void endScene();

	public abstract void clearBuffers();

	public abstract void executeState();

	public abstract void drawElements();

	public abstract void setWidth();

	public abstract void setHeight();

	public abstract void setBackgroundColor(float[] color);

	public abstract void setClearDepth(int i);

	public abstract void enableDepthTest();

	public abstract void enableLight();

	public abstract void enableVertexArrays();

	public abstract void enableTextures();

	public abstract void initialBufferObjects();

	public abstract void initialProjection();

	public abstract void setRenderMode(int mode);

	public abstract void processMouseSelection(int x, int y);
	
	public abstract void initialDrawing();

	public abstract void flushBuffers();
	
	public abstract void enableFog();
	
	public abstract Object getRenderingDevice();
	
	public abstract void enableAntialiasing();
	
	public Renderer() {
	}

	public final void initRenderer() {
		this.enableDepthTest();
		this.enableLight();
		this.initialBufferObjects();
		this.enableVertexArrays();
		this.enableFog();
		this.enableAntialiasing();
		this.enableTextures();
		this.initialProjection();
		this.initialDrawing();
	}

	public final void drawScene(Node scene) {
		if (scene != null) {
			scene.onDraw(this, false);
			this.flushBuffers();
		} else {
			throw new NullPointerException();
		}
	}

	public final void draw(Geometry geometry) {
		this.geometry = geometry;

		drawPrimitive();
	}

	public final void drawPrimitive() {
		this.executeState();
		this.drawElements();
	}

	public final void setCamera(Camera camera) {
		this.camera = camera;
	}

	public final void setDrawable(GLDrawable drawable) {
		this.drawable = drawable;
	}

	public final SceneGraph getSceneGraph() {
		return this.sceneGraph;
	}

	public final void addUserInputEventListener(Object object) {
		this.userInputEventListener.addElement(object);
	}

	public final void removeUserInputEventListener(Object object) {
		this.userInputEventListener.removeElement(object);
	}

	public final void fireUserInputEvent(UserInputEvent userInputEvent) {

		for (int i = 0; i < this.userInputEventListener.size(); i++) {
			UserInputListener userInputListener = (UserInputListener) this.userInputEventListener
					.elementAt(i);

			userInputListener.executeUserEvent(userInputEvent);
		}

	}

}
