package game.engine.renderer;

import game.core.TexturesProperties;

import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;
import java.awt.image.Raster;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import javax.imageio.ImageIO;

public class Texture {

	public BufferedImage image = null;

	public ByteBuffer textureBuffer = null;

	public int imgWidth = 0;

	public int imgHeight = 0;

	public int bindID = 0;

	public Texture(String name, int bindID) {
		try {
			this.bindID = bindID;
			this.createImage(name);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private final void createImage(String name) throws IOException {
		String texturePath = TexturesProperties.getInstance().getProperty(name);
		InputStream is = ClassLoader.getSystemResourceAsStream(texturePath);

		BufferedInputStream bis = new BufferedInputStream(is);
		image = ImageIO.read(bis);

		Raster r = image.getRaster();

		DataBufferByte dbi = (DataBufferByte) r.getDataBuffer();
		byte b[] = dbi.getData();
		textureBuffer = null;

		int texSize = b.length;

		textureBuffer = ByteBuffer.allocateDirect(texSize);
		ByteOrder newOrder = ByteOrder.nativeOrder();

		textureBuffer.order(newOrder);
		textureBuffer.put(b, 0, texSize);

		imgWidth = image.getWidth();
		imgHeight = image.getHeight();
	}

	public final void releaseBuffer() {
		this.textureBuffer.clear();
		this.textureBuffer = null;
	}
}
