package game.engine.renderer;

import net.java.games.jogl.GL;
import game.engine.SceneGraph;
import game.engine.geometry.BoundingVolume;
import game.engine.spatial.LinearAlgebra4D;

public class ViewFrustrum {

	private static ViewFrustrum instance = null;

	private Camera camera = null;

	private SceneGraph sceneGraph = null;

	private float[] clipMatrix = null;

	private float[] projMatrix = null;

	private float[] modelViewMatrix = null;

	private float[][] viewFrustrum = null;

	private ViewFrustrum(Camera camera, SceneGraph sceneGraph) {
		this.camera = camera;
		this.sceneGraph = sceneGraph;
		clipMatrix = new float[16];
		projMatrix = new float[16];
		modelViewMatrix = new float[16];
		viewFrustrum = new float[6][4];
	}

	public static void createViewFrustrum(Camera camera, SceneGraph sceneGraph) {
		if (instance == null) {
			instance = new ViewFrustrum(camera, sceneGraph);
		}
	}

	public static ViewFrustrum getInstance() {
		return instance;
	}

	public final void updateFrustrum() {
		GL gl = (GL) this.sceneGraph.renderer.getRenderingDevice();

		gl.glGetFloatv(GL.GL_PROJECTION_MATRIX, projMatrix);
		gl.glGetFloatv(GL.GL_MODELVIEW_MATRIX, modelViewMatrix);

		clipMatrix[0] = modelViewMatrix[0] * projMatrix[0];
		clipMatrix[1] = modelViewMatrix[1] * projMatrix[5];
		clipMatrix[2] = modelViewMatrix[2] * projMatrix[10]
				+ modelViewMatrix[3] * projMatrix[14];
		clipMatrix[3] = modelViewMatrix[2] * projMatrix[11];

		clipMatrix[4] = modelViewMatrix[4] * projMatrix[0];
		clipMatrix[5] = modelViewMatrix[5] * projMatrix[5];
		clipMatrix[6] = modelViewMatrix[6] * projMatrix[10]
				+ modelViewMatrix[7] * projMatrix[14];
		clipMatrix[7] = modelViewMatrix[6] * projMatrix[11];

		clipMatrix[8] = modelViewMatrix[8] * projMatrix[0];
		clipMatrix[9] = modelViewMatrix[9] * projMatrix[5];
		clipMatrix[10] = modelViewMatrix[10] * projMatrix[10]
				+ modelViewMatrix[11] * projMatrix[14];
		clipMatrix[11] = modelViewMatrix[10] * projMatrix[11];

		clipMatrix[12] = modelViewMatrix[12] * projMatrix[0];
		clipMatrix[13] = modelViewMatrix[13] * projMatrix[5];
		clipMatrix[14] = modelViewMatrix[14] * projMatrix[10]
				+ modelViewMatrix[15] * projMatrix[14];
		clipMatrix[15] = modelViewMatrix[14] * projMatrix[11];

		// right plane
		viewFrustrum[0][0] = clipMatrix[3] - clipMatrix[0];
		viewFrustrum[0][1] = clipMatrix[7] - clipMatrix[4];
		viewFrustrum[0][2] = clipMatrix[11] - clipMatrix[8];
		viewFrustrum[0][3] = clipMatrix[15] - clipMatrix[12];

		float t = 0.0f;

		t = (float) (Math.sqrt(viewFrustrum[0][0] * viewFrustrum[0][0]
				+ viewFrustrum[0][1] * viewFrustrum[0][1] + viewFrustrum[0][2]
				* viewFrustrum[0][2]));

		viewFrustrum[0][0] /= t;
		viewFrustrum[0][1] /= t;
		viewFrustrum[0][2] /= t;
		viewFrustrum[0][3] /= t;

		// left plane
		viewFrustrum[1][0] = clipMatrix[3] + clipMatrix[0];
		viewFrustrum[1][1] = clipMatrix[7] + clipMatrix[4];
		viewFrustrum[1][2] = clipMatrix[11] + clipMatrix[8];
		viewFrustrum[1][3] = clipMatrix[15] + clipMatrix[12];

		t = (float) (Math.sqrt(viewFrustrum[1][0] * viewFrustrum[1][0]
				+ viewFrustrum[1][1] * viewFrustrum[1][1] + viewFrustrum[1][2]
				* viewFrustrum[1][2]));

		viewFrustrum[1][0] /= t;
		viewFrustrum[1][1] /= t;
		viewFrustrum[1][2] /= t;
		viewFrustrum[1][3] /= t;

		// bottom plane
		viewFrustrum[2][0] = clipMatrix[3] + clipMatrix[1];
		viewFrustrum[2][1] = clipMatrix[7] + clipMatrix[5];
		viewFrustrum[2][2] = clipMatrix[11] + clipMatrix[9];
		viewFrustrum[2][3] = clipMatrix[15] + clipMatrix[13];

		t = (float) (Math.sqrt(viewFrustrum[2][0] * viewFrustrum[2][0]
				+ viewFrustrum[2][1] * viewFrustrum[2][1] + viewFrustrum[2][2]
				* viewFrustrum[2][2]));

		viewFrustrum[2][0] /= t;
		viewFrustrum[2][1] /= t;
		viewFrustrum[2][2] /= t;
		viewFrustrum[2][3] /= t;

		// top plane
		viewFrustrum[3][0] = clipMatrix[3] - clipMatrix[1];
		viewFrustrum[3][1] = clipMatrix[7] - clipMatrix[5];
		viewFrustrum[3][2] = clipMatrix[11] - clipMatrix[9];
		viewFrustrum[3][3] = clipMatrix[15] - clipMatrix[13];

		t = (float) (Math.sqrt(viewFrustrum[3][0] * viewFrustrum[3][0]
				+ viewFrustrum[3][1] * viewFrustrum[3][1] + viewFrustrum[3][2]
				* viewFrustrum[3][2]));

		viewFrustrum[3][0] /= t;
		viewFrustrum[3][1] /= t;
		viewFrustrum[3][2] /= t;
		viewFrustrum[3][3] /= t;

		// Far plane
		viewFrustrum[4][0] = clipMatrix[3] - clipMatrix[2];
		viewFrustrum[4][1] = clipMatrix[7] - clipMatrix[6];
		viewFrustrum[4][2] = clipMatrix[11] - clipMatrix[10];
		viewFrustrum[4][3] = clipMatrix[15] - clipMatrix[14];

		t = (float) (Math.sqrt(viewFrustrum[4][0] * viewFrustrum[4][0]
				+ viewFrustrum[4][1] * viewFrustrum[4][1] + viewFrustrum[4][2]
				* viewFrustrum[4][2]));

		viewFrustrum[4][0] /= t;
		viewFrustrum[4][1] /= t;
		viewFrustrum[4][2] /= t;
		viewFrustrum[4][3] /= t;

		// Near plane
		viewFrustrum[5][0] = clipMatrix[3] + clipMatrix[2];
		viewFrustrum[5][1] = clipMatrix[7] + clipMatrix[6];
		viewFrustrum[5][2] = clipMatrix[11] + clipMatrix[10];
		viewFrustrum[5][3] = clipMatrix[15] + clipMatrix[14];

		t = (float) (Math.sqrt(viewFrustrum[5][0] * viewFrustrum[5][0]
				+ viewFrustrum[5][1] * viewFrustrum[5][1] + viewFrustrum[5][2]
				* viewFrustrum[5][2]));

		viewFrustrum[5][0] /= t;
		viewFrustrum[5][1] /= t;
		viewFrustrum[5][2] /= t;
		viewFrustrum[5][3] /= t;

	}

	public boolean isBoundingSphererInFrustrum(BoundingVolume sphere) {
		for (int i = 0; i < 6; i++) {
			if (viewFrustrum[i][0] * sphere.center[0] + viewFrustrum[i][1]
					* sphere.center[1] + viewFrustrum[i][2] * sphere.center[2]
					+ viewFrustrum[i][3] <= -sphere.radius[0]) {

				return false;
			}
		}

		return true;
	}
}
