package game.engine.spatial;

import game.engine.renderer.Renderer;

public class LinearAlgebra4D {

	public static final float[] UNION_VECTOR_X = { 1, 0, 0, 1 };

	public static final float[] UNION_VECTOR_Y = { 0, 1, 0, 1 };

	public static final float[] UNION_VECTOR_Z = { 0, 0, 1, 1 };

	public static final float[] ORIGIN = { 0, 0, 0, 1 };

	public static final void calculateTableVectorProduct(float[][] table,
			float[] vector, float[] destVector) {

		float[] temp = new float[vector.length];

		temp[0] = table[0][0] * vector[0] + table[0][1] * vector[1]
				+ table[0][2] * vector[2] + table[0][3] * vector[3];
		temp[1] = table[1][0] * vector[0] + table[1][1] * vector[1]
				+ table[1][2] * vector[2] + table[1][3] * vector[3];
		temp[2] = table[2][0] * vector[0] + table[2][1] * vector[1]
				+ table[2][2] * vector[2] + table[2][3] * vector[3];
		temp[3] = table[3][0] * vector[0] + table[3][1] * vector[1]
				+ table[3][2] * vector[2] + table[3][3] * vector[3];

		destVector[0] = temp[0];
		destVector[1] = temp[1];
		destVector[2] = temp[2];
		destVector[3] = temp[3];
	}

	public static final void calculateTableTableProduct(float[][] table1,
			float[][] table2, float[][] destTable) {
		float[][] table = new float[4][4];

		for (int row = 0; row < 4; row++) {
			for (int col = 0; col < 4; col++) {
				table[row][col] = table1[row][0] * table2[0][col]
						+ table1[row][1] * table2[1][col] + table1[row][2]
						* table2[2][col] + table1[row][3] * table2[3][col];
			}
		}

		for (int i = 0; i < 4; i++) {
			for (int j = 0; j < 4; j++) {
				destTable[i][j] = table[i][j];
			}
		}

	}

	public static final void calculateVectorFromPoints(float[] p1, float[] p2,
			float[] vector) {

		float[] temp = new float[4];

		temp[0] = p2[0] - p1[0];
		temp[1] = p2[1] - p1[1];
		temp[2] = p2[2] - p1[2];
		temp[3] = 1;

		for (int i = 0; i < 4; i++) {
			vector[i] = temp[i];
		}

	}

	public static final void calculateVectorCrossProduct(float[] vector1,
			float[] vector2, float[] destVector) {

		float[] temp = new float[4];

		temp[0] = vector1[1] * vector2[2] - vector1[2] * vector2[1];
		temp[1] = vector1[2] * vector2[0] - vector1[0] * vector2[2];
		temp[2] = vector1[0] * vector2[1] - vector1[1] * vector2[0];
		temp[3] = 1;

		for (int i = 0; i < 4; i++) {
			destVector[i] = temp[i];
		}
	}

	public static final void substractVectorVector(float[] point1,
			float[] point2, float[] dest) {

		dest[0] = point2[0] - point1[0];
		dest[1] = point2[1] - point1[1];
		dest[2] = point2[2] - point1[2];
		dest[3] = 1;

	}

	public static final void calculatePointVectorSum(float[] point,
			float[] vector, float[] destPoint) {

		float[] temp = new float[point.length];

		for (int i = 0; i < 4; i++) {
			temp[i] = point[i] + vector[i];
		}

		temp[3] = 1;

		for (int i = 0; i < point.length; i++) {
			destPoint[i] = temp[i];
		}
	}

	public static final void setIdentity(float[][] table) {

		for (int i = 0; i < table.length; i++) {
			for (int j = 0; j < table[i].length; j++) {
				if (i == j) {
					table[i][j] = 1;
				} else {
					table[i][j] = 0;
				}
			}
		}
	}

	public static final void displayTable(float[][] table) {

		for (int i = 0; i < table.length; i++) {
			for (int j = 0; j < table[i].length; j++) {
				System.out.print(table[i][j] + " ");
			}
			System.out.println();
		}
	}

	public static final void normalizeVector(float[] vector, float[] destVector) {

		float d = (float) Math.sqrt(vector[0] * vector[0] + vector[1]
				* vector[1] + vector[2] * vector[2]);

		destVector[0] = vector[0] / d;
		destVector[1] = vector[1] / d;
		destVector[2] = vector[2] / d;
		destVector[3] = 1;

	}

	public static final void calculateVector(float[] p1, float[] p2,
			float[] destVector) {

		destVector[0] = p2[0] - p1[0];
		destVector[1] = p2[1] - p1[1];
		destVector[2] = p2[2] - p1[2];
		destVector[3] = 1;
	}

	public static final void calculateInversVector(float[] vector,
			float[] destVector) {

		destVector[0] = -vector[0];
		destVector[1] = -vector[1];
		destVector[2] = -vector[2];
		destVector[3] = 1;
	}

	public static final void scaleVector(float[] vector, float scale,
			float[] destVector) {

		destVector[0] = vector[0] * scale;
		destVector[1] = vector[1] * scale;
		destVector[2] = vector[2] * scale;
		destVector[3] = 1;

	}

	public static final void calculateNormal(float[] p1, float[] p2,
			float[] p3, float[] normal) {

		float[] v1 = new float[4];
		float[] v2 = new float[4];

		LinearAlgebra4D.calculateVector(p1, p2, v1);
		LinearAlgebra4D.calculateVector(p1, p3, v2);

		LinearAlgebra4D.calculateVectorCrossProduct(v1, v2, normal);
		LinearAlgebra4D.normalizeVector(normal, normal);
	}

	public static final void toRadian(float angle_degree, float[] angle_radian) {
		angle_radian[0] = (float) (angle_degree * ((2 * Math.PI) / 360));
	}

	public static final void calculateVectorDotProduct(float[] vector,
			float[] vector2, float[] product) {

		float sum = 0;

		for (int i = 0; i < 3; i++) {
			sum += vector[i] * vector2[i];
		}

		product[0] = sum;
	}

	public static final void calculatePlaneEquation(float[] p1, float[] p2,
			float[] p3, float[] planeEquation) {

		float[] temp = new float[4];

		temp[0] = ((p2[1] - p1[1]) * (p3[2] - p1[2]))
				- ((p2[2] - p1[2]) * (p3[1] - p1[1]));
		temp[1] = ((p2[2] - p1[2]) * (p3[0] - p1[0]))
				- ((p2[0] - p1[0]) * (p3[2] - p1[2]));
		temp[2] = ((p2[0] - p1[0]) * (p3[1] - p1[1]))
				- ((p2[1] - p1[1]) * (p3[0] - p1[0]));
		temp[3] = temp[0] * p1[0] + temp[1] * p1[1] + temp[2] * p1[2];

		for (int i = 0; i < temp.length; i++) {
			planeEquation[i] = temp[i];
		}
	}

	public static final void calculateProjectionMatrix(float[] p1, float[] p2,
			float[] p3, float[] lightPos, float[][] shadowMatrix) {

		float[] planeEquation = new float[4];
		LinearAlgebra4D.calculatePlaneEquation(p1, p2, p3, planeEquation);

		float[] dot = new float[1];
		dot[0] = planeEquation[0] * lightPos[0] + planeEquation[1]
				* lightPos[1] + planeEquation[2] * lightPos[2]
				+ planeEquation[3] * lightPos[3];

		shadowMatrix[0][0] = dot[0] - lightPos[0] * planeEquation[0];
		shadowMatrix[0][1] = 0.0f - lightPos[0] * planeEquation[1];
		shadowMatrix[0][2] = 0.0f - lightPos[0] * planeEquation[2];
		shadowMatrix[0][3] = 0.0f - lightPos[0] * planeEquation[3];

		shadowMatrix[1][0] = 0.0f - lightPos[1] * planeEquation[0];
		shadowMatrix[1][1] = dot[0] - lightPos[1] * planeEquation[1];
		shadowMatrix[1][2] = 0.0f - lightPos[1] * planeEquation[2];
		shadowMatrix[1][3] = 0.0f - lightPos[1] * planeEquation[3];

		shadowMatrix[2][0] = 0.0f - lightPos[2] * planeEquation[0];
		shadowMatrix[2][1] = 0.0f - lightPos[2] * planeEquation[1];
		shadowMatrix[2][2] = dot[0] - lightPos[2] * planeEquation[2];
		shadowMatrix[2][3] = 0.0f - lightPos[2] * planeEquation[3];

		shadowMatrix[3][0] = 0.0f - lightPos[3] * planeEquation[0];
		shadowMatrix[3][1] = 0.0f - lightPos[3] * planeEquation[1];
		shadowMatrix[3][2] = 0.0f - lightPos[3] * planeEquation[2];
		shadowMatrix[3][3] = dot[0] - lightPos[3] * planeEquation[3];
	}

	public static void main(String args[]) {

		System.out.println("hallo welt");

		// LinearAlgebra4D.calculateProjectionMatrix(p1, p2, p3, lightPos,
		// shadowMatrix);
		// LinearAlgebra4D.displayTable(shadowMatrix);
	}
}
