package game.engine.spatial;

import java.util.Vector;

import game.engine.GameflowEvent;
import game.engine.Node;
import game.engine.PlayGroundElement;
import game.engine.renderer.Renderer;

public abstract class Spatial {
	
	public static final int STATIC = 1;

	public static final int CHANGE_POSITION = 2;
	
	public int geometricalState = Spatial.STATIC;
	
	public Node parent = null;

	public Transformation localTransformation = null;

	public Transformation transformationMatrix = null;

	boolean isWorldBoundCurrent = true;

	boolean isCullForced = false;

	public int selectionID = -1;

	public String gameflowControllerTableID = null;

	public PlayGroundElement playGroundelement = null;

	public abstract void draw(Renderer renderer, boolean isCulled);

	public Spatial() {
		localTransformation = new Transformation();
		transformationMatrix = new Transformation();
	}

	public void onDraw(Renderer renderer, boolean isCulled) {
		if (this.isCullForced) {
			return;
		}

		if (!isCulled) {
			draw(renderer, isCulled);
		}
	}

	public final void setParent(Node parent) {
		this.parent = parent;
	}

	public final void setIsWorldBoundCurrent(boolean isWorldBoundCurrent) {
		this.isWorldBoundCurrent = isWorldBoundCurrent;
	}

	public abstract void updateTransformationState(
			Transformation newWorldTranstormation);

}
