package game.engine.geometry;

import game.engine.GameflowController;
import game.engine.MoveMapElement;
import game.engine.Node;
import game.engine.PlayGround;
import game.engine.PlayGroundElement;
import game.engine.SelectionController;
import game.engine.ai.PathFinder;
import game.engine.creatures.Creature;
import game.engine.creatures.CreatureFactory;
import game.engine.renderer.BufferController;
import game.engine.renderer.Renderer;
import game.engine.spatial.LinearAlgebra4D;
import game.engine.spatial.Transformation;

import java.util.Vector;

public class GeometryFactory {

	public static final int LEVEL_1 = 1;

	public static final int LEVEL_2 = 2;

	public static final int LEVEL_3 = 3;

	public static final int LEVEL_4 = 4;

	public static final int LEVEL_5 = 5;

	public static final int LEVEL_6 = 6;

	public static final int LEVEL_7 = 7;

	public static final int BANE_SPIDER = 8;

	public static final int DWARF_WORRIOR = 9;

	public static final int DWARF_RANGED = 10;

	public static final int ELF = 11;

	public static final int DWARF_GEODE = 12;

	public static final int MOVE_MAP = 13;

	public static final int MOVE_PLATE = 14;

	public static final int BOMB = 15;

	public static final int MARK = 16;

	public static final int OCTOPUS = 17;

	public static final int GATE = 18;

	public static final int FIRE_ELEMENTAL = 19;

	public static final int INIT_SELECTION = 20;

	private static GeometryFactory instance = null;

	private Renderer renderer = null;

	public Vector entryPointList = new Vector();
	
	private GeometryFactory(Renderer renderer) {
		this.renderer = renderer;

		entryPointList.add(new PlayGroundElement(4, 2, 12,
				GeometryFactory.BANE_SPIDER));
		entryPointList.add(new PlayGroundElement(6, 2, 13,
				GeometryFactory.BANE_SPIDER));
		
		entryPointList.add(new PlayGroundElement(34, 2, 27,
				GeometryFactory.BANE_SPIDER));
		entryPointList.add(new PlayGroundElement(36, 2, 28,
				GeometryFactory.BANE_SPIDER));
		
	}

	public final static GeometryFactory createFactory(Renderer renderer) {
		instance = new GeometryFactory(renderer);

		return instance;
	}

	public final static GeometryFactory getInstance() {
		if (instance == null) {
			throw new NullPointerException();
		}

		return instance;
	}

	public final Node createGeometry(PlayGroundElement playGroundElement) {

		if (playGroundElement.geometryFactoryType == GeometryFactory.BANE_SPIDER) {
			return this.createSpider(playGroundElement);
		}

		if (playGroundElement.geometryFactoryType == GeometryFactory.DWARF_WORRIOR) {
			return this.createWorrior(playGroundElement);
		}

		if (playGroundElement.geometryFactoryType == GeometryFactory.DWARF_RANGED) {
			return this.createDwarf(playGroundElement);
		}

		if (playGroundElement.geometryFactoryType == GeometryFactory.DWARF_GEODE) {
			return this.createMagician(playGroundElement);
		}

		if (playGroundElement.geometryFactoryType == GeometryFactory.MOVE_MAP) {
			return this.createMoveMap(playGroundElement);
		}

		if (playGroundElement.geometryFactoryType == GeometryFactory.BOMB) {
			return this.createBomb(playGroundElement);
		}

		if (playGroundElement.geometryFactoryType == GeometryFactory.MARK) {
			return this.createMark(playGroundElement);
		}

		if (playGroundElement.geometryFactoryType == GeometryFactory.OCTOPUS) {
			return this.createOctopus(playGroundElement);
		}

		if (playGroundElement.geometryFactoryType == GeometryFactory.GATE) {
			return this.createGate(playGroundElement);
		}

		if (playGroundElement.geometryFactoryType == GeometryFactory.FIRE_ELEMENTAL) {
			return this.createFireElemental(playGroundElement);
		}

		return null;
	}

	private final Node createGate(PlayGroundElement playGroundElement) {
		BufferController bufferController = BufferController.getInstance();

		Node gate = new Node();

		int selectionID = SelectionController.getInstance().addMonster(gate);

		gate.gameflowControllerTableID = gate.toString();
		gate.playGroundelement = playGroundElement;

		playGroundElement.playGroundInfo = new int[2][1];
		playGroundElement.playGroundInfo[0][0] = PlayGround.MONSTER;
		playGroundElement.playGroundInfo[1][0] = PlayGround.MONSTER;
		playGroundElement.creatureInfo = GeometryFactory.GATE;

		Creature creatureInfo = CreatureFactory
				.getCreature(playGroundElement.creatureInfo);

		playGroundElement.element = gate;
		playGroundElement.wounds = creatureInfo.wounds;
		PlayGround.getInstance().addNewElement(playGroundElement);

		Transformation transformation = new Transformation();

		Cube gatePost = new Cube(0.1f, 6, 0.1f, 0.1f, 1);

		gatePost.isSelectable = true;
		gatePost.selectionID = selectionID;
		gatePost.setBufferID(bufferController.getBufferID());
		gatePost.setTextureID("octopus");

		gatePost.setAmbientFactor(Materials.AMBIENT_MATAL);
		gatePost.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		gatePost.setSpecularFactor(Materials.SPECULAR_METAL);
		gatePost.shinessFactor = Materials.SHINESS_METAL;

		transformation.setIdentity();

		float[] a1 = { 0.1f, 0, 0, 1 };

		transformation.translate(a1);
		gatePost.updateTransformationState(transformation);

		Node.addChildrenToNode(gate, gatePost);

		for (int i = 0; i < 9; i++) {
			gatePost = new Cube(0.1f, 6, 0.1f, 0.1f, 1);

			gatePost.isSelectable = true;
			gatePost.selectionID = selectionID;
			gatePost.setBufferID(bufferController.getBufferID());
			gatePost.setTextureID("octopus");

			gatePost.setAmbientFactor(Materials.AMBIENT_MATAL);
			gatePost.setDiffuseFactor(Materials.DIFFUSE_MATAL);
			gatePost.setSpecularFactor(Materials.SPECULAR_METAL);
			gatePost.shinessFactor = Materials.SHINESS_METAL;

			transformation.setIdentity();

			float[] t1 = { 0.1f, 0, (i + 1) * 0.2f, 1 };

			transformation.translate(t1);
			gatePost.updateTransformationState(transformation);

			Node.addChildrenToNode(gate, gatePost);
		}

		for (int i = 0; i < 15; i++) {
			gatePost = new Cube(0.1f, 0.2f, 2f, 1f, 0.1f);

			gatePost.isSelectable = true;
			gatePost.selectionID = selectionID;
			gatePost.setTextureID("octopus");

			gatePost.setAmbientFactor(Materials.AMBIENT_MATAL);
			gatePost.setDiffuseFactor(Materials.DIFFUSE_MATAL);
			gatePost.setSpecularFactor(Materials.SPECULAR_METAL);
			gatePost.shinessFactor = Materials.SHINESS_METAL;

			gatePost.setBufferID(bufferController.getBufferID());

			transformation.setIdentity();

			float[] t1 = { 0.1f, 0 + (i * 0.4f), 0f, 1 };

			transformation.translate(t1);
			gatePost.updateTransformationState(transformation);

			Node.addChildrenToNode(gate, gatePost);
		}

		return gate;
	}

	private final Node createFireElemental(PlayGroundElement playGroundElement) {
		BufferController bufferController = BufferController.getInstance();

		Node fireElemental = new Node();

		int selectionID = SelectionController.getInstance().addMonster(
				fireElemental);

		fireElemental.gameflowControllerTableID = fireElemental.toString();
		fireElemental.playGroundelement = playGroundElement;

		playGroundElement.playGroundInfo = new int[1][1];
		playGroundElement.playGroundInfo[0][0] = PlayGround.MONSTER;

		playGroundElement.creatureInfo = GeometryFactory.FIRE_ELEMENTAL;

		playGroundElement.element = fireElemental;
		PlayGround.getInstance().addNewElement(playGroundElement);

		Transformation transformation = new Transformation();

		Ellipsoid chest = new Ellipsoid(0.5f, 1f, 0.5f, 15, 15);

		chest.isSelectable = true;
		chest.selectionID = selectionID;
		chest.setBufferID(bufferController.getBufferID());
		chest.setTextureID("rock");

		chest.setAmbientFactor(Materials.AMBIENT_GROUND);
		chest.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		chest.setSpecularFactor(Materials.SPECULAR_GROUND);
		chest.shinessFactor = Materials.SHINESS_GROUND;

		transformation.setIdentity();

		float[] a1 = { 0f, 0, 0, 1 };

		transformation.translate(a1);
		chest.updateTransformationState(transformation);

		Node.addChildrenToNode(fireElemental, chest);

		EllipsoidArc chestArc = new EllipsoidArc(0.4f, 1f, 0.4f,
				-(float) Math.PI / 2, (float) Math.PI / 2, 0, (float) Math.PI,
				20, 20);

		chestArc.isSelectable = true;
		chestArc.selectionID = selectionID;
		chestArc.setBufferID(bufferController.getBufferID());
		chestArc.setTextureID("rock");

		chestArc.setAmbientFactor(Materials.AMBIENT_GROUND);
		chestArc.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		chestArc.setSpecularFactor(Materials.SPECULAR_GROUND);
		chestArc.shinessFactor = Materials.SHINESS_GROUND;

		transformation.setIdentity();

		float[] a3 = { 0f, 0.6f, 0, 1 };

		transformation.translate(a3);
		chestArc.updateTransformationState(transformation);

		Node.addChildrenToNode(fireElemental, chestArc);

		Ellipsoid breast = new Ellipsoid(0.25f, 0.5f, 0.25f, 10, 10);

		breast.isSelectable = true;
		breast.selectionID = selectionID;
		breast.setBufferID(bufferController.getBufferID());
		breast.setTextureID("rock");

		breast.setAmbientFactor(Materials.AMBIENT_GROUND);
		breast.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		breast.setSpecularFactor(Materials.SPECULAR_GROUND);
		breast.shinessFactor = Materials.SHINESS_GROUND;

		transformation.setIdentity();

		float[] a2 = { -0.15f, 0.8f, -0.2f, 1 };

		transformation.translate(a2);
		breast.updateTransformationState(transformation);

		Node.addChildrenToNode(fireElemental, breast);

		breast = new Ellipsoid(0.25f, 0.5f, 0.25f, 10, 10);

		breast.isSelectable = true;
		breast.selectionID = selectionID;
		breast.setBufferID(bufferController.getBufferID());
		breast.setTextureID("rock");

		breast.setAmbientFactor(Materials.AMBIENT_GROUND);
		breast.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		breast.setSpecularFactor(Materials.SPECULAR_GROUND);
		breast.shinessFactor = Materials.SHINESS_GROUND;

		transformation.setIdentity();

		float[] a7 = { 0.15f, 0.8f, -0.2f, 1 };

		transformation.translate(a7);
		breast.updateTransformationState(transformation);

		Node.addChildrenToNode(fireElemental, breast);

		Ellipsoid leftEye = new Ellipsoid(0.1f, 0.05f, 0.05f, 10, 10);

		leftEye.isSelectable = true;
		leftEye.selectionID = selectionID;
		leftEye.setBufferID(bufferController.getBufferID());
		leftEye.setTextureID("lava");

		leftEye.setAmbientFactor(Materials.AMBIENT_LAVA);
		leftEye.setDiffuseFactor(Materials.DIFFUSE_LAVA);
		leftEye.setSpecularFactor(Materials.SPECULAR_LAVA);
		leftEye.shinessFactor = Materials.SHINESS_LAVA;

		transformation.setIdentity();

		float[] a5 = { 0f, 0.4f, -0.2f, 1 };

		transformation.translate(a5);
		leftEye.updateTransformationState(transformation);

		Node.addChildrenToNode(fireElemental, leftEye);

		return fireElemental;
	}

	private final Node createOctopus(PlayGroundElement playGroundElement) {
		BufferController bufferController = BufferController.getInstance();

		Node octopus = new Node();

		int selectionID = SelectionController.getInstance().addMonster(octopus);

		octopus.gameflowControllerTableID = octopus.toString();
		octopus.playGroundelement = playGroundElement;

		playGroundElement.creatureInfo = GeometryFactory.OCTOPUS;
		playGroundElement.playGroundInfo = new int[3][3];

		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				playGroundElement.playGroundInfo[i][j] = PlayGround.STRUCTURE;
			}
		}

		playGroundElement.element = octopus;
		PlayGround.getInstance().addNewElement(playGroundElement);

		Ellipsoid head = new Ellipsoid(1, 1.5f, 2.5f, 15, 15);

		head.isSelectable = true;
		head.selectionID = selectionID;
		head.setTextureID("octopus");

		head.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		head.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		head.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		head.shinessFactor = Materials.SHINESS_OCTOPUS;

		head.setBufferID(bufferController.getBufferID());

		Transformation transformation = new Transformation();

		Node.addChildrenToNode(octopus, head);

		EllipsoidArc headDecor1 = new EllipsoidArc(0.5f, 0.8f, 0.9f,
				(float) -Math.PI / 2, 0, (float) -Math.PI, 0, 10, 10);

		headDecor1.isSelectable = true;
		headDecor1.selectionID = selectionID;
		headDecor1.setTextureID("octopus");

		headDecor1.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		headDecor1.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		headDecor1.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		headDecor1.shinessFactor = Materials.SHINESS_OCTOPUS;

		headDecor1.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] t7 = { -.4f, .5f, -0.9f, 1 };

		transformation.translate(t7);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -90);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, -30);
		headDecor1.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, headDecor1);

		EllipsoidArc headDecor2 = new EllipsoidArc(0.5f, 0.8f, 0.9f,
				(float) -Math.PI / 2, 0, (float) -Math.PI, 0, 10, 10);

		headDecor2.isSelectable = true;
		headDecor2.selectionID = selectionID;
		headDecor2.setTextureID("octopus");

		headDecor2.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		headDecor2.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		headDecor2.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		headDecor2.shinessFactor = Materials.SHINESS_OCTOPUS;

		headDecor2.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] t8 = { -.5f, .5f, -0.4f, 1 };

		transformation.translate(t8);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -90);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, -20);
		headDecor2.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, headDecor2);

		EllipsoidArc headDecor3 = new EllipsoidArc(0.5f, 0.8f, 0.9f,
				(float) -Math.PI / 2, 0, (float) -Math.PI, 0, 10, 10);

		headDecor3.isSelectable = true;
		headDecor3.selectionID = selectionID;
		headDecor3.setTextureID("octopus");

		headDecor3.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		headDecor3.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		headDecor3.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		headDecor3.shinessFactor = Materials.SHINESS_OCTOPUS;

		headDecor3.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] t9 = { -.6f, .5f, 0.1f, 1 };

		transformation.translate(t9);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -90);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, -10);
		headDecor3.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, headDecor3);

		EllipsoidArc headDecor4 = new EllipsoidArc(0.5f, 0.8f, 0.9f,
				(float) -Math.PI / 2, 0, (float) -Math.PI, 0, 10, 10);

		headDecor4.isSelectable = true;
		headDecor4.selectionID = selectionID;
		headDecor4.setTextureID("octopus");

		headDecor4.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		headDecor4.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		headDecor4.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		headDecor4.shinessFactor = Materials.SHINESS_OCTOPUS;

		headDecor4.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] t10 = { .4f, .5f, -0.9f, 1 };

		transformation.translate(t10);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 90);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 30);
		headDecor4.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, headDecor4);

		EllipsoidArc headDecor5 = new EllipsoidArc(0.5f, 0.8f, 0.9f,
				(float) -Math.PI / 2, 0, (float) -Math.PI, 0, 10, 10);

		headDecor5.isSelectable = true;
		headDecor5.selectionID = selectionID;
		headDecor5.setTextureID("octopus");

		headDecor5.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		headDecor5.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		headDecor5.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		headDecor5.shinessFactor = Materials.SHINESS_OCTOPUS;

		headDecor5.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] t11 = { .5f, .5f, -0.5f, 1 };

		transformation.translate(t11);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 90);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 20);
		headDecor5.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, headDecor5);

		EllipsoidArc headDecor6 = new EllipsoidArc(0.5f, 0.8f, 0.9f,
				(float) -Math.PI / 2, 0, (float) -Math.PI, 0, 10, 10);

		headDecor6.isSelectable = true;
		headDecor6.selectionID = selectionID;
		headDecor6.setTextureID("octopus");

		headDecor6.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		headDecor6.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		headDecor6.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		headDecor6.shinessFactor = Materials.SHINESS_OCTOPUS;

		headDecor6.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] t12 = { .6f, .5f, -0.1f, 1 };

		transformation.translate(t12);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 90);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 10);
		headDecor6.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, headDecor6);

		Ellipsoid leftEye = new Ellipsoid(0.6f, 1.6f, 0.4f, 10, 10);

		leftEye.isSelectable = true;
		leftEye.selectionID = selectionID;
		leftEye.setTextureID("octopus");

		leftEye.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		leftEye.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		leftEye.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		leftEye.shinessFactor = Materials.SHINESS_OCTOPUS;

		leftEye.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] t1 = { -0.4f, .4f, 1.0f, 1 };

		transformation.translate(t1);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 15);
		leftEye.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, leftEye);

		Ellipsoid leftEyePupill = new Ellipsoid(0.1f, 0.2f, 0.3f, 10, 10);

		leftEyePupill.isSelectable = true;
		leftEyePupill.selectionID = selectionID;
		leftEyePupill.setTextureID("lava");

		leftEyePupill.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		leftEyePupill.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		leftEyePupill.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		leftEyePupill.shinessFactor = Materials.SHINESS_OCTOPUS;

		leftEyePupill.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] t2 = { -1.0f, 1.0f, 1.1f, 1 };

		transformation.translate(t2);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 30);
		leftEyePupill.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, leftEyePupill);

		Ellipsoid rigthEye = new Ellipsoid(0.6f, 1.6f, 0.4f, 10, 10);

		rigthEye.isSelectable = true;
		rigthEye.selectionID = selectionID;
		rigthEye.setTextureID("octopus");

		rigthEye.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		rigthEye.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		rigthEye.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		rigthEye.shinessFactor = Materials.SHINESS_OCTOPUS;

		rigthEye.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] t3 = { 0.4f, .4f, 1.0f, 1 };

		transformation.translate(t3);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -15);
		rigthEye.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, rigthEye);

		Ellipsoid rightEyePupill = new Ellipsoid(0.1f, 0.2f, 0.3f, 10, 10);

		rightEyePupill.isSelectable = true;
		rightEyePupill.selectionID = selectionID;
		rightEyePupill.setTextureID("lava");

		rightEyePupill.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		rightEyePupill.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		rightEyePupill.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		rightEyePupill.shinessFactor = Materials.SHINESS_OCTOPUS;

		rightEyePupill.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] t4 = { 1.0f, 1.0f, 1.1f, 1 };

		transformation.translate(t4);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, -30);
		rightEyePupill.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, rightEyePupill);

		EllipsoidArc peak = new EllipsoidArc(0.4f, 0.6f, 0.9f,
				(float) -Math.PI / 2, 0, (float) -Math.PI, 0, 10, 10);

		peak.isSelectable = true;
		peak.selectionID = selectionID;
		peak.setTextureID("peak");

		peak.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		peak.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		peak.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		peak.shinessFactor = Materials.SHINESS_OCTOPUS;

		peak.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] t5 = { 0f, 1.2f, 1.5f, 1 };

		transformation.translate(t5);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 110);
		peak.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, peak);

		EllipsoidArc littlePeak = new EllipsoidArc(0.3f, 0.6f, 0.9f,
				(float) -Math.PI / 2, 0, (float) -Math.PI, 0, 10, 10);

		littlePeak.isSelectable = true;
		littlePeak.selectionID = selectionID;
		littlePeak.setTextureID("peak");

		littlePeak.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		littlePeak.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		littlePeak.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		littlePeak.shinessFactor = Materials.SHINESS_OCTOPUS;

		littlePeak.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] t6 = { 0f, 1.0f, 1.3f, 1 };

		transformation.translate(t6);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 180);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, -50);
		littlePeak.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, littlePeak);

		// ----------------------------------------------------------------
		Node tentacle1 = new Node();

		Node tentacle1_1 = new Node();

		Cylinder tentacle = new Cylinder(8, 0.25f, 2, 0.2f, 1f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("octopus");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(tentacle1_1, tentacle);

		tentacle = new Cylinder(8, 0.25f, 2, 0.2f, 1.4f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("bowl");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] b1 = { 0, 0, 0.1f, 1 };

		transformation.translate(b1);
		tentacle.updateTransformationState(transformation);

		Node.addChildrenToNode(tentacle1_1, tentacle);

		for (int i = 0; i < 6; i++) {
			Arc tentacleDecor = new Arc(0, (float) Math.PI, 0.25f, 0.1f, 0.15f,
					8, 0.1f, 1);

			tentacleDecor.isSelectable = true;
			tentacleDecor.selectionID = selectionID;
			tentacleDecor.setTextureID("bowl");

			tentacleDecor.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
			tentacleDecor.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
			tentacleDecor.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
			tentacleDecor.shinessFactor = Materials.SHINESS_OCTOPUS;

			tentacleDecor.setBufferID(bufferController.getBufferID());

			transformation.setIdentity();

			float[] c1 = { 0, i * (0.4f), .2f, 1 };

			transformation.translate(c1);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -45);
			tentacleDecor.updateTransformationState(transformation);

			Node.addChildrenToNode(tentacle1_1, tentacleDecor);

			tentacleDecor = new Arc(0, (float) Math.PI, 0.25f, 0.1f, 0.15f, 10,
					0.1f, 1);

			tentacleDecor.isSelectable = true;
			tentacleDecor.selectionID = selectionID;
			tentacleDecor.setTextureID("bowl");

			tentacleDecor.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
			tentacleDecor.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
			tentacleDecor.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
			tentacleDecor.shinessFactor = Materials.SHINESS_OCTOPUS;

			tentacleDecor.setBufferID(bufferController.getBufferID());

			transformation.setIdentity();

			float[] c2 = { 0 - 0.3f, i * (0.4f), .2f, 1 };

			transformation.translate(c2);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -45);
			tentacleDecor.updateTransformationState(transformation);

			Node.addChildrenToNode(tentacle1_1, tentacleDecor);

		}

		transformation.setIdentity();

		float[] a1 = { 0, 0, 1.8f, 1 };

		transformation.translate(a1);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -45);
		tentacle1_1.updateTransformationState(transformation);

		Node.addChildrenToNode(tentacle1, tentacle1_1);
		// ------------------------------------------------------
		Node tentacle1_2 = new Node();

		tentacle = new Cylinder(8, 0.25f, 2, 0.2f, 1f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("octopus");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(tentacle1_2, tentacle);

		tentacle = new Cylinder(8, 0.25f, 2, 0.2f, 1f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("bowl");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] a2 = { -0.1f, 0.f, 0f, 1 };

		transformation.translate(a2);
		tentacle.updateTransformationState(transformation);

		Node.addChildrenToNode(tentacle1_2, tentacle);

		for (int i = 1; i < 5; i++) {
			Arc tentacleDecor = new Arc(0, (float) Math.PI, 0.10f, 0.1f, 0.15f,
					6, 0.1f, 1);

			tentacleDecor.isSelectable = true;
			tentacleDecor.selectionID = selectionID;
			tentacleDecor.setTextureID("bowl");

			tentacleDecor.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
			tentacleDecor.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
			tentacleDecor.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
			tentacleDecor.shinessFactor = Materials.SHINESS_OCTOPUS;

			tentacleDecor.setBufferID(bufferController.getBufferID());

			transformation.setIdentity();

			float[] c1 = { -0.4f, i * (0.4f), -.1f, 1 };

			transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
			transformation.translate(c1);

			tentacleDecor.updateTransformationState(transformation);

			Node.addChildrenToNode(tentacle1_2, tentacleDecor);

		}

		transformation.setIdentity();
		float[] c3 = { 1.5f, 1.2f, 1.7f, 1 };

		transformation.translate(c3);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 50);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 30);
		tentacle1_2.updateTransformationState(transformation);

		Node.addChildrenToNode(tentacle1, tentacle1_2);

		// ------------------------------------------------------------------------

		EllipsoidArc tentacleEnd = new EllipsoidArc(0.25f, 0.25f, 1.4f,
				(float) -Math.PI / 2, 0, (float) -Math.PI, (float) Math.PI, 10,
				10);

		tentacleEnd.isSelectable = true;
		tentacleEnd.selectionID = selectionID;
		tentacleEnd.setTextureID("octopus");

		tentacleEnd.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacleEnd.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacleEnd.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacleEnd.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacleEnd.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] a3 = { 2.2f, 2.4f, 3.3f, 1 };

		transformation.translate(a3);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 180);
		tentacleEnd.updateTransformationState(transformation);

		Node.addChildrenToNode(tentacle1, tentacleEnd);

		Node.addChildrenToNode(octopus, tentacle1);
		// ------------------------------------------------------
		// tentacle2
		Node tentacle2 = new Node();
		Node tentacle2_1 = new Node();

		tentacle = new Cylinder(8, 0.25f, 2, 0.2f, 1f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("octopus");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(tentacle2_1, tentacle);

		tentacle = new Cylinder(8, 0.25f, 2, 0.2f, 1f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("bowl");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] d4 = { 0, 0, 0.1f, 1 };

		transformation.translate(d4);
		tentacle.updateTransformationState(transformation);

		Node.addChildrenToNode(tentacle2_1, tentacle);

		for (int i = 0; i < 6; i++) {
			Arc tentacleDecor = new Arc(0, (float) Math.PI, 0.25f, 0.1f, 0.15f,
					6, 0.1f, 1);

			tentacleDecor.isSelectable = true;
			tentacleDecor.selectionID = selectionID;
			tentacleDecor.setTextureID("bowl");

			tentacleDecor.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
			tentacleDecor.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
			tentacleDecor.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
			tentacleDecor.shinessFactor = Materials.SHINESS_OCTOPUS;

			tentacleDecor.setBufferID(bufferController.getBufferID());

			transformation.setIdentity();

			float[] c1 = { 0.1f, i * (0.4f), .15f, 1 };

			transformation.translate(c1);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -45);
			tentacleDecor.updateTransformationState(transformation);

			Node.addChildrenToNode(tentacle2_1, tentacleDecor);

			tentacleDecor = new Arc(0, (float) Math.PI, 0.25f, 0.1f, 0.15f, 6,
					0.1f, 1);

			tentacleDecor.isSelectable = true;
			tentacleDecor.selectionID = selectionID;
			tentacleDecor.setTextureID("bowl");

			tentacleDecor.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
			tentacleDecor.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
			tentacleDecor.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
			tentacleDecor.shinessFactor = Materials.SHINESS_OCTOPUS;

			tentacleDecor.setBufferID(bufferController.getBufferID());

			transformation.setIdentity();

			float[] c2 = { 0 - 0.2f, i * (0.4f), .15f, 1 };

			transformation.translate(c2);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -45);
			tentacleDecor.updateTransformationState(transformation);

			Node.addChildrenToNode(tentacle2_1, tentacleDecor);

		}

		transformation.setIdentity();

		float[] a4 = { 0, 0, 1.8f, 1 };

		transformation.translate(a1);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 45);
		tentacle2_1.updateTransformationState(transformation);

		Node.addChildrenToNode(tentacle2, tentacle2_1);

		Node tentacle2_2 = new Node();

		tentacle = new Cylinder(8, 0.25f, 2, 0.2f, 1f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("octopus");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(tentacle2_2, tentacle);

		tentacle = new Cylinder(8, 0.25f, 2, 0.2f, 1f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("bowl");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] d5 = { .1f, 0, 0, 1 };

		transformation.translate(d5);
		tentacle.updateTransformationState(transformation);

		Node.addChildrenToNode(tentacle2_2, tentacle);

		for (int i = 1; i < 5; i++) {
			Arc tentacleDecor = new Arc(0, (float) Math.PI, 0.2f, 0.1f, 0.15f,
					6, 0.1f, 1);

			tentacleDecor.isSelectable = true;
			tentacleDecor.selectionID = selectionID;
			tentacleDecor.setTextureID("bowl");

			tentacleDecor.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
			tentacleDecor.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
			tentacleDecor.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
			tentacleDecor.shinessFactor = Materials.SHINESS_OCTOPUS;

			tentacleDecor.setBufferID(bufferController.getBufferID());

			transformation.setIdentity();

			float[] c1 = { 0.2f, i * (0.4f), -.0f, 1 };

			transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
			transformation.translate(c1);

			tentacleDecor.updateTransformationState(transformation);

			Node.addChildrenToNode(tentacle2_2, tentacleDecor);

		}

		transformation.setIdentity();

		float[] a5 = { -1.4f, 1.2f, 1.7f, 1 };

		transformation.translate(a5);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 50);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, -30);
		tentacle2_2.updateTransformationState(transformation);

		Node.addChildrenToNode(tentacle2, tentacle2_2);

		tentacleEnd = new EllipsoidArc(0.25f, 0.25f, 1.4f,
				(float) -Math.PI / 2, 0, (float) -Math.PI, (float) Math.PI, 10,
				10);

		tentacleEnd.isSelectable = true;
		tentacleEnd.selectionID = selectionID;
		tentacleEnd.setTextureID("octopus");

		tentacleEnd.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacleEnd.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacleEnd.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacleEnd.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacleEnd.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] a6 = { -2.1f, 2.4f, 3.3f, 1 };

		transformation.translate(a6);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 180);
		tentacleEnd.updateTransformationState(transformation);

		Node.addChildrenToNode(tentacle2, tentacleEnd);
		Node.addChildrenToNode(octopus, tentacle2);
		// ---------------------------------------------------------------
		// tentacle 3
		Node tentacle3 = new Node();

		tentacle = new Cylinder(8, 0.25f, 3, 0.2f, 1.4f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("octopus");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(tentacle3, tentacle);

		tentacle = new Cylinder(8, 0.25f, 3, 0.2f, 1.4f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("bowl");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] d7 = { 0.0f, 0.0f, .1f, 1 };

		transformation.translate(d7);
		tentacle.updateTransformationState(transformation);

		Node.addChildrenToNode(tentacle3, tentacle);

		for (int i = 0; i < 8; i++) {
			Arc tentacleDecor = new Arc(0, (float) Math.PI, 0.3f, 0.1f, 0.15f,
					6, 0.1f, 1);

			tentacleDecor.isSelectable = true;
			tentacleDecor.selectionID = selectionID;
			tentacleDecor.setTextureID("bowl");

			tentacleDecor.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
			tentacleDecor.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
			tentacleDecor.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
			tentacleDecor.shinessFactor = Materials.SHINESS_OCTOPUS;

			tentacleDecor.setBufferID(bufferController.getBufferID());

			transformation.setIdentity();

			float[] c1 = { 0, i * (0.4f), .12f, 1 };

			transformation.translate(c1);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -90);
			tentacleDecor.updateTransformationState(transformation);

			Node.addChildrenToNode(tentacle3, tentacleDecor);

			tentacleDecor = new Arc(0, (float) Math.PI, 0.3f, 0.1f, 0.15f, 6,
					0.1f, 1);

			tentacleDecor.isSelectable = true;
			tentacleDecor.selectionID = selectionID;
			tentacleDecor.setTextureID("bowl");

			tentacleDecor.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
			tentacleDecor.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
			tentacleDecor.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
			tentacleDecor.shinessFactor = Materials.SHINESS_OCTOPUS;

			tentacleDecor.setBufferID(bufferController.getBufferID());

			transformation.setIdentity();

			float[] c2 = { 0 - 0.3f, i * (0.4f), .1f, 1 };

			transformation.translate(c2);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -90);
			tentacleDecor.updateTransformationState(transformation);

			Node.addChildrenToNode(tentacle3, tentacleDecor);

		}

		transformation.setIdentity();

		float[] a7 = { 0, 0.8f, 1.8f, 1 };

		transformation.translate(a7);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 135);
		tentacle3.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, tentacle3);
		// ---------------------------------------------------------------
		// tentacle 4
		Node tentacle4 = new Node();

		tentacle = new Cylinder(8, 0.25f, 3, 0.2f, 1.4f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("octopus");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(tentacle4, tentacle);
		tentacle = new Cylinder(8, 0.25f, 3, 0.2f, 1.4f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("bowl");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] d8 = { 0, 0f, 0.1f, 1 };

		transformation.translate(d8);
		tentacle.updateTransformationState(transformation);

		Node.addChildrenToNode(tentacle4, tentacle);

		for (int i = 0; i < 8; i++) {
			Arc tentacleDecor = new Arc(0, (float) Math.PI, 0.25f, 0.1f, 0.15f,
					6, 0.1f, 1);

			tentacleDecor.isSelectable = true;
			tentacleDecor.selectionID = selectionID;
			tentacleDecor.setTextureID("bowl");

			tentacleDecor.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
			tentacleDecor.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
			tentacleDecor.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
			tentacleDecor.shinessFactor = Materials.SHINESS_OCTOPUS;

			tentacleDecor.setBufferID(bufferController.getBufferID());

			transformation.setIdentity();

			float[] c1 = { 0, i * (0.4f), .2f, 1 };

			transformation.translate(c1);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -45);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 180);
			tentacleDecor.updateTransformationState(transformation);

			Node.addChildrenToNode(tentacle4, tentacleDecor);

			tentacleDecor = new Arc(0, (float) Math.PI, 0.25f, 0.1f, 0.15f, 6,
					0.1f, 1);

			tentacleDecor.isSelectable = true;
			tentacleDecor.selectionID = selectionID;
			tentacleDecor.setTextureID("bowl");

			tentacleDecor.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
			tentacleDecor.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
			tentacleDecor.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
			tentacleDecor.shinessFactor = Materials.SHINESS_OCTOPUS;

			tentacleDecor.setBufferID(bufferController.getBufferID());

			transformation.setIdentity();

			float[] c2 = { 0 - 0.3f, i * (0.4f), .2f, 1 };

			transformation.translate(c2);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -45);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 180);
			tentacleDecor.updateTransformationState(transformation);

			Node.addChildrenToNode(tentacle4, tentacleDecor);

		}

		transformation.setIdentity();

		float[] a8 = { 0, 0.8f, 1.8f, 1 };

		transformation.translate(a8);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -135);
		tentacle4.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, tentacle4);
		// ---------------------------------------------------------------
		// tentacle 5
		Node tentacle5 = new Node();

		tentacle = new Cylinder(10, 0.25f, 3, 0.2f, 1.4f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("octopus");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(tentacle5, tentacle);

		tentacle = new Cylinder(8, 0.25f, 3, 0.2f, 1.4f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("bowl");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] d9 = { 0, 0f, .1f, 1 };

		transformation.translate(d9);
		tentacle.updateTransformationState(transformation);

		Node.addChildrenToNode(tentacle5, tentacle);

		for (int i = 0; i < 8; i++) {
			Arc tentacleDecor = new Arc(0, (float) Math.PI, 0.25f, 0.1f, 0.15f,
					6, 0.1f, 1);

			tentacleDecor.isSelectable = true;
			tentacleDecor.selectionID = selectionID;
			tentacleDecor.setTextureID("bowl");

			tentacleDecor.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
			tentacleDecor.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
			tentacleDecor.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
			tentacleDecor.shinessFactor = Materials.SHINESS_OCTOPUS;

			tentacleDecor.setBufferID(bufferController.getBufferID());

			transformation.setIdentity();

			float[] c1 = { 0, i * (0.4f), .2f, 1 };

			transformation.translate(c1);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -45);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 180);
			tentacleDecor.updateTransformationState(transformation);

			Node.addChildrenToNode(tentacle5, tentacleDecor);

			tentacleDecor = new Arc(0, (float) Math.PI, 0.25f, 0.1f, 0.15f, 6,
					0.1f, 1);

			tentacleDecor.isSelectable = true;
			tentacleDecor.selectionID = selectionID;
			tentacleDecor.setTextureID("bowl");

			tentacleDecor.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
			tentacleDecor.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
			tentacleDecor.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
			tentacleDecor.shinessFactor = Materials.SHINESS_OCTOPUS;

			tentacleDecor.setBufferID(bufferController.getBufferID());

			transformation.setIdentity();

			float[] c2 = { 0 - 0.3f, i * (0.4f), .2f, 1 };

			transformation.translate(c2);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -45);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 180);
			tentacleDecor.updateTransformationState(transformation);

			Node.addChildrenToNode(tentacle5, tentacleDecor);

		}

		transformation.setIdentity();

		float[] a9 = { 0, 0.5f, 1.8f, 1 };

		transformation.translate(a9);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -135);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, -70);
		tentacle5.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, tentacle5);
		// ---------------------------------------------------------------
		// tentacle 6
		Node tentacle6 = new Node();

		tentacle = new Cylinder(8, 0.25f, 3, 0.2f, 1.4f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("octopus");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(tentacle6, tentacle);

		tentacle = new Cylinder(8, 0.25f, 3, 0.2f, 1.4f);

		tentacle.isSelectable = true;
		tentacle.selectionID = selectionID;
		tentacle.setTextureID("bowl");

		tentacle.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
		tentacle.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
		tentacle.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
		tentacle.shinessFactor = Materials.SHINESS_OCTOPUS;

		tentacle.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();

		float[] d10 = { 0, 0f, .1f, 1 };

		transformation.translate(d10);
		tentacle.updateTransformationState(transformation);

		Node.addChildrenToNode(tentacle6, tentacle);

		for (int i = 0; i < 8; i++) {
			Arc tentacleDecor = new Arc(0, (float) Math.PI, 0.15f, 0.1f, 0.15f,
					6, 0.1f, 1);

			tentacleDecor.isSelectable = true;
			tentacleDecor.selectionID = selectionID;
			tentacleDecor.setTextureID("bowl");

			tentacleDecor.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
			tentacleDecor.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
			tentacleDecor.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
			tentacleDecor.shinessFactor = Materials.SHINESS_OCTOPUS;

			tentacleDecor.setBufferID(bufferController.getBufferID());

			transformation.setIdentity();

			float[] c1 = { 0.2f, i * (0.4f), .2f, 1 };

			transformation.translate(c1);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 180);
			tentacleDecor.updateTransformationState(transformation);

			Node.addChildrenToNode(tentacle6, tentacleDecor);

			tentacleDecor = new Arc(0, (float) Math.PI, 0.15f, 0.1f, 0.15f, 6,
					0.1f, 1);

			tentacleDecor.isSelectable = true;
			tentacleDecor.selectionID = selectionID;
			tentacleDecor.setTextureID("bowl");

			tentacleDecor.setAmbientFactor(Materials.AMBIENT_OCTOPUS);
			tentacleDecor.setDiffuseFactor(Materials.DIFFUSE_OCTOPUS);
			tentacleDecor.setSpecularFactor(Materials.SPECULAR_OCTOPUS);
			tentacleDecor.shinessFactor = Materials.SHINESS_OCTOPUS;

			tentacleDecor.setBufferID(bufferController.getBufferID());

			transformation.setIdentity();

			float[] c2 = { 0.5f, i * (0.4f), .2f, 1 };

			transformation.translate(c2);
			transformation
					.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 180);
			tentacleDecor.updateTransformationState(transformation);

			Node.addChildrenToNode(tentacle6, tentacleDecor);

		}

		transformation.setIdentity();

		float[] a10 = { 0, 0.5f, 1.8f, 1 };

		transformation.translate(a10);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 135);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 70);
		tentacle6.updateTransformationState(transformation);

		Node.addChildrenToNode(octopus, tentacle6);

		return octopus;
	}

	private final Node createMark(PlayGroundElement playGroundElement) {

		Node markNode = new Node();

		Circle circle = new Circle(20, 4f, 5f, 1, 1);

		circle.setTextureID("hero_mark");

		circle.setAmbientFactor(Materials.AMBIENT_MARMOR);
		circle.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		circle.setSpecularFactor(Materials.SPECULAR_MARMOR);
		circle.shinessFactor = Materials.SHINESS_MARMOR;

		circle.isSelectable = false;

		Transformation transform = new Transformation();
		float[] translate = { 0, 10f, 0, 1 };
		transform.translate(translate);

		circle.updateTransformationState(transform);

		Node.addChildrenToNode(markNode, circle);
		playGroundElement.mark = markNode;

		return markNode;

	}

	private final Node createBomb(PlayGroundElement playGroundElement) {

		Node bombNode = new Node();

		Ellipsoid cube = new Ellipsoid(0.2f, 5f, 2f, 40, 40);

		cube.setTextureID("lava");
		cube.isSelectable = false;

		Transformation transform = new Transformation();
		float[] translate = { 0, 0.1f, 0, 1 };
		transform.translate(translate);

		cube.updateTransformationState(transform);

		Node.addChildrenToNode(bombNode, cube);
		playGroundElement.element = bombNode;

		return bombNode;
	}

	private final Node createMoveMap(PlayGroundElement playGroundElement) {
		BufferController bufferController = BufferController.getInstance();

		Node node_movemap = new Node();

		node_movemap.playGroundelement = playGroundElement;
		Vector moveMapList = playGroundElement.moveMapList;
		Node hero = (Node) playGroundElement.element;

		for (int i = 0; i < moveMapList.size(); i++) {

			MoveMapElement element = (MoveMapElement) moveMapList.elementAt(i);

			if (element.posX <= 0
					|| element.posZ <= 0
					|| element.posX > PlayGround.getInstance().playGround[0].length - 1
					|| element.posZ > PlayGround.getInstance().playGround.length - 1) {

				continue;
			}

			if (PlayGround.getInstance().playGround[element.posZ][element.posX] != PlayGround.FREE) {
				continue;
			}

			if (!PathFinder.getInstance().generatePath(
					hero.playGroundelement.posX, hero.playGroundelement.posZ,
					element.posX, element.posZ)) {

				continue;
			}

			PlayGroundElement playGroundElementPlate = new PlayGroundElement(
					element.posX, 0, element.posZ, GeometryFactory.MOVE_PLATE);

			playGroundElementPlate.element = hero;
			hero.playGroundelement.moveMap = node_movemap;

			Cube cube = new Cube(0.5f, 0.2f, 0.5f, 1, 1);

			cube.setAmbientFactor(Materials.AMBIENT_MARMOR);
			cube.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			cube.setSpecularFactor(Materials.SPECULAR_MARMOR);
			cube.shinessFactor = Materials.SHINESS_MARMOR;

			cube.playGroundelement = playGroundElementPlate;

			Transformation transformation = new Transformation();
			float[] translate = { element.posX, 2.0f, element.posZ, 1 };
			transformation.translate(translate);

			cube.updateTransformationState(transformation);

			cube.setTextureID("lava");
			cube.setBufferID(bufferController.getBufferID());
			cube.isSelectable = true;

			int selectionID = SelectionController.getInstance()
					.addMoveMapPlate(cube);
			cube.selectionID = selectionID;

			Node.addChildrenToNode(node_movemap, cube);

		}

		return node_movemap;
	}

	private final Node createSpider(PlayGroundElement playGroundElement) {
		BufferController bufferController = BufferController.getInstance();

		Node node_rat = new Node();
		int selectionID = SelectionController.getInstance()
				.addMonster(node_rat);

		node_rat.gameflowControllerTableID = node_rat.toString();
		node_rat.playGroundelement = playGroundElement;

		playGroundElement.playGroundInfo = new int[1][1];
		playGroundElement.creatureInfo = GeometryFactory.BANE_SPIDER;
		playGroundElement.playGroundInfo[0][0] = PlayGround.MONSTER;
		playGroundElement.element = node_rat;

		PlayGround.getInstance().addNewElement(playGroundElement);

		Ellipsoid chest = new Ellipsoid(0.5f, 1.5f, 0.5f, 10, 10);
		chest.setTextureID("spider");

		chest.setAmbientFactor(Materials.AMBIENT_SPIDER);
		chest.setDiffuseFactor(Materials.DIFFUSE_SPIDER);
		chest.setSpecularFactor(Materials.SPECULAR_SPIDER);
		chest.shinessFactor = Materials.SHINESS_SPIDER;

		chest.setBufferID(bufferController.getBufferID());
		chest.isSelectable = true;
		chest.selectionID = selectionID;

		Transformation transform = new Transformation();

		transform.setIdentity();
		float[] translate0 = { 0.f, 1.0f, 0f, 1 };
		transform.translate(translate0);

		chest.updateTransformationState(transform);
		Node.addChildrenToNode(node_rat, chest);

		Ellipsoid body = new Ellipsoid(0.3f, 0.4f, 0.3f, 10, 10);
		body.setTextureID("spider");

		body.setAmbientFactor(Materials.AMBIENT_SPIDER);
		body.setDiffuseFactor(Materials.DIFFUSE_SPIDER);
		body.setSpecularFactor(Materials.SPECULAR_SPIDER);
		body.shinessFactor = Materials.SHINESS_SPIDER;

		body.setBufferID(bufferController.getBufferID());
		body.isSelectable = true;
		body.selectionID = selectionID;

		transform.setIdentity();
		float[] translate = { 0.f, 1.5f, 0.7f, 1 };
		transform.translate(translate);

		body.updateTransformationState(transform);
		Node.addChildrenToNode(node_rat, body);

		SolidDeformedCylinder horn1 = new SolidDeformedCylinder(0.08f, 0.02f,
				0.7f, 10, 0.1f, 1f);
		horn1.isSelectable = true;
		horn1.selectionID = selectionID;
		horn1.setTextureID("spear");

		horn1.setAmbientFactor(Materials.AMBIENT_HORN);
		horn1.setDiffuseFactor(Materials.DIFFUSE_HORN);
		horn1.setSpecularFactor(Materials.SPECULAR_HORN);
		horn1.shinessFactor = Materials.SHINESS_HORN;

		horn1.setBufferID(bufferController.getBufferID());

		transform.setIdentity();
		float[] t15 = { 0.f, 1.6f, .8f, 1 };

		transform.translate(t15);
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 160);
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 45);
		horn1.updateTransformationState(transform);

		Node.addChildrenToNode(node_rat, horn1);

		SolidDeformedCylinder horn2 = new SolidDeformedCylinder(0.08f, 0.02f,
				0.7f, 6, 0.1f, 1f);
		horn2.isSelectable = true;
		horn2.selectionID = selectionID;
		horn2.setTextureID("spear");

		horn2.setAmbientFactor(Materials.AMBIENT_HORN);
		horn2.setDiffuseFactor(Materials.DIFFUSE_HORN);
		horn2.setSpecularFactor(Materials.SPECULAR_HORN);
		horn2.shinessFactor = Materials.SHINESS_HORN;

		horn2.setBufferID(bufferController.getBufferID());

		transform.setIdentity();
		float[] t16 = { 0.f, 1.6f, .8f, 1 };

		transform.translate(t16);
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 160);
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, -45);
		horn2.updateTransformationState(transform);

		Node.addChildrenToNode(node_rat, horn2);

		SolidDeformedCylinder horn3 = new SolidDeformedCylinder(0.08f, 0.02f,
				0.7f, 6, 0.1f, 1f);
		horn3.isSelectable = true;
		horn3.selectionID = selectionID;
		horn3.setTextureID("spear");

		horn3.setAmbientFactor(Materials.AMBIENT_HORN);
		horn3.setDiffuseFactor(Materials.DIFFUSE_HORN);
		horn3.setSpecularFactor(Materials.SPECULAR_HORN);
		horn3.shinessFactor = Materials.SHINESS_HORN;

		horn3.setBufferID(bufferController.getBufferID());

		transform.setIdentity();
		float[] t17 = { 0.f, 1.6f, .8f, 1 };

		transform.translate(t17);
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 30);
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 45);
		horn3.updateTransformationState(transform);

		// Node.addChildrenToNode(node_rat, horn3);

		SolidDeformedCylinder horn4 = new SolidDeformedCylinder(0.08f, 0.02f,
				0.7f, 6, 0.1f, 1f);
		horn4.isSelectable = true;
		horn4.selectionID = selectionID;
		horn4.setTextureID("spear");

		horn4.setAmbientFactor(Materials.AMBIENT_HORN);
		horn4.setDiffuseFactor(Materials.DIFFUSE_HORN);
		horn4.setSpecularFactor(Materials.SPECULAR_HORN);
		horn4.shinessFactor = Materials.SHINESS_HORN;

		horn4.setBufferID(bufferController.getBufferID());

		transform.setIdentity();
		float[] t18 = { 0.f, 1.6f, .8f, 1 };

		transform.translate(t18);
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 30);
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, -45);
		horn4.updateTransformationState(transform);

		// Node.addChildrenToNode(node_rat, horn4);

		SolidDeformedCylinder leg1 = new SolidDeformedCylinder(0.1f, 0.08f, 1f,
				6, 0.1f, 1);

		leg1.setTextureID("spider");

		leg1.setAmbientFactor(Materials.AMBIENT_SPIDER);
		leg1.setDiffuseFactor(Materials.DIFFUSE_SPIDER);
		leg1.setSpecularFactor(Materials.SPECULAR_SPIDER);
		leg1.shinessFactor = Materials.SHINESS_SPIDER;

		leg1.isSelectable = true;
		leg1.selectionID = selectionID;
		leg1.setBufferID(bufferController.getBufferID());

		transform.setIdentity();
		float[] t1 = { 0.f, 0.8f, 0.7f, 1 };
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 45);
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 45);
		transform.translate(t1);

		leg1.updateTransformationState(transform);
		Node.addChildrenToNode(node_rat, leg1);

		SolidDeformedCylinder leg2 = new SolidDeformedCylinder(0.1f, 0.08f, 1f,
				6, 0.1f, 1);

		leg2.setTextureID("spider");

		leg2.setAmbientFactor(Materials.AMBIENT_SPIDER);
		leg2.setDiffuseFactor(Materials.DIFFUSE_SPIDER);
		leg2.setSpecularFactor(Materials.SPECULAR_SPIDER);
		leg2.shinessFactor = Materials.SHINESS_SPIDER;

		leg2.isSelectable = true;
		leg2.selectionID = selectionID;
		leg2.setBufferID(bufferController.getBufferID());

		transform.setIdentity();
		float[] t2 = { 0.f, 0.8f, 0.7f, 1 };
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 45);
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, -45);
		transform.translate(t2);

		leg2.updateTransformationState(transform);
		Node.addChildrenToNode(node_rat, leg2);

		SolidDeformedCylinder leg3 = new SolidDeformedCylinder(0.1f, 0.08f, 1f,
				6, 0.1f, 1);

		leg3.setTextureID("spider");

		leg3.setAmbientFactor(Materials.AMBIENT_SPIDER);
		leg3.setDiffuseFactor(Materials.DIFFUSE_SPIDER);
		leg3.setSpecularFactor(Materials.SPECULAR_SPIDER);
		leg3.shinessFactor = Materials.SHINESS_SPIDER;

		leg3.isSelectable = true;
		leg3.selectionID = selectionID;
		leg3.setBufferID(bufferController.getBufferID());

		transform.setIdentity();
		float[] t3 = { 0.f, 0.8f, 0.7f, 1 };
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 45);
		transform.translate(t3);

		leg3.updateTransformationState(transform);
		Node.addChildrenToNode(node_rat, leg3);

		SolidDeformedCylinder leg4 = new SolidDeformedCylinder(0.1f, 0.08f, 1f,
				6, 0.1f, 1);

		leg4.setTextureID("spider");

		leg4.setAmbientFactor(Materials.AMBIENT_SPIDER);
		leg4.setDiffuseFactor(Materials.DIFFUSE_SPIDER);
		leg4.setSpecularFactor(Materials.SPECULAR_SPIDER);
		leg4.shinessFactor = Materials.SHINESS_SPIDER;

		leg4.isSelectable = true;
		leg4.selectionID = selectionID;
		leg4.setBufferID(bufferController.getBufferID());

		transform.setIdentity();
		float[] t4 = { 0.f, 0.8f, 0.7f, 1 };
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -45);
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, -45);
		transform.translate(t4);

		leg4.updateTransformationState(transform);
		Node.addChildrenToNode(node_rat, leg4);

		SolidDeformedCylinder leg5 = new SolidDeformedCylinder(0.1f, 0.08f, 1f,
				6, 0.1f, 1);

		leg5.setTextureID("spider");

		leg5.setAmbientFactor(Materials.AMBIENT_SPIDER);
		leg5.setDiffuseFactor(Materials.DIFFUSE_SPIDER);
		leg5.setSpecularFactor(Materials.SPECULAR_SPIDER);
		leg5.shinessFactor = Materials.SHINESS_SPIDER;

		leg5.setBufferID(bufferController.getBufferID());
		leg5.isSelectable = true;
		leg5.selectionID = selectionID;

		transform.setIdentity();
		float[] t5 = { 0.f, 0.8f, 0.7f, 1 };
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -45);
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 45);
		transform.translate(t5);

		leg5.updateTransformationState(transform);
		Node.addChildrenToNode(node_rat, leg5);

		SolidDeformedCylinder leg6 = new SolidDeformedCylinder(0.1f, 0.08f, 1f,
				6, 0.1f, 1);

		leg6.setTextureID("spider");

		leg6.setAmbientFactor(Materials.AMBIENT_SPIDER);
		leg6.setDiffuseFactor(Materials.DIFFUSE_SPIDER);
		leg6.setSpecularFactor(Materials.SPECULAR_SPIDER);
		leg6.shinessFactor = Materials.SHINESS_SPIDER;

		leg6.setBufferID(bufferController.getBufferID());
		leg6.isSelectable = true;
		leg6.selectionID = selectionID;

		transform.setIdentity();
		float[] t6 = { 0.f, 0.8f, 0.7f, 1 };
		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -45);
		transform.translate(t6);

		leg6.updateTransformationState(transform);
		Node.addChildrenToNode(node_rat, leg6);

		SolidDeformedCylinder mid_leg1 = new SolidDeformedCylinder(0.08f,
				0.05f, 2f, 6, 0.1f, 1);

		mid_leg1.setTextureID("spider");

		mid_leg1.setAmbientFactor(Materials.AMBIENT_SPIDER);
		mid_leg1.setDiffuseFactor(Materials.DIFFUSE_SPIDER);
		mid_leg1.setSpecularFactor(Materials.SPECULAR_SPIDER);
		mid_leg1.shinessFactor = Materials.SHINESS_SPIDER;

		mid_leg1.setBufferID(bufferController.getBufferID());
		mid_leg1.isSelectable = true;
		mid_leg1.selectionID = selectionID;

		transform.setIdentity();
		float[] mid_t1 = { 0.f, 0.8f, 0.7f, 1 };
		float[] temp = { -0.55f, 0.9f, -0.4f, 1 };

		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 170);
		transform.translate(mid_t1);
		transform.translate(temp);

		mid_leg1.updateTransformationState(transform);
		Node.addChildrenToNode(node_rat, mid_leg1);

		SolidDeformedCylinder mid_leg2 = new SolidDeformedCylinder(0.08f,
				0.05f, 2f, 6, 0.1f, 1);

		mid_leg2.setTextureID("spider");

		mid_leg2.setAmbientFactor(Materials.AMBIENT_SPIDER);
		mid_leg2.setDiffuseFactor(Materials.DIFFUSE_SPIDER);
		mid_leg2.setSpecularFactor(Materials.SPECULAR_SPIDER);
		mid_leg2.shinessFactor = Materials.SHINESS_SPIDER;

		mid_leg2.setBufferID(bufferController.getBufferID());
		mid_leg2.isSelectable = true;
		mid_leg2.selectionID = selectionID;

		transform.setIdentity();
		float[] mid_t2 = { 0.f, 0.8f, 0.7f, 1 };
		float[] temp2 = { -0.65f, 0.7f, -0f, 1 };

		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 170);
		transform.translate(mid_t2);
		transform.translate(temp2);

		mid_leg2.updateTransformationState(transform);
		Node.addChildrenToNode(node_rat, mid_leg2);

		SolidDeformedCylinder mid_leg3 = new SolidDeformedCylinder(0.08f,
				0.05f, 2f, 6, 0.1f, 1);

		mid_leg3.setTextureID("spider");

		mid_leg3.setAmbientFactor(Materials.AMBIENT_SPIDER);
		mid_leg3.setDiffuseFactor(Materials.DIFFUSE_SPIDER);
		mid_leg3.setSpecularFactor(Materials.SPECULAR_SPIDER);
		mid_leg3.shinessFactor = Materials.SHINESS_SPIDER;

		mid_leg3.setBufferID(bufferController.getBufferID());
		mid_leg3.isSelectable = true;
		mid_leg3.selectionID = selectionID;

		transform.setIdentity();
		float[] mid_t3 = { 0.f, 0.8f, 0.7f, 1 };
		float[] temp3 = { -0.45f, 0.8f, 0.5f, 1 };

		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 170);
		transform.translate(mid_t3);
		transform.translate(temp3);

		mid_leg3.updateTransformationState(transform);
		Node.addChildrenToNode(node_rat, mid_leg3);

		SolidDeformedCylinder mid_leg4 = new SolidDeformedCylinder(0.08f,
				0.05f, 2f, 6, 0.1f, 1);

		mid_leg4.setTextureID("spider");

		mid_leg4.setAmbientFactor(Materials.AMBIENT_SPIDER);
		mid_leg4.setDiffuseFactor(Materials.DIFFUSE_SPIDER);
		mid_leg4.setSpecularFactor(Materials.SPECULAR_SPIDER);
		mid_leg4.shinessFactor = Materials.SHINESS_SPIDER;

		mid_leg4.setBufferID(bufferController.getBufferID());
		mid_leg4.isSelectable = true;
		mid_leg4.selectionID = selectionID;

		transform.setIdentity();
		float[] mid_t4 = { 0.f, 0.8f, 0.7f, 1 };
		float[] temp4 = { 0.55f, .9f, -0.4f, 1 };

		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -170);
		transform.translate(mid_t4);
		transform.translate(temp4);

		mid_leg4.updateTransformationState(transform);
		Node.addChildrenToNode(node_rat, mid_leg4);

		SolidDeformedCylinder mid_leg5 = new SolidDeformedCylinder(0.08f,
				0.05f, 2f, 6, 0.1f, 1);

		mid_leg5.setTextureID("spider");

		mid_leg5.setAmbientFactor(Materials.AMBIENT_SPIDER);
		mid_leg5.setDiffuseFactor(Materials.DIFFUSE_SPIDER);
		mid_leg5.setSpecularFactor(Materials.SPECULAR_SPIDER);
		mid_leg5.shinessFactor = Materials.SHINESS_SPIDER;

		mid_leg5.setBufferID(bufferController.getBufferID());
		mid_leg5.isSelectable = true;
		mid_leg5.selectionID = selectionID;

		transform.setIdentity();
		float[] mid_t5 = { 0.f, 0.8f, 0.7f, 1 };
		float[] temp5 = { 0.7f, 0.8f, -0f, 1 };

		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -170);
		transform.translate(mid_t5);
		transform.translate(temp5);

		mid_leg5.updateTransformationState(transform);
		Node.addChildrenToNode(node_rat, mid_leg5);

		SolidDeformedCylinder mid_leg6 = new SolidDeformedCylinder(0.08f,
				0.05f, 2f, 6, 0.1f, 1);

		mid_leg6.setTextureID("spider");

		mid_leg6.setAmbientFactor(Materials.AMBIENT_SPIDER);
		mid_leg6.setDiffuseFactor(Materials.DIFFUSE_SPIDER);
		mid_leg6.setSpecularFactor(Materials.SPECULAR_SPIDER);
		mid_leg6.shinessFactor = Materials.SHINESS_SPIDER;

		mid_leg6.setBufferID(bufferController.getBufferID());
		mid_leg6.isSelectable = true;
		mid_leg6.selectionID = selectionID;

		transform.setIdentity();
		float[] mid_t6 = { 0.f, 0.8f, 0.7f, 1 };
		float[] temp6 = { 0.45f, .8f, 0.5f, 1 };

		transform.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -170);
		transform.translate(mid_t6);
		transform.translate(temp6);

		mid_leg6.updateTransformationState(transform);
		Node.addChildrenToNode(node_rat, mid_leg6);

		playGroundElement.creatureInfo = GeometryFactory.BANE_SPIDER;

		Creature creature = CreatureFactory
				.getCreature(playGroundElement.creatureInfo);
		playGroundElement.wounds = creature.wounds;

		GameflowController.getInstance().monsterList.addElement(node_rat);

		return node_rat;
	}

	private final Node createWorrior(PlayGroundElement playGroundElement) {
		BufferController bufferController = BufferController.getInstance();
		Node node_worrior = new Node();

		node_worrior.gameflowControllerTableID = Integer
				.toString(GeometryFactory.DWARF_WORRIOR);
		node_worrior.playGroundelement = playGroundElement;
		node_worrior.selectionID = GeometryFactory.DWARF_WORRIOR;

		EllipsoidArc legs = new EllipsoidArc(0.25f, .8f, .25f,
				(float) -Math.PI / 2, (float) Math.PI / 2, 0, (float) Math.PI,
				20, 20);
		legs.isSelectable = true;
		legs.selectionID = GeometryFactory.DWARF_WORRIOR;
		legs.setTextureID("chainmail");

		legs.setAmbientFactor(Materials.AMBIENT_CHAINMAIL_LEGS);
		legs.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL_LEGS);
		legs.setSpecularFactor(Materials.SPECULAR_CHAINMAIL_LEGS);
		legs.shinessFactor = Materials.SHINESS_CHAINMAIL_LEGS;

		legs.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(node_worrior, legs);

		EllipsoidArc chest = new EllipsoidArc(0.2f, 1.0f, .2f,
				(float) -Math.PI / 2, (float) Math.PI / 2, -(float) Math.PI, 0,
				20, 20);
		chest.isSelectable = true;
		chest.selectionID = GeometryFactory.DWARF_WORRIOR;
		chest.setTextureID("chainmail");

		chest.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		chest.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		chest.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		chest.shinessFactor = Materials.SHINESS_CHAINMAIL;
		chest.setBufferID(bufferController.getBufferID());

		Transformation transformation = new Transformation();
		transformation.setIdentity();
		float[] t1 = { 0, 1.2f, 0, 1 };

		transformation.translate(t1);
		chest.updateTransformationState(transformation);

		Node.addChildrenToNode(node_worrior, chest);

		Node leftShoulderNode = new Node();

		EllipsoidArc leftShoulder = new EllipsoidArc(0.25f, .45f, .25f,
				(float) -Math.PI / 2, 0, 0, (float) Math.PI, 20, 20);

		leftShoulder.isSelectable = true;
		leftShoulder.selectionID = GeometryFactory.DWARF_WORRIOR;
		leftShoulder.setTextureID("chainmail");

		leftShoulder.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		leftShoulder.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		leftShoulder.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		leftShoulder.shinessFactor = Materials.SHINESS_CHAINMAIL;

		leftShoulder.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t3 = { 0.1f, 1.0f, 0, 1 };

		transformation.translate(t3);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, -90);
		leftShoulder.updateTransformationState(transformation);

		Node.addChildrenToNode(leftShoulderNode, leftShoulder);

		SolidDeformedCylinder leftArm = new SolidDeformedCylinder(0.1f, 0.15f,
				.7f, 10, 0.1f, 0.5f);
		leftArm.isSelectable = true;
		leftArm.selectionID = GeometryFactory.DWARF_WORRIOR;
		leftArm.setTextureID("chainmail");

		leftArm.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		leftArm.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		leftArm.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		leftArm.shinessFactor = Materials.SHINESS_CHAINMAIL;

		leftArm.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t6 = { 0.3f, 0.8f, 0, 1 };

		transformation.translate(t6);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 30);
		leftArm.updateTransformationState(transformation);

		Node.addChildrenToNode(leftShoulderNode, leftArm);

		SolidDeformedCylinder leftHand = new SolidDeformedCylinder(0.07f,
				0.08f, .35f, 10, 0.1f, 0.5f);
		leftHand.isSelectable = true;
		leftHand.selectionID = GeometryFactory.DWARF_WORRIOR;
		leftHand.setTextureID("chainmail");

		leftHand.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		leftHand.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		leftHand.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		leftHand.shinessFactor = Materials.SHINESS_CHAINMAIL;
		leftHand.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t12 = { 0.25f, 0.7f, -0.3f, 1 };

		transformation.translate(t12);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 90);
		leftHand.updateTransformationState(transformation);

		Node.addChildrenToNode(leftShoulderNode, leftHand);
		Node.addChildrenToNode(node_worrior, leftShoulderNode);

		Node rightShoulderNode = new Node();
		EllipsoidArc rightShoulder = new EllipsoidArc(0.25f, .45f, .25f,
				(float) -Math.PI / 2, 0, 0, (float) Math.PI, 20, 20);

		rightShoulder.isSelectable = true;
		rightShoulder.selectionID = GeometryFactory.DWARF_WORRIOR;
		rightShoulder.setTextureID("chainmail");

		rightShoulder.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		rightShoulder.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		rightShoulder.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		rightShoulder.shinessFactor = Materials.SHINESS_CHAINMAIL;

		rightShoulder.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t4 = { -0.10f, 1f, 0, 1 };

		transformation.translate(t4);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		rightShoulder.updateTransformationState(transformation);

		Node.addChildrenToNode(rightShoulderNode, rightShoulder);

		SolidDeformedCylinder rightArm = new SolidDeformedCylinder(0.1f, 0.15f,
				.7f, 10, 0.1f, 0.5f);
		rightArm.isSelectable = true;
		rightArm.selectionID = GeometryFactory.DWARF_WORRIOR;
		rightArm.setTextureID("chainmail");

		rightArm.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		rightArm.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		rightArm.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		rightArm.shinessFactor = Materials.SHINESS_CHAINMAIL;

		rightArm.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t8 = { -0.3f, 0.8f, 0, 1 };

		transformation.translate(t8);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -30);
		rightArm.updateTransformationState(transformation);

		Node.addChildrenToNode(rightShoulderNode, rightArm);

		SolidDeformedCylinder rightHand = new SolidDeformedCylinder(0.07f,
				0.08f, .35f, 10, 0.1f, 0.5f);
		rightHand.isSelectable = true;
		rightHand.selectionID = GeometryFactory.DWARF_WORRIOR;
		rightHand.setTextureID("chainmail");

		rightHand.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		rightHand.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		rightHand.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		rightHand.shinessFactor = Materials.SHINESS_CHAINMAIL;

		rightHand.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t13 = { -0.25f, 0.7f, -0.3f, 1 };

		transformation.translate(t13);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 90);
		rightHand.updateTransformationState(transformation);

		Node.addChildrenToNode(leftShoulderNode, rightHand);
		Node.addChildrenToNode(node_worrior, rightShoulderNode);

		EllipsoidArc beard = new EllipsoidArc(0.15f, .6f, .05f,
				(float) -Math.PI / 2, (float) Math.PI / 2, -(float) Math.PI, 0,
				20, 20);

		beard.isSelectable = true;
		beard.selectionID = GeometryFactory.DWARF_WORRIOR;
		beard.setTextureID("beard");

		beard.setAmbientFactor(Materials.AMBIENT_BEARD_WOORIOR);
		beard.setDiffuseFactor(Materials.DIFFUSE_BEARD_WOORIOR);
		beard.setSpecularFactor(Materials.SPECULAR_BEARD_WOORIOR);
		beard.shinessFactor = Materials.SHINESS_BEARD_WOORIOR;

		beard.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t9 = { 0.f, 1.4f, -0.1f, 1 };

		transformation.translate(t9);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 30);
		beard.updateTransformationState(transformation);

		Node.addChildrenToNode(node_worrior, beard);

		EllipsoidArc helmet = new EllipsoidArc(0.15f, 0.3f, 0.15f,
				(float) -(Math.PI ) / 2, (float) (Math.PI) / 2, (float) 0,
				(float) (Math.PI), 20, 20);

		helmet.isSelectable = true;
		helmet.selectionID = GeometryFactory.DWARF_WORRIOR;
		helmet.setTextureID("chainmail");

		helmet.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		helmet.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		helmet.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		helmet.shinessFactor = Materials.SHINESS_CHAINMAIL;

		helmet.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t10 = { 0f, 1.5f, 0f, 1 };

		transformation.translate(t10);
		helmet.updateTransformationState(transformation);

		Node.addChildrenToNode(node_worrior, helmet);

		SolidDeformedCylinder horn1 = new SolidDeformedCylinder(0.08f, 0.02f,
				0.5f, 10, 0.1f, 1f);
		horn1.isSelectable = true;
		horn1.selectionID = GeometryFactory.DWARF_WORRIOR;
		horn1.setTextureID("spear");

		horn1.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		horn1.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		horn1.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		horn1.shinessFactor = Materials.SHINESS_CHAINMAIL;

		horn1.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t7 = { 0.0f, 1.5f, 0f, 1 };

		transformation.translate(t7);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -45);
		horn1.updateTransformationState(transformation);

		Node.addChildrenToNode(node_worrior, horn1);

		SolidDeformedCylinder horn2 = new SolidDeformedCylinder(0.08f, 0.02f,
				0.5f, 10, 0.1f, 1f);
		horn2.isSelectable = true;
		horn2.selectionID = GeometryFactory.DWARF_WORRIOR;
		horn2.setTextureID("spear");

		horn2.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		horn2.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		horn2.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		horn2.shinessFactor = Materials.SHINESS_CHAINMAIL;

		horn2.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t16 = { -0f, 1.5f, 0f, 1 };

		transformation.translate(t16);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 45);
		horn2.updateTransformationState(transformation);

		Node.addChildrenToNode(node_worrior, horn2);

		SolidDeformedCylinder axe = new SolidDeformedCylinder(0.07f, 0.08f, 2f,
				10, 0.1f, 1f);
		axe.isSelectable = true;
		axe.selectionID = GeometryFactory.DWARF_WORRIOR;
		axe.setTextureID("spear");

		axe.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		axe.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		axe.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		axe.shinessFactor = Materials.SHINESS_CHAINMAIL;

		axe.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t14 = { 0.25f, 0.f, -0.2f, 1 };

		transformation.translate(t14);
		axe.updateTransformationState(transformation);

		Node.addChildrenToNode(node_worrior, axe);

		Arc axeShaft = new Arc(0f, (float) Math.PI, 0.1f, 0.20f, 0.3f, 10, 1f,
				0.1f);
		axeShaft.isSelectable = true;
		axeShaft.selectionID = GeometryFactory.DWARF_WORRIOR;
		axeShaft.setTextureID("spear");

		axeShaft.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		axeShaft.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		axeShaft.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		axeShaft.shinessFactor = Materials.SHINESS_CHAINMAIL;

		axeShaft.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t28 = { 0.27f, 1.5f, -0.26f, 1 };

		transformation.translate(t28);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -90);
		axeShaft.updateTransformationState(transformation);

		Node.addChildrenToNode(node_worrior, axeShaft);

		Cube shield = new Cube(0.4f, 1.5f, 0.1f, 1f, 1f);
		shield.isSelectable = true;
		shield.selectionID = GeometryFactory.DWARF_WORRIOR;
		shield.setTextureID("shield");

		shield.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		shield.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		shield.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		shield.shinessFactor = Materials.SHINESS_CHAINMAIL;

		shield.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t21 = { -0.45f, 0.f, 0.1f, 1 };

		transformation.translate(t21);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		shield.updateTransformationState(transformation);

		Node.addChildrenToNode(node_worrior, shield);

		SolidDeformedCylinder horn3 = new SolidDeformedCylinder(0.08f, 0.02f,
				0.35f, 10, 0.1f, 1f);
		horn3.isSelectable = true;
		horn3.selectionID = GeometryFactory.DWARF_WORRIOR;
		horn3.setTextureID("spear");

		horn3.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		horn3.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		horn3.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		horn3.shinessFactor = Materials.SHINESS_CHAINMAIL;

		horn3.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t20 = { -0.35f, 0.7f, -0.1f, 1 };

		transformation.translate(t20);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 90);
		horn3.updateTransformationState(transformation);

		Node.addChildrenToNode(node_worrior, horn3);

		SelectionController.getInstance().addHero(
				GeometryFactory.DWARF_WORRIOR, node_worrior);
		playGroundElement.creatureInfo = GeometryFactory.DWARF_WORRIOR;

		Creature creature = CreatureFactory
				.getCreature(playGroundElement.creatureInfo);
		playGroundElement.wounds = creature.wounds;

		GameflowController.getInstance().heroTable.put(
				node_worrior.gameflowControllerTableID, node_worrior);
		GameflowController.getInstance().heroList.add(node_worrior);

		playGroundElement.playGroundInfo = new int[1][1];
		playGroundElement.playGroundInfo[0][0] = PlayGround.HERO;
		playGroundElement.element = node_worrior;

		PlayGround.getInstance().addNewElement(playGroundElement);

		return node_worrior;
	}

	private final Node createDwarf(PlayGroundElement playGroundElement) {
		BufferController bufferController = BufferController.getInstance();

		Node node_dwarf = new Node();

		node_dwarf.gameflowControllerTableID = Integer
				.toString(GeometryFactory.DWARF_RANGED);
		node_dwarf.playGroundelement = playGroundElement;
		node_dwarf.selectionID = GeometryFactory.DWARF_RANGED;

		SolidDeformedCylinder legs = new SolidDeformedCylinder(0.25f, 0.2f,
				.6f, 10, 0.1f, 1f);
		legs.isSelectable = true;
		legs.selectionID = GeometryFactory.DWARF_RANGED;
		legs.setTextureID("archer_armor");

		legs.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		legs.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		legs.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		legs.shinessFactor = Materials.SHINESS_CHAINMAIL;

		legs.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(node_dwarf, legs);

		SolidDeformedCylinder chest = new SolidDeformedCylinder(0.2f, 0.25f,
				0.6f, 10, 0.1f, 1f);
		chest.isSelectable = true;
		chest.selectionID = GeometryFactory.DWARF_RANGED;
		chest.setTextureID("archer_armor");

		chest.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		chest.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		chest.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		chest.shinessFactor = Materials.SHINESS_CHAINMAIL;

		chest.setBufferID(bufferController.getBufferID());

		Transformation transformation = new Transformation();
		transformation.setIdentity();
		float[] t1 = { 0, 0.6f, 0, 1 };

		transformation.translate(t1);
		chest.updateTransformationState(transformation);

		Node.addChildrenToNode(node_dwarf, chest);

		Node leftShoulderNode = new Node();

		DeformedConeArc leftShoulder = new DeformedConeArc(0.25f, 0.25f, 0.4f,
				(float) Math.PI, 10, 0.1f, 1f);

		leftShoulder.isSelectable = true;
		leftShoulder.selectionID = GeometryFactory.DWARF_RANGED;
		leftShoulder.setTextureID("leather_poket");

		leftShoulder.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		leftShoulder.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		leftShoulder.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		leftShoulder.shinessFactor = Materials.SHINESS_CHAINMAIL;

		leftShoulder.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t3 = { 0.f, 1.2f, 0, 1 };

		transformation.translate(t3);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -60);
		leftShoulder.updateTransformationState(transformation);

		Node.addChildrenToNode(leftShoulderNode, leftShoulder);

		SolidDeformedCylinder leftArm = new SolidDeformedCylinder(0.1f, 0.15f,
				.7f, 10, 0.1f, 0.5f);
		leftArm.isSelectable = true;
		leftArm.selectionID = GeometryFactory.DWARF_RANGED;
		leftArm.setTextureID("chainmail");

		leftArm.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		leftArm.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		leftArm.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		leftArm.shinessFactor = Materials.SHINESS_CHAINMAIL;

		leftArm.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t6 = { 0.3f, 0.8f, 0, 1 };

		transformation.translate(t6);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 30);
		leftArm.updateTransformationState(transformation);

		Node.addChildrenToNode(leftShoulderNode, leftArm);

		SolidDeformedCylinder leftHand = new SolidDeformedCylinder(0.07f,
				0.08f, .35f, 10, 0.1f, 0.5f);
		leftHand.isSelectable = true;
		leftHand.selectionID = GeometryFactory.DWARF_RANGED;
		leftHand.setTextureID("chainmail");

		leftHand.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		leftHand.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		leftHand.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		leftHand.shinessFactor = Materials.SHINESS_CHAINMAIL;

		leftHand.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t12 = { 0.25f, 0.7f, -0.3f, 1 };

		transformation.translate(t12);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 90);
		leftHand.updateTransformationState(transformation);

		Node.addChildrenToNode(leftShoulderNode, leftHand);
		Node.addChildrenToNode(node_dwarf, leftShoulderNode);

		Node rightShoulderNode = new Node();
		DeformedConeArc rightShoulder = new DeformedConeArc(0.25f, 0.25f, 0.4f,
				(float) Math.PI, 10, 0.1f, 1f);

		rightShoulder.isSelectable = true;
		rightShoulder.selectionID = GeometryFactory.DWARF_RANGED;
		rightShoulder.setTextureID("leather_poket");

		rightShoulder.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		rightShoulder.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		rightShoulder.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		rightShoulder.shinessFactor = Materials.SHINESS_CHAINMAIL;

		rightShoulder.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t4 = { -0.20f, 1f, 0, 1 };

		transformation.translate(t4);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 60);
		rightShoulder.updateTransformationState(transformation);

		Node.addChildrenToNode(rightShoulderNode, rightShoulder);

		SolidDeformedCylinder rightArm = new SolidDeformedCylinder(0.1f, 0.15f,
				.7f, 10, 0.1f, 0.5f);
		rightArm.isSelectable = true;
		rightArm.selectionID = GeometryFactory.DWARF_RANGED;
		rightArm.setTextureID("chainmail");

		rightArm.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		rightArm.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		rightArm.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		rightArm.shinessFactor = Materials.SHINESS_CHAINMAIL;

		rightArm.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t8 = { -0.3f, 0.8f, 0, 1 };

		transformation.translate(t8);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -30);
		rightArm.updateTransformationState(transformation);

		Node.addChildrenToNode(rightShoulderNode, rightArm);

		SolidDeformedCylinder rightHand = new SolidDeformedCylinder(0.07f,
				0.08f, .35f, 10, 0.1f, 0.5f);
		rightHand.isSelectable = true;
		rightHand.selectionID = GeometryFactory.DWARF_RANGED;
		rightHand.setTextureID("chainmail");

		rightHand.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		rightHand.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		rightHand.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		rightHand.shinessFactor = Materials.SHINESS_CHAINMAIL;

		rightHand.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t13 = { -0.25f, 0.7f, -0.3f, 1 };

		transformation.translate(t13);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 90);
		rightHand.updateTransformationState(transformation);

		Node.addChildrenToNode(leftShoulderNode, rightHand);
		Node.addChildrenToNode(node_dwarf, rightShoulderNode);

		DeformedConeArc beard = new DeformedConeArc(0.1f, 0.25f, 0.6f,
				(float) Math.PI, 10, 1f, 3f);

		beard.isSelectable = true;
		beard.selectionID = GeometryFactory.DWARF_RANGED;
		beard.setTextureID("beard");

		beard.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		beard.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		beard.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		beard.shinessFactor = Materials.SHINESS_CHAINMAIL;

		beard.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t9 = { 0.f, 0.8f, -0.3f, 1 };

		transformation.translate(t9);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, -60);
		beard.updateTransformationState(transformation);

		Node.addChildrenToNode(node_dwarf, beard);

		EllipsoidArc helmet = new EllipsoidArc(0.15f, 0.3f, 0.15f,
				(float) -(Math.PI ) / 2, (float) (Math.PI) / 2, (float) 0,
				(float) (Math.PI), 20, 20);

		helmet.isSelectable = true;
		helmet.selectionID = GeometryFactory.DWARF_WORRIOR;
		helmet.setTextureID("chainmail");

		helmet.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		helmet.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		helmet.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		helmet.shinessFactor = Materials.SHINESS_CHAINMAIL;

		helmet.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t10 = { 0f, 1.5f, 0f, 1 };

		transformation.translate(t10);
		helmet.updateTransformationState(transformation);

		Node.addChildrenToNode(node_dwarf, helmet);

		SolidDeformedCylinder axe = new SolidDeformedCylinder(0.07f, 0.08f, 1f,
				10, 0.1f, 1f);
		axe.isSelectable = true;
		axe.selectionID = GeometryFactory.DWARF_RANGED;
		axe.setTextureID("spear");

		axe.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		axe.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		axe.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		axe.shinessFactor = Materials.SHINESS_CHAINMAIL;

		axe.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t14 = { 0.4f, 1.f, -0.8f, 1 };

		transformation.translate(t14);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 90);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, -45);
		axe.updateTransformationState(transformation);

		Node.addChildrenToNode(node_dwarf, axe);

		Arc axeShaft = new Arc(0f, (float) Math.PI, 0.1f, 0.10f, 0.5f, 20, 1f,
				0.1f);
		axeShaft.isSelectable = true;
		axeShaft.selectionID = GeometryFactory.DWARF_RANGED;
		axeShaft.setTextureID("spear");

		axeShaft.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		axeShaft.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		axeShaft.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		axeShaft.shinessFactor = Materials.SHINESS_CHAINMAIL;

		axeShaft.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t28 = { 0.f, 1.f, -0.46f, 1 };

		transformation.translate(t28);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, -90);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, -45);
		axeShaft.updateTransformationState(transformation);

		Node.addChildrenToNode(node_dwarf, axeShaft);

		Cube shield = new Cube(0.4f, 1.5f, 0.1f, 1f, 1f);
		shield.isSelectable = true;
		shield.selectionID = GeometryFactory.DWARF_RANGED;
		shield.setTextureID("shield");

		shield.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		shield.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		shield.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		shield.shinessFactor = Materials.SHINESS_CHAINMAIL;

		shield.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t21 = { -0.45f, 0.2f, 0.1f, 1 };

		transformation.translate(t21);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		shield.updateTransformationState(transformation);

		// Node.addChildrenToNode(node_dwarf, shield);

		SelectionController.getInstance().addHero(node_dwarf.selectionID,
				node_dwarf);
		playGroundElement.creatureInfo = GeometryFactory.DWARF_RANGED;

		Creature creature = CreatureFactory
				.getCreature(playGroundElement.creatureInfo);
		playGroundElement.wounds = creature.wounds;

		GameflowController.getInstance().heroTable.put(
				node_dwarf.gameflowControllerTableID, node_dwarf);
		GameflowController.getInstance().heroList.add(node_dwarf);

		playGroundElement.playGroundInfo = new int[1][1];
		playGroundElement.playGroundInfo[0][0] = PlayGround.HERO;
		playGroundElement.element = node_dwarf;

		PlayGround.getInstance().addNewElement(playGroundElement);

		return node_dwarf;
	}

	private final Node createMagician(PlayGroundElement playGroundElement) {
		BufferController bufferController = BufferController.getInstance();
		Node node_magician = new Node();

		node_magician.gameflowControllerTableID = Integer
				.toString(GeometryFactory.DWARF_GEODE);
		node_magician.playGroundelement = playGroundElement;
		node_magician.selectionID = GeometryFactory.DWARF_GEODE;

		SolidDeformedCylinder legs = new SolidDeformedCylinder(0.25f, 0.2f,
				.6f, 10, 0.1f, 1f);
		legs.isSelectable = true;
		legs.selectionID = GeometryFactory.DWARF_GEODE;
		legs.setTextureID("lava");

		legs.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		legs.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		legs.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		legs.shinessFactor = Materials.SHINESS_CHAINMAIL;

		legs.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(node_magician, legs);

		SolidDeformedCylinder chest = new SolidDeformedCylinder(0.2f, 0.25f,
				0.6f, 10, 0.1f, 1f);
		chest.isSelectable = true;
		chest.selectionID = GeometryFactory.DWARF_GEODE;
		chest.setTextureID("lava");

		chest.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		chest.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		chest.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		chest.shinessFactor = Materials.SHINESS_CHAINMAIL;

		chest.setBufferID(bufferController.getBufferID());

		Transformation transformation = new Transformation();
		transformation.setIdentity();
		float[] t1 = { 0, 0.6f, 0, 1 };

		transformation.translate(t1);
		chest.updateTransformationState(transformation);

		Node.addChildrenToNode(node_magician, chest);

		DeformedConeArc pela = new DeformedConeArc(0.25f, 0.4f, 1.2f,
				(float) Math.PI, 10, 0.1f, 1f);

		pela.isSelectable = true;
		pela.selectionID = GeometryFactory.DWARF_GEODE;
		pela.setTextureID("lava");

		pela.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		pela.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		pela.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		pela.shinessFactor = Materials.SHINESS_CHAINMAIL;

		pela.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t31 = { 0, 1.2f, 0.1f, 1 };

		transformation.translate(t31);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 90);
		pela.updateTransformationState(transformation);

		Node.addChildrenToNode(node_magician, pela);

		Node leftShoulderNode = new Node();

		DeformedConeArc leftShoulder = new DeformedConeArc(0.25f, 0.25f, 0.4f,
				(float) Math.PI, 10, 0.1f, 1f);

		leftShoulder.isSelectable = true;
		leftShoulder.selectionID = GeometryFactory.DWARF_GEODE;
		leftShoulder.setTextureID("lava");

		leftShoulder.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		leftShoulder.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		leftShoulder.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		leftShoulder.shinessFactor = Materials.SHINESS_CHAINMAIL;

		leftShoulder.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t3 = { 0.f, 1.2f, 0, 1 };

		transformation.translate(t3);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -60);
		leftShoulder.updateTransformationState(transformation);

		Node.addChildrenToNode(leftShoulderNode, leftShoulder);

		SolidDeformedCylinder leftArm = new SolidDeformedCylinder(0.1f, 0.15f,
				.7f, 10, 0.1f, 0.5f);
		leftArm.isSelectable = true;
		leftArm.selectionID = GeometryFactory.DWARF_GEODE;
		leftArm.setTextureID("chainmail");

		leftArm.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		leftArm.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		leftArm.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		leftArm.shinessFactor = Materials.SHINESS_CHAINMAIL;

		leftArm.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t6 = { 0.3f, 0.8f, 0, 1 };

		transformation.translate(t6);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 30);
		leftArm.updateTransformationState(transformation);

		Node.addChildrenToNode(leftShoulderNode, leftArm);

		SolidDeformedCylinder leftHand = new SolidDeformedCylinder(0.07f,
				0.08f, .35f, 10, 0.1f, 0.5f);
		leftHand.isSelectable = true;
		leftHand.selectionID = GeometryFactory.DWARF_GEODE;
		leftHand.setTextureID("chainmail");

		leftHand.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		leftHand.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		leftHand.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		leftHand.shinessFactor = Materials.SHINESS_CHAINMAIL;

		leftHand.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t12 = { 0.25f, 0.7f, -0.3f, 1 };

		transformation.translate(t12);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 90);
		leftHand.updateTransformationState(transformation);

		Node.addChildrenToNode(leftShoulderNode, leftHand);
		Node.addChildrenToNode(node_magician, leftShoulderNode);

		Node rightShoulderNode = new Node();
		DeformedConeArc rightShoulder = new DeformedConeArc(0.25f, 0.25f, 0.4f,
				(float) Math.PI, 10, 0.1f, 1f);

		rightShoulder.isSelectable = true;
		rightShoulder.selectionID = GeometryFactory.DWARF_GEODE;
		rightShoulder.setTextureID("lava");

		rightShoulder.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		rightShoulder.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		rightShoulder.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		rightShoulder.shinessFactor = Materials.SHINESS_CHAINMAIL;

		rightShoulder.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t4 = { -0.20f, 1f, 0, 1 };

		transformation.translate(t4);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 60);
		rightShoulder.updateTransformationState(transformation);

		Node.addChildrenToNode(rightShoulderNode, rightShoulder);

		SolidDeformedCylinder rightArm = new SolidDeformedCylinder(0.1f, 0.15f,
				.7f, 10, 0.1f, 0.5f);
		rightArm.isSelectable = true;
		rightArm.selectionID = GeometryFactory.DWARF_GEODE;
		rightArm.setTextureID("chainmail");

		rightArm.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		rightArm.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		rightArm.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		rightArm.shinessFactor = Materials.SHINESS_CHAINMAIL;

		rightArm.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t8 = { -0.3f, 0.8f, 0, 1 };

		transformation.translate(t8);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -30);
		rightArm.updateTransformationState(transformation);

		Node.addChildrenToNode(rightShoulderNode, rightArm);

		SolidDeformedCylinder rightHand = new SolidDeformedCylinder(0.07f,
				0.08f, .35f, 10, 0.1f, 0.5f);
		rightHand.isSelectable = true;
		rightHand.selectionID = GeometryFactory.DWARF_GEODE;
		rightHand.setTextureID("chainmail");

		rightHand.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		rightHand.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		rightHand.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		rightHand.shinessFactor = Materials.SHINESS_CHAINMAIL;

		rightHand.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t13 = { -0.25f, 0.7f, -0.3f, 1 };

		transformation.translate(t13);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 90);
		rightHand.updateTransformationState(transformation);

		Node.addChildrenToNode(leftShoulderNode, rightHand);
		Node.addChildrenToNode(node_magician, rightShoulderNode);

		DeformedConeArc beard = new DeformedConeArc(0.1f, 0.25f, 0.6f,
				(float) Math.PI, 10, 1f, 3f);

		beard.isSelectable = true;
		beard.selectionID = GeometryFactory.DWARF_GEODE;
		beard.setTextureID("beard");

		beard.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		beard.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		beard.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		beard.shinessFactor = Materials.SHINESS_CHAINMAIL;

		beard.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t9 = { 0.f, 0.8f, -0.3f, 1 };

		transformation.translate(t9);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, -60);
		beard.updateTransformationState(transformation);

		Node.addChildrenToNode(node_magician, beard);

		EllipsoidArc helmet = new EllipsoidArc(0.3f, 0.15f, 0.15f,
				(float) -(Math.PI ), (float) (Math.PI), (float) -(Math.PI / 2),
				(float) (Math.PI / 2), 10, 10);

		helmet.isSelectable = true;
		helmet.selectionID = GeometryFactory.DWARF_GEODE;
		helmet.setTextureID("chainmail");

		helmet.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		helmet.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		helmet.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		helmet.shinessFactor = Materials.SHINESS_CHAINMAIL;

		helmet.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t10 = { 0f, 1.5f, 0f, 1 };

		transformation.translate(t10);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 90);
		helmet.updateTransformationState(transformation);

		Node.addChildrenToNode(node_magician, helmet);

		SolidDeformedCylinder axe = new SolidDeformedCylinder(0.07f, 0.08f, 2f,
				10, 0.1f, 1f);
		axe.isSelectable = true;
		axe.selectionID = GeometryFactory.DWARF_GEODE;
		axe.setTextureID("spear");

		axe.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		axe.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		axe.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		axe.shinessFactor = Materials.SHINESS_CHAINMAIL;

		axe.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t14 = { 0.25f, 0.f, -0.2f, 1 };

		transformation.translate(t14);
		axe.updateTransformationState(transformation);

		Node.addChildrenToNode(node_magician, axe);

		Ellipsoid axeShaft = new Ellipsoid(0.2f, 0.2f, 0.2f, 20, 20);
		axeShaft.isSelectable = true;
		axeShaft.selectionID = GeometryFactory.DWARF_GEODE;
		axeShaft.setTextureID("spear");

		axeShaft.setAmbientFactor(Materials.AMBIENT_CHAINMAIL);
		axeShaft.setDiffuseFactor(Materials.DIFFUSE_CHAINMAIL);
		axeShaft.setSpecularFactor(Materials.SPECULAR_CHAINMAIL);
		axeShaft.shinessFactor = Materials.SHINESS_CHAINMAIL;

		axeShaft.setBufferID(bufferController.getBufferID());

		transformation.setIdentity();
		float[] t28 = { 0.25f, 2.f, -0.2f, 1 };

		transformation.translate(t28);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, -90);
		transformation.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, -45);
		axeShaft.updateTransformationState(transformation);

		Node.addChildrenToNode(node_magician, axeShaft);

		SelectionController.getInstance().addHero(GeometryFactory.DWARF_GEODE,
				node_magician);
		playGroundElement.creatureInfo = GeometryFactory.DWARF_GEODE;

		Creature creature = CreatureFactory
				.getCreature(playGroundElement.creatureInfo);
		playGroundElement.wounds = creature.wounds;

		GameflowController.getInstance().heroTable.put(
				node_magician.gameflowControllerTableID, node_magician);
		GameflowController.getInstance().heroList.add(node_magician);

		playGroundElement.playGroundInfo = new int[1][1];
		playGroundElement.playGroundInfo[0][0] = PlayGround.HERO;
		playGroundElement.element = node_magician;

		PlayGround.getInstance().addNewElement(playGroundElement);

		return node_magician;
	}

	public void createCylinder(Node node) {

		// Icosahedron cube = new Icosahedron(0.1f, 0.1f);
		// SolidDeformedCylinder cube = new SolidDeformedCylinder(3, 2, 4,
		// 20,0.5f,4);
		// SolidDeformedPolyCube cube = new SolidDeformedPolyCube(3, 2, 4,
		// 10,0.5f, 5.0f);
		// Arc cube = new Arc(0, (float) Math.PI, 2, 0.2f, 1, 40, 0.5f, 1.0f);
		// DeformedConeArc cube = new DeformedConeArc(2, 1, 4,(float) (2 *
		// Math.PI), 20, 0.5f, 1);
		// Ellipsoid cube = new Ellipsoid(1, 2, 1, 40, 40);
		// EllipsoidArc cube = new EllipsoidArc(1, 2, 1, (float) -(Math.PI /
		// 4),(float) (Math.PI / 4), (float) -(Math.PI), (float) (Math.PI), 40,
		// 40);
		// SolidDeformedConeArc cube = new SolidDeformedConeArc(2, 1, 4,(float)
		// (Math.PI), 10);
		// Cube cube = new Cube(3, 3, 3, 1, 1);
		// Cylinder cube = new Cylinder(30, 3, 10, 2, 1);
		// SolidPolyCube cube = new SolidPolyCube(5, 3, 10, 2, 1, 1, 1.4f);
		Cube cube = new Cube(1.5f, 3, 1.5f, 1, 1);
		cube.setTextureID("lava");

		Node.addChildrenToNode(node, cube);
	}

	public final void createPlayGround(Node node, int level) {

		if (level == GeometryFactory.LEVEL_1) {
			this.createLevel1(node);
		}
	}

	private final void createUniqueMonsters_1() {
		PlayGroundElement playGroundElement_octopus = new PlayGroundElement(19,
				2, 10, GeometryFactory.OCTOPUS);
		this.renderer.sceneGraph.addNewElement(playGroundElement_octopus);

		/*PlayGroundElement playGroundElement_gate1 = new PlayGroundElement(15,
				2, 15, GeometryFactory.GATE);
		this.renderer.sceneGraph.addNewElement(playGroundElement_gate1);

		PlayGroundElement playGroundElement_gate2 = new PlayGroundElement(25,
				2, 15, GeometryFactory.GATE);
		this.renderer.sceneGraph.addNewElement(playGroundElement_gate2);*/
	}

	private final void initializePlayGround_1() {
		PlayGround.getInstance().initialPlayGround_1();
	}

	private final void createStructure_level_1(Node node) {
		BufferController bufferController = BufferController.getInstance();
		Node groundNode = new Node();
		Node.addChildrenToNode(node, groundNode);
		// ---------------------------------------------------------------------
		// wheel
		Node wheel = new Node();

		Cube wheelGround = new Cube(2, 0.5f, 2, 1f, 1f);
		wheelGround.setTextureID("ground");
		wheelGround.setAmbientFactor(Materials.AMBIENT_GROUND);
		wheelGround.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		wheelGround.setSpecularFactor(Materials.SPECULAR_GROUND);
		wheelGround.shinessFactor = Materials.SHINESS_GROUND;

		Transformation transWheel_1 = new Transformation();
		float[] yx1 = { 0, 0, 0, 1 };

		transWheel_1.translate(yx1);

		wheelGround.updateTransformationState(transWheel_1);
		wheelGround.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(wheel, wheelGround);

		SolidDeformedCylinder wheelDecor = new SolidDeformedCylinder(0.7f,
				0.7f, 1.f, 15, 0.5f, 0.1f);
		wheelDecor.setTextureID("brick_wall");
		wheelDecor.setAmbientFactor(Materials.AMBIENT_WALL);
		wheelDecor.setDiffuseFactor(Materials.DIFFUSE_WALL);
		wheelDecor.setSpecularFactor(Materials.SPECULAR_WALL);
		wheelDecor.shinessFactor = Materials.SHINESS_WALL;

		transWheel_1 = new Transformation();
		float[] yx2 = { 1, 0.5f, 1, 1 };

		transWheel_1.translate(yx2);

		wheelDecor.updateTransformationState(transWheel_1);
		wheelDecor.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(wheel, wheelDecor);

		SolidDeformedCylinder wheelSt = new SolidDeformedCylinder(0.55f, 0.55f,
				2.f, 15, 0.4f, 0.2f);
		wheelSt.setTextureID("brick_wall");
		wheelSt.setAmbientFactor(Materials.AMBIENT_WALL);
		wheelSt.setDiffuseFactor(Materials.DIFFUSE_WALL);
		wheelSt.setSpecularFactor(Materials.SPECULAR_WALL);
		wheelSt.shinessFactor = Materials.SHINESS_WALL;

		transWheel_1 = new Transformation();
		float[] yx3 = { 1, 1.5f, 1, 1 };

		transWheel_1.translate(yx3);

		wheelSt.updateTransformationState(transWheel_1);
		wheelSt.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(wheel, wheelSt);

		Arc wheelTop = new Arc(0, (float) Math.PI, 1.f, 0.2f, 0.7f, 15, 0.5f,
				0.2f);
		wheelTop.setTextureID("brick_wall");
		wheelTop.setAmbientFactor(Materials.AMBIENT_WALL);
		wheelTop.setDiffuseFactor(Materials.DIFFUSE_WALL);
		wheelTop.setSpecularFactor(Materials.SPECULAR_WALL);
		wheelTop.shinessFactor = Materials.SHINESS_WALL;

		transWheel_1 = new Transformation();
		float[] yx4 = { 1, 3.8f, 1, 1 };

		transWheel_1.translate(yx4);
		transWheel_1.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 90);

		wheelTop.updateTransformationState(transWheel_1);
		wheelTop.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(wheel, wheelTop);

		wheelTop = new Arc(0, (float) Math.PI, 1.f, 0.2f, 0.7f, 15, 0.5f, 0.2f);
		wheelTop.setTextureID("brick_wall");
		wheelTop.setAmbientFactor(Materials.AMBIENT_WALL);
		wheelTop.setDiffuseFactor(Materials.DIFFUSE_WALL);
		wheelTop.setSpecularFactor(Materials.SPECULAR_WALL);
		wheelTop.shinessFactor = Materials.SHINESS_WALL;

		transWheel_1 = new Transformation();
		float[] yx5 = { 1, 3.8f, 1, 1 };

		transWheel_1.translate(yx5);
		transWheel_1.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, -90);
		transWheel_1.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, 180);

		wheelTop.updateTransformationState(transWheel_1);
		wheelTop.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(wheel, wheelTop);

		Cube wheelCube = new Cube(0.1f, 3.3f, 0.1f, 1f, 1f);
		wheelCube.setTextureID("ground");
		wheelCube.setAmbientFactor(Materials.AMBIENT_MATAL);
		wheelCube.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		wheelCube.setSpecularFactor(Materials.SPECULAR_METAL);
		wheelCube.shinessFactor = Materials.SHINESS_METAL;

		transWheel_1 = new Transformation();
		float[] yx6 = { 0.34f, 3.7f, 1, 1 };

		transWheel_1.translate(yx6);

		wheelCube.updateTransformationState(transWheel_1);
		wheelCube.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(wheel, wheelCube);

		wheelCube = new Cube(0.1f, 3.3f, 0.1f, 1f, 1f);
		wheelCube.setTextureID("ground");
		wheelCube.setAmbientFactor(Materials.AMBIENT_MATAL);
		wheelCube.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		wheelCube.setSpecularFactor(Materials.SPECULAR_METAL);
		wheelCube.shinessFactor = Materials.SHINESS_METAL;

		transWheel_1 = new Transformation();
		float[] yx7 = { 2 - 0.53f, 3.7f, 1, 1 };

		transWheel_1.translate(yx7);

		wheelCube.updateTransformationState(transWheel_1);
		wheelCube.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(wheel, wheelCube);

		wheelCube = new Cube(1.33f, 0.2f, 1.33f, 2f, 1f);
		wheelCube.setTextureID("wood");
		wheelCube.setAmbientFactor(Materials.AMBIENT_WOOD);
		wheelCube.setDiffuseFactor(Materials.DIFFUSE_WOOD);
		wheelCube.setSpecularFactor(Materials.SPECULAR_WOOD);
		wheelCube.shinessFactor = Materials.SHINESS_WOOD;

		transWheel_1 = new Transformation();
		float[] yx8 = { 0.33f, 7.f, 1, 1 };

		transWheel_1.translate(yx8);
		transWheel_1.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 45);

		wheelCube.updateTransformationState(transWheel_1);
		wheelCube.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(wheel, wheelCube);

		wheelCube = new Cube(1.33f, 0.2f, 1.33f, 2f, 1f);
		wheelCube.setTextureID("wood");
		wheelCube.setAmbientFactor(Materials.AMBIENT_WOOD);
		wheelCube.setDiffuseFactor(Materials.DIFFUSE_WOOD);
		wheelCube.setSpecularFactor(Materials.SPECULAR_WOOD);
		wheelCube.shinessFactor = Materials.SHINESS_WOOD;

		transWheel_1 = new Transformation();
		float[] yx9 = { 0.33f, 6.f, 0.2f, 1 };

		transWheel_1.translate(yx9);
		transWheel_1.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, -45);

		wheelCube.updateTransformationState(transWheel_1);
		wheelCube.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(wheel, wheelCube);

		Transformation h = new Transformation();

		float[] d = { 8, 2, 24, 1 };
		h.translate(d);

		wheel.updateTransformationState(h);

		Node.addChildrenToNode(groundNode, wheel);

		// -------------------------------------------
		float horizontalBoxCount = 40;
		float verticalBoxCount = 40;

		float wallStrength = 0.2f;
		float wallHeight = 20;

		float sTextureRangeGround = 5;
		float tTextureRangeGround = 5;

		float sTextureRangeWall = 4;
		float tTextureRangeWall = 2;

		float x = horizontalBoxCount;
		float y = wallStrength;
		float z = verticalBoxCount;

		// --------------------------------------------------------------

		Cube leftGround = new Cube(x, 2, 5, 4f, 0.4f);
		leftGround.setTextureID("ground");
		leftGround.setAmbientFactor(Materials.AMBIENT_GROUND);
		leftGround.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		leftGround.setSpecularFactor(Materials.SPECULAR_GROUND);
		leftGround.shinessFactor = Materials.SHINESS_GROUND;

		leftGround.setBufferID(bufferController.getBufferID());

		Cube decorWall1 = new Cube(30f, 3, 0.2f, 2.5f, 0.4f);

		Transformation trans6 = new Transformation();
		float[] t48 = { 5, 0, 5, 1 };

		trans6.translate(t48);
		decorWall1.updateTransformationState(trans6);
		decorWall1.setTextureID("brick_wall");
		decorWall1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall1.shinessFactor = Materials.SHINESS_WALL;

		decorWall1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall1);

		for (int i = 0; i < 16; i++) {
			Cube decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

			trans6.setIdentity();
			float[] t7 = { 5 + (i * 2.0f) - 0.1f, 3f, 5f - 0.1f, 1 };
			trans6.translate(t7);

			decorStone.updateTransformationState(trans6);
			decorStone.setTextureID("brick_wall");

			decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
			decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
			decorStone.shinessFactor = Materials.SHINESS_WALL;

			decorStone.setBufferID(bufferController.getBufferID());
			Node.addChildrenToNode(groundNode, decorStone);

			Cube decorCube = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

			trans6.setIdentity();
			float[] t4 = { 5 + (i * 2.0f), 0, 5f, 1 };

			trans6.translate(t4);
			decorCube.updateTransformationState(trans6);
			decorCube.setTextureID("brick_wall");

			decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
			decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
			decorCube.shinessFactor = Materials.SHINESS_WALL;

			decorCube.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, decorCube);
		}
		// -----------------------------------------------------------------
		Cube decorStone9 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans6.setIdentity();
		float[] kq7 = { 3 - 0.1f, 3f, 5f - 0.1f, 1 };
		trans6.translate(kq7);

		decorStone9.updateTransformationState(trans6);
		decorStone9.setTextureID("brick_wall");

		decorStone9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone9.shinessFactor = Materials.SHINESS_WALL;

		decorStone9.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone9);

		Cube decorCube9 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

		trans6.setIdentity();
		float[] kq4 = { 3, 0, 5f, 1 };

		trans6.translate(kq4);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] kqq4 = { 3, 3.7f, 5f, 1 };

		trans6.translate(kqq4);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] ksqq4 = { 5, 3.7f, 5f, 1 };

		trans6.translate(ksqq4);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		Arc decorArc13 = new Arc(0, (float) Math.PI, 0.3f, 0.6f, 1.15f, 20,
				0.15f, 0.2f);

		Transformation trans45 = new Transformation();
		trans45.setIdentity();
		float[] eh34 = { 4.15f, 8.7f, 5f, 1 };

		trans45.translate(eh34);
		decorArc13.updateTransformationState(trans45);
		decorArc13.setTextureID("brick_wall");

		decorArc13.setAmbientFactor(Materials.AMBIENT_WALL);
		decorArc13.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorArc13.setSpecularFactor(Materials.SPECULAR_WALL);
		decorArc13.shinessFactor = Materials.SHINESS_WALL;

		decorArc13.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorArc13);

		// -----------------------
		decorStone9 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans6.setIdentity();
		float[] kqq7 = { 3 - 0.1f, 3f, 10f - 0.3f - 0.1f, 1 };
		trans6.translate(kqq7);

		decorStone9.updateTransformationState(trans6);
		decorStone9.setTextureID("brick_wall");

		decorStone9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone9.shinessFactor = Materials.SHINESS_WALL;

		decorStone9.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone9);

		decorCube9 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] kqqq4 = { 3, 0, 10f - 0.3f, 1 };

		trans6.translate(kqqq4);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] aa4 = { 3, 3.7f, 10f - 0.3f, 1 };

		trans6.translate(aa4);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] ksqqq4 = { 5, 3.7f, 10f - 0.3f, 1 };

		trans6.translate(ksqqq4);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorArc13 = new Arc(0, (float) Math.PI, 0.3f, 0.6f, 1.15f, 20, 0.15f,
				0.2f);

		trans45.setIdentity();
		float[] ehe34 = { 4.15f, 8.7f, 10f - 0.3f, 1 };

		trans45.translate(ehe34);
		decorArc13.updateTransformationState(trans45);
		decorArc13.setTextureID("brick_wall");

		decorArc13.setAmbientFactor(Materials.AMBIENT_WALL);
		decorArc13.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorArc13.setSpecularFactor(Materials.SPECULAR_WALL);
		decorArc13.shinessFactor = Materials.SHINESS_WALL;

		decorArc13.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorArc13);

		// -----------------------
		decorStone9 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans6.setIdentity();
		float[] akqq7 = { 3 - 0.1f, 3f, 30f - 0.1f, 1 };
		trans6.translate(akqq7);

		decorStone9.updateTransformationState(trans6);
		decorStone9.setTextureID("brick_wall");

		decorStone9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone9.shinessFactor = Materials.SHINESS_WALL;

		decorStone9.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone9);

		decorCube9 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] aakqqq4 = { 3, 0, 30f, 1 };

		trans6.translate(aakqqq4);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] aaa4 = { 3, 3.7f, 30f, 1 };

		trans6.translate(aaa4);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] aksqqq4 = { 5, 3.7f, 30f, 1 };

		trans6.translate(aksqqq4);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorArc13 = new Arc(0, (float) Math.PI, 0.3f, 0.6f, 1.15f, 20, 0.15f,
				0.2f);

		trans45.setIdentity();
		float[] aehe34 = { 4.15f, 8.7f, 30f, 1 };

		trans45.translate(aehe34);
		decorArc13.updateTransformationState(trans45);
		decorArc13.setTextureID("brick_wall");

		decorArc13.setAmbientFactor(Materials.AMBIENT_WALL);
		decorArc13.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorArc13.setSpecularFactor(Materials.SPECULAR_WALL);
		decorArc13.shinessFactor = Materials.SHINESS_WALL;

		decorArc13.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorArc13);

		// -----------------------
		decorStone9 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans6.setIdentity();
		float[] akqqe7 = { 3 - 0.1f, 3f, 34.6f - 0.1f, 1 };
		trans6.translate(akqqe7);

		decorStone9.updateTransformationState(trans6);
		decorStone9.setTextureID("brick_wall");

		decorStone9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone9.shinessFactor = Materials.SHINESS_WALL;

		decorStone9.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone9);

		decorCube9 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] aakqqqe4 = { 3, 0, 34.6f, 1 };

		trans6.translate(aakqqqe4);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] aaa4e = { 3, 3.7f, 34.6f, 1 };

		trans6.translate(aaa4e);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] aksqqq4e = { 5, 3.7f, 34.6f, 1 };

		trans6.translate(aksqqq4e);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorArc13 = new Arc(0, (float) Math.PI, 0.3f, 0.6f, 1.15f, 20, 0.15f,
				0.2f);

		trans45.setIdentity();
		float[] aehe34e = { 4.15f, 8.7f, 34.6f, 1 };

		trans45.translate(aehe34e);
		decorArc13.updateTransformationState(trans45);
		decorArc13.setTextureID("brick_wall");

		decorArc13.setAmbientFactor(Materials.AMBIENT_WALL);
		decorArc13.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorArc13.setSpecularFactor(Materials.SPECULAR_WALL);
		decorArc13.shinessFactor = Materials.SHINESS_WALL;

		decorArc13.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorArc13);
		// -----------------------
		decorStone9 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans6.setIdentity();
		float[] akqqe7w = { 37 - 0.1f, 3f, 34.6f - 0.1f, 1 };
		trans6.translate(akqqe7w);

		decorStone9.updateTransformationState(trans6);
		decorStone9.setTextureID("brick_wall");

		decorStone9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone9.shinessFactor = Materials.SHINESS_WALL;

		decorStone9.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone9);

		decorCube9 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] aakqqqe4w = { 37, 0, 34.6f, 1 };

		trans6.translate(aakqqqe4w);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] aaa4ew = { 37, 3.7f, 34.6f, 1 };

		trans6.translate(aaa4ew);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] aksqqq4ew = { 35, 3.7f, 34.6f, 1 };

		trans6.translate(aksqqq4ew);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorArc13 = new Arc(0, (float) Math.PI, 0.3f, 0.6f, 1.15f, 20, 0.15f,
				0.2f);

		trans45.setIdentity();
		float[] aehe34ew = { 36.15f, 8.7f, 34.6f, 1 };

		trans45.translate(aehe34ew);
		decorArc13.updateTransformationState(trans45);
		decorArc13.setTextureID("brick_wall");

		decorArc13.setAmbientFactor(Materials.AMBIENT_WALL);
		decorArc13.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorArc13.setSpecularFactor(Materials.SPECULAR_WALL);
		decorArc13.shinessFactor = Materials.SHINESS_WALL;

		decorArc13.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorArc13);
		// -----------------------
		decorStone9 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans6.setIdentity();
		float[] sakqqe7w = { 37 - 0.1f - 0.2f, 3f, 30f - 0.1f, 1 };
		trans6.translate(sakqqe7w);

		decorStone9.updateTransformationState(trans6);
		decorStone9.setTextureID("brick_wall");

		decorStone9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone9.shinessFactor = Materials.SHINESS_WALL;

		decorStone9.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone9);

		decorCube9 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] saakqqqe4w = { 37 - 0.2f, 0, 30f, 1 };

		trans6.translate(saakqqqe4w);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] ssaaa4ew = { 37 - 0.2f, 3.7f, 30f, 1 };

		trans6.translate(ssaaa4ew);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] saksqqq4ew = { 35 - 0.2f, 3.7f, 30f, 1 };

		trans6.translate(saksqqq4ew);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorArc13 = new Arc(0, (float) Math.PI, 0.3f, 0.6f, 1.15f, 20, 0.15f,
				0.2f);

		trans45.setIdentity();
		float[] saehe34ew = { 36.15f - 0.2f, 8.7f, 30f, 1 };

		trans45.translate(saehe34ew);
		decorArc13.updateTransformationState(trans45);
		decorArc13.setTextureID("brick_wall");

		decorArc13.setAmbientFactor(Materials.AMBIENT_WALL);
		decorArc13.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorArc13.setSpecularFactor(Materials.SPECULAR_WALL);
		decorArc13.shinessFactor = Materials.SHINESS_WALL;

		decorArc13.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorArc13);

		// -----------------------
		decorStone9 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans6.setIdentity();
		float[] sakqqe7wb = { 37 - 0.1f - 0.2f, 3f, 9.7f - 0.1f, 1 };
		trans6.translate(sakqqe7wb);

		decorStone9.updateTransformationState(trans6);
		decorStone9.setTextureID("brick_wall");

		decorStone9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone9.shinessFactor = Materials.SHINESS_WALL;

		decorStone9.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone9);

		decorCube9 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] saakqqqe4wb = { 37 - 0.2f, 0, 9.7f, 1 };

		trans6.translate(saakqqqe4wb);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] ssaaa4ewb = { 37 - 0.2f, 3.7f, 9.7f, 1 };

		trans6.translate(ssaaa4ewb);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] saksqqq4ewb = { 35 - 0.2f, 3.7f, 9.7f, 1 };

		trans6.translate(saksqqq4ewb);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorArc13 = new Arc(0, (float) Math.PI, 0.3f, 0.6f, 1.15f, 20, 0.15f,
				0.2f);

		trans45.setIdentity();
		float[] saehe34ewb = { 36.15f - 0.2f, 8.7f, 9.7f, 1 };

		trans45.translate(saehe34ewb);
		decorArc13.updateTransformationState(trans45);
		decorArc13.setTextureID("brick_wall");

		decorArc13.setAmbientFactor(Materials.AMBIENT_WALL);
		decorArc13.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorArc13.setSpecularFactor(Materials.SPECULAR_WALL);
		decorArc13.shinessFactor = Materials.SHINESS_WALL;

		decorArc13.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorArc13);

		// -----------------------
		decorStone9 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans6.setIdentity();
		float[] ccsakqqe7wb = { 37 - 0.1f, 3f, 5f - 0.1f, 1 };
		trans6.translate(ccsakqqe7wb);

		decorStone9.updateTransformationState(trans6);
		decorStone9.setTextureID("brick_wall");

		decorStone9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone9.shinessFactor = Materials.SHINESS_WALL;

		decorStone9.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone9);

		decorCube9 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] csaakqqqe4wb = { 37, 0, 5f, 1 };

		trans6.translate(csaakqqqe4wb);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] cssaaa4ewb = { 37, 3.7f, 5f, 1 };

		trans6.translate(cssaaa4ewb);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorCube9 = new Cube(0.3f, 5f, 0.3f, 0.1f, 1.2f);

		trans6.setIdentity();
		float[] csaksqqq4ewb = { 35, 3.7f, 5f, 1 };

		trans6.translate(csaksqqq4ewb);
		decorCube9.updateTransformationState(trans6);
		decorCube9.setTextureID("brick_wall");

		decorCube9.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube9.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube9.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube9.shinessFactor = Materials.SHINESS_WALL;

		decorCube9.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube9);

		decorArc13 = new Arc(0, (float) Math.PI, 0.3f, 0.6f, 1.15f, 20, 0.15f,
				0.2f);

		trans45.setIdentity();
		float[] csaehe34ewb = { 36.15f, 8.7f, 5f, 1 };

		trans45.translate(csaehe34ewb);
		decorArc13.updateTransformationState(trans45);
		decorArc13.setTextureID("brick_wall");

		decorArc13.setAmbientFactor(Materials.AMBIENT_WALL);
		decorArc13.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorArc13.setSpecularFactor(Materials.SPECULAR_WALL);
		decorArc13.shinessFactor = Materials.SHINESS_WALL;

		decorArc13.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorArc13);
		// ---------------------------------------------------------------------

		decorWall1 = new Cube(30f, 3, 0.2f, 2.5f, 0.2f);

		trans6 = new Transformation();
		float[] t49 = { 5, 0, 35 - 0.2f, 1 };

		trans6.translate(t49);
		decorWall1.updateTransformationState(trans6);
		decorWall1.setTextureID("brick_wall");

		decorWall1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall1.shinessFactor = Materials.SHINESS_WALL;

		decorWall1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall1);

		for (int i = 0; i < 16; i++) {
			Cube decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

			trans6.setIdentity();
			float[] t7 = { 5 + (i * 2.0f) - 0.1f, 3f, 35 - 0.5f, 1 };
			trans6.translate(t7);

			decorStone.updateTransformationState(trans6);
			decorStone.setTextureID("brick_wall");

			decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
			decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
			decorStone.shinessFactor = Materials.SHINESS_WALL;

			decorStone.setBufferID(bufferController.getBufferID());
			Node.addChildrenToNode(groundNode, decorStone);

			Cube decorCube = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

			trans6.setIdentity();
			float[] t4 = { 5 + (i * 2.0f), 0, 35 - 0.4f, 1 };

			trans6.translate(t4);
			decorCube.updateTransformationState(trans6);
			decorCube.setTextureID("brick_wall");

			decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
			decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
			decorCube.shinessFactor = Materials.SHINESS_WALL;

			decorCube.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, decorCube);
		}

		// ---------------------------------------------------------------

		Cube rightGround = new Cube(x, 2, 5, 4f, 0.4f);
		rightGround.setTextureID("ground");

		rightGround.setAmbientFactor(Materials.AMBIENT_GROUND);
		rightGround.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		rightGround.setSpecularFactor(Materials.SPECULAR_GROUND);
		rightGround.shinessFactor = Materials.SHINESS_GROUND;

		rightGround.setBufferID(bufferController.getBufferID());

		Transformation trans = new Transformation();
		float[] translate = { 0, 0, 35, 1 };

		trans.translate(translate);
		rightGround.updateTransformationState(trans);

		// --------------------------------------------------------------

		// -----------------------------------------------------------------
		// barrels

		for (int i = 0; i < 5; i++) {
			EllipsoidArc barrel = new EllipsoidArc(0.5f, 0.7f, 0.5f,
					(float) -(Math.PI / 4), (float) (Math.PI / 4),
					(float) -(Math.PI), (float) (Math.PI), 10, 10);

			trans.setIdentity();
			float[] t = { 7 - i, 3, 25, 1 };
			trans.translate(t);

			barrel.setTextureID("barrel");

			barrel.setAmbientFactor(Materials.AMBIENT_WOOD);
			barrel.setDiffuseFactor(Materials.DIFFUSE_WOOD);
			barrel.setSpecularFactor(Materials.SPECULAR_WOOD);
			barrel.shinessFactor = Materials.SHINESS_WOOD;

			barrel.setBufferID(bufferController.getBufferID());
			barrel.updateTransformationState(trans);
			Node.addChildrenToNode(groundNode, barrel);
		}

		for (int i = 0; i < 3; i++) {
			EllipsoidArc barrel = new EllipsoidArc(0.5f, 0.7f, 0.5f,
					(float) -(Math.PI / 4), (float) (Math.PI / 4),
					(float) -(Math.PI), (float) (Math.PI), 10, 10);

			trans.setIdentity();
			float[] t = { 5.7f - i, 3, 24, 1 };
			trans.translate(t);

			barrel.setTextureID("barrel");

			barrel.setAmbientFactor(Materials.AMBIENT_WOOD);
			barrel.setDiffuseFactor(Materials.DIFFUSE_WOOD);
			barrel.setSpecularFactor(Materials.SPECULAR_WOOD);
			barrel.shinessFactor = Materials.SHINESS_WOOD;

			barrel.setBufferID(bufferController.getBufferID());
			barrel.updateTransformationState(trans);
			Node.addChildrenToNode(groundNode, barrel);
		}
		// ----------------------------------------------------------------------

		// ---------------------------------------------------------------------
		Cube firstPlayGround = new Cube(15, 2, 20, 1.5f, 2f);

		trans.setIdentity();
		float[] translate2 = { 0, 0, 10, 1 };

		trans.translate(translate2);

		firstPlayGround.updateTransformationState(trans);
		firstPlayGround.setTextureID("ground");

		firstPlayGround.setAmbientFactor(Materials.AMBIENT_GROUND);
		firstPlayGround.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		firstPlayGround.setSpecularFactor(Materials.SPECULAR_GROUND);
		firstPlayGround.shinessFactor = Materials.SHINESS_GROUND;

		firstPlayGround.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, firstPlayGround);
		// --------------------------------------------------------------------
		// pillarShadows

		float[][] shadowMatrix = new float[4][4];
		float[] lightPos = { 20, 40, 20, 1f };
		float[] p1 = { 1, -2.0f, 1, 1 };
		float[] p2 = { 2, -2.0f, 4, 1 };
		float[] p3 = { 4, -2.0f, 2, 1 };

		LinearAlgebra4D.calculateProjectionMatrix(p1, p2, p3, lightPos,
				shadowMatrix);
		Transformation shadowTransformation = new Transformation(shadowMatrix);

		// -----------------------------------------------------------------
		// pillars

		Cube pillar1 = new Cube(1, 20, 1, 0.15f, 4f);

		trans.setIdentity();
		float[] t10 = { 10, 2, 12, 1 };
		trans.translate(t10);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(2.2f, 2.2f, 2.2f, 0.3f, 0.4f);

		trans.setIdentity();
		float[] tq10 = { 10 - 0.6f, 2, 12 - 0.6f, 1 };
		trans.translate(tq10);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		// -------------------------------------------------------------------

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 4f);

		trans.setIdentity();
		float[] t14 = { 9 + 0.5f, 2, 13, 1 };
		trans.translate(t14);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_MARMOR);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		pillar1.setSpecularFactor(Materials.SPECULAR_MARMOR);
		pillar1.shinessFactor = Materials.SHINESS_MARMOR;
		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] t13 = { 11f, 2, 13, 1 };
		trans.translate(t13);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_MARMOR);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		pillar1.setSpecularFactor(Materials.SPECULAR_MARMOR);
		pillar1.shinessFactor = Materials.SHINESS_MARMOR;
		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] t11 = { 9 + 0.5f, 2, 11 + 0.5f, 1 };
		trans.translate(t11);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_MARMOR);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		pillar1.setSpecularFactor(Materials.SPECULAR_MARMOR);
		pillar1.shinessFactor = Materials.SHINESS_MARMOR;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] t12 = { 11f, 2, 11 + 0.5f, 1 };
		trans.translate(t12);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_MARMOR);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		pillar1.setSpecularFactor(Materials.SPECULAR_MARMOR);
		pillar1.shinessFactor = Materials.SHINESS_MARMOR;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(1, 20, 1, 0.15f, 5f);

		trans.setIdentity();
		float[] t15 = { 10, 2, 27, 1 };
		trans.translate(t15);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(2.2f, 2.2f, 2.2f, 0.4f, 0.6f);

		trans.setIdentity();
		float[] tqr10 = { 10 - 0.6f, 2, 27 - 0.6f, 1 };
		trans.translate(tqr10);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] t16 = { 9 + 0.5f, 2, 26 + 0.5f, 1 };
		trans.translate(t16);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] t17 = { 11f, 2, 26 + 0.5f, 1 };
		trans.translate(t17);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] t18 = { 11f, 2, 28, 1 };
		trans.translate(t18);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] t19 = { 9 + 0.5f, 2, 28, 1 };
		trans.translate(t19);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(1, 20, 1, 0.15f, 4f);

		trans.setIdentity();
		float[] e10 = { 10, 2, 19.5f, 1 };
		trans.translate(e10);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(2.2f, 2.2f, 2.2f, 0.4f, 0.6f);

		trans.setIdentity();
		float[] tqd10 = { 10 - 0.6f, 2, 19.5f - 0.6f, 1 };
		trans.translate(tqd10);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] e11 = { 9 + 0.5f, 2, 19, 1 };
		trans.translate(e11);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] e12 = { 11f, 2, 19, 1 };
		trans.translate(e12);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] e13 = { 11f, 2, 20.5f, 1 };
		trans.translate(e13);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] e14 = { 9 + 0.5f, 2, 20.5f, 1 };
		trans.translate(e14);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		Arc bridgeArc4 = new Arc(0, (float) Math.PI, 0.8f, 1.f, 3.8f, 30, 0.1f,
				0.2f);

		trans.setIdentity();
		float[] e34 = { 10.1f, 14f, 16.2f, 1 };

		trans.translate(e34);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		bridgeArc4.updateTransformationState(trans);
		bridgeArc4.setTextureID("brick_wall");

		bridgeArc4.setAmbientFactor(Materials.AMBIENT_WALL);
		bridgeArc4.setDiffuseFactor(Materials.DIFFUSE_WALL);
		bridgeArc4.setSpecularFactor(Materials.SPECULAR_WALL);
		bridgeArc4.shinessFactor = Materials.SHINESS_WALL;

		bridgeArc4.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc4);

		bridgeArc4 = new Arc(0, (float) Math.PI, 0.8f, 1.f, 3.8f, 30, 0.1f,
				0.2f);

		trans.setIdentity();
		float[] e35 = { 10.1f, 14f, 23.9f, 1 };

		trans.translate(e35);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		bridgeArc4.updateTransformationState(trans);
		bridgeArc4.setTextureID("brick_wall");

		bridgeArc4.setAmbientFactor(Materials.AMBIENT_WALL);
		bridgeArc4.setDiffuseFactor(Materials.DIFFUSE_WALL);
		bridgeArc4.setSpecularFactor(Materials.SPECULAR_WALL);
		bridgeArc4.shinessFactor = Materials.SHINESS_WALL;

		bridgeArc4.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc4);

		bridgeArc4 = new Arc(0, (float) Math.PI, 0.8f, 1.f, 5.5f, 30, 0.1f,
				0.2f);

		trans.setIdentity();
		float[] e36 = { 5f, 11.5f, 27.1f, 1 };

		trans.translate(e36);
		bridgeArc4.updateTransformationState(trans);
		bridgeArc4.setTextureID("brick_wall");

		bridgeArc4.setAmbientFactor(Materials.AMBIENT_WALL);
		bridgeArc4.setDiffuseFactor(Materials.DIFFUSE_WALL);
		bridgeArc4.setSpecularFactor(Materials.SPECULAR_WALL);
		bridgeArc4.shinessFactor = Materials.SHINESS_WALL;

		bridgeArc4.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc4);

		bridgeArc4 = new Arc(0, (float) Math.PI, 0.8f, 1.f, 5.5f, 30, 0.1f,
				0.2f);

		trans.setIdentity();
		float[] e37 = { 5f, 11.5f, 12.1f, 1 };

		trans.translate(e37);
		bridgeArc4.updateTransformationState(trans);
		bridgeArc4.setTextureID("brick_wall");

		bridgeArc4.setAmbientFactor(Materials.AMBIENT_WALL);
		bridgeArc4.setDiffuseFactor(Materials.DIFFUSE_WALL);
		bridgeArc4.setSpecularFactor(Materials.SPECULAR_WALL);
		bridgeArc4.shinessFactor = Materials.SHINESS_WALL;

		bridgeArc4.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc4);

		// decorwalls
		Cube decorWall = new Cube(0.2f, 3, 13, 1, 0.3f);

		trans.setIdentity();
		float[] t30 = { 15, 0, 17, 1 };

		trans.translate(t30);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		for (int i = 0; i < 8; i++) {
			Cube decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

			trans.setIdentity();
			float[] t7 = { 15 - 0.1f, 3f, 17 + (i * 1.86f) - 0.1f, 1 };
			trans.translate(t7);

			decorStone.updateTransformationState(trans);
			decorStone.setTextureID("brick_wall");

			decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
			decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
			decorStone.shinessFactor = Materials.SHINESS_WALL;

			decorStone.setBufferID(bufferController.getBufferID());

			Cube decorCube = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

			Node.addChildrenToNode(groundNode, decorStone);

			trans.setIdentity();
			float[] t4 = { 15, 0, 17 + (i * 1.86f), 1 };

			trans.translate(t4);
			decorCube.updateTransformationState(trans);
			decorCube.setTextureID("brick_wall");

			decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
			decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
			decorCube.shinessFactor = Materials.SHINESS_WALL;

			decorCube.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, decorCube);
		}

		Arc bridgeArc1 = new Arc(0, (float) Math.PI, 0.8f, 0.4f, 0.7f, 10, 1f,
				1);

		trans.setIdentity();
		float[] t34 = { 15, 1.2f, 20f, 1 };

		trans.translate(t34);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		bridgeArc1.updateTransformationState(trans);
		bridgeArc1.setTextureID("ground");

		bridgeArc1.setAmbientFactor(Materials.AMBIENT_GROUND);
		bridgeArc1.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		bridgeArc1.setSpecularFactor(Materials.SPECULAR_GROUND);
		bridgeArc1.shinessFactor = Materials.SHINESS_GROUND;

		bridgeArc1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc1);

		bridgeArc1 = new Arc(0, (float) Math.PI, 0.8f, 0.4f, 0.7f, 10, 1f, 1);

		trans.setIdentity();
		float[] t35 = { 15, 1.2f, 27.5f, 1 };

		trans.translate(t35);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		bridgeArc1.updateTransformationState(trans);
		bridgeArc1.setTextureID("ground");

		bridgeArc1.setAmbientFactor(Materials.AMBIENT_GROUND);
		bridgeArc1.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		bridgeArc1.setSpecularFactor(Materials.SPECULAR_GROUND);
		bridgeArc1.shinessFactor = Materials.SHINESS_GROUND;

		bridgeArc1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc1);

		decorWall = new Cube(0.2f, 3, 5, 1, 0.3f);

		trans.setIdentity();
		float[] t38 = { 15, 0, 10, 1 };

		trans.translate(t38);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		for (int i = 0; i < 3; i++) {
			Cube decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

			trans.setIdentity();
			float[] t7 = { 15 - 0.1f, 3f, 14.6f - (i * 2.5f), 1 };
			trans.translate(t7);

			decorStone.updateTransformationState(trans);
			decorStone.setTextureID("brick_wall");

			decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
			decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
			decorStone.shinessFactor = Materials.SHINESS_WALL;

			decorStone.setBufferID(bufferController.getBufferID());
			Node.addChildrenToNode(groundNode, decorStone);

			Cube decorCube = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1);

			trans.setIdentity();
			float[] t4 = { 15, 0, 14.7f - (i * 2.5f), 1 };

			trans.translate(t4);
			decorCube.updateTransformationState(trans);
			decorCube.setTextureID("brick_wall");

			decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
			decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
			decorCube.shinessFactor = Materials.SHINESS_WALL;

			decorCube.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, decorCube);
		}

		bridgeArc1 = new Arc(0, (float) Math.PI, 0.8f, 0.4f, 0.7f, 10, 1f, 1);

		trans.setIdentity();
		float[] t39 = { 15, 1.2f, 11f, 1 };

		trans.translate(t39);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		bridgeArc1.updateTransformationState(trans);
		bridgeArc1.setTextureID("ground");

		bridgeArc1.setAmbientFactor(Materials.AMBIENT_GROUND);
		bridgeArc1.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		bridgeArc1.setSpecularFactor(Materials.SPECULAR_GROUND);
		bridgeArc1.shinessFactor = Materials.SHINESS_GROUND;

		bridgeArc1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc1);

		decorWall = new Cube(10f, 3, 0.2f, 1, 0.3f);

		trans.setIdentity();
		float[] t42 = { 5, 0, 9.8f, 1 };

		trans.translate(t42);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		for (int i = 0; i < 6; i++) {
			Cube decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

			trans.setIdentity();
			float[] t7 = { 15 - 0.1f - (i * 2.0f), 3f, 9.7f, 1 };
			trans.translate(t7);

			decorStone.updateTransformationState(trans);
			decorStone.setTextureID("brick_wall");

			decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
			decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
			decorStone.shinessFactor = Materials.SHINESS_WALL;

			decorStone.setBufferID(bufferController.getBufferID());
			Node.addChildrenToNode(groundNode, decorStone);

			Cube decorCube = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

			trans.setIdentity();
			float[] t4 = { 15 - (i * 2.0f), 0, 9.7f, 1 };

			trans.translate(t4);
			decorCube.updateTransformationState(trans);
			decorCube.setTextureID("brick_wall");

			decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
			decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
			decorCube.shinessFactor = Materials.SHINESS_WALL;

			decorCube.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, decorCube);
		}

		decorWall = new Cube(10f, 3, 0.2f, 1, 0.3f);

		trans.setIdentity();
		float[] t43 = { 5, 0, 30f, 1 };

		trans.translate(t43);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		for (int i = 0; i < 6; i++) {
			Cube decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

			trans.setIdentity();
			float[] t7 = { 15 - 0.1f - (i * 2.0f), 3f, 30f, 1 };
			trans.translate(t7);

			decorStone.updateTransformationState(trans);
			decorStone.setTextureID("brick_wall");

			decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
			decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
			decorStone.shinessFactor = Materials.SHINESS_WALL;

			decorStone.setBufferID(bufferController.getBufferID());
			Node.addChildrenToNode(groundNode, decorStone);

			Cube decorCube = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

			trans.setIdentity();
			float[] t4 = { 15 - (i * 2.0f), 0, 30f, 1 };

			trans.translate(t4);
			decorCube.updateTransformationState(trans);
			decorCube.setTextureID("brick_wall");

			decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
			decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
			decorCube.shinessFactor = Materials.SHINESS_WALL;

			decorCube.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, decorCube);
		}
		// ------------------------------------------------------------------
		Cube secondPlayGround = new Cube(15, 2, 20, 1.5f, 2f);

		trans.setIdentity();
		float[] translate3 = { 25, 0, 10, 1 };

		trans.translate(translate3);
		secondPlayGround.updateTransformationState(trans);
		secondPlayGround.setTextureID("ground");

		secondPlayGround.setAmbientFactor(Materials.AMBIENT_GROUND);
		secondPlayGround.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		secondPlayGround.setSpecularFactor(Materials.SPECULAR_GROUND);
		secondPlayGround.shinessFactor = Materials.SHINESS_GROUND;

		secondPlayGround.setBufferID(bufferController.getBufferID());

		// ---------------------------------------------------------------------
		// tempel

		Cube templeFloor = new Cube(4, 1, 4, 1, 1);

		trans.setIdentity();
		float[] transla3 = { 35, 2, 14, 1 };

		trans.translate(transla3);
		templeFloor.updateTransformationState(trans);
		templeFloor.setTextureID("ground");

		templeFloor.setAmbientFactor(Materials.AMBIENT_GROUND);
		templeFloor.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		templeFloor.setSpecularFactor(Materials.SPECULAR_GROUND);
		templeFloor.shinessFactor = Materials.SHINESS_GROUND;

		secondPlayGround.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, templeFloor);

		decorWall = new Cube(0.2f, 3, 4.4f, 2, 0.3f);

		trans.setIdentity();
		float[] t331 = { 39, 2, 14 - 0.2f, 1 };

		trans.translate(t331);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		Cube decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t7q = { 39 - 0.1f, 2, 14 - 0.4f, 1 };
		trans.translate(t7q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t1q = { 39 - 0.1f, 5, 14 - 0.4f, 1 };
		trans.translate(t1q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone1);

		Cube decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] t4q = { 39, 2, 14 - 0.3f, 1 };

		trans.translate(t4q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t8q = { 39 - 0.1f, 2, 18 - 0.1f, 1 };
		trans.translate(t8q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t2q = { 39 - 0.1f, 5, 18 - 0.1f, 1 };
		trans.translate(t2q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] t9q = { 39, 2, 18, 1 };

		trans.translate(t9q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		decorWall = new Cube(1f, 3, 0.2f, 0.1f, 0.3f);

		trans.setIdentity();
		float[] t3311 = { 38, 2, 14 - 0.2f, 1 };

		trans.translate(t3311);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t11q = { 38 - 0.1f, 2, 14 - 0.3f, 1 };
		trans.translate(t11q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t3q = { 38 - 0.1f, 5, 14 - 0.3f, 1 };
		trans.translate(t3q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone1);

		decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);
		trans.setIdentity();
		float[] t10q = { 38, 2, 14 - 0.2f, 1 };

		trans.translate(t10q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		decorWall = new Cube(1f, 3, 0.2f, 0.1f, 0.3f);

		trans.setIdentity();
		float[] a3311 = { 38, 2, 18, 1 };

		trans.translate(a3311);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] a11q = { 38 - 0.1f, 2, 18 - 0.1f, 1 };
		trans.translate(a11q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] a3q = { 38 - 0.1f, 5, 18 - 0.1f, 1 };
		trans.translate(a3q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone1);

		decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);
		trans.setIdentity();
		float[] a10q = { 38, 2, 18f, 1 };

		trans.translate(a10q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		decorWall = new Cube(0.2f, 3, 1f, 0.1f, 0.3f);

		trans.setIdentity();
		float[] t3331 = { 38, 2, 14 - 1.2f, 1 };

		trans.translate(t3331);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t12q = { 38 - 0.1f, 2, 14 - 1.5f, 1 };
		trans.translate(t12q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t14q = { 38 - 0.1f, 5, 14 - 1.5f, 1 };
		trans.translate(t14q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t15q = { 38 - 0.1f, 5, 14 - 1.5f, 1 };
		trans.translate(t1q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] t13q = { 38, 2, 14 - 1.4f, 1 };

		trans.translate(t13q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		decorWall = new Cube(0.2f, 3, 1f, 0.1f, 0.3f);

		trans.setIdentity();
		float[] a3331 = { 38, 2, 18.2f, 1 };

		trans.translate(a3331);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] a12q = { 38 - 0.1f, 2, 19.1f, 1 };
		trans.translate(a12q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] a14q = { 38 - 0.1f, 5, 19.1f, 1 };
		trans.translate(a14q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] a15q = { 38 - 0.1f, 5, 19.1f, 1 };
		trans.translate(a15q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] a13q = { 38, 2, 19.2f, 1 };

		trans.translate(a13q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		decorWall = new Cube(1f, 3, 0.2f, 0.1f, 0.3f);

		trans.setIdentity();
		float[] t3321 = { 35, 2, 14 - 0.2f, 1 };

		trans.translate(t3321);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t18q = { 36 - 0.4f, 2, 14 - 0.4f, 1 };
		trans.translate(t18q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t19q = { 36 - 0.4f, 5, 14 - 0.4f, 1 };
		trans.translate(t19q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] t20q = { 36 - 0.3f, 2, 14 - 0.3f, 1 };

		trans.translate(t20q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		decorWall = new Cube(0.2f, 3, 1f, 0.1f, 0.3f);

		trans.setIdentity();
		float[] a3321 = { 36 - 0.2f, 2, 18.2f, 1 };

		trans.translate(a3321);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] a18q = { 36 - 0.4f, 2, 19.1f, 1 };
		trans.translate(a18q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] a19q = { 36 - 0.4f, 5, 19.1f, 1 };
		trans.translate(a19q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] a20q = { 36 - 0.3f, 2, 19.2f, 1 };

		trans.translate(a20q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] b18q = { 36 - 0.4f, 2, 18.f, 1 };
		trans.translate(b18q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] b19q = { 36 - 0.4f, 5, 18.f, 1 };
		trans.translate(b19q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] b20q = { 36 - 0.3f, 2, 18.1f, 1 };

		trans.translate(b20q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] c18q = { 35 - 0.4f, 2, 18.f, 1 };
		trans.translate(c18q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] c19q = { 35 - 0.4f, 5, 18.f, 1 };
		trans.translate(c19q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] c20q = { 35 - 0.3f, 2, 18.1f, 1 };

		trans.translate(c20q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] q18q = { 35 - 0.4f, 2, 17.1f, 1 };
		trans.translate(q18q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] q19q = { 35 - 0.4f, 5, 17.1f, 1 };
		trans.translate(q19q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] q20q = { 35 - 0.3f, 2, 17.2f, 1 };

		trans.translate(q20q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] d18q = { 33.4f, 2, 17.1f, 1 };
		trans.translate(d18q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] d19q = { 33.4f, 5, 17.1f, 1 };
		trans.translate(d19q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] d20q = { 33.5f, 2, 17.2f, 1 };

		trans.translate(d20q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		decorWall = new Cube(1f, 3, 0.2f, 0.1f, 0.3f);

		trans.setIdentity();
		float[] a3332 = { 35f, 2, 18f, 1 };

		trans.translate(a3332);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		decorWall = new Cube(0.2f, 3, 1f, 0.1f, 0.3f);

		trans.setIdentity();
		float[] c3332 = { 35f - 0.2f, 2, 17.2f, 1 };

		trans.translate(c3332);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		decorWall = new Cube(1f, 3, 0.2f, 0.1f, 0.3f);

		trans.setIdentity();
		float[] d3332 = { 33.8f, 2, 17.2f, 1 };

		trans.translate(d3332);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		decorWall = new Cube(0.2f, 3, 1f, 0.1f, 0.3f);

		trans.setIdentity();
		float[] t3332 = { 36 - 0.2f, 2, 12.8f, 1 };

		trans.translate(t3332);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t22q = { 36 - 0.4f, 2, 12.8f - 0.4f, 1 };
		trans.translate(t22q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t23q = { 36 - 0.4f, 5, 12.8f - 0.4f, 1 };
		trans.translate(t23q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] t21q = { 36 - 0.3f, 2, 12.8f - 0.3f, 1 };

		trans.translate(t21q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		decorWall = new Cube(0.2f, 3, 1f, 0.1f, 0.3f);

		trans.setIdentity();
		float[] t1321 = { 35 - 0.2f, 2, 14 - 0.2f, 1 };

		trans.translate(t1321);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t25q = { 35 - 0.4f, 2, 14 - 0.4f, 1 };
		trans.translate(t25q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t26q = { 35 - 0.4f, 5, 14 - 0.4f, 1 };
		trans.translate(t26q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] t27q = { 35 - 0.3f, 2, 14 - 0.3f, 1 };

		trans.translate(t27q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t28q = { 35 - 0.4f, 2, 14 + 0.4f, 1 };
		trans.translate(t28q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t29q = { 35 - 0.4f, 5, 14 + 0.4f, 1 };
		trans.translate(t29q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] t230q = { 35 - 0.3f, 2, 14 + 0.5f, 1 };

		trans.translate(t230q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		decorWall = new Cube(1f, 3, 0.2f, 0.1f, 0.3f);

		trans.setIdentity();
		float[] t1332 = { 33.8f, 2, 14.5f, 1 };

		trans.translate(t1332);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t33q = { 33.8f - 0.4f, 2, 14.5f - 0.1f, 1 };
		trans.translate(t33q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorStone1 = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] t34q = { 33.8f - 0.4f, 5, 14.5f - 0.1f, 1 };
		trans.translate(t34q);

		decorStone1.updateTransformationState(trans);
		decorStone1.setTextureID("brick_wall");

		decorStone1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone1.shinessFactor = Materials.SHINESS_WALL;

		decorStone1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorStone1);

		decorCube1 = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] t31q = { 33.8f - 0.3f, 2, 14.5f, 1 };

		trans.translate(t31q);
		decorCube1.updateTransformationState(trans);
		decorCube1.setTextureID("brick_wall");

		decorCube1.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube1.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube1.shinessFactor = Materials.SHINESS_WALL;

		decorCube1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube1);

		Cylinder pillar3 = new Cylinder(15, 0.25f, 5, 0.1f, 1);

		trans.setIdentity();
		float[] h6 = { 36, 3, 15, 1 };

		trans.translate(h6);
		pillar3.updateTransformationState(trans);
		pillar3.setTextureID("marmor");

		pillar3.setAmbientFactor(Materials.AMBIENT_MARMOR);
		pillar3.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		pillar3.setSpecularFactor(Materials.SPECULAR_MARMOR);
		pillar3.shinessFactor = Materials.SHINESS_MARMOR;

		pillar3.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar3);

		SolidDeformedCylinder cubeDecor3 = new SolidDeformedCylinder(0.25f,
				0.4f, 0.7f, 15, 0.1f, 0.3f);

		trans.setIdentity();
		float[] h7 = { 36, 3 + 4.3f, 15, 1 };

		trans.translate(h7);
		cubeDecor3.updateTransformationState(trans);
		cubeDecor3.setTextureID("marmor");

		cubeDecor3.setAmbientFactor(Materials.AMBIENT_MARMOR);
		cubeDecor3.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		cubeDecor3.setSpecularFactor(Materials.SPECULAR_MARMOR);
		cubeDecor3.shinessFactor = Materials.SHINESS_MARMOR;

		cubeDecor3.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, cubeDecor3);

		cubeDecor3 = new SolidDeformedCylinder(0.35f, 0.25f, 0.7f, 15, 0.1f,
				0.3f);

		trans.setIdentity();
		float[] h8 = { 36, 3, 15, 1 };

		trans.translate(h8);
		cubeDecor3.updateTransformationState(trans);
		cubeDecor3.setTextureID("marmor");

		cubeDecor3.setAmbientFactor(Materials.AMBIENT_MARMOR);
		cubeDecor3.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		cubeDecor3.setSpecularFactor(Materials.SPECULAR_MARMOR);
		cubeDecor3.shinessFactor = Materials.SHINESS_MARMOR;

		cubeDecor3.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, cubeDecor3);

		pillar3 = new Cylinder(15, 0.25f, 5, 0.1f, 1);

		trans.setIdentity();
		float[] y6 = { 38, 3, 15, 1 };

		trans.translate(y6);
		pillar3.updateTransformationState(trans);
		pillar3.setTextureID("marmor");

		pillar3.setAmbientFactor(Materials.AMBIENT_MARMOR);
		pillar3.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		pillar3.setSpecularFactor(Materials.SPECULAR_MARMOR);
		pillar3.shinessFactor = Materials.SHINESS_MARMOR;

		pillar3.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar3);

		cubeDecor3 = new SolidDeformedCylinder(0.25f, 0.4f, 0.7f, 15, 0.1f,
				0.3f);

		trans.setIdentity();
		float[] y7 = { 38, 3 + 4.3f, 15, 1 };

		trans.translate(y7);
		cubeDecor3.updateTransformationState(trans);
		cubeDecor3.setTextureID("marmor");

		cubeDecor3.setAmbientFactor(Materials.AMBIENT_MARMOR);
		cubeDecor3.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		cubeDecor3.setSpecularFactor(Materials.SPECULAR_MARMOR);
		cubeDecor3.shinessFactor = Materials.SHINESS_MARMOR;

		cubeDecor3.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, cubeDecor3);

		cubeDecor3 = new SolidDeformedCylinder(0.35f, 0.25f, 0.7f, 15, 0.1f,
				0.3f);

		trans.setIdentity();
		float[] y8 = { 38, 3, 15, 1 };

		trans.translate(y8);
		cubeDecor3.updateTransformationState(trans);
		cubeDecor3.setTextureID("marmor");

		cubeDecor3.setAmbientFactor(Materials.AMBIENT_MARMOR);
		cubeDecor3.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		cubeDecor3.setSpecularFactor(Materials.SPECULAR_MARMOR);
		cubeDecor3.shinessFactor = Materials.SHINESS_MARMOR;

		cubeDecor3.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, cubeDecor3);

		pillar3 = new Cylinder(15, 0.25f, 5, 0.1f, 1);

		trans.setIdentity();
		float[] r61 = { 38, 3, 17, 1 };

		trans.translate(r61);
		pillar3.updateTransformationState(trans);
		pillar3.setTextureID("marmor");

		pillar3.setAmbientFactor(Materials.AMBIENT_MARMOR);
		pillar3.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		pillar3.setSpecularFactor(Materials.SPECULAR_MARMOR);
		pillar3.shinessFactor = Materials.SHINESS_MARMOR;

		pillar3.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar3);

		cubeDecor3 = new SolidDeformedCylinder(0.25f, 0.4f, 0.7f, 15, 0.1f,
				0.3f);

		trans.setIdentity();
		float[] r71 = { 38, 3 + 4.3f, 17, 1 };

		trans.translate(r71);
		cubeDecor3.updateTransformationState(trans);
		cubeDecor3.setTextureID("marmor");

		cubeDecor3.setAmbientFactor(Materials.AMBIENT_MARMOR);
		cubeDecor3.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		cubeDecor3.setSpecularFactor(Materials.SPECULAR_MARMOR);
		cubeDecor3.shinessFactor = Materials.SHINESS_MARMOR;

		cubeDecor3.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, cubeDecor3);

		cubeDecor3 = new SolidDeformedCylinder(0.35f, 0.25f, 0.7f, 15, 0.1f,
				0.3f);

		trans.setIdentity();
		float[] r81 = { 38, 3, 17, 1 };

		trans.translate(r81);
		cubeDecor3.updateTransformationState(trans);
		cubeDecor3.setTextureID("marmor");

		cubeDecor3.setAmbientFactor(Materials.AMBIENT_MARMOR);
		cubeDecor3.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		cubeDecor3.setSpecularFactor(Materials.SPECULAR_MARMOR);
		cubeDecor3.shinessFactor = Materials.SHINESS_MARMOR;

		cubeDecor3.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, cubeDecor3);

		pillar3 = new Cylinder(15, 0.25f, 5, 0.1f, 1);

		trans.setIdentity();
		float[] rw61 = { 36, 3, 17, 1 };

		trans.translate(rw61);
		pillar3.updateTransformationState(trans);
		pillar3.setTextureID("marmor");

		pillar3.setAmbientFactor(Materials.AMBIENT_MARMOR);
		pillar3.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		pillar3.setSpecularFactor(Materials.SPECULAR_MARMOR);
		pillar3.shinessFactor = Materials.SHINESS_MARMOR;

		pillar3.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar3);

		cubeDecor3 = new SolidDeformedCylinder(0.25f, 0.4f, 0.7f, 15, 0.1f,
				0.3f);

		trans.setIdentity();
		float[] rw71 = { 36, 3 + 4.3f, 17, 1 };

		trans.translate(rw71);
		cubeDecor3.updateTransformationState(trans);
		cubeDecor3.setTextureID("marmor");

		cubeDecor3.setAmbientFactor(Materials.AMBIENT_MARMOR);
		cubeDecor3.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		cubeDecor3.setSpecularFactor(Materials.SPECULAR_MARMOR);
		cubeDecor3.shinessFactor = Materials.SHINESS_MARMOR;

		cubeDecor3.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, cubeDecor3);

		cubeDecor3 = new SolidDeformedCylinder(0.35f, 0.25f, 0.7f, 15, 0.1f,
				0.3f);

		trans.setIdentity();
		float[] rw81 = { 36, 3, 17, 1 };

		trans.translate(rw81);
		cubeDecor3.updateTransformationState(trans);
		cubeDecor3.setTextureID("marmor");

		cubeDecor3.setAmbientFactor(Materials.AMBIENT_MARMOR);
		cubeDecor3.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		cubeDecor3.setSpecularFactor(Materials.SPECULAR_MARMOR);
		cubeDecor3.shinessFactor = Materials.SHINESS_MARMOR;

		cubeDecor3.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, cubeDecor3);

		SolidDeformedCylinder decorCylinder = new SolidDeformedCylinder(2f, 2f,
				0.5f, 20, 1, 0.1f);

		trans.setIdentity();
		float[] rwq81 = { 37, 8, 16, 1 };

		trans.translate(rwq81);
		decorCylinder.updateTransformationState(trans);
		decorCylinder.setTextureID("marmor");

		decorCylinder.setAmbientFactor(Materials.AMBIENT_MARMOR);
		decorCylinder.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		decorCylinder.setSpecularFactor(Materials.SPECULAR_MARMOR);
		decorCylinder.shinessFactor = Materials.SHINESS_MARMOR;

		decorCylinder.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCylinder);

		EllipsoidArc kuppel = new EllipsoidArc(1.6f, 3.2f, 1.6f,
				(float) -Math.PI / 2, (float) Math.PI / 2, 0, (float) Math.PI,
				40, 40);

		trans.setIdentity();
		float[] rwaaq81 = { 37, 8.5f, 16, 1 };

		trans.translate(rwq81);

		kuppel.updateTransformationState(trans);
		kuppel.setTextureID("marmor");

		kuppel.setAmbientFactor(Materials.AMBIENT_MARMOR);
		kuppel.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		kuppel.setSpecularFactor(Materials.SPECULAR_MARMOR);
		kuppel.shinessFactor = Materials.SHINESS_MARMOR;

		kuppel.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, kuppel);

		Arc templeArc = new Arc(0, (float) Math.PI, 0.3f, 0.4f, 1.55f, 20,
				0.2f, 1.0f);

		trans.setIdentity();
		float[] tat23 = { 33.5f, 5.5f, 16f, 1 };

		trans.translate(tat23);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		templeArc.updateTransformationState(trans);
		templeArc.setTextureID("brick_wall");

		templeArc.setAmbientFactor(Materials.AMBIENT_WALL);
		templeArc.setDiffuseFactor(Materials.DIFFUSE_WALL);
		templeArc.setSpecularFactor(Materials.SPECULAR_WALL);
		templeArc.shinessFactor = Materials.SHINESS_WALL;

		templeArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, templeArc);

		templeArc = new Arc(0, (float) Math.PI, 0.3f, 0.3f, 1.3f, 20, 0.2f,
				1.0f);

		trans.setIdentity();
		float[] ta23 = { 37f, 5.5f, 12.6f, 1 };

		trans.translate(ta23);
		templeArc.updateTransformationState(trans);
		templeArc.setTextureID("brick_wall");

		templeArc.setAmbientFactor(Materials.AMBIENT_WALL);
		templeArc.setDiffuseFactor(Materials.DIFFUSE_WALL);
		templeArc.setSpecularFactor(Materials.SPECULAR_WALL);
		templeArc.shinessFactor = Materials.SHINESS_WALL;

		templeArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, templeArc);

		templeArc = new Arc(0, (float) Math.PI, 0.3f, 0.3f, 1.3f, 20, 0.2f,
				1.0f);

		trans.setIdentity();
		float[] ta23d = { 37f, 5.5f, 19.2f, 1 };

		trans.translate(ta23d);
		templeArc.updateTransformationState(trans);
		templeArc.setTextureID("brick_wall");

		templeArc.setAmbientFactor(Materials.AMBIENT_WALL);
		templeArc.setDiffuseFactor(Materials.DIFFUSE_WALL);
		templeArc.setSpecularFactor(Materials.SPECULAR_WALL);
		templeArc.shinessFactor = Materials.SHINESS_WALL;

		templeArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, templeArc);
		// --------------------------------------------------------------------
		// exit
		Cube door = new Cube(0.2f, 2.5f, 1.1f, 1, 1);

		trans.setIdentity();
		float[] translat20 = { 40 - 0.2f - 0.2f, 2f, 25.7f, 1 };

		trans.translate(translat20);
		door.updateTransformationState(trans);
		door.setTextureID("door");

		door.setAmbientFactor(Materials.AMBIENT_DOOR);
		door.setDiffuseFactor(Materials.DIFFUSE_DOOR);
		door.setSpecularFactor(Materials.SPECULAR_DOOR);
		door.shinessFactor = Materials.SHINESS_DOOR;

		door.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, door);

		Cube doorDecor = new Cube(0.8f, 2.5f, 0.2f, 1, 1);

		trans.setIdentity();
		float[] translat21 = { 40 - 1f, 2f, 25.5f, 1 };

		trans.translate(translat21);
		doorDecor.updateTransformationState(trans);
		doorDecor.setTextureID("marmor");

		doorDecor.setAmbientFactor(Materials.AMBIENT_MARMOR);
		doorDecor.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		doorDecor.setSpecularFactor(Materials.SPECULAR_MARMOR);
		doorDecor.shinessFactor = Materials.SHINESS_MARMOR;

		doorDecor.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, doorDecor);

		doorDecor = new Cube(0.8f, 2.5f, 0.2f, 1, 1);

		trans.setIdentity();
		float[] translat22 = { 40 - 1f, 2f, 26.8f, 1 };

		trans.translate(translat22);
		doorDecor.updateTransformationState(trans);
		doorDecor.setTextureID("marmor");

		doorDecor.setAmbientFactor(Materials.AMBIENT_MARMOR);
		doorDecor.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		doorDecor.setSpecularFactor(Materials.SPECULAR_MARMOR);
		doorDecor.shinessFactor = Materials.SHINESS_MARMOR;

		doorDecor.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, doorDecor);

		Arc doorArc = new Arc(0, (float) Math.PI, 0.8f, 0.4f, 0.8f, 20, 0.2f,
				1.0f);

		trans.setIdentity();
		float[] translat23 = { 40 - 1f, 2.f + 2.5f, 26.25f, 1 };

		trans.translate(translat23);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		doorArc.updateTransformationState(trans);
		doorArc.setTextureID("marmor");

		doorArc.setAmbientFactor(Materials.AMBIENT_MARMOR);
		doorArc.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		doorArc.setSpecularFactor(Materials.SPECULAR_MARMOR);
		doorArc.shinessFactor = Materials.SHINESS_MARMOR;

		doorArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, doorArc);

		// -------------------------------------------------------------------
		// decorwalls
		decorWall = new Cube(0.2f, 3, 13, 1, 0.3f);

		trans.setIdentity();
		float[] t31 = { 24.8f, 0, 17, 1 };

		trans.translate(t31);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		for (int i = 0; i < 8; i++) {
			Cube decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

			trans.setIdentity();
			float[] t7 = { 24.8f - 0.2f, 3f, 17 + (i * 1.86f) - 0.1f, 1 };
			trans.translate(t7);

			decorStone.updateTransformationState(trans);
			decorStone.setTextureID("brick_wall");

			decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
			decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
			decorStone.shinessFactor = Materials.SHINESS_WALL;

			decorStone.setBufferID(bufferController.getBufferID());

			Cube decorCube = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

			Node.addChildrenToNode(groundNode, decorStone);

			trans.setIdentity();
			float[] t4 = { 24.8f - 0.1f, 0, 17 + (i * 1.86f), 1 };

			trans.translate(t4);
			decorCube.updateTransformationState(trans);
			decorCube.setTextureID("brick_wall");

			decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
			decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
			decorCube.shinessFactor = Materials.SHINESS_WALL;

			decorCube.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, decorCube);
		}

		bridgeArc1 = new Arc(0, (float) Math.PI, 0.8f, 0.4f, 0.7f, 10, 1f, 1);

		trans.setIdentity();
		float[] t36 = { 24.3f, 1.2f, 20f, 1 };

		trans.translate(t36);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		bridgeArc1.updateTransformationState(trans);
		bridgeArc1.setTextureID("ground");

		bridgeArc1.setAmbientFactor(Materials.AMBIENT_GROUND);
		bridgeArc1.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		bridgeArc1.setSpecularFactor(Materials.SPECULAR_GROUND);
		bridgeArc1.shinessFactor = Materials.SHINESS_GROUND;

		bridgeArc1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc1);

		bridgeArc1 = new Arc(0, (float) Math.PI, 0.8f, 0.4f, 0.7f, 10, 1f, 1);

		trans.setIdentity();
		float[] t37 = { 24.3f, 1.2f, 27.5f, 1 };

		trans.translate(t37);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		bridgeArc1.updateTransformationState(trans);
		bridgeArc1.setTextureID("ground");

		bridgeArc1.setAmbientFactor(Materials.AMBIENT_GROUND);
		bridgeArc1.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		bridgeArc1.setSpecularFactor(Materials.SPECULAR_GROUND);
		bridgeArc1.shinessFactor = Materials.SHINESS_GROUND;

		bridgeArc1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc1);

		decorWall = new Cube(0.2f, 3, 5, 1, 0.3f);

		trans.setIdentity();
		float[] t40 = { 24.8f, 0, 10, 1 };

		trans.translate(t40);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		for (int i = 0; i < 3; i++) {
			Cube decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

			trans.setIdentity();
			float[] t7 = { 24.8f - 0.2f, 3f, 14.6f - (i * 2.5f), 1 };
			trans.translate(t7);

			decorStone.updateTransformationState(trans);
			decorStone.setTextureID("brick_wall");

			decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
			decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
			decorStone.shinessFactor = Materials.SHINESS_WALL;

			decorStone.setBufferID(bufferController.getBufferID());
			Node.addChildrenToNode(groundNode, decorStone);

			Cube decorCube = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

			trans.setIdentity();
			float[] t4 = { 24.8f - 0.1f, 0, 14.7f - (i * 2.5f), 1 };

			trans.translate(t4);
			decorCube.updateTransformationState(trans);
			decorCube.setTextureID("brick_wall");

			decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
			decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
			decorCube.shinessFactor = Materials.SHINESS_WALL;

			decorCube.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, decorCube);
		}

		bridgeArc1 = new Arc(0, (float) Math.PI, 0.8f, 0.4f, 0.7f, 10, 1f, 1);

		trans.setIdentity();
		float[] t41 = { 24.3f, 1.2f, 11f, 1 };

		trans.translate(t41);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		bridgeArc1.updateTransformationState(trans);
		bridgeArc1.setTextureID("ground");

		bridgeArc1.setAmbientFactor(Materials.AMBIENT_GROUND);
		bridgeArc1.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		bridgeArc1.setSpecularFactor(Materials.SPECULAR_GROUND);
		bridgeArc1.shinessFactor = Materials.SHINESS_GROUND;

		bridgeArc1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc1);

		decorWall = new Cube(10f, 3, 0.2f, 1, 0.3f);

		trans.setIdentity();
		float[] t44 = { 25, 0, 30f, 1 };

		trans.translate(t44);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		for (int i = 0; i < 6; i++) {
			Cube decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

			trans.setIdentity();
			float[] t7 = { 24.8f - 0.1f + (i * 2.0f), 3f, 30f, 1 };
			trans.translate(t7);

			decorStone.updateTransformationState(trans);
			decorStone.setTextureID("brick_wall");

			decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
			decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
			decorStone.shinessFactor = Materials.SHINESS_WALL;

			decorStone.setBufferID(bufferController.getBufferID());
			Node.addChildrenToNode(groundNode, decorStone);

			Cube decorCube = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

			trans.setIdentity();
			float[] t4 = { 24.8f + (i * 2.0f), 0, 30f, 1 };

			trans.translate(t4);
			decorCube.updateTransformationState(trans);
			decorCube.setTextureID("brick_wall");

			decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
			decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
			decorCube.shinessFactor = Materials.SHINESS_WALL;

			decorCube.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, decorCube);
		}

		decorWall = new Cube(10f, 3, 0.2f, 1, 0.3f);

		trans.setIdentity();
		float[] t45 = { 25, 0, 10f - 0.2f, 1 };

		trans.translate(t45);
		decorWall.updateTransformationState(trans);
		decorWall.setTextureID("brick_wall");

		decorWall.setAmbientFactor(Materials.AMBIENT_WALL);
		decorWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorWall.setSpecularFactor(Materials.SPECULAR_WALL);
		decorWall.shinessFactor = Materials.SHINESS_WALL;

		decorWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorWall);

		for (int i = 0; i < 6; i++) {
			Cube decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

			trans.setIdentity();
			float[] t7 = { 24.8f - 0.1f + (i * 2.0f), 3f, 10f - 0.4f, 1 };
			trans.translate(t7);

			decorStone.updateTransformationState(trans);
			decorStone.setTextureID("brick_wall");

			decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
			decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
			decorStone.shinessFactor = Materials.SHINESS_WALL;

			decorStone.setBufferID(bufferController.getBufferID());
			Node.addChildrenToNode(groundNode, decorStone);

			Cube decorCube = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

			trans.setIdentity();
			float[] t4 = { 24.8f + (i * 2.0f), 0, 10f - 0.4f, 1 };

			trans.translate(t4);
			decorCube.updateTransformationState(trans);
			decorCube.setTextureID("brick_wall");

			decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
			decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
			decorCube.shinessFactor = Materials.SHINESS_WALL;

			decorCube.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, decorCube);
		}
		// -----------------------------------------------------------------------
		// pillars
		pillar1 = new Cube(1, 20, 1, 0.15f, 4f);

		trans.setIdentity();
		float[] t20 = { 30, 2, 12, 1 };
		trans.translate(t20);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(2.2f, 2.2f, 2.2f, 0.3f, 0.4f);

		trans.setIdentity();
		float[] tqa10 = { 30 - 0.6f, 2, 12 - 0.6f, 1 };
		trans.translate(tqa10);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] t21 = { 29 + 0.5f, 2, 11 + 0.5f, 1 };
		trans.translate(t21);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] t22 = { 31f, 2, 11 + 0.5f, 1 };
		trans.translate(t22);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] t23 = { 31f, 2, 13, 1 };
		trans.translate(t23);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] t24 = { 29 + 0.5f, 2, 13, 1 };
		trans.translate(t24);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(1, 20, 1, 0.15f, 5f);

		trans.setIdentity();
		float[] t25 = { 30, 2, 27, 1 };
		trans.translate(t25);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(2.2f, 2.2f, 2.2f, 0.3f, 0.4f);

		trans.setIdentity();
		float[] tqw10 = { 30 - 0.6f, 2, 27 - 0.6f, 1 };
		trans.translate(tqw10);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] t26 = { 29 + 0.5f, 2, 26 + 0.5f, 1 };
		trans.translate(t26);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] t27 = { 31f, 2, 26 + 0.5f, 1 };
		trans.translate(t27);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] t28 = { 31f, 2, 28, 1 };
		trans.translate(t28);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] t29 = { 29 + 0.5f, 2, 28, 1 };
		trans.translate(t29);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(1, 20, 1, 0.15f, 4f);

		trans.setIdentity();
		float[] k10 = { 30, 2, 19.5f, 1 };
		trans.translate(k10);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(2.2f, 2.2f, 2.2f, 0.3f, 0.4f);

		trans.setIdentity();
		float[] tq10s = { 30 - 0.6f, 2, 19.5f - 0.6f, 1 };
		trans.translate(tq10s);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] k11 = { 29 + 0.5f, 2, 19, 1 };
		trans.translate(k11);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] k12 = { 31f, 2, 19, 1 };
		trans.translate(k12);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] k13 = { 31f, 2, 20.5f, 1 };
		trans.translate(k13);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 20, 0.5f, 0.15f, 5f);

		trans.setIdentity();
		float[] k14 = { 29 + 0.5f, 2, 20.5f, 1 };
		trans.translate(k14);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		bridgeArc4 = new Arc(0, (float) Math.PI, 0.8f, 1.f, 3.8f, 30, 0.1f,
				0.2f);

		trans.setIdentity();
		float[] k34 = { 30.1f, 14f, 16.2f, 1 };

		trans.translate(k34);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		bridgeArc4.updateTransformationState(trans);
		bridgeArc4.setTextureID("brick_wall");

		bridgeArc4.setAmbientFactor(Materials.AMBIENT_WALL);
		bridgeArc4.setDiffuseFactor(Materials.DIFFUSE_WALL);
		bridgeArc4.setSpecularFactor(Materials.SPECULAR_WALL);
		bridgeArc4.shinessFactor = Materials.SHINESS_WALL;

		bridgeArc4.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc4);

		bridgeArc4 = new Arc(0, (float) Math.PI, 0.8f, 1.f, 3.8f, 30, 0.1f,
				0.2f);

		trans.setIdentity();
		float[] k35 = { 30.1f, 14f, 23.9f, 1 };

		trans.translate(k35);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		bridgeArc4.updateTransformationState(trans);
		bridgeArc4.setTextureID("brick_wall");

		bridgeArc4.setAmbientFactor(Materials.AMBIENT_WALL);
		bridgeArc4.setDiffuseFactor(Materials.DIFFUSE_WALL);
		bridgeArc4.setSpecularFactor(Materials.SPECULAR_WALL);
		bridgeArc4.shinessFactor = Materials.SHINESS_WALL;

		bridgeArc4.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc4);

		bridgeArc4 = new Arc(0, (float) Math.PI, 0.8f, 1.f, 5.5f, 30, 0.1f,
				0.2f);

		trans.setIdentity();
		float[] k36 = { 35.5f, 12.5f, 27.1f, 1 };

		trans.translate(k36);
		bridgeArc4.updateTransformationState(trans);
		bridgeArc4.setTextureID("brick_wall");

		bridgeArc4.setAmbientFactor(Materials.AMBIENT_WALL);
		bridgeArc4.setDiffuseFactor(Materials.DIFFUSE_WALL);
		bridgeArc4.setSpecularFactor(Materials.SPECULAR_WALL);
		bridgeArc4.shinessFactor = Materials.SHINESS_WALL;

		bridgeArc4.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc4);

		bridgeArc4 = new Arc(0, (float) Math.PI, 0.8f, 1.f, 5.5f, 30, 0.1f,
				0.2f);

		trans.setIdentity();
		float[] k37 = { 35.5f, 12.5f, 12.1f, 1 };

		trans.translate(k37);
		bridgeArc4.updateTransformationState(trans);
		bridgeArc4.setTextureID("brick_wall");

		bridgeArc4.setAmbientFactor(Materials.AMBIENT_WALL);
		bridgeArc4.setDiffuseFactor(Materials.DIFFUSE_WALL);
		bridgeArc4.setSpecularFactor(Materials.SPECULAR_WALL);
		bridgeArc4.shinessFactor = Materials.SHINESS_WALL;

		bridgeArc4.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc4);
		// ------------------------------------------------------------------------
		// wheel

		Arc wheel_top = new Arc(0, (float) Math.PI, 0.8f, 1.f, 3f, 30, 0.1f,
				0.2f);

		trans.setIdentity();
		float[] k57 = { 20f, 3.f, 25.1f, 1 };

		trans.translate(k57);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		wheel_top.updateTransformationState(trans);
		wheel_top.setTextureID("wood");

		wheel_top.setAmbientFactor(Materials.AMBIENT_WOOD);
		wheel_top.setDiffuseFactor(Materials.DIFFUSE_WOOD);
		wheel_top.setSpecularFactor(Materials.SPECULAR_WOOD);
		wheel_top.shinessFactor = Materials.SHINESS_WOOD;

		wheel_top.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, wheel_top);

		Arc wheel_bottom = new Arc(0, (float) Math.PI, 0.8f, 1.f, 3f, 30, 0.1f,
				0.2f);

		trans.setIdentity();
		float[] k58 = { 20f, 3.f, 25.1f, 1 };

		trans.translate(k58);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 180);
		wheel_bottom.updateTransformationState(trans);
		wheel_bottom.setTextureID("wood");

		wheel_bottom.setAmbientFactor(Materials.AMBIENT_WOOD);
		wheel_bottom.setDiffuseFactor(Materials.DIFFUSE_WOOD);
		wheel_bottom.setSpecularFactor(Materials.SPECULAR_WOOD);
		wheel_bottom.shinessFactor = Materials.SHINESS_WOOD;

		wheel_bottom.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, wheel_bottom);
		float alpha = 0;

		Cube wheelAxe = new Cube(0.25f, 5, 0.25f, 0.1f, 1.5f);

		trans.setIdentity();
		float[] k201 = { 20.2f, 0.f, 24.9f, 1 };
		trans.translate(k201);

		wheelAxe.updateTransformationState(trans);
		wheelAxe.setTextureID("wood");

		wheelAxe.setAmbientFactor(Materials.AMBIENT_WOOD);
		wheelAxe.setDiffuseFactor(Materials.DIFFUSE_WOOD);
		wheelAxe.setSpecularFactor(Materials.SPECULAR_WOOD);
		wheelAxe.shinessFactor = Materials.SHINESS_WOOD;

		wheelAxe.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, wheelAxe);

		wheelAxe = new Cube(0.25f, 5.5f, 0.25f, 0.1f, 1.5f);

		trans.setIdentity();
		float[] k202 = { 20.2f, 3.f, 22.1f, 1 };
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 90);
		trans.translate(k202);

		wheelAxe.updateTransformationState(trans);
		wheelAxe.setTextureID("wood");

		wheelAxe.setAmbientFactor(Materials.AMBIENT_WOOD);
		wheelAxe.setDiffuseFactor(Materials.DIFFUSE_WOOD);
		wheelAxe.setSpecularFactor(Materials.SPECULAR_WOOD);
		wheelAxe.shinessFactor = Materials.SHINESS_WOOD;

		wheelAxe.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, wheelAxe);

		Cylinder wheelCyl = new Cylinder(10, 0.15f, 4.f, 0.1f, 1f);

		trans.setIdentity();
		float[] k203 = { 18.5f, 3.f, 25.0f, 1 };
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -90);
		trans.translate(k203);

		wheelCyl.updateTransformationState(trans);
		wheelCyl.setTextureID("wood");

		wheelCyl.setAmbientFactor(Materials.AMBIENT_WOOD);
		wheelCyl.setDiffuseFactor(Materials.DIFFUSE_WOOD);
		wheelCyl.setSpecularFactor(Materials.SPECULAR_WOOD);
		wheelCyl.shinessFactor = Materials.SHINESS_WOOD;

		wheelCyl.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, wheelCyl);

		for (int i = 0; alpha < 2 * Math.PI; i++) {
			Cube wheel_decor = new Cube(0.8f, 0.7f, 0.3f, 1f, 1f);

			float zt = 3 * (float) Math.cos(alpha);
			float yt = 3 * (float) Math.sin(alpha);

			trans.setIdentity();
			float[] k59 = { 20f, yt, zt, 1 };
			float[] k60 = { .8f, 2.6f, 25.3f, 1 };

			trans.translate(k59);
			trans.translate(k60);
			trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 180);
			wheel_decor.updateTransformationState(trans);
			wheel_decor.setTextureID("wood");

			wheel_decor.setAmbientFactor(Materials.AMBIENT_WOOD);
			wheel_decor.setDiffuseFactor(Materials.DIFFUSE_WOOD);
			wheel_decor.setSpecularFactor(Materials.SPECULAR_WOOD);
			wheel_decor.shinessFactor = Materials.SHINESS_WOOD;

			wheel_decor.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, wheel_decor);

			alpha += 0.4f;
		}
		// ------------------------------------------------------
		// structure on wheel
		Cube templeGround = new Cube(3.7f, 0.3f, 3.7f, 1f, 1f);

		trans.setIdentity();
		float[] ak2021 = { 18.65f, 10 + 0.2f, 23.15f, 1 };
		trans.translate(ak2021);

		templeGround.updateTransformationState(trans);
		templeGround.setTextureID("marmor");

		templeGround.setAmbientFactor(Materials.AMBIENT_MARMOR);
		templeGround.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		templeGround.setSpecularFactor(Materials.SPECULAR_MARMOR);
		templeGround.shinessFactor = Materials.SHINESS_MARMOR;

		templeGround.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, templeGround);

		templeGround = new Cube(3.2f, 2.1f, 0.2f, 1f, 1f);

		trans.setIdentity();
		float[] a21 = { 19f - 0.1f, 10 + 0.2f + 0.4f, 25f, 1 };
		trans.translate(a21);

		templeGround.updateTransformationState(trans);
		templeGround.setTextureID("marmor");

		templeGround.setAmbientFactor(Materials.AMBIENT_MARMOR);
		templeGround.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		templeGround.setSpecularFactor(Materials.SPECULAR_MARMOR);
		templeGround.shinessFactor = Materials.SHINESS_MARMOR;

		templeGround.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, templeGround);

		templeGround = new Cube(0.3f, 0.3f, 0.3f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] a21e = { 19f - 0.15f, 10 + 0.2f + 0.35f, 25f - 0.05f, 1 };
		trans.translate(a21e);

		templeGround.updateTransformationState(trans);
		templeGround.setTextureID("marmor");

		templeGround.setAmbientFactor(Materials.AMBIENT_MARMOR);
		templeGround.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		templeGround.setSpecularFactor(Materials.SPECULAR_MARMOR);
		templeGround.shinessFactor = Materials.SHINESS_MARMOR;

		templeGround.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, templeGround);

		templeGround = new Cube(0.3f, 0.3f, 0.3f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] a21ee = { 19f - 0.15f, 10 + 0.2f + 2.2f, 25f - 0.05f, 1 };
		trans.translate(a21ee);

		templeGround.updateTransformationState(trans);
		templeGround.setTextureID("marmor");

		templeGround.setAmbientFactor(Materials.AMBIENT_MARMOR);
		templeGround.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		templeGround.setSpecularFactor(Materials.SPECULAR_MARMOR);
		templeGround.shinessFactor = Materials.SHINESS_MARMOR;

		templeGround.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, templeGround);

		templeGround = new Cube(0.3f, 0.3f, 0.3f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] a21ef = { 19f - 0.15f + 3, 10 + 0.2f + 0.35f, 25f - 0.05f, 1 };
		trans.translate(a21ef);

		templeGround.updateTransformationState(trans);
		templeGround.setTextureID("marmor");

		templeGround.setAmbientFactor(Materials.AMBIENT_MARMOR);
		templeGround.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		templeGround.setSpecularFactor(Materials.SPECULAR_MARMOR);
		templeGround.shinessFactor = Materials.SHINESS_MARMOR;

		templeGround.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, templeGround);

		templeGround = new Cube(0.3f, 0.3f, 0.3f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] a21eef = { 19f - 0.15f + 3, 10 + 0.2f + 2.2f, 25f - 0.05f, 1 };
		trans.translate(a21eef);

		templeGround.updateTransformationState(trans);
		templeGround.setTextureID("marmor");

		templeGround.setAmbientFactor(Materials.AMBIENT_MARMOR);
		templeGround.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		templeGround.setSpecularFactor(Materials.SPECULAR_MARMOR);
		templeGround.shinessFactor = Materials.SHINESS_MARMOR;

		templeGround.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, templeGround);

		templeGround = new Cube(0.2f, 2.1f, 3.2f, 1f, 1f);

		trans.setIdentity();
		float[] a215 = { 18.65f + 1.6f + 0.2f, 10 + 0.2f + 0.4f,
				23.15f + 0.25f, 1 };
		trans.translate(a215);

		templeGround.updateTransformationState(trans);
		templeGround.setTextureID("marmor");

		templeGround.setAmbientFactor(Materials.AMBIENT_MARMOR);
		templeGround.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		templeGround.setSpecularFactor(Materials.SPECULAR_MARMOR);
		templeGround.shinessFactor = Materials.SHINESS_MARMOR;

		templeGround.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, templeGround);

		templeGround = new Cube(0.3f, 0.3f, 0.3f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] a2f = { 20.45f - 0.05f, 10.6f, 23.4f - 0.05f, 1 };
		trans.translate(a2f);

		templeGround.updateTransformationState(trans);
		templeGround.setTextureID("marmor");

		templeGround.setAmbientFactor(Materials.AMBIENT_MARMOR);
		templeGround.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		templeGround.setSpecularFactor(Materials.SPECULAR_MARMOR);
		templeGround.shinessFactor = Materials.SHINESS_MARMOR;

		templeGround.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, templeGround);

		templeGround = new Cube(0.3f, 0.3f, 0.3f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] a21f = { 20.45f - 0.05f, 10 + 0.2f + 2.2f, 23.4f - 0.05f, 1 };
		trans.translate(a21f);

		templeGround.updateTransformationState(trans);
		templeGround.setTextureID("marmor");

		templeGround.setAmbientFactor(Materials.AMBIENT_MARMOR);
		templeGround.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		templeGround.setSpecularFactor(Materials.SPECULAR_MARMOR);
		templeGround.shinessFactor = Materials.SHINESS_MARMOR;

		templeGround.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, templeGround);

		templeGround = new Cube(0.3f, 0.3f, 0.3f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] a2sf = { 20.45f - 0.05f, 10.6f, 23.4f - 0.05f + 3, 1 };
		trans.translate(a2sf);

		templeGround.updateTransformationState(trans);
		templeGround.setTextureID("marmor");

		templeGround.setAmbientFactor(Materials.AMBIENT_MARMOR);
		templeGround.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		templeGround.setSpecularFactor(Materials.SPECULAR_MARMOR);
		templeGround.shinessFactor = Materials.SHINESS_MARMOR;

		templeGround.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, templeGround);

		templeGround = new Cube(0.3f, 0.3f, 0.3f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] a21sf = { 20.45f - 0.05f, 10 + 0.2f + 2.2f, 23.4f - 0.05f + 3,
				1 };
		trans.translate(a21sf);

		templeGround.updateTransformationState(trans);
		templeGround.setTextureID("marmor");

		templeGround.setAmbientFactor(Materials.AMBIENT_MARMOR);
		templeGround.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		templeGround.setSpecularFactor(Materials.SPECULAR_MARMOR);
		templeGround.shinessFactor = Materials.SHINESS_MARMOR;

		templeGround.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, templeGround);

		EllipsoidArc tempelKuppel = new EllipsoidArc(1.6f, 2.2f, 1.6f,
				(float) -Math.PI / 2, (float) Math.PI / 2, 0, (float) Math.PI,
				40, 40);

		trans.setIdentity();
		float[] r812 = { 20.5f, 10 + 0.2f + 0.4f, 25, 1 };

		trans.translate(r812);

		tempelKuppel.updateTransformationState(trans);
		tempelKuppel.setTextureID("marmor");

		tempelKuppel.setAmbientFactor(Materials.AMBIENT_MARMOR);
		tempelKuppel.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		tempelKuppel.setSpecularFactor(Materials.SPECULAR_MARMOR);
		tempelKuppel.shinessFactor = Materials.SHINESS_MARMOR;

		tempelKuppel.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, tempelKuppel);
		// ------------------------------------------------------

		// ------------------------------------------------------
		Cube pillarGround = new Cube(5.7f, 0.3f, 6.7f, 1f, 1f);

		trans.setIdentity();
		float[] k2021 = { 18 - 0.35f, 10, 22 - 0.35f, 1 };
		trans.translate(k2021);

		pillarGround.updateTransformationState(trans);
		pillarGround.setTextureID("marmor");

		pillarGround.setAmbientFactor(Materials.AMBIENT_MARMOR);
		pillarGround.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		pillarGround.setSpecularFactor(Materials.SPECULAR_MARMOR);
		pillarGround.shinessFactor = Materials.SHINESS_MARMOR;

		pillarGround.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillarGround);

		pillar1 = new Cube(0.5f, 10, 0.5f, 0.15f, 2.5f);

		trans.setIdentity();
		float[] k20 = { 18, 0, 22, 1 };
		trans.translate(k20);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(1.25f, 1f, 1.25f, 0.3f, 0.3f);

		trans.setIdentity();
		float[] k20d = { 18 - 0.35f, 9, 22 - 0.35f, 1 };
		trans.translate(k20d);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(1.25f, 1f, 1.25f, 0.3f, 0.3f);

		trans.setIdentity();
		float[] k20df = { 18 - 0.35f, 0, 22 - 0.35f, 1 };
		trans.translate(k20df);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k21 = { 18 - 0.25f, 0, 22 - 0.25f, 1 };
		trans.translate(k21);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k22 = { 18 - 0.25f, 0, 22 + 0.5f, 1 };
		trans.translate(k22);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k23 = { 18 + 0.5f, 0, 22 - 0.25f, 1 };
		trans.translate(k23);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k24 = { 18 + 0.5f, 0, 22 + 0.5f, 1 };
		trans.translate(k24);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 10, 0.5f, 0.15f, 2.5f);

		trans.setIdentity();
		float[] k25 = { 18, 0, 22 + 5.5f, 1 };
		trans.translate(k25);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(1.25f, 1f, 1.25f, 0.3f, 0.3f);

		trans.setIdentity();
		float[] k20ad = { 18 - 0.35f, 9, 28.35f - 1.25f, 1 };
		trans.translate(k20ad);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(1.25f, 1f, 1.25f, 0.3f, 0.3f);

		trans.setIdentity();
		float[] k20acd = { 18 - 0.35f, 0, 28.35f - 1.25f, 1 };
		trans.translate(k20acd);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k26 = { 18 - 0.25f, 0, 22 - 0.25f + 5.5f, 1 };
		trans.translate(k26);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k27 = { 18 - 0.25f, 0, 22 + 0.5f + 5.5f, 1 };
		trans.translate(k27);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k28 = { 18 + 0.5f, 0, 22 - 0.25f + 5.5f, 1 };
		trans.translate(k28);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k29 = { 18 + 0.5f, 0, 22 + 0.5f + 5.5f, 1 };
		trans.translate(k29);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 10, 0.5f, 0.15f, 2.5f);

		trans.setIdentity();
		float[] k30 = { 18 + 4.5f, 0, 22, 1 };
		trans.translate(k30);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(1.25f, 1f, 1.25f, 0.3f, 0.3f);

		trans.setIdentity();
		float[] k20asd = { 22.1f, 9, 28.35f - 1.25f, 1 };
		trans.translate(k20asd);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(1.25f, 1f, 1.25f, 0.3f, 0.3f);

		trans.setIdentity();
		float[] kx20asd = { 22.1f, 0, 28.35f - 1.25f, 1 };
		trans.translate(kx20asd);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k31 = { 18 - 0.25f + 4.5f, 0, 22 - 0.25f, 1 };
		trans.translate(k31);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");
		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k32 = { 18 - 0.25f + 4.5f, 0, 22 + 0.5f, 1 };
		trans.translate(k32);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k33 = { 18 + 0.5f + 4.5f, 0, 22 - 0.25f, 1 };
		trans.translate(k33);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k44 = { 18 + 0.5f + 4.5f, 0, 22 + 0.5f, 1 };
		trans.translate(k44);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 10, 0.5f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k40 = { 18 + 4.5f, 0, 22 + 5.5f, 1 };
		trans.translate(k40);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(1.25f, 1f, 1.25f, 0.3f, 0.3f);

		trans.setIdentity();
		float[] k20dsd = { 22.1f, 9, 22 - 0.35f, 1 };
		trans.translate(k20dsd);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(1.25f, 1f, 1.25f, 0.3f, 0.3f);

		trans.setIdentity();
		float[] k20dssd = { 22.1f, 0, 22 - 0.35f, 1 };
		trans.translate(k20dssd);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k41 = { 18 - 0.25f + 4.5f, 0, 22 - 0.25f + 5.5f, 1 };
		trans.translate(k41);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k42 = { 18 - 0.25f + 4.5f, 0, 22 + 0.5f + 5.5f, 1 };
		trans.translate(k42);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k43 = { 18 + 0.5f + 4.5f, 0, 22 - 0.25f + 5.5f, 1 };
		trans.translate(k43);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.25f, 10, 0.25f, 0.07f, 2.5f);

		trans.setIdentity();
		float[] k45 = { 18 + 0.5f + 4.5f, 0, 22 + 0.5f + 5.5f, 1 };
		trans.translate(k45);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 5.5f, 0.5f, 0.15f, 2.5f);

		trans.setIdentity();
		float[] k46 = { 18, 3, 22, 1 };
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 90);
		trans.translate(k46);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 5.5f, 0.5f, 0.15f, 2.5f);

		trans.setIdentity();
		float[] k47 = { 18 + 4.5f, 3, 22, 1 };
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 90);
		trans.translate(k47);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 4.f, 0.5f, 0.15f, 2.5f);

		trans.setIdentity();
		float[] k48 = { 18 + 0.5f, 7, 22, 1 };
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -90);
		trans.translate(k48);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		pillar1 = new Cube(0.5f, 4.f, 0.5f, 0.15f, 2.5f);

		trans.setIdentity();
		float[] k49 = { 18 + 0.5f, 7, 22 + 5.5f, 1 };
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Z, -90);
		trans.translate(k49);

		pillar1.updateTransformationState(trans);
		pillar1.setTextureID("brick_wall");

		pillar1.setAmbientFactor(Materials.AMBIENT_WALL);
		pillar1.setDiffuseFactor(Materials.DIFFUSE_WALL);
		pillar1.setSpecularFactor(Materials.SPECULAR_WALL);
		pillar1.shinessFactor = Materials.SHINESS_WALL;

		pillar1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, pillar1);

		// -------------------------------------------------------------------------
		// bridge
		Cube bridge1 = new Cube(10, 2, 2, 1f, 0.3f);

		trans.setIdentity();
		float[] translate4 = { 15, 0, 15, 1 };

		trans.translate(translate4);
		bridge1.updateTransformationState(trans);
		bridge1.setTextureID("ground");

		bridge1.setAmbientFactor(Materials.AMBIENT_GROUND);
		bridge1.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		bridge1.setSpecularFactor(Materials.SPECULAR_GROUND);
		bridge1.shinessFactor = Materials.SHINESS_GROUND;

		bridge1.setBufferID(bufferController.getBufferID());

		Cube leftBridgeWall = new Cube(10, 3, 0.2f, 1, 0.3f);

		trans.setIdentity();
		float[] t2 = { 15, 0, 14.8f, 1 };

		trans.translate(t2);
		leftBridgeWall.updateTransformationState(trans);
		leftBridgeWall.setTextureID("brick_wall");

		leftBridgeWall.setAmbientFactor(Materials.AMBIENT_WALL);
		leftBridgeWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		leftBridgeWall.setSpecularFactor(Materials.SPECULAR_WALL);
		leftBridgeWall.shinessFactor = Materials.SHINESS_WALL;

		leftBridgeWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, leftBridgeWall);

		Cube rightBridgeWall = new Cube(10, 3, 0.2f, 1, 0.3f);

		trans.setIdentity();
		float[] t3 = { 15, 0, 17, 1 };

		trans.translate(t3);
		rightBridgeWall.updateTransformationState(trans);
		rightBridgeWall.setTextureID("brick_wall");

		rightBridgeWall.setAmbientFactor(Materials.AMBIENT_WALL);
		rightBridgeWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		rightBridgeWall.setSpecularFactor(Materials.SPECULAR_WALL);
		rightBridgeWall.shinessFactor = Materials.SHINESS_WALL;

		rightBridgeWall.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, rightBridgeWall);

		Cube decorGate = new Cube(0.3f, 5.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] r7 = { 15, 3f, 17, 1 };
		trans.translate(r7);

		decorGate.updateTransformationState(trans);
		decorGate.setTextureID("brick_wall");

		decorGate.setAmbientFactor(Materials.AMBIENT_WALL);
		decorGate.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorGate.setSpecularFactor(Materials.SPECULAR_WALL);
		decorGate.shinessFactor = Materials.SHINESS_WALL;

		decorGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorGate);

		decorGate = new Cube(0.3f, 5.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] r8 = { 15, 3f, 14.7f, 1 };
		trans.translate(r8);

		decorGate.updateTransformationState(trans);
		decorGate.setTextureID("brick_wall");

		decorGate.setAmbientFactor(Materials.AMBIENT_WALL);
		decorGate.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorGate.setSpecularFactor(Materials.SPECULAR_WALL);
		decorGate.shinessFactor = Materials.SHINESS_WALL;

		decorGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorGate);

		decorGate = new Cube(0.3f, 5.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] r9 = { 15 + (5 * 1.93f), 3f, 17, 1 };
		trans.translate(r9);

		decorGate.updateTransformationState(trans);
		decorGate.setTextureID("brick_wall");

		decorGate.setAmbientFactor(Materials.AMBIENT_WALL);
		decorGate.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorGate.setSpecularFactor(Materials.SPECULAR_WALL);
		decorGate.shinessFactor = Materials.SHINESS_WALL;

		decorGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorGate);

		decorGate = new Cube(0.3f, 5.7f, 0.3f, 0.1f, 1f);

		trans.setIdentity();
		float[] r10 = { 15 + (5 * 1.93f), 3f, 14.7f, 1 };
		trans.translate(r10);

		decorGate.updateTransformationState(trans);
		decorGate.setTextureID("brick_wall");

		decorGate.setAmbientFactor(Materials.AMBIENT_WALL);
		decorGate.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorGate.setSpecularFactor(Materials.SPECULAR_WALL);
		decorGate.shinessFactor = Materials.SHINESS_WALL;

		decorGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorGate);

		Arc decorArc = new Arc(0, (float) Math.PI, 0.3f, 0.51f, 1.29f, 20,
				0.1f, 0.1f);

		trans.setIdentity();
		float[] r11 = { 15, 8.7f, 16f, 1 };
		trans.translate(r11);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);

		decorArc.updateTransformationState(trans);
		decorArc.setTextureID("brick_wall");

		decorArc.setAmbientFactor(Materials.AMBIENT_WALL);
		decorArc.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorArc.setSpecularFactor(Materials.SPECULAR_WALL);
		decorGate.shinessFactor = Materials.SHINESS_WALL;

		decorArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorArc);

		decorArc = new Arc(0, (float) Math.PI, 0.3f, 0.51f, 1.29f, 20, 0.1f,
				0.1f);

		trans.setIdentity();
		float[] r12 = { 15 + (5 * 1.93f), 8.7f, 16f, 1 };
		trans.translate(r12);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);

		decorArc.updateTransformationState(trans);
		decorArc.setTextureID("brick_wall");

		decorArc.setAmbientFactor(Materials.AMBIENT_WALL);
		decorArc.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorArc.setSpecularFactor(Materials.SPECULAR_WALL);
		decorArc.shinessFactor = Materials.SHINESS_WALL;

		decorArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorArc);

		for (int i = 0; i < 6; i++) {
			Cube decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

			trans.setIdentity();
			float[] t7 = { 15 + (i * 1.93f) - 0.1f, 3f, 17 - 0.1f, 1 };
			trans.translate(t7);

			decorStone.updateTransformationState(trans);
			decorStone.setTextureID("brick_wall");

			decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
			decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
			decorStone.shinessFactor = Materials.SHINESS_WALL;

			decorStone.setBufferID(bufferController.getBufferID());

			Cube decorCube = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

			Node.addChildrenToNode(groundNode, decorStone);

			trans.setIdentity();
			float[] t4 = { 15 + (i * 1.93f), 0, 17, 1 };

			trans.translate(t4);
			decorCube.updateTransformationState(trans);
			decorCube.setTextureID("brick_wall");

			decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
			decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
			decorCube.shinessFactor = Materials.SHINESS_WALL;

			decorCube.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, decorCube);
		}

		for (int i = 0; i < 6; i++) {
			Cube decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

			trans.setIdentity();
			float[] t7 = { 15 + (i * 1.93f) - 0.1f, 3f, 14.6f, 1 };
			trans.translate(t7);

			decorStone.updateTransformationState(trans);
			decorStone.setTextureID("brick_wall");

			decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
			decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
			decorStone.shinessFactor = Materials.SHINESS_WALL;

			decorStone.setBufferID(bufferController.getBufferID());
			Node.addChildrenToNode(groundNode, decorStone);

			Cube decorCube = new Cube(0.3f, 3.7f, 0.3f, 0.1f, 1f);

			trans.setIdentity();
			float[] t4 = { 15 + (i * 1.93f), 0, 14.7f, 1 };

			trans.translate(t4);
			decorCube.updateTransformationState(trans);
			decorCube.setTextureID("brick_wall");

			decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
			decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
			decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
			decorCube.shinessFactor = Materials.SHINESS_WALL;

			decorCube.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, decorCube);
		}

		Arc bridgeArc = new Arc(0, (float) Math.PI, 0.8f, 0.4f, 0.7f, 10, 1f, 1);

		trans.setIdentity();
		float[] t5 = { 18f, 1.2f, 17f, 1 };

		trans.translate(t5);
		bridgeArc.updateTransformationState(trans);
		bridgeArc.setTextureID("ground");

		bridgeArc.setAmbientFactor(Materials.AMBIENT_GROUND);
		bridgeArc.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		bridgeArc.setSpecularFactor(Materials.SPECULAR_GROUND);
		bridgeArc.shinessFactor = Materials.SHINESS_GROUND;

		bridgeArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc);

		bridgeArc = new Arc(0, (float) Math.PI, 0.8f, 0.4f, 0.7f, 10, 1f, 1);

		trans.setIdentity();
		float[] t6 = { 22f, 1.2f, 17f, 1 };

		trans.translate(t6);
		bridgeArc.updateTransformationState(trans);
		bridgeArc.setTextureID("ground");
		bridgeArc.setBufferID(bufferController.getBufferID());

		bridgeArc.setAmbientFactor(Materials.AMBIENT_GROUND);
		bridgeArc.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		bridgeArc.setSpecularFactor(Materials.SPECULAR_GROUND);
		bridgeArc.shinessFactor = Materials.SHINESS_GROUND;

		Node.addChildrenToNode(groundNode, bridgeArc);

		bridgeArc = new Arc(0, (float) Math.PI, 0.8f, 0.4f, 0.7f, 20, 1f, 1);

		trans.setIdentity();
		float[] t7 = { 22f, 1.2f, 14.2f, 1 };

		trans.translate(t7);
		bridgeArc.updateTransformationState(trans);
		bridgeArc.setTextureID("ground");

		bridgeArc.setAmbientFactor(Materials.AMBIENT_GROUND);
		bridgeArc.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		bridgeArc.setSpecularFactor(Materials.SPECULAR_GROUND);
		bridgeArc.shinessFactor = Materials.SHINESS_GROUND;

		bridgeArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc);

		bridgeArc = new Arc(0, (float) Math.PI, 0.8f, 0.4f, 0.7f, 10, 1f, 1);

		trans.setIdentity();
		float[] t8 = { 18f, 1.2f, 14.2f, 1 };

		trans.translate(t8);
		bridgeArc.updateTransformationState(trans);
		bridgeArc.setTextureID("ground");

		bridgeArc.setAmbientFactor(Materials.AMBIENT_GROUND);
		bridgeArc.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		bridgeArc.setSpecularFactor(Materials.SPECULAR_GROUND);
		bridgeArc.shinessFactor = Materials.SHINESS_GROUND;

		bridgeArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, bridgeArc);

		// -----------------------------------------------------------------------------------------
		Node.addChildrenToNode(groundNode, leftGround);
		Node.addChildrenToNode(groundNode, rightGround);

		Node.addChildrenToNode(groundNode, secondPlayGround);
		Node.addChildrenToNode(groundNode, bridge1);

		Arc sewerArc = new Arc(0, (float) Math.PI, 2.5f, 0.4f, 2.5f, 20, 0.2f,
				0.5f);

		trans.setIdentity();
		float[] translate15 = { 0f, 2, 7.5f, 1 };

		trans.translate(translate15);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		sewerArc.updateTransformationState(trans);
		sewerArc.setTextureID("ground");

		sewerArc.setAmbientFactor(Materials.AMBIENT_GROUND);
		sewerArc.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		sewerArc.setSpecularFactor(Materials.SPECULAR_GROUND);
		sewerArc.shinessFactor = Materials.SHINESS_GROUND;

		sewerArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerArc);

		sewerArc = new Arc(0, (float) Math.PI, .2f, 1.7f, 1.5f, 20, 0.2f, 0.5f);

		trans.setIdentity();
		float[] translate151 = { 0.3f, 1.2f, 7.5f, 1 };

		trans.translate(translate151);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		sewerArc.updateTransformationState(trans);
		sewerArc.setTextureID("door");

		sewerArc.setAmbientFactor(Materials.AMBIENT_DOOR);
		sewerArc.setDiffuseFactor(Materials.DIFFUSE_DOOR);
		sewerArc.setSpecularFactor(Materials.SPECULAR_DOOR);
		sewerArc.shinessFactor = Materials.SHINESS_DOOR;

		sewerArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerArc);

		// ------------------------------------------------------------------

		for (int i = 0, j = 0; i < 15; i++) {

			if (i < 7) {
				Cube sewerGate = new Cube(0.1f, 3, 0.1f, 1, 1);

				trans.setIdentity();
				float[] translate30 = { 2, 0 + i * (0.15f), 9.5f - i * (0.3f),
						1 };

				trans.translate(translate30);
				sewerGate.updateTransformationState(trans);
				sewerGate.setTextureID("octopus");

				sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
				sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
				sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
				sewerGate.shinessFactor = Materials.SHINESS_METAL;

				sewerGate.setBufferID(bufferController.getBufferID());

				Node.addChildrenToNode(groundNode, sewerGate);
			} else {
				Cube sewerGate = new Cube(0.1f, 3.2f, 0.1f, 1, 1);

				trans.setIdentity();
				float[] translate30 = { 2, 1.0f - j++ * (0.2f),
						9.5f - i * (0.3f), 1 };

				trans.translate(translate30);
				sewerGate.updateTransformationState(trans);
				sewerGate.setTextureID("octopus");

				sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
				sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
				sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
				sewerGate.shinessFactor = Materials.SHINESS_METAL;

				sewerGate.setBufferID(bufferController.getBufferID());

				Node.addChildrenToNode(groundNode, sewerGate);
			}

		}
		Cube sewerGate = new Cube(0.1f, 0.1f, 5, 1, 1);

		trans.setIdentity();
		float[] translate30 = { 2, 1f, 5, 1 };

		trans.translate(translate30);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);

		sewerGate = new Cube(0.1f, 0.1f, 5, 1, 1);

		trans.setIdentity();
		float[] translate31 = { 2, 1.6f, 5, 1 };

		trans.translate(translate31);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);

		sewerGate = new Cube(0.1f, 0.1f, 4.6f, 1, 1);

		trans.setIdentity();
		float[] translate32 = { 2, 2.2f, 5.2f, 1 };

		trans.translate(translate32);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);

		sewerGate = new Cube(0.1f, 0.1f, 4.4f, 1, 1);

		trans.setIdentity();
		float[] translate33 = { 2, 2.8f, 5.4f, 1 };

		trans.translate(translate33);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);

		sewerGate = new Cube(0.1f, 0.1f, 3.8f, 1, 1);

		trans.setIdentity();
		float[] translate34 = { 2, 3.4f, 5.6f, 1 };

		trans.translate(translate34);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);

		// ------------------------------------------------------------------
		for (int i = 0, j = 0; i < 15; i++) {

			if (i < 7) {
				Cube sewerGate1 = new Cube(0.1f, 3, 0.1f, 1, 1);

				trans.setIdentity();
				float[] translate35 = { 2, 0 + i * (0.15f), 34.5f - i * (0.3f),
						1 };

				trans.translate(translate35);
				sewerGate1.updateTransformationState(trans);
				sewerGate1.setTextureID("octopus");

				sewerGate1.setAmbientFactor(Materials.AMBIENT_MATAL);
				sewerGate1.setDiffuseFactor(Materials.DIFFUSE_MATAL);
				sewerGate1.setSpecularFactor(Materials.SPECULAR_METAL);
				sewerGate1.shinessFactor = Materials.SHINESS_METAL;

				sewerGate1.setBufferID(bufferController.getBufferID());

				Node.addChildrenToNode(groundNode, sewerGate1);
			} else {
				Cube sewerGate1 = new Cube(0.1f, 3.2f, 0.1f, 1, 1);

				trans.setIdentity();
				float[] translate35 = { 2, 1.0f - j++ * (0.2f),
						34.5f - i * (0.3f), 1 };

				trans.translate(translate35);
				sewerGate1.updateTransformationState(trans);
				sewerGate1.setTextureID("octopus");

				sewerGate1.setAmbientFactor(Materials.AMBIENT_MATAL);
				sewerGate1.setDiffuseFactor(Materials.DIFFUSE_MATAL);
				sewerGate1.setSpecularFactor(Materials.SPECULAR_METAL);

				sewerGate1.shinessFactor = Materials.SHINESS_METAL;
				sewerGate1.setBufferID(bufferController.getBufferID());

				Node.addChildrenToNode(groundNode, sewerGate1);
			}

		}
		Cube sewerGate1 = new Cube(0.1f, 0.1f, 5, 1, 1);

		trans.setIdentity();
		float[] translate35 = { 2, 1f, 30, 1 };

		trans.translate(translate35);
		sewerGate1.updateTransformationState(trans);
		sewerGate1.setTextureID("octopus");

		sewerGate1.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate1.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate1.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate1.shinessFactor = Materials.SHINESS_METAL;

		sewerGate1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate1);

		sewerGate1 = new Cube(0.1f, 0.1f, 5, 1, 1);

		trans.setIdentity();
		float[] translate36 = { 2, 1.6f, 30, 1 };

		trans.translate(translate36);
		sewerGate1.updateTransformationState(trans);
		sewerGate1.setTextureID("octopus");

		sewerGate1.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate1.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate1.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate1.shinessFactor = Materials.SHINESS_METAL;

		sewerGate1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate1);

		sewerGate = new Cube(0.1f, 0.1f, 4.6f, 1, 1);

		trans.setIdentity();
		float[] translate37 = { 2, 2.2f, 30.2f, 1 };

		trans.translate(translate37);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);

		sewerGate = new Cube(0.1f, 0.1f, 4.4f, 1, 1);

		trans.setIdentity();
		float[] translate38 = { 2, 2.8f, 30.4f, 1 };

		trans.translate(translate38);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);

		sewerGate = new Cube(0.1f, 0.1f, 3.8f, 1, 1);

		trans.setIdentity();
		float[] translate39 = { 2, 3.4f, 30.6f, 1 };

		trans.translate(translate39);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);

		// ------------------------------------------------------------------
		// 3.
		for (int i = 0, j = 0; i < 15; i++) {

			if (i < 7) {
				sewerGate = new Cube(0.1f, 3, 0.1f, 1, 1);

				trans.setIdentity();
				float[] translate40 = { 38, 0 + i * (0.15f), 9.5f - i * (0.3f),
						1 };

				trans.translate(translate40);
				sewerGate.updateTransformationState(trans);
				sewerGate.setTextureID("octopus");

				sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
				sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
				sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
				sewerGate.shinessFactor = Materials.SHINESS_METAL;

				sewerGate.setBufferID(bufferController.getBufferID());

				Node.addChildrenToNode(groundNode, sewerGate);
			} else {
				sewerGate = new Cube(0.1f, 3.2f, 0.1f, 1, 1);

				trans.setIdentity();
				float[] translate40 = { 38, 1.0f - j++ * (0.2f),
						9.5f - i * (0.3f), 1 };

				trans.translate(translate40);
				sewerGate.updateTransformationState(trans);
				sewerGate.setTextureID("octopus");

				sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
				sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
				sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
				sewerGate.shinessFactor = Materials.SHINESS_METAL;

				sewerGate.setBufferID(bufferController.getBufferID());

				Node.addChildrenToNode(groundNode, sewerGate);
			}

		}
		sewerGate = new Cube(0.1f, 0.1f, 5, 1, 1);

		trans.setIdentity();
		float[] translate41 = { 38, 1f, 5, 1 };

		trans.translate(translate41);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);

		sewerGate = new Cube(0.1f, 0.1f, 5, 1, 1);

		trans.setIdentity();
		float[] translate42 = { 38, 1.6f, 5, 1 };

		trans.translate(translate42);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);

		sewerGate = new Cube(0.1f, 0.1f, 4.6f, 1, 1);

		trans.setIdentity();
		float[] translate43 = { 38, 2.2f, 5.2f, 1 };

		trans.translate(translate43);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);

		sewerGate = new Cube(0.1f, 0.1f, 4.4f, 1, 1);

		trans.setIdentity();
		float[] translate44 = { 38, 2.8f, 5.4f, 1 };

		trans.translate(translate44);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);

		sewerGate = new Cube(0.1f, 0.1f, 3.8f, 1, 1);

		trans.setIdentity();
		float[] translate45 = { 38, 3.4f, 5.6f, 1 };

		trans.translate(translate45);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);

		// ------------------------------------------------------------------
		for (int i = 0, j = 0; i < 15; i++) {

			if (i < 7) {
				sewerGate1 = new Cube(0.1f, 3, 0.1f, 1, 1);

				trans.setIdentity();
				float[] translate46 = { 38, 0 + i * (0.15f),
						34.5f - i * (0.3f), 1 };

				trans.translate(translate46);
				sewerGate1.updateTransformationState(trans);
				sewerGate1.setTextureID("octopus");

				sewerGate1.setAmbientFactor(Materials.AMBIENT_MATAL);
				sewerGate1.setDiffuseFactor(Materials.DIFFUSE_MATAL);
				sewerGate1.setSpecularFactor(Materials.SPECULAR_METAL);
				sewerGate1.shinessFactor = Materials.SHINESS_METAL;

				sewerGate1.setBufferID(bufferController.getBufferID());

				Node.addChildrenToNode(groundNode, sewerGate1);
			} else {
				sewerGate1 = new Cube(0.1f, 3.2f, 0.1f, 1, 1);

				trans.setIdentity();
				float[] translate46 = { 38, 1.0f - j++ * (0.2f),
						34.5f - i * (0.3f), 1 };

				trans.translate(translate46);
				sewerGate1.updateTransformationState(trans);
				sewerGate1.setTextureID("octopus");

				sewerGate1.setAmbientFactor(Materials.AMBIENT_MATAL);
				sewerGate1.setDiffuseFactor(Materials.DIFFUSE_MATAL);
				sewerGate1.setSpecularFactor(Materials.SPECULAR_METAL);
				sewerGate1.shinessFactor = Materials.SHINESS_METAL;

				sewerGate1.setBufferID(bufferController.getBufferID());

				Node.addChildrenToNode(groundNode, sewerGate1);
			}

		}
		sewerGate1 = new Cube(0.1f, 0.1f, 5, 1, 1);

		trans.setIdentity();
		float[] translate47 = { 38, 1f, 30, 1 };

		trans.translate(translate47);
		sewerGate1.updateTransformationState(trans);
		sewerGate1.setTextureID("octopus");

		sewerGate1.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate1.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate1.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate1.shinessFactor = Materials.SHINESS_METAL;

		sewerGate1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate1);

		sewerGate1 = new Cube(0.1f, 0.1f, 5, 1, 1);

		trans.setIdentity();
		float[] translate48 = { 38, 1.6f, 30, 1 };

		trans.translate(translate48);
		sewerGate1.updateTransformationState(trans);
		sewerGate1.setTextureID("octopus");

		sewerGate1.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate1.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate1.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate1.shinessFactor = Materials.SHINESS_METAL;

		sewerGate1.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate1);

		sewerGate = new Cube(0.1f, 0.1f, 4.6f, 1, 1);

		trans.setIdentity();
		float[] translate49 = { 38, 2.2f, 30.2f, 1 };

		trans.translate(translate49);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);

		sewerGate = new Cube(0.1f, 0.1f, 4.4f, 1, 1);

		trans.setIdentity();
		float[] translate50 = { 38, 2.8f, 30.4f, 1 };

		trans.translate(translate50);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);

		sewerGate = new Cube(0.1f, 0.1f, 3.8f, 1, 1);

		trans.setIdentity();
		float[] translate51 = { 38, 3.4f, 30.6f, 1 };

		trans.translate(translate51);
		sewerGate.updateTransformationState(trans);
		sewerGate.setTextureID("octopus");

		sewerGate.setAmbientFactor(Materials.AMBIENT_MATAL);
		sewerGate.setDiffuseFactor(Materials.DIFFUSE_MATAL);
		sewerGate.setSpecularFactor(Materials.SPECULAR_METAL);
		sewerGate.shinessFactor = Materials.SHINESS_METAL;

		sewerGate.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerGate);
		// ------------------------------------------------------------------
		sewerArc = new Arc(0, (float) Math.PI, 2.5f, 0.2f, 2.5f, 20, 0.2f, 0.5f);

		trans.setIdentity();
		float[] translate16 = { 0f, 2, 32.5f, 1 };

		trans.translate(translate16);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		sewerArc.updateTransformationState(trans);
		sewerArc.setTextureID("ground");

		sewerArc.setAmbientFactor(Materials.AMBIENT_GROUND);
		sewerArc.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		sewerArc.setSpecularFactor(Materials.SPECULAR_GROUND);
		sewerArc.shinessFactor = Materials.SHINESS_GROUND;

		sewerArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerArc);

		sewerArc = new Arc(0, (float) Math.PI, .2f, 1.5f, 1.5f, 20, 0.2f, 0.5f);

		trans.setIdentity();
		float[] translate152 = { 0.3f, 1.2f, 32.5f, 1 };

		trans.translate(translate152);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		sewerArc.updateTransformationState(trans);
		sewerArc.setTextureID("door");

		sewerArc.setAmbientFactor(Materials.AMBIENT_DOOR);
		sewerArc.setDiffuseFactor(Materials.DIFFUSE_DOOR);
		sewerArc.setSpecularFactor(Materials.SPECULAR_DOOR);
		sewerArc.shinessFactor = Materials.SHINESS_DOOR;

		sewerArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerArc);

		sewerArc = new Arc(0, (float) Math.PI, 2.5f, 0.2f, 2.5f, 20, 0.2f, 0.5f);

		trans.setIdentity();
		float[] translate17 = { 37.5f, 2, 32.5f, 1 };

		trans.translate(translate17);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		sewerArc.updateTransformationState(trans);
		sewerArc.setTextureID("ground");

		sewerArc.setAmbientFactor(Materials.AMBIENT_GROUND);
		sewerArc.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		sewerArc.setSpecularFactor(Materials.SPECULAR_GROUND);
		sewerArc.shinessFactor = Materials.SHINESS_GROUND;

		sewerArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerArc);

		sewerArc = new Arc(0, (float) Math.PI, .2f, 1.5f, 1.5f, 20, 0.2f, 0.5f);

		trans.setIdentity();
		float[] translate153 = { 39.4f, 1.2f, 32.5f, 1 };

		trans.translate(translate153);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		sewerArc.updateTransformationState(trans);
		sewerArc.setTextureID("door");

		sewerArc.setAmbientFactor(Materials.AMBIENT_DOOR);
		sewerArc.setDiffuseFactor(Materials.DIFFUSE_DOOR);
		sewerArc.setSpecularFactor(Materials.SPECULAR_DOOR);

		sewerArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerArc);

		sewerArc = new Arc(0, (float) Math.PI, 2.5f, 0.2f, 2.5f, 20, 0.2f, 0.5f);

		trans.setIdentity();
		float[] translate18 = { 37.5f, 2, 7.5f, 1 };

		trans.translate(translate18);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		sewerArc.updateTransformationState(trans);
		sewerArc.setTextureID("ground");

		sewerArc.setAmbientFactor(Materials.AMBIENT_GROUND);
		sewerArc.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		sewerArc.setSpecularFactor(Materials.SPECULAR_GROUND);
		sewerArc.shinessFactor = Materials.SHINESS_GROUND;

		sewerArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerArc);

		sewerArc = new Arc(0, (float) Math.PI, .2f, 1.5f, 1.5f, 20, 0.2f, 0.5f);

		trans.setIdentity();
		float[] translate154 = { 39.4f, 1.2f, 7.5f, 1 };

		trans.translate(translate154);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		sewerArc.updateTransformationState(trans);
		sewerArc.setTextureID("door");

		sewerArc.setAmbientFactor(Materials.AMBIENT_DOOR);
		sewerArc.setDiffuseFactor(Materials.DIFFUSE_DOOR);
		sewerArc.setSpecularFactor(Materials.SPECULAR_DOOR);
		sewerArc.shinessFactor = Materials.SHINESS_DOOR;

		sewerArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, sewerArc);

		// ------------------------------------------------------------------
		// Stairs

		door = new Cube(0.2f, 2.5f, 1.1f, 1, 1);

		trans.setIdentity();
		float[] translat201 = { 0.2f, 10 + 0.4f, 20.7f, 1 };

		trans.translate(translat201);
		door.updateTransformationState(trans);
		door.setTextureID("door");

		door.setAmbientFactor(Materials.AMBIENT_DOOR);
		door.setDiffuseFactor(Materials.DIFFUSE_DOOR);
		door.setSpecularFactor(Materials.SPECULAR_DOOR);
		door.shinessFactor = Materials.SHINESS_DOOR;

		door.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, door);

		doorDecor = new Cube(0.8f, 2.5f, 0.2f, 1, 1);

		trans.setIdentity();
		float[] translat211 = { 0.2f, 10 + 0.4f, 20.5f, 1 };

		trans.translate(translat211);
		doorDecor.updateTransformationState(trans);
		doorDecor.setTextureID("marmor");

		doorDecor.setAmbientFactor(Materials.AMBIENT_MARMOR);
		doorDecor.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		doorDecor.setSpecularFactor(Materials.SPECULAR_MARMOR);
		doorDecor.shinessFactor = Materials.SHINESS_MARMOR;

		doorDecor.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, doorDecor);

		doorDecor = new Cube(0.8f, 2.5f, 0.2f, 1, 1);

		trans.setIdentity();
		float[] translat221 = { 0.2f, 10 + 0.4f, 21.8f, 1 };

		trans.translate(translat221);
		doorDecor.updateTransformationState(trans);
		doorDecor.setTextureID("marmor");

		doorDecor.setAmbientFactor(Materials.AMBIENT_MARMOR);
		doorDecor.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		doorDecor.setSpecularFactor(Materials.SPECULAR_MARMOR);
		doorDecor.shinessFactor = Materials.SHINESS_MARMOR;

		doorDecor.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, doorDecor);

		doorArc = new Arc(0, (float) Math.PI, 0.8f, 0.3f, 0.8f, 20, 0.2f, 1.0f);

		trans.setIdentity();
		float[] translat231 = { 0.2f, 10 + 0.4f + 2.5f, 21.25f, 1 };

		trans.translate(translat231);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		doorArc.updateTransformationState(trans);
		doorArc.setTextureID("marmor");

		doorArc.setAmbientFactor(Materials.AMBIENT_MARMOR);
		doorArc.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		doorArc.setSpecularFactor(Materials.SPECULAR_MARMOR);
		doorArc.shinessFactor = Materials.SHINESS_MARMOR;

		doorArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, doorArc);

		Cube stair = new Cube(2, 10 + 0.4f, 2f, 0.2f, 1);

		trans.setIdentity();
		float[] translate20 = { 0, 0, 20f + 0.2f, 1 };

		trans.translate(translate20);
		stair.updateTransformationState(trans);
		stair.setTextureID("ground");

		stair.setAmbientFactor(Materials.AMBIENT_MARMOR);
		stair.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		stair.setSpecularFactor(Materials.SPECULAR_MARMOR);
		stair.shinessFactor = Materials.SHINESS_MARMOR;

		stair.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, stair);

		Cube stairDecor = new Cube(2, 11 + 0.4f, 0.2f, 0.4f, 2);

		trans.setIdentity();
		float[] b20 = { 0, 0, 22f + 0.2f, 1 };

		trans.translate(b20);
		stairDecor.updateTransformationState(trans);
		stairDecor.setTextureID("brick_wall");

		stairDecor.setAmbientFactor(Materials.AMBIENT_WALL);
		stairDecor.setDiffuseFactor(Materials.DIFFUSE_WALL);
		stairDecor.setSpecularFactor(Materials.SPECULAR_WALL);
		stairDecor.shinessFactor = Materials.SHINESS_WALL;

		stairDecor.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, stairDecor);

		stairDecor = new Cube(0.2f, 11 + 0.4f, 4f - 0.2f, 0.7f, 2.0f);

		trans.setIdentity();
		float[] b21 = { 2, 0, 18f + 0.8f, 1 };

		trans.translate(b21);
		stairDecor.updateTransformationState(trans);
		stairDecor.setTextureID("brick_wall");

		stairDecor.setAmbientFactor(Materials.AMBIENT_WALL);
		stairDecor.setDiffuseFactor(Materials.DIFFUSE_WALL);
		stairDecor.setSpecularFactor(Materials.SPECULAR_WALL);
		stairDecor.shinessFactor = Materials.SHINESS_WALL;

		stairDecor.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, stairDecor);

		stairDecor = new Cube(2.4f, 9 + 0.4f, 0.2f, 0.4f, 2.0f);

		trans.setIdentity();
		float[] b22 = { 2.2f, 0, 22f - 4 + 0.6f, 1 };

		trans.translate(b22);
		stairDecor.updateTransformationState(trans);
		stairDecor.setTextureID("brick_wall");

		stairDecor.setAmbientFactor(Materials.AMBIENT_WALL);
		stairDecor.setDiffuseFactor(Materials.DIFFUSE_WALL);
		stairDecor.setSpecularFactor(Materials.SPECULAR_WALL);
		stairDecor.shinessFactor = Materials.SHINESS_WALL;

		stairDecor.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, stairDecor);

		stairDecor = new Cube(4.6f, 9 + 0.4f, 0.2f, 0.7f, 2.0f);

		trans.setIdentity();
		float[] b23 = { 0f, 0, 22f - 6 + 0.4f, 1 };

		trans.translate(b23);
		stairDecor.updateTransformationState(trans);
		stairDecor.setTextureID("brick_wall");

		stairDecor.setAmbientFactor(Materials.AMBIENT_WALL);
		stairDecor.setDiffuseFactor(Materials.DIFFUSE_WALL);
		stairDecor.setSpecularFactor(Materials.SPECULAR_WALL);
		stairDecor.shinessFactor = Materials.SHINESS_WALL;

		stairDecor.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, stairDecor);

		Cube decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] v = { 0.1f, 9.5f, 16.2f, 1 };
		trans.translate(v);

		decorStone.updateTransformationState(trans);
		decorStone.setTextureID("brick_wall");

		decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone.shinessFactor = Materials.SHINESS_WALL;

		decorStone.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone);

		decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] v1 = { 0.1f, 2f, 16.2f, 1 };
		trans.translate(v1);

		decorStone.updateTransformationState(trans);
		decorStone.setTextureID("brick_wall");

		decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone.shinessFactor = Materials.SHINESS_WALL;

		decorStone.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone);

		Cube decorCube = new Cube(0.3f, 10.2f, 0.3f, 0.1f, 2f);

		trans.setIdentity();
		float[] v3 = { 0.2f, 0, 16.3f, 1 };

		trans.translate(v3);
		decorCube.updateTransformationState(trans);
		decorCube.setTextureID("brick_wall");

		decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube.shinessFactor = Materials.SHINESS_WALL;

		decorCube.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube);

		decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] v6 = { 4.5f, 9.5f, 16.2f, 1 };
		trans.translate(v6);

		decorStone.updateTransformationState(trans);
		decorStone.setTextureID("brick_wall");

		decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone.shinessFactor = Materials.SHINESS_WALL;

		decorStone.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone);

		decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] v4 = { 4.5f, 2f, 16.2f, 1 };
		trans.translate(v4);

		decorStone.updateTransformationState(trans);
		decorStone.setTextureID("brick_wall");

		decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone.shinessFactor = Materials.SHINESS_WALL;

		decorStone.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone);

		decorCube = new Cube(0.3f, 10.2f, 0.3f, 0.1f, 2f);

		trans.setIdentity();
		float[] v5 = { 4.6f, 0, 16.3f, 1 };

		trans.translate(v5);
		decorCube.updateTransformationState(trans);
		decorCube.setTextureID("brick_wall");

		decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube.shinessFactor = Materials.SHINESS_WALL;

		decorCube.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube);

		decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] v11 = { 4.5f, 9.5f, 18.5f, 1 };
		trans.translate(v11);

		decorStone.updateTransformationState(trans);
		decorStone.setTextureID("brick_wall");

		decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone.shinessFactor = Materials.SHINESS_WALL;

		decorStone.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone);

		decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] v10 = { 4.5f, 2f, 18.5f, 1 };
		trans.translate(v10);

		decorStone.updateTransformationState(trans);
		decorStone.setTextureID("brick_wall");

		decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone.shinessFactor = Materials.SHINESS_WALL;

		decorStone.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone);

		decorCube = new Cube(0.3f, 10.2f, 0.3f, 0.1f, 2f);

		trans.setIdentity();
		float[] v9 = { 4.6f, 0, 18.6f, 1 };

		trans.translate(v9);
		decorCube.updateTransformationState(trans);
		decorCube.setTextureID("brick_wall");

		decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube.shinessFactor = Materials.SHINESS_WALL;

		decorCube.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube);

		Arc stairDecorArc = new Arc(0, (float) Math.PI, 0.3f, 0.3f, 1.3f, 20,
				0.2f, 0.3f);

		trans.setIdentity();
		float[] v13 = { 4.6f, 10.2f, 17.6f, 1 };

		trans.translate(v13);
		trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 90);
		stairDecorArc.updateTransformationState(trans);
		stairDecorArc.setTextureID("brick_wall");

		stairDecorArc.setAmbientFactor(Materials.AMBIENT_WALL);
		stairDecorArc.setDiffuseFactor(Materials.DIFFUSE_WALL);
		stairDecorArc.setSpecularFactor(Materials.SPECULAR_WALL);
		stairDecorArc.shinessFactor = Materials.SHINESS_WALL;

		stairDecorArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, stairDecorArc);

		decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] v17 = { 1.9f, 11.2f, 18.5f, 1 };
		trans.translate(v17);

		decorStone.updateTransformationState(trans);
		decorStone.setTextureID("brick_wall");

		decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone.shinessFactor = Materials.SHINESS_WALL;

		decorStone.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone);

		decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] v16 = { 1.9f, 2f, 18.5f, 1 };
		trans.translate(v16);

		decorStone.updateTransformationState(trans);
		decorStone.setTextureID("brick_wall");

		decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone.shinessFactor = Materials.SHINESS_WALL;

		decorStone.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone);

		decorCube = new Cube(0.3f, 11.9f, 0.3f, 0.1f, 2f);

		trans.setIdentity();
		float[] v15 = { 2.f, 0, 18.6f, 1 };

		trans.translate(v15);
		decorCube.updateTransformationState(trans);
		decorCube.setTextureID("brick_wall");

		decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube.shinessFactor = Materials.SHINESS_WALL;

		decorCube.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube);

		decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] v117 = { 0.f, 11.2f, 18.5f, 1 };
		trans.translate(v117);

		decorStone.updateTransformationState(trans);
		decorStone.setTextureID("brick_wall");

		decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone.shinessFactor = Materials.SHINESS_WALL;

		decorStone.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone);

		decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] v116 = { 0.f, 2f, 18.5f, 1 };
		trans.translate(v116);

		decorStone.updateTransformationState(trans);
		decorStone.setTextureID("brick_wall");

		decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone.shinessFactor = Materials.SHINESS_WALL;

		decorStone.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone);

		decorCube = new Cube(0.3f, 11.9f, 0.3f, 0.1f, 2f);

		trans.setIdentity();
		float[] v115 = { 0.1f, 0, 18.6f, 1 };

		trans.translate(v115);
		decorCube.updateTransformationState(trans);
		decorCube.setTextureID("brick_wall");

		decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube.shinessFactor = Materials.SHINESS_WALL;

		decorCube.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube);

		stairDecorArc = new Arc(0, (float) Math.PI, 0.3f, 0.3f, 1.1f, 20, 0.2f,
				0.3f);

		trans.setIdentity();
		float[] v20 = { 1.2f, 11.9f, 18.6f, 1 };

		trans.translate(v20);
		stairDecorArc.updateTransformationState(trans);
		stairDecorArc.setTextureID("brick_wall");

		stairDecorArc.setAmbientFactor(Materials.AMBIENT_WALL);
		stairDecorArc.setDiffuseFactor(Materials.DIFFUSE_WALL);
		stairDecorArc.setSpecularFactor(Materials.SPECULAR_WALL);
		stairDecorArc.shinessFactor = Materials.SHINESS_WALL;

		stairDecorArc.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, stairDecorArc);

		decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] v23 = { 1.9f, 11.2f, 22.3f, 1 };
		trans.translate(v23);

		decorStone.updateTransformationState(trans);
		decorStone.setTextureID("brick_wall");

		decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone.shinessFactor = Materials.SHINESS_WALL;

		decorStone.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone);

		decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] v22 = { 1.9f, 2f, 22.3f, 1 };
		trans.translate(v22);

		decorStone.updateTransformationState(trans);
		decorStone.setTextureID("brick_wall");

		decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone.shinessFactor = Materials.SHINESS_WALL;

		decorStone.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone);

		decorCube = new Cube(0.3f, 11.9f, 0.3f, 0.1f, 2f);

		trans.setIdentity();
		float[] v21 = { 2.f, 0, 22.4f, 1 };

		trans.translate(v21);
		decorCube.updateTransformationState(trans);
		decorCube.setTextureID("brick_wall");

		decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube.shinessFactor = Materials.SHINESS_WALL;

		decorCube.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube);

		decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] v33 = { 0.1f, 11.2f, 22.3f, 1 };
		trans.translate(v33);

		decorStone.updateTransformationState(trans);
		decorStone.setTextureID("brick_wall");

		decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone.shinessFactor = Materials.SHINESS_WALL;

		decorStone.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone);

		decorStone = new Cube(0.5f, 0.6f, 0.5f, 0.1f, 0.1f);

		trans.setIdentity();
		float[] v32 = { 0.1f, 2f, 22.3f, 1 };
		trans.translate(v32);

		decorStone.updateTransformationState(trans);
		decorStone.setTextureID("brick_wall");

		decorStone.setAmbientFactor(Materials.AMBIENT_WALL);
		decorStone.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorStone.setSpecularFactor(Materials.SPECULAR_WALL);
		decorStone.shinessFactor = Materials.SHINESS_WALL;

		decorStone.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, decorStone);

		decorCube = new Cube(0.3f, 11.9f, 0.3f, 0.1f, 2f);

		trans.setIdentity();
		float[] v31 = { 0.2f, 0, 22.4f, 1 };

		trans.translate(v31);
		decorCube.updateTransformationState(trans);
		decorCube.setTextureID("brick_wall");

		decorCube.setAmbientFactor(Materials.AMBIENT_WALL);
		decorCube.setDiffuseFactor(Materials.DIFFUSE_WALL);
		decorCube.setSpecularFactor(Materials.SPECULAR_WALL);
		decorCube.shinessFactor = Materials.SHINESS_WALL;

		decorCube.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, decorCube);

		for (int i = 0; i < 9; i++) {
			Cube stairStep = new Cube(2, 0.4f, 0.2f, 0.4f, 0.2f);

			trans.setIdentity();
			float[] translate19 = { 0, 10 - (i * 0.4f), 20f - (i * 0.2f), 1 };

			trans.translate(translate19);
			stairStep.updateTransformationState(trans);
			stairStep.setTextureID("ground");

			stairStep.setAmbientFactor(Materials.AMBIENT_MARMOR);
			stairStep.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			stairStep.setSpecularFactor(Materials.SPECULAR_MARMOR);
			stairStep.shinessFactor = Materials.SHINESS_MARMOR;

			stairStep.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, stairStep);
		}

		stair = new Cube(2, 10 - (8 * 0.4f), 2f, 0.4f, 1f);

		trans.setIdentity();
		float[] translate21 = { 0, 0, 20f - (17 * 0.2f), 1 };

		trans.translate(translate21);
		stair.updateTransformationState(trans);
		stair.setTextureID("ground");

		stair.setAmbientFactor(Materials.AMBIENT_MARMOR);
		stair.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
		stair.setSpecularFactor(Materials.SPECULAR_MARMOR);
		stair.shinessFactor = Materials.SHINESS_MARMOR;

		stair.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, stair);

		for (int i = 10, j = 0; i < 22; i++, j++) {
			Cube stairStep = new Cube(0.2f, 0.4f, 2, 0.1f, 0.5f);

			trans.setIdentity();
			float[] translate19 = { 2 + (j * 0.2f), 10 - (i * 0.4f),
					20f - (17 * 0.2f), 1 };

			trans.translate(translate19);
			stairStep.updateTransformationState(trans);
			stairStep.setTextureID("ground");

			stairStep.setAmbientFactor(Materials.AMBIENT_MARMOR);
			stairStep.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			stairStep.setSpecularFactor(Materials.SPECULAR_MARMOR);
			stairStep.shinessFactor = Materials.SHINESS_MARMOR;

			stairStep.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, stairStep);
		}
		// ------------------------------------------------------------------
		// left arcwall

		for (int i = 0; i < 39; i += 2) {
			Cylinder pillar = new Cylinder(15, 0.25f, 5, 0.1f, 1);

			trans.setIdentity();
			float[] translate6 = { 1 + i, 2, 2, 1 };

			trans.translate(translate6);
			pillar.updateTransformationState(trans);
			pillar.setTextureID("marmor");

			pillar.setAmbientFactor(Materials.AMBIENT_MARMOR);
			pillar.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			pillar.setSpecularFactor(Materials.SPECULAR_MARMOR);
			pillar.shinessFactor = Materials.SHINESS_MARMOR;

			pillar.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, pillar);

			SolidDeformedCylinder cubeDecor = new SolidDeformedCylinder(0.35f,
					0.35f, 0.5f, 15, 0.1f, 0.3f);

			trans.setIdentity();
			float[] translate7 = { 1 + i, 2 + 4.5f, 2, 1 };

			trans.translate(translate7);
			cubeDecor.updateTransformationState(trans);
			cubeDecor.setTextureID("marmor");

			cubeDecor.setAmbientFactor(Materials.AMBIENT_MARMOR);
			cubeDecor.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			cubeDecor.setSpecularFactor(Materials.SPECULAR_MARMOR);
			cubeDecor.shinessFactor = Materials.SHINESS_MARMOR;

			cubeDecor.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, cubeDecor);

			cubeDecor = new SolidDeformedCylinder(0.35f, 0.35f, 0.5f, 15, 0.1f,
					0.3f);

			trans.setIdentity();
			float[] translate8 = { 1 + i, 2, 2, 1 };

			trans.translate(translate8);
			cubeDecor.updateTransformationState(trans);
			cubeDecor.setTextureID("marmor");

			cubeDecor.setAmbientFactor(Materials.AMBIENT_MARMOR);
			cubeDecor.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			cubeDecor.setSpecularFactor(Materials.SPECULAR_MARMOR);
			cubeDecor.shinessFactor = Materials.SHINESS_MARMOR;

			cubeDecor.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, cubeDecor);
		}

		for (int i = 0; i < 38; i += 2) {
			Arc arc = new Arc(0, (float) Math.PI, 0.2f, 0.3f, 1, 8, 0.2f, 0.3f);

			trans.setIdentity();
			float[] translate7 = { 2f + i, 7, 1.9f, 1 };

			trans.translate(translate7);
			trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 180);
			arc.updateTransformationState(trans);
			arc.setTextureID("marmor");

			arc.setAmbientFactor(Materials.AMBIENT_MARMOR);
			arc.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			arc.setSpecularFactor(Materials.SPECULAR_MARMOR);
			arc.shinessFactor = Materials.SHINESS_MARMOR;

			arc.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, arc);

		}

		Cube arcGround = new Cube(40, 0.2f, 2f, 4f, 0.2f);

		trans.setIdentity();
		float[] translate8 = { 0, 8, 0.2f, 1 };

		trans.translate(translate8);
		arcGround.updateTransformationState(trans);
		arcGround.setTextureID("ground");

		arcGround.setAmbientFactor(Materials.AMBIENT_GROUND);
		arcGround.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		arcGround.setSpecularFactor(Materials.SPECULAR_GROUND);
		arcGround.shinessFactor = Materials.SHINESS_GROUND;

		arcGround.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, arcGround);

		for (int i = 0; i < 39; i += 2) {
			Cylinder pillar = new Cylinder(15, 0.25f, 5, 0.1f, 1);

			trans.setIdentity();
			float[] translate6 = { 1 + i, 8.2f, 1.2f, 1 };

			trans.translate(translate6);
			pillar.updateTransformationState(trans);
			pillar.setTextureID("marmor");

			pillar.setAmbientFactor(Materials.AMBIENT_MARMOR);
			pillar.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			pillar.setSpecularFactor(Materials.SPECULAR_MARMOR);
			pillar.shinessFactor = Materials.SHINESS_MARMOR;

			pillar.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, pillar);

			SolidDeformedCylinder cubeDecor = new SolidDeformedCylinder(0.35f,
					0.35f, 0.5f, 15, 0.1f, 0.1f);

			trans.setIdentity();
			float[] translate7 = { 1 + i, 8.2f + 4.5f, 1.2f, 1 };

			trans.translate(translate7);
			cubeDecor.updateTransformationState(trans);
			cubeDecor.setTextureID("marmor");

			cubeDecor.setAmbientFactor(Materials.AMBIENT_MARMOR);
			cubeDecor.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			cubeDecor.setSpecularFactor(Materials.SPECULAR_MARMOR);
			cubeDecor.shinessFactor = Materials.SHINESS_MARMOR;

			cubeDecor.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, cubeDecor);

			cubeDecor = new SolidDeformedCylinder(0.35f, 0.35f, 0.5f, 15, 0.1f,
					0.3f);

			trans.setIdentity();
			float[] translate9 = { 1 + i, 8.2f, 1.2f, 1 };

			trans.translate(translate9);
			cubeDecor.updateTransformationState(trans);
			cubeDecor.setTextureID("marmor");

			cubeDecor.setAmbientFactor(Materials.AMBIENT_MARMOR);
			cubeDecor.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			cubeDecor.setSpecularFactor(Materials.SPECULAR_MARMOR);
			cubeDecor.shinessFactor = Materials.SHINESS_MARMOR;

			cubeDecor.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, cubeDecor);
		}

		int counter = 0;

		for (int i = 0; i < 38; i += 2) {
			Arc arc = new Arc(0, (float) Math.PI, 0.2f, 0.3f, 1, 8, 0.2f, 0.3f);

			trans.setIdentity();
			float[] translate7 = { 2f + i, 13.2f, 1.f, 1 };

			trans.translate(translate7);
			trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 180);
			arc.updateTransformationState(trans);
			arc.setTextureID("marmor");

			arc.setAmbientFactor(Materials.AMBIENT_MARMOR);
			arc.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			arc.setSpecularFactor(Materials.SPECULAR_MARMOR);
			arc.shinessFactor = Materials.SHINESS_MARMOR;

			arc.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, arc);

			if (counter % 2 == 0) {
				SolidDeformedPolyCube cubeDecor = new SolidDeformedPolyCube(
						0.55f, 0.45f, 0.5f, 4, 0.2f, 0.2f);

				trans.setIdentity();
				float[] translate9 = { 2.f + i, 8.2f, 1.2f, 1 };

				trans.translate(translate9);
				trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 45);
				cubeDecor.updateTransformationState(trans);
				cubeDecor.setTextureID("marmor");

				cubeDecor.setAmbientFactor(Materials.AMBIENT_MARMOR);
				cubeDecor.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
				cubeDecor.setSpecularFactor(Materials.SPECULAR_MARMOR);
				cubeDecor.shinessFactor = Materials.SHINESS_MARMOR;

				cubeDecor.setBufferID(bufferController.getBufferID());

				Node.addChildrenToNode(groundNode, cubeDecor);

				Cube cubeDecor2 = new Cube(0.6f, 2.7f, 0.16f, 1f, 1f);

				trans.setIdentity();
				float[] translate10 = { 1.69f + i, 8.6f, 1.1f, 1 };

				trans.translate(translate10);
				cubeDecor2.updateTransformationState(trans);
				cubeDecor2.setTextureID("statue");

				cubeDecor2.setAmbientFactor(Materials.AMBIENT_MARMOR);
				cubeDecor2.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
				cubeDecor2.setSpecularFactor(Materials.SPECULAR_MARMOR);
				cubeDecor2.shinessFactor = Materials.SHINESS_MARMOR;

				cubeDecor2.setBufferID(bufferController.getBufferID());

				Node.addChildrenToNode(groundNode, cubeDecor2);
			}

			++counter;
		}

		Cube arcGround2 = new Cube(40, 0.2f, 1.8f, 4f, 0.3f);

		trans.setIdentity();
		float[] translate9 = { 0, 14.2f, 0.2f, 1 };

		trans.translate(translate9);
		arcGround2.updateTransformationState(trans);
		arcGround2.setTextureID("ground");

		arcGround2.setAmbientFactor(Materials.AMBIENT_GROUND);
		arcGround2.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		arcGround2.setSpecularFactor(Materials.SPECULAR_GROUND);
		arcGround2.shinessFactor = Materials.SHINESS_GROUND;

		arcGround2.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, arcGround2);

		// -----------------------------------------------------------------
		// right arcwall

		for (int i = 0; i < 39; i += 2) {
			Cylinder pillar = new Cylinder(15, 0.25f, 5, 0.1f, 1);

			trans.setIdentity();
			float[] translate6 = { 1 + i, 2, 38, 1 };

			trans.translate(translate6);
			pillar.updateTransformationState(trans);
			pillar.setTextureID("marmor");

			pillar.setAmbientFactor(Materials.AMBIENT_MARMOR);
			pillar.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			pillar.setSpecularFactor(Materials.SPECULAR_MARMOR);
			pillar.shinessFactor = Materials.SHINESS_MARMOR;

			pillar.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, pillar);

			SolidDeformedCylinder cubeDecor = new SolidDeformedCylinder(0.35f,
					0.35f, 0.5f, 15, 0.1f, 0.1f);

			trans.setIdentity();
			float[] translate7 = { 1 + i, 2 + 4.5f, 38, 1 };

			trans.translate(translate7);
			cubeDecor.updateTransformationState(trans);
			cubeDecor.setTextureID("marmor");

			cubeDecor.setAmbientFactor(Materials.AMBIENT_MARMOR);
			cubeDecor.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			cubeDecor.setSpecularFactor(Materials.SPECULAR_MARMOR);
			cubeDecor.shinessFactor = Materials.SHINESS_MARMOR;

			cubeDecor.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, cubeDecor);

			cubeDecor = new SolidDeformedCylinder(0.35f, 0.35f, 0.5f, 15, 0.1f,
					0.3f);

			trans.setIdentity();
			float[] translate10 = { 1 + i, 2f, 38f, 1 };

			trans.translate(translate10);
			cubeDecor.updateTransformationState(trans);
			cubeDecor.setTextureID("marmor");

			cubeDecor.setAmbientFactor(Materials.AMBIENT_MARMOR);
			cubeDecor.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			cubeDecor.setSpecularFactor(Materials.SPECULAR_MARMOR);
			cubeDecor.shinessFactor = Materials.SHINESS_MARMOR;

			cubeDecor.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, cubeDecor);
		}

		for (int i = 0; i < 38; i += 2) {
			Arc arc = new Arc(0, (float) Math.PI, 0.2f, 0.3f, 1, 8, 0.2f, 0.3f);

			trans.setIdentity();
			float[] translate7 = { 2f + i, 7, 38.0f, 1 };

			trans.translate(translate7);
			arc.updateTransformationState(trans);
			arc.setTextureID("marmor");

			arc.setAmbientFactor(Materials.AMBIENT_MARMOR);
			arc.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			arc.setSpecularFactor(Materials.SPECULAR_MARMOR);
			arc.shinessFactor = Materials.SHINESS_MARMOR;

			arc.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, arc);

		}

		arcGround = new Cube(40, 0.2f, 1.8f, 4f, 0.3f);

		trans.setIdentity();
		float[] translate10 = { 0, 8, 37.8f, 1 };

		trans.translate(translate10);
		arcGround.updateTransformationState(trans);
		arcGround.setTextureID("ground");

		arcGround.setAmbientFactor(Materials.AMBIENT_GROUND);
		arcGround.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		arcGround.setSpecularFactor(Materials.SPECULAR_GROUND);
		arcGround.shinessFactor = Materials.SHINESS_GROUND;

		arcGround.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, arcGround);

		for (int i = 0; i < 39; i += 2) {
			Cylinder pillar = new Cylinder(15, 0.25f, 5, 0.1f, 1);

			trans.setIdentity();
			float[] translate6 = { 1 + i, 8.2f, 38.7f, 1 };

			trans.translate(translate6);
			pillar.updateTransformationState(trans);
			pillar.setTextureID("marmor");

			pillar.setAmbientFactor(Materials.AMBIENT_MARMOR);
			pillar.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			pillar.setSpecularFactor(Materials.SPECULAR_MARMOR);
			pillar.shinessFactor = Materials.SHINESS_MARMOR;

			pillar.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, pillar);

			SolidDeformedCylinder cubeDecor = new SolidDeformedCylinder(0.35f,
					0.35f, 0.5f, 15, 0.1f, 0.1f);

			trans.setIdentity();
			float[] translate7 = { 1 + i, 8.2f + 4.5f, 38.7f, 1 };

			trans.translate(translate7);
			cubeDecor.updateTransformationState(trans);
			cubeDecor.setTextureID("marmor");

			cubeDecor.setAmbientFactor(Materials.AMBIENT_MARMOR);
			cubeDecor.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			cubeDecor.setSpecularFactor(Materials.SPECULAR_MARMOR);
			cubeDecor.shinessFactor = Materials.SHINESS_MARMOR;

			cubeDecor.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, cubeDecor);

			cubeDecor = new SolidDeformedCylinder(0.35f, 0.35f, 0.5f, 15, 0.1f,
					0.3f);

			trans.setIdentity();
			float[] translate11 = { 1 + i, 8.2f, 38.7f, 1 };

			trans.translate(translate11);
			cubeDecor.updateTransformationState(trans);
			cubeDecor.setTextureID("marmor");

			cubeDecor.setAmbientFactor(Materials.AMBIENT_MARMOR);
			cubeDecor.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			cubeDecor.setSpecularFactor(Materials.SPECULAR_MARMOR);
			cubeDecor.shinessFactor = Materials.SHINESS_MARMOR;

			cubeDecor.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, cubeDecor);
		}

		int count = 0;

		for (int i = 0; i < 38; i += 2) {
			Arc arc = new Arc(0, (float) Math.PI, 0.2f, 0.3f, 1, 8, 0.2f, 0.3f);

			trans.setIdentity();
			float[] translate7 = { 2f + i, 13.2f, 38.7f, 1 };

			trans.translate(translate7);
			arc.updateTransformationState(trans);
			arc.setTextureID("marmor");

			arc.setAmbientFactor(Materials.AMBIENT_MARMOR);
			arc.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
			arc.setSpecularFactor(Materials.SPECULAR_MARMOR);
			arc.shinessFactor = Materials.SHINESS_MARMOR;

			arc.setBufferID(bufferController.getBufferID());

			Node.addChildrenToNode(groundNode, arc);

			if (count % 2 == 0) {
				SolidDeformedCylinder cubeDecor = new SolidDeformedCylinder(
						0.55f, 0.45f, 0.5f, 4, 0.2f, 0.2f);

				trans.setIdentity();
				float[] translate19 = { 2.f + i, 8.2f, 38.7f, 1 };

				trans.translate(translate19);
				trans.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_Y, 45);
				cubeDecor.updateTransformationState(trans);
				cubeDecor.setTextureID("marmor");

				cubeDecor.setAmbientFactor(Materials.AMBIENT_MARMOR);
				cubeDecor.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
				cubeDecor.setSpecularFactor(Materials.SPECULAR_MARMOR);
				cubeDecor.shinessFactor = Materials.SHINESS_MARMOR;

				cubeDecor.setBufferID(bufferController.getBufferID());

				Node.addChildrenToNode(groundNode, cubeDecor);

				Cube cubeDecor2 = new Cube(0.6f, 2.7f, 0.16f, 1f, 1f);

				trans.setIdentity();
				float[] translate22 = { 1.69f + i, 8.6f, 38.7f, 1 };

				trans.translate(translate22);
				cubeDecor2.updateTransformationState(trans);
				cubeDecor2.setTextureID("statue");

				cubeDecor2.setAmbientFactor(Materials.AMBIENT_MARMOR);
				cubeDecor2.setDiffuseFactor(Materials.DIFFUSE_MARMOR);
				cubeDecor2.setSpecularFactor(Materials.SPECULAR_MARMOR);
				cubeDecor2.shinessFactor = Materials.SHINESS_MARMOR;

				cubeDecor2.setBufferID(bufferController.getBufferID());

				Node.addChildrenToNode(groundNode, cubeDecor2);
			}

			++count;

		}

		arcGround2 = new Cube(40, 0.2f, 1.8f, 4f, 0.3f);

		trans.setIdentity();
		float[] translate11 = { 0, 14.2f, 37.8f, 1 };

		trans.translate(translate11);
		arcGround2.updateTransformationState(trans);
		arcGround2.setTextureID("ground");

		arcGround2.setAmbientFactor(Materials.AMBIENT_GROUND);
		arcGround2.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		arcGround2.setSpecularFactor(Materials.SPECULAR_GROUND);
		arcGround2.shinessFactor = Materials.SHINESS_GROUND;

		arcGround2.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, arcGround2);

		// *----------------------------------------------------------------

		x = horizontalBoxCount;
		y = wallHeight;
		z = wallStrength;

		Cube frontWall = new Cube(x, y, z, 4f, 2);
		frontWall.setTextureID("rock");

		frontWall.setAmbientFactor(Materials.AMBIENT_WALL);
		frontWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		frontWall.setSpecularFactor(Materials.SPECULAR_WALL);
		frontWall.shinessFactor = Materials.SHINESS_WALL;

		frontWall.setBufferID(bufferController.getBufferID());

		Transformation r = new Transformation();
		r.rotateAtPosition(LinearAlgebra4D.UNION_VECTOR_X, 90);

		Node.addChildrenToNode(groundNode, frontWall);

		x = wallStrength;
		y = wallHeight;
		z = verticalBoxCount;

		Cube rightWall = new Cube(x, y, z, 4f, 2f);
		rightWall.setTextureID("rock");

		rightWall.setAmbientFactor(Materials.AMBIENT_WALL);
		rightWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		rightWall.setSpecularFactor(Materials.SPECULAR_WALL);
		rightWall.shinessFactor = Materials.SHINESS_WALL;

		rightWall.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(groundNode, rightWall);
		// -----------------------------------------------------------------------------------

		x = horizontalBoxCount;
		y = wallHeight;
		z = wallStrength;

		Cube backWall = new Cube(x, y, z, 4, 2);

		backWall.setTextureID("rock");

		backWall.setAmbientFactor(Materials.AMBIENT_WALL);
		backWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		backWall.setSpecularFactor(Materials.SPECULAR_WALL);
		backWall.shinessFactor = Materials.SHINESS_WALL;

		backWall.setBufferID(bufferController.getBufferID());

		Transformation backWallTrans = backWall.localTransformation;
		float[] translate_back = { 0, 0, verticalBoxCount, 1 };

		backWallTrans.translate(translate_back);
		Node.addChildrenToNode(groundNode, backWall);
		// --------------------------------------------------------------------
		x = wallStrength;
		y = wallHeight;
		z = verticalBoxCount;

		Cube leftWall = new Cube(x, y, z, 4, 2);
		leftWall.setTextureID("rock");

		leftWall.setAmbientFactor(Materials.AMBIENT_WALL);
		leftWall.setDiffuseFactor(Materials.DIFFUSE_WALL);
		leftWall.setSpecularFactor(Materials.SPECULAR_WALL);
		leftWall.shinessFactor = Materials.SHINESS_WALL;

		leftWall.setBufferID(bufferController.getBufferID());

		Transformation leftWallTrans = leftWall.localTransformation;

		float[] trans4 = { horizontalBoxCount, 0, 0, 1 };

		leftWallTrans.translate(trans4);

		Node.addChildrenToNode(groundNode, leftWall);

		Cube waterGround = new Cube(40 + 2, 0.3f, 40 + 2f, 14, 14.5f);

		Transformation transformation = new Transformation();
		float[] transs = { -1f, 0.f, 4, 1 };
		transformation.translate(transs);

		waterGround.updateTransformationState(transformation);
		waterGround.setTextureID("waterbank");

		waterGround.setAmbientFactor(Materials.AMBIENT_GROUND);
		waterGround.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		waterGround.setSpecularFactor(Materials.SPECULAR_GROUND);
		waterGround.shinessFactor = Materials.SHINESS_GROUND;

		waterGround.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, waterGround);

		Cube waterBank = new Cube(40 + 2, 0.3f, 40 + 2f, 5, 5f);

		transformation = new Transformation();
		float[] transd = { -1f, 1.f, 4, 1 };
		transformation.translate(transd);

		waterBank.updateTransformationState(transformation);
		waterBank.setTextureID("water");

		waterBank.setAmbientFactor(Materials.AMBIENT_WATER);
		waterBank.setDiffuseFactor(Materials.DIFFUSE_WATER);
		waterBank.setSpecularFactor(Materials.SPECULAR_WATER);
		waterBank.shinessFactor = Materials.SHINESS_WATER;

		waterBank.setBufferID(bufferController.getBufferID());

		Node.addChildrenToNode(groundNode, waterBank);

		// ------------------------------------------
		// latern

		Node laternNode = new Node();

		wheelDecor = new SolidDeformedCylinder(0.35f, 0.35f, 1.f, 15, 0.5f,
				0.3f);
		wheelDecor.setTextureID("lava");
		wheelDecor.setAmbientFactor(Materials.AMBIENT_GROUND);
		wheelDecor.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		wheelDecor.setSpecularFactor(Materials.SPECULAR_GROUND);
		wheelDecor.shinessFactor = Materials.SHINESS_GROUND;

		transWheel_1 = new Transformation();
		float[] awx2 = { 0, 0f, 0, 1 };

		transWheel_1.translate(awx2);

		wheelDecor.updateTransformationState(transWheel_1);
		wheelDecor.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, wheelDecor);

		wheelDecor = new SolidDeformedCylinder(0.3f, 0.3f, 10.f, 15, 0.7f, 2f);
		wheelDecor.setTextureID("lava");
		wheelDecor.setAmbientFactor(Materials.AMBIENT_GROUND);
		wheelDecor.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		wheelDecor.setSpecularFactor(Materials.SPECULAR_GROUND);
		wheelDecor.shinessFactor = Materials.SHINESS_GROUND;

		transWheel_1 = new Transformation();
		float[] asx3 = { 0, 0f, 0, 1 };

		transWheel_1.translate(asx3);

		wheelDecor.updateTransformationState(transWheel_1);
		wheelDecor.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, wheelDecor);

		wheelDecor = new SolidDeformedCylinder(0.35f, 0.35f, 1.f, 15, 1f, 0.3f);
		wheelDecor.setTextureID("lava");
		wheelDecor.setAmbientFactor(Materials.AMBIENT_GROUND);
		wheelDecor.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		wheelDecor.setSpecularFactor(Materials.SPECULAR_GROUND);
		wheelDecor.shinessFactor = Materials.SHINESS_GROUND;

		transWheel_1 = new Transformation();
		float[] asx4 = { 0, 9f, 0, 1 };

		transWheel_1.translate(asx4);

		wheelDecor.updateTransformationState(transWheel_1);
		wheelDecor.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, wheelDecor);

		Ellipsoid ball = new Ellipsoid(0.35f, 0.7f, 0.35f, 20, 20);

		ball.setTextureID("lava");
		ball.setAmbientFactor(Materials.AMBIENT_GROUND);
		ball.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		ball.setSpecularFactor(Materials.SPECULAR_GROUND);
		ball.shinessFactor = Materials.SHINESS_GROUND;
		ball.emmisionFactor = 100;

		transWheel_1 = new Transformation();
		float[] ax6 = { 0, 10.5f, 0, 1 };

		transWheel_1.translate(ax6);

		ball.updateTransformationState(transWheel_1);
		ball.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, ball);

		ball = new Ellipsoid(1f, 2f, 1f, 40, 40);

		ball.setTextureID("ground");
		ball.setAmbientFactor(Materials.AMBIENT_LIGHT);
		ball.setDiffuseFactor(Materials.DIFFUSE_LIGHT);
		ball.setSpecularFactor(Materials.SPECULAR_LIGHT);
		ball.shinessFactor = Materials.SHINESS_LIGHT;

		transWheel_1 = new Transformation();
		float[] ax7 = { 0, 10.5f, 0, 1 };

		transWheel_1.translate(ax7);

		ball.updateTransformationState(transWheel_1);
		ball.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, ball);

		Node.addChildrenToNode(groundNode, laternNode);

		Transformation tLatern = new Transformation();

		float[] e = { 14, 2, 11, 1 };
		tLatern.translate(e);

		laternNode.updateTransformationState(tLatern);

		laternNode = new Node();

		wheelDecor = new SolidDeformedCylinder(0.35f, 0.35f, 1.f, 15, 1f, 0.3f);
		wheelDecor.setTextureID("lava");
		wheelDecor.setAmbientFactor(Materials.AMBIENT_GROUND);
		wheelDecor.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		wheelDecor.setSpecularFactor(Materials.SPECULAR_GROUND);
		wheelDecor.shinessFactor = Materials.SHINESS_GROUND;

		transWheel_1 = new Transformation();
		float[] ax8 = { 0, 0f, 0, 1 };

		transWheel_1.translate(ax8);

		wheelDecor.updateTransformationState(transWheel_1);
		wheelDecor.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, wheelDecor);

		wheelDecor = new SolidDeformedCylinder(0.3f, 0.3f, 10.f, 15, 0.7f, 2f);
		wheelDecor.setTextureID("lava");
		wheelDecor.setAmbientFactor(Materials.AMBIENT_GROUND);
		wheelDecor.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		wheelDecor.setSpecularFactor(Materials.SPECULAR_GROUND);
		wheelDecor.shinessFactor = Materials.SHINESS_GROUND;

		transWheel_1 = new Transformation();
		float[] ax9 = { 0, 0f, 0, 1 };

		transWheel_1.translate(ax9);

		wheelDecor.updateTransformationState(transWheel_1);
		wheelDecor.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, wheelDecor);

		wheelDecor = new SolidDeformedCylinder(0.35f, 0.35f, 1.f, 15, 1f, 0.3f);
		wheelDecor.setTextureID("lava");
		wheelDecor.setAmbientFactor(Materials.AMBIENT_GROUND);
		wheelDecor.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		wheelDecor.setSpecularFactor(Materials.SPECULAR_GROUND);
		wheelDecor.shinessFactor = Materials.SHINESS_GROUND;

		transWheel_1 = new Transformation();
		float[] ax10 = { 0, 9f, 0, 1 };

		transWheel_1.translate(ax10);

		wheelDecor.updateTransformationState(transWheel_1);
		wheelDecor.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, wheelDecor);

		ball = new Ellipsoid(0.35f, 0.7f, 0.35f, 20, 20);

		ball.setTextureID("lava");
		ball.setAmbientFactor(Materials.AMBIENT_GROUND);
		ball.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		ball.setSpecularFactor(Materials.SPECULAR_GROUND);
		ball.shinessFactor = Materials.SHINESS_GROUND;
		ball.emmisionFactor = 100;

		transWheel_1 = new Transformation();
		float[] ax11 = { 0, 10.5f, 0, 1 };

		transWheel_1.translate(ax11);

		ball.updateTransformationState(transWheel_1);
		ball.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, ball);

		ball = new Ellipsoid(1f, 2f, 1f, 40, 40);

		ball.setTextureID("ground");
		ball.setAmbientFactor(Materials.AMBIENT_LIGHT);
		ball.setDiffuseFactor(Materials.DIFFUSE_LIGHT);
		ball.setSpecularFactor(Materials.SPECULAR_LIGHT);
		ball.shinessFactor = Materials.SHINESS_LIGHT;

		transWheel_1 = new Transformation();
		float[] ax12 = { 0, 10.5f, 0, 1 };

		transWheel_1.translate(ax12);

		ball.updateTransformationState(transWheel_1);
		ball.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, ball);

		Node.addChildrenToNode(groundNode, laternNode);

		tLatern.setIdentity();

		float[] j = { 14, 2, 29, 1 };
		tLatern.translate(j);

		laternNode.updateTransformationState(tLatern);

		laternNode = new Node();

		wheelDecor = new SolidDeformedCylinder(0.35f, 0.35f, 1.f, 15, 1f, 0.3f);
		wheelDecor.setTextureID("lava");
		wheelDecor.setAmbientFactor(Materials.AMBIENT_GROUND);
		wheelDecor.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		wheelDecor.setSpecularFactor(Materials.SPECULAR_GROUND);
		wheelDecor.shinessFactor = Materials.SHINESS_GROUND;

		transWheel_1 = new Transformation();

		transWheel_1.translate(ax8);

		wheelDecor.updateTransformationState(transWheel_1);
		wheelDecor.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, wheelDecor);

		wheelDecor = new SolidDeformedCylinder(0.3f, 0.3f, 10.f, 15, 0.7f, 2f);
		wheelDecor.setTextureID("lava");
		wheelDecor.setAmbientFactor(Materials.AMBIENT_GROUND);
		wheelDecor.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		wheelDecor.setSpecularFactor(Materials.SPECULAR_GROUND);
		wheelDecor.shinessFactor = Materials.SHINESS_GROUND;

		transWheel_1 = new Transformation();

		transWheel_1.translate(ax9);

		wheelDecor.updateTransformationState(transWheel_1);
		wheelDecor.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, wheelDecor);

		wheelDecor = new SolidDeformedCylinder(0.35f, 0.35f, 1.f, 15, 1f, 0.3f);
		wheelDecor.setTextureID("lava");
		wheelDecor.setAmbientFactor(Materials.AMBIENT_GROUND);
		wheelDecor.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		wheelDecor.setSpecularFactor(Materials.SPECULAR_GROUND);
		wheelDecor.shinessFactor = Materials.SHINESS_GROUND;

		transWheel_1 = new Transformation();

		transWheel_1.translate(ax10);

		wheelDecor.updateTransformationState(transWheel_1);
		wheelDecor.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, wheelDecor);

		ball = new Ellipsoid(0.35f, 0.7f, 0.35f, 20, 20);

		ball.setTextureID("lava");
		ball.setAmbientFactor(Materials.AMBIENT_GROUND);
		ball.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		ball.setSpecularFactor(Materials.SPECULAR_GROUND);
		ball.shinessFactor = Materials.SHINESS_GROUND;
		ball.emmisionFactor = 100;

		transWheel_1 = new Transformation();
		transWheel_1.translate(ax11);

		ball.updateTransformationState(transWheel_1);
		ball.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, ball);

		ball = new Ellipsoid(1f, 2f, 1f, 40, 40);

		ball.setTextureID("ground");
		ball.setAmbientFactor(Materials.AMBIENT_LIGHT);
		ball.setDiffuseFactor(Materials.DIFFUSE_LIGHT);
		ball.setSpecularFactor(Materials.SPECULAR_LIGHT);
		ball.shinessFactor = Materials.SHINESS_LIGHT;

		transWheel_1 = new Transformation();

		transWheel_1.translate(ax12);

		ball.updateTransformationState(transWheel_1);
		ball.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, ball);

		Node.addChildrenToNode(groundNode, laternNode);

		tLatern.setIdentity();

		float[] mj = { 26, 2, 29, 1 };
		tLatern.translate(mj);

		laternNode.updateTransformationState(tLatern);

		laternNode = new Node();

		wheelDecor = new SolidDeformedCylinder(0.35f, 0.35f, 1.f, 15, 1f, 0.3f);
		wheelDecor.setTextureID("lava");
		wheelDecor.setAmbientFactor(Materials.AMBIENT_GROUND);
		wheelDecor.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		wheelDecor.setSpecularFactor(Materials.SPECULAR_GROUND);
		wheelDecor.shinessFactor = Materials.SHINESS_GROUND;

		transWheel_1 = new Transformation();

		transWheel_1.translate(ax8);

		wheelDecor.updateTransformationState(transWheel_1);
		wheelDecor.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, wheelDecor);

		wheelDecor = new SolidDeformedCylinder(0.3f, 0.3f, 10.f, 15, 0.7f, 2f);
		wheelDecor.setTextureID("lava");
		wheelDecor.setAmbientFactor(Materials.AMBIENT_GROUND);
		wheelDecor.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		wheelDecor.setSpecularFactor(Materials.SPECULAR_GROUND);
		wheelDecor.shinessFactor = Materials.SHINESS_GROUND;

		transWheel_1 = new Transformation();

		transWheel_1.translate(ax9);

		wheelDecor.updateTransformationState(transWheel_1);
		wheelDecor.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, wheelDecor);

		wheelDecor = new SolidDeformedCylinder(0.35f, 0.35f, 1.f, 15, 1f, 0.3f);
		wheelDecor.setTextureID("lava");
		wheelDecor.setAmbientFactor(Materials.AMBIENT_GROUND);
		wheelDecor.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		wheelDecor.setSpecularFactor(Materials.SPECULAR_GROUND);
		wheelDecor.shinessFactor = Materials.SHINESS_GROUND;

		transWheel_1 = new Transformation();

		transWheel_1.translate(ax10);

		wheelDecor.updateTransformationState(transWheel_1);
		wheelDecor.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, wheelDecor);

		ball = new Ellipsoid(0.35f, 0.7f, 0.35f, 20, 20);

		ball.setTextureID("lava");
		ball.setAmbientFactor(Materials.AMBIENT_GROUND);
		ball.setDiffuseFactor(Materials.DIFFUSE_GROUND);
		ball.setSpecularFactor(Materials.SPECULAR_GROUND);
		ball.shinessFactor = Materials.SHINESS_GROUND;
		ball.emmisionFactor = 50;

		transWheel_1 = new Transformation();
		transWheel_1.translate(ax11);

		ball.updateTransformationState(transWheel_1);
		ball.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, ball);

		ball = new Ellipsoid(1f, 2f, 1f, 40, 40);

		ball.setTextureID("ground");
		ball.setAmbientFactor(Materials.AMBIENT_LIGHT);
		ball.setDiffuseFactor(Materials.DIFFUSE_LIGHT);
		ball.setSpecularFactor(Materials.SPECULAR_LIGHT);
		ball.shinessFactor = Materials.SHINESS_LIGHT;

		transWheel_1 = new Transformation();

		transWheel_1.translate(ax12);

		ball.updateTransformationState(transWheel_1);
		ball.setBufferID(bufferController.getBufferID());
		Node.addChildrenToNode(laternNode, ball);
		
		Node.addChildrenToNode(groundNode, laternNode);

		tLatern.setIdentity();

		float[] msj = { 26, 2, 11, 1 };
		tLatern.translate(msj);

		laternNode.updateTransformationState(tLatern);

		// ----------------------------------------------------------------------------
		// create entry Points
		/*
		 * Vector entryPointList =
		 * GameflowController.getInstance().entryPointList;
		 * 
		 * for(int i = 0; i < entryPointList.size();i++){ PlayGroundElement
		 * entryPoint = (PlayGroundElement)entryPointList.elementAt(i);
		 * 
		 * SolidDeformedCylinder entryPointStructure = new
		 * SolidDeformedCylinder(0.5f, 0.5f, 2,15, 0.5f, 0.5f);
		 * 
		 * entryPointStructure.setTextureID("ground");
		 * entryPointStructure.setAmbientFactor(Materials.AMBIENT_LIGHT);
		 * entryPointStructure.setDiffuseFactor(Materials.DIFFUSE_LIGHT);
		 * entryPointStructure.setSpecularFactor(Materials.SPECULAR_LIGHT);
		 * entryPointStructure.shinessFactor = Materials.SHINESS_LIGHT;
		 * 
		 * transWheel_1.setIdentity(); float[] s = {entryPoint.posX + 0.5f ,
		 * entryPoint.posY, entryPoint.posZ + 0.5f, 1};
		 * 
		 * 
		 * transWheel_1.translate(s);
		 * 
		 * entryPointStructure.updateTransformationState(transWheel_1);
		 * entryPointStructure.setBufferID(bufferController.getBufferID());
		 * Node.addChildrenToNode(groundNode, entryPointStructure); }
		 */
		// -----------------------------------------------------------------------------
	}

	private final void createStructure_sewer(Node node) {

	}

	private final void createLevel1(Node node) {
		this.initializePlayGround_1();
		this.createUniqueMonsters_1();
		this.createStructure_level_1(node);
		this.createEntryPoints_level_1();
		this.createHeros_level_1();
		SelectionController.getInstance().selectHero(
				(Node) GameflowController.getInstance().heroList.elementAt(0));
	}

	public final void createEntryPoints(int level) {

		if (level == GeometryFactory.LEVEL_1) {
			this.createEntryPoints_level_1();
			return;
		}
	}

	

	private final void createEntryPoints_level_1() {
		Vector entryPointList = GameflowController.getInstance().entryPointList;

		entryPointList.removeAllElements();

		for (int i = 0; i < this.entryPointList.size(); i++) {
			entryPointList.add(((PlayGroundElement)this.entryPointList.elementAt(i)).clone());
		}
	}

	private final void createHeros_level_1() {
		PlayGroundElement playGroundElement = new PlayGroundElement(6, 2, 12,
				GeometryFactory.DWARF_WORRIOR);
		this.renderer.sceneGraph.addNewElement(playGroundElement);

		playGroundElement = new PlayGroundElement(5, 2, 12,
				GeometryFactory.DWARF_RANGED);
		this.renderer.sceneGraph.addNewElement(playGroundElement);

		playGroundElement = new PlayGroundElement(13, 2, 13,
				GeometryFactory.DWARF_GEODE);
		this.renderer.sceneGraph.addNewElement(playGroundElement);
	}
}
